* FeynHiggs.F
* the command-line front end for FeynHiggs
* this file is part of FeynHiggs
* last modified 29 Jul 07 th

#include "FeynHiggs.h"
#include "debug.h"
#include "version.h"


	program FeynHiggs
	implicit none

	integer argc, error, i
	character argv*100

#if U77EXT
	integer iargc, lnblnk
	external iargc, lnblnk
#endif

	integer mssmpart, fieldren, tanbren, higgsmix, p2approx
	integer looplevel, tlRunningMT, tlBotResum, tlCplxApprox
	double precision scalefactor

	character*55 dmssmpart(-1:5), dfieldren(-1:5),
     &    dtanbren(-1:3), dhiggsmix(0:5), dp2approx(-1:5),
     &    dlooplevel(-1:3), dtlRunningMT(-1:2),
     &    dtlBotResum(-1:2), dtlCplxApprox(-1:4)

	data mssmpart /4/
	data fieldren /0/
	data tanbren /0/
	data higgsmix /2/
	data p2approx /0/
	data looplevel /2/
	data tlRunningMT /1/
	data tlBotResum /1/
	data tlCplxApprox /3/

	data scalefactor /1/

	data dmssmpart /
     &    "???",
     &    "MT^4 approximation (for real parameters only)",
     &    "top/stop sector",
     &    "top/stop + bottom/sbottom sector",
     &    "full (s)quark/(s)lepton sector",
     &    "full MSSM [recommended]",
     &    "???" /

	data dfieldren /
     &    "???",
     &    "DRbar field ren. [strongly recommended]",
     &    "on-shell field ren., Dabelstein convention",
     &    "on-shell field ren., `Goldstone-pole' version",
     &    "on-shell field ren., MF I",
     &    "on-shell field ren., MF II",
     &    "???" /

	data dtanbren /
     &    "???",
     &    "DRbar TB-ren. [strongly recommended]",
     &    "field renormalization part only for TB (Dabelstein)",
     &    "mixed field/on-shell renormalization for TB",
     &    "???" /

	data dhiggsmix /
     &    "???",
     &    "no mixing in the neutral Higgs sector",
     &    "2x2 (h0-HH) mixing in the neutral Higgs sector",
     &    "3x3 (h0-HH-A0) mixing in the neutral Higgs sector",
     &    "4x4 (G0-h0-HH-A0) mixing [experimental]",
     &    "???" /

	data dp2approx /
     &    "???",
     &    "no approximation [recommended]",
     &    "p^2 = 0 approximation",
     &    "self-energies are calculated at p^2 = M_tree^2",
     &    "imaginary parts of self-energies are discarded",
     &    "UHiggs is evaluated at p^2 = 0",
     &    "???" /

	data dlooplevel /
     &    "???",
     &    "tree level only",
     &    "include 1-loop corrections",
     &    "include 2-loop corrections",
     &    "???" /

	data dtlRunningMT /
     &    "???",
     &    "pole top mass in 2-loop corrections",
     &    "running top mass in 2-loop corrections [recommended]",
     &    "???" /

	data dtlBotResum /
     &    "???",
     &    "non-resummed MB in 2-loop corrections",
     &    "resummed MB in 2-loop corrections [recommended]",
     &    "???" /

	data dtlCplxApprox /
     &    "???",
     &    "all corrections evaluated in the rMSSM",
     &    "a_s a_t in cMSSM only",
     &    "a_s a_t in cMSSM, rest in rMSSM",
     &    "a_s a_t in cMSSM, interpolation in phase for rest",
     &    "???" /

	call pageron

	argc = iargc()
	if( argc .lt. 1 .or. argc .gt. 3 ) goto 999

#define GetFlag(i) ichar(argv(i:i))-48

	if( argc .ge. 2 ) then
	  call getarg(2, argv)
	  if( lnblnk(argv) .ne. 9 ) goto 999
	  mssmpart = GetFlag(1)
	  fieldren = GetFlag(2)
	  tanbren = GetFlag(3)
	  higgsmix = GetFlag(4)
	  p2approx = GetFlag(5)
	  looplevel = GetFlag(6)
	  tlRunningMT = GetFlag(7)
	  tlBotResum = GetFlag(8)
	  tlCplxApprox = GetFlag(9)
	endif

#define ShowFlag(t,f,lo,hi) t, f, d f(min(max(f,lo-1),hi+1))

101	format(A, " = ", FLAG_FMT, T22, "# ", A)
	print 101,
     &    ShowFlag("mssmpart",     mssmpart,       0, 4),
     &    ShowFlag("fieldren",     fieldren,       0, 4),
     &    ShowFlag("tanbren",      tanbren,        0, 2),
     &    ShowFlag("higgsmix",     higgsmix,       1, 4),
     &    ShowFlag("p2approx",     p2approx,       0, 4),
     &    ShowFlag("looplevel",    looplevel,      0, 2),
     &    ShowFlag("tlRunningMT",  tlRunningMT,  0, 1),
     &    ShowFlag("tlBotResum",   tlBotResum,   0, 1),
     &    ShowFlag("tlCplxApprox", tlCplxApprox, 0, 3)

	call FHSetFlags(error,
     &    mssmpart, fieldren, tanbren, higgsmix, p2approx,
     &    looplevel, tlRunningMT, tlBotResum, tlCplxApprox)
	if( error .ne. 0 ) stop

	if( argc .ge. 3 ) then
	  call getarg(3, argv)
	  read(argv, *, err=99) scalefactor
99	  continue
	  if( scalefactor .le. 0 ) scalefactor = 1
	endif


	call getarg(1, argv)

	call SLHAMode(error, argv, scalefactor)

* File format not recognized:
	if( error .eq. 2 )
     &    call FHMode(error, argv, scalefactor, higgsmix)

	if( error .gt. 0 ) Err("Could not process "//argv)
	goto 1


#define DESC_FMT (/I5,": ",A)

100	format ("FeynHiggs ", FHVERSION
     &    /"built on ", FHBUILDDATE
     &    //"USAGE:  ", A, " <inputfile> [<flags> [<scalefactor>]]"
     &    //"<inputfile> is a file containing the parameters"
     &    //"<flags> is a string of 9 digits with"
     &    /"  #1 = mssmpart = scope of the calculation",
     &    5 DESC_FMT
     &    /"  #2 = fieldren = 1-loop field renormalization",
     &    5 DESC_FMT
     &    /"  #3 = tanbren = 1-loop tan(beta) renormalization",
     &    3 DESC_FMT
     &    /"  #4 = higgsmix = mixing in the neutral Higgs sector",
     &    4 DESC_FMT
     &    /"  #5 = p2approx = approximation for the 1-loop result",
     &    5 DESC_FMT
     &    /"  #6 = looplevel, inclusion of higher-order corrections",
     &    3 DESC_FMT
     &    /"  #7 = tlRunningMT, MT in the 2-loop corrections",
     &    2 DESC_FMT
     &    /"  #8 = tlBotResum, MB in the 2-loop corrections",
     &    2 DESC_FMT
     &    /"  #9 = tlCplxApprox, complex parameters in the 2-loop corrections",
     &    4 DESC_FMT
     &    //"<scalefactor> is an optional factor that multiplies"
     &    /"  the scale at which the corrections are computed (MT)"/)

999	call getarg(0, argv)
	print 100, Strip(argv),
     &    (i, dmssmpart(i),     i = 0, 4),
     &    (i, dfieldren(i),     i = 0, 4),
     &    (i, dtanbren(i),      i = 0, 2),
     &    (i, dhiggsmix(i),     i = 1, 4),
     &    (i, dp2approx(i),     i = 0, 4),
     &    (i, dlooplevel(i),    i = 0, 2),
     &    (i, dtlRunningMT(i),  i = 0, 1),
     &    (i, dtlBotResum(i),   i = 0, 1),
     &    (i, dtlCplxApprox(i), i = 0, 3)

1	call flush(6)
	call pageroff
	end


************************************************************************

	subroutine e_(msg)
	implicit none
	character*(*) msg

#if VT100
100	format(3A)
	print 100, BOLD//RED//"Error: ", msg, RESET
#else
100	format(A)
	print 100, "Error: ", msg
#endif
	end


************************************************************************

	subroutine w_(msg)
	implicit none
	character*(*) msg

#if VT100
100	format(3A)
	print 100, BOLD//BLUE//"Warning: ", msg, RESET
#else
100	format(2A)
	print 100, "Warning: ", msg
#endif
	end
