* DeltaMB.F
* calculation of the Delta-MB corrections
* this file is part of FeynHiggs
* last modified 24 Apr 08 th

#include "externals.h"
#include "debug.h"

* Note:
* Mdy(3,1) and DeltaMBc(1) are the values used in the 1L corrections
*   (and must `fit' to the 2L part)
* Mdy(3,2) and DeltaMBc(2) are the values used in decays etc.


	subroutine MBfromPara(error)
	implicit none
	integer error

#include "FH.h"
#define __SUBROUTINE__ "MBfromPara"

	double precision old
	integer i
	logical fail

	if( tlBotResum .ne. 0 ) then
	  do i = 1, DeltaMB_maxpass
	    call EigenSf(fail, MSf(1,4,3), MSf2(1,4,3), USf(1,1,4,3),
     &        DSS2(1,4,3), Mdy(3,2), Xf(4,3,3), 1)

	    old = Mdy(3,2)
	    call ResumMB

	    if( debuglevel .ge. 2 )
     &        DPARA "MBfromPara: Mdy(3,2) =", Mdy(3,2) ENDL

	    if( abs(Mdy(3,2) - old) .lt. 1D-8 ) goto 1
	  enddo

	  if( DeltaMB_maxpass .gt. 1 )
     &      Warning("resummation of MB did not converge")

1	  continue
	endif

	if( looplevel .ge. 2 ) then
	  call TLShifts(error)
	  if( debuglevel .ge. 2 )
     &      DPARA "MBfromPara: Mdy(3,1) =", Mdy(3,1) ENDL
	  call CalcRenSETL(error)
	endif

	call BUpdate
	end

#undef __SUBROUTINE__


************************************************************************

	subroutine MBfromMasses(error, X)
	implicit none
	integer error
	double complex X(2,2)

#include "FH.h"
#define __SUBROUTINE__ "MBfromMasses"

	double precision old
	integer i, err
	logical fail

	if( tlBotResum .ne. 0 ) call ResumMB

	if( debuglevel .ge. 2 )
     &    DPARA "MBfromMasses: Mdy(3,2) =", Mdy(3,2) ENDL

	if( looplevel .ge. 2 ) then
	  do i = 1, DeltaMB_maxpass
	    DSS2(1,4,3) = DBLE(X(1,1)) - DBLE(X(2,2))
	    DSS2(1,5,3) = DSS2(1,4,3)
	    DSS2(2,4,3) = DBLE(X(2,2)) - Mdy(3,1)**2
	    DSS2(2,5,3) = DSS2(2,4,3)
c	    Xf(4,3,3) = DCONJG(X(1,2))/Mdy(3,1)

	    old = Mdy(3,1)
	    err = error
	    call TLShifts(err)
	    if( debuglevel .ge. 2 )
     &        DPARA "MBfromMasses: Mdy(3,1) =", Mdy(3,1) ENDL
	    if( abs(Mdy(3,1) - old) .lt. 1D-8 ) goto 1
	  enddo

	  if( DeltaMB_maxpass .gt. 1 )
     &      Warning("resummation of MB did not converge")

1	  error = err
	  call EigenSf(fail, MSf(1,4,3), MSf2(1,4,3), USf(1,1,4,3),
     &      DSS2(1,4,3), Mdy(3,1), Xf(4,3,3), 1)
	  if( fail ) Error("negative sbottom mass squares")

	  call CalcRenSETL(error)
	endif

	call BUpdate

	DSS2(1,4,3) = DBLE(X(1,1)) - DBLE(X(2,2))
	DSS2(1,5,3) = DSS2(1,4,3)
	DSS2(2,4,3) = DBLE(X(2,2)) - Mdy2(3,1)
	DSS2(2,5,3) = DSS2(2,4,3)
c	Xf(4,3,3) = DCONJG(X(1,2))/Mdy(3,1)
	end


************************************************************************

	subroutine ResumMB
	implicit none

#include "FH.h"

	double precision Tdmb
	external Tdmb

	DeltaMBc(1) = TB*DCONJG(MUE)*(
     &    2/(3*pi)*Alfas*DCONJG(M_3)*
     &      Tdmb(MSf2(1,4,3), MSf2(2,4,3), Sq(M_3)) +
     &    (MTrun/174.1D0)**2/(4*pi)**2*DCONJG(Af(3,3,3))*
     &      Tdmb(MSf2(1,3,3), MSf2(2,3,3), Sq(MUE)) )
	DeltaMBc(2) = DeltaMBc(1)

	Mdy(3,1) = MBrun/abs(1 + DeltaMBc(1))
	Mdy(3,2) = Mdy(3,1)
	end


************************************************************************
	
	double precision function Tdmb(x, y, z)
	implicit none
	double precision x, y, z

	double precision n, d

	n = 0
	d = 1

	if( x .ne. y ) then
	  if( x*y .ne. 0 ) n = n + x*y*log(x/y)
	  d = d*(x - y)
	endif
	if( x .ne. z ) then
	  if( x*z .ne. 0 ) n = n + x*z*log(z/x)
	  d = d*(x - z)
	endif
	if( y .ne. z ) then
	  if( y*z .ne. 0 ) n = n + y*z*log(y/z)
	  d = d*(y - z)
	endif

	Tdmb = n/d
	end       


************************************************************************
	
	subroutine BUpdate
	implicit none

#include "FH.h"

	Mdy2(3,1) = Mdy(3,1)**2

	Mdy(3,2) = MBrun/abs(1 + DeltaMBc(2))
	Mdy2(3,2) = Mdy(3,2)**2

	Hff(1,3,1) = 1 - CA/(SA*TB)*DeltaMBc(1)
	Hff(2,3,1) = 1 + SA/(CA*TB)*DeltaMBc(1)

	Hff(1,3,2) = 1 - CA/(SA*TB)*DeltaMBc(2)
	Hff(2,3,2) = 1 + SA/(CA*TB)*DeltaMBc(2)
	end

