* HiggsProd.F
* approximate calculation of the Higgs production cross-sections
* neutral-Higgs cross-section data are from Fabio Maltoni:
*   http://maltoni.home.cern.ch/maltoni/TeV4LHC
* charged-Higgs cross-section data are from Tilman Plehn
*   http://www.ph.ed.ac.uk/~tplehn/charged_higgs
* this file is part of FeynHiggs
* last modified 4 Feb 08 th

#include "externals.h"
#include "debug.h"
#include "FHCouplings.h"


	subroutine FHHiggsProd(error, prodxs)
	implicit none
	integer error
	double precision prodxs(nprodxs)

#include "FH.h"
#define __SUBROUTINE__ "FHHiggsProd"

	integer i, h
	double precision sqrtm, logm, m, gSM, c

	double precision kTopTev, kBotReTev, kBotImTev
	double precision kTopLHC, kBotReLHC, kBotImLHC

	logical blurb
	character*2 higgs(4)
	data blurb /.TRUE./
	data higgs /"h0", "HH", "A0", "Hp"/

#include "kfactorTev.f"
#include "kfactorLHC.f"

	call CheckCouplings(error)
	if( error .ne. 0 ) return

	if( blurb ) then
	  print *, "FHHiggsProd contains code by"
	  print *, "F. Maltoni et al. (SM Higgs production cross-sections)"
	  print *, "http://maltoni.home.cern.ch/maltoni/TeV4LHC"
	  print *, "T. Plehn et al. (charged Higgs production cross-sections)"
	  print *, "http://www.ph.ed.ac.uk/~tplehn/charged_higgs"
	  blurb = .FALSE.
	endif

	do i = 1, nprodxs
	  prodxs(i) = -999
	enddo

	do h = 1, 3
	  if( MHiggs(h) .gt. 80 ) then
	    sqrtm = sqrt(MHiggs(h))
	    logm = log(MHiggs(h))

#include "NHiggsProdFits.f"

	    gSM = GammaSM(H0FF(h,4,3))
	    if( gSM .eq. 0 ) then
	      bbhTev(h) = 0
	      bbhTevSM(h) = 0
	      bbhLHC(h) = 0
	      bbhLHCSM(h) = 0
	      btagbhTev(h) = 0
	      btagbhTevSM(h) = 0
	      btagbhLHC(h) = 0
	      btagbhLHCSM(h) = 0
	    else
	      c = Gamma(H0FF(h,4,3))/gSM
	      bbhTev(h) = c*bbhTevSM(h)
	      bbhLHC(h) = c*bbhLHCSM(h)
	      btagbhTev(h) = c*btagbhTevSM(h)
	      btagbhLHC(h) = c*btagbhLHCSM(h)
	    endif

	    c = ratios(H0FF(h,3,3))
	    tthTev(h) = c*tthTevSM(h)
	    tthLHC(h) = c*tthLHCSM(h)

	    hggU(1,3) = kTopTev(MHiggs(h), logm)
	    hggU(2,3) = hggU(1,3)
	    hggU(3,3) = hggU(1,3)
	    hggDRe(1,3) = kBotReTev(MHiggs(h), logm)
	    hggDRe(2,3) = hggDRe(1,3)
	    hggDRe(3,3) = hggDRe(1,3)
	    hggDIm(1,3) = kBotImTev(MHiggs(h), logm)
	    hggDIm(2,3) = hggDIm(1,3)
	    hggDIm(3,3) = hggDIm(1,3)
	    call hggSM(h, gSM)
	    if( gSM .eq. 0 ) then
	      gghTev(h) = 0
	      gghTevSM(h) = 0
	    else
	      call hgg(h, c)
	      gghTev(h) = c/gSM*gghTevSM(h)
	    endif

	    hggU(1,3) = kTopLHC(MHiggs(h), logm)
	    hggU(2,3) = hggU(1,3)
	    hggU(3,3) = hggU(1,3)
	    hggDRe(1,3) = kBotReLHC(MHiggs(h), logm)
	    hggDRe(2,3) = hggDRe(1,3)
	    hggDRe(3,3) = hggDRe(1,3)
	    hggDIm(1,3) = kBotImLHC(MHiggs(h), logm)
	    hggDIm(2,3) = hggDIm(1,3)
	    hggDIm(3,3) = hggDIm(1,3)
	    call hggSM(h, gSM)
	    if( gSM .eq. 0 ) then
	      gghLHC(h) = 0
	      gghLHCSM(h) = 0
	    else
	      call hgg(h, c)
	      gghLHC(h) = c/gSM*gghLHCSM(h)
	    endif

	    c = ratios(H0VV(h,4))
	    qqhTev(h) = c*qqhTevSM(h)
	    qqhLHC(h) = c*qqhLHCSM(h)
	    WhTev(h) = c*WhTevSM(h)
	    WhLHC(h) = c*WhLHCSM(h)

	    c = ratios(H0VV(h,3))
	    ZhTev(h) = c*ZhTevSM(h)
	    ZhLHC(h) = c*ZhLHCSM(h)

	    StSthLHC(h) = StSthLHC(h)*
     &        Sq(Coupling(H0SfSf(h,1,1,3,3)))
	  endif

	  if( debuglevel .ge. 2 ) then
	    DPROD higgs(h), " production cross-sections (MSSM, SM):" ENDL
	    DPROD "bbhTev    =", bbhTev(h), bbhTevSM(h)		ENDL
	    DPROD "bbhLHC    =", bbhLHC(h), bbhLHCSM(h)		ENDL
	    DPROD "btagbhTev =", btagbhTev(h), btagbhTevSM(h)	ENDL
	    DPROD "btagbhLHC =", btagbhLHC(h), btagbhLHCSM(h)	ENDL
	    DPROD "gghTev    =", gghTev(h), gghTevSM(h)		ENDL
	    DPROD "gghLHC    =", gghLHC(h), gghLHCSM(h)		ENDL
	    DPROD "qqhTev    =", qqhTev(h), qqhTevSM(h)		ENDL
	    DPROD "qqhLHC    =", qqhLHC(h), qqhLHCSM(h)		ENDL
	    DPROD "tthTev    =", tthTev(h), tthTevSM(h)		ENDL
	    DPROD "tthLHC    =", tthLHC(h), tthLHCSM(h)		ENDL
	    DPROD "WhTev     =", WhTev(h), WhTevSM(h)		ENDL
	    DPROD "WhLHC     =", WhLHC(h), WhLHCSM(h)		ENDL
	    DPROD "ZhTev     =", ZhTev(h), ZhTevSM(h)		ENDL
	    DPROD "ZhLHC     =", ZhLHC(h), ZhLHCSM(h)		ENDL
	    DPROD "StSthLHC  =", StSthLHC(h)			ENDL
	  endif
	enddo

	if( MHiggs(4) .gt. 80 ) then
	  sqrtm = sqrt(MHiggs(4))

#include "CHiggsProdFits.f"

	  tHmLHC = tHmLHC/abs(1 + DeltaMBc(2))**2
	endif

	if( debuglevel .ge. 2 )
     &    DPROD "tHmLHC    =", tHmLHC ENDL
	end

