* Para.F
* initialization routines for the various parameter blocks
* this file is part of FeynHiggs
* last modified 8 Apr 08 th

* based on mssm_ini.F from FormCalc:
* Christian Schappacher, 17 Feb 99
* last modified 15 Jun 01 th
* (almost) completely rewritten to be independent
* of any external libraries and
* adapted to Higgs mass routines by Markus Frank.
* 01.10.2002 sh: inclusion of O(als alb) resummation

#include "externals.h"
#include "debug.h"

#define __SUBROUTINE__ "FHSetPara"


	subroutine BeginPara(error, scalefactor_, MT_, TB_, M_3_)
	implicit none
	integer error
	double precision scalefactor_, MT_, TB_
	double complex M_3_

#include "FH.h"
#include "FHRecord.h"
#include "looptools.h"

	external recordnamesini

	integer i, g
	double precision i3part, qpart

	double precision Run
	external Run

	para_valid = 0
	higgs_valid = 0
	coup_valid = 0

	error = 0

	CheckFlags()

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "BeginPara input parameters:"	ENDL
	  DPARA "----------------------------"	ENDL
	  DPARA "scalefactor =", scalefactor_	ENDL
	  DTAG(iMT) MT_
	  DTAG(iTB) TB_
	  DTAG(iRe(iM3)) DBLE(M_3_)
	  DTAG(iIm(iM3)) DIMAG(M_3_)
	endif

	MT = MT_
	if( MT .lt. 0 ) Error("invalid MT")
	MT2 = MT**2

	DeltaMBc(1) = 0
	DeltaMBc(2) = 0
	DeltaMB_maxpass = 20

	do g = 1, 3
	  MdySM(g) = Mf(4,g)
	  Mdy(g,1) = Mf(4,g)
	  Mdy(g,2) = Mdy(g,1)
	  Mdy2(g,1) = Mf2(4,g)
	  Mdy2(g,2) = Mdy2(g,1)
	enddo

	do i = 1, 2*3*2
	  Hff_flat(i) = 1
	enddo

	do i = 1, 3*3
	  hggU_flat(i) = 1
	  hggDRe_flat(i) = 1
	  hggDIm_flat(i) = 1
	enddo

	call ffini
	scalefactor = scalefactor_
	call setmudim(MT2*scalefactor**2)

	Alfas = AlfasMZ/(1 + (11 - 10/3D0)/(4*pi)*AlfasMZ*log(MT2/MZ2))
c	Alfas = AlphaS(MT)
	Alfas2 = Alfas**2
	GS = sqrt(4*pi*Alfas)

	MTrun = MT/(1 - .075D0/(2*pi) + 4/(3*pi)*Alfas)
	MBrun = MB*Run(MB, MT)

	M_3 = M_3_
	MGl = abs(M_3)
	MGl2 = MGl**2

	TB  = TB_
	if( TB .lt. 0 ) Error("invalid TB")
	TB2 = TB**2
	CB2 = 1/(1 + TB2)
	CB  = sqrt(CB2)
	SB2 = (1 - CB)*(1 + CB)
	SB  = CB*TB
	S2B = 2*TB*CB2
	C2B = (CB - SB)*(CB + SB)

	i3part = .5D0*MZ2*C2B
	qpart = SW2*MZ2*C2B
	DSf(2,1) = 0
	DSf(1,1) = +i3part
	DSf(2,2) = Qe*qpart
	DSf(1,2) = -i3part - DSf(2,2)
	DSf(2,3) = Qu*qpart
	DSf(1,3) = +i3part - DSf(2,3)
	DSf(2,4) = Qd*qpart
	DSf(1,4) = -i3part - DSf(2,4)

	nmfv = 0
	do i = 1, 3*3*3
	  Af_flat(i) = 0
	enddo
	do i = 1, 6*6*2
	  UASf_flat(i) = 0
	enddo
	do i = 1, 2*6*6
	  deltaSf_flat(i) = 0
	enddo

	do i = 1, semax
	  se2R(i) = 0
	enddo

	Qtau = 0
	Qt = 0
	Qb = 0
	end


************************************************************************

	subroutine HiggsMasses(error, MA0_, MHp_, QA0_)
	implicit none
	integer error
	double precision MA0_, MHp_, QA0_

#include "FH.h"
#include "FHRecord.h"

	double precision avgMH2, deltaMH2

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "HiggsMasses input:"		ENDL
	  DPARA "----------------------------"	ENDL
	  DTAG(iMA0) MA0_
	  DTAG(iMHp) MHp_
	endif

	if( MA0_ .ge. 0 ) then
	  MA0 = max(MA0_, 1D-3)
	  MA02 = MA0**2
	  MHp2 = MA02 + MW2
	  MHp = sqrt(MHp2)
	  inputmass = 3
	else if( MHp_ .gt. MW ) then
	  MHp = MHp_
	  MHp2 = MHp**2
	  MA02 = (MHp - MW)*(MHp + MW)
	  MA0 = sqrt(MA02)
	  inputmass = 4
	else
	  Error("neither MA0 nor MHp given")
	  MA02 = 1
	  return
	endif

* these are the tree-level Higgs masses:
	avgMH2 = .5D0*(MA02 + MZ2)
	deltaMH2 = MA0*MZ*C2B
	deltaMH2 = sqrt((avgMH2 - deltaMH2)*(avgMH2 + deltaMH2))
	Mh02 = avgMH2 - deltaMH2
	MHH2 = avgMH2 + deltaMH2
	Mh0 = -1
	if( Mh02 .ge. 0 ) Mh0 = sqrt(Mh02)
	MHH = -1
	if( MHH2 .ge. 0 ) MHH = sqrt(MHH2)

	CA2 = .5D0*(SB2*MA02 + CB2*MZ2 - Mh02)/deltaMH2
	CA  = sqrt(CA2)
	SA  = -SB*CB*avgMH2/(CA*deltaMH2)
	SA2 = SA**2
	S2A = 2*SA*CA
	C2A = (CA - SA)*(CA + SA)
	SAB = SA*CB + CA*SB
	CAB = CA*CB - SA*SB
	SBA = SB*CA - CB*SA
	CBA = CB*CA + SB*SA
	end


************************************************************************

	subroutine CharginoMasses(error, M_2_, MUE_)
	implicit none
	integer error
	double complex M_2_, MUE_

#include "FH.h"
#include "FHRecord.h"

	double complex X(2,2)

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "CharginoMasses input:"		ENDL
	  DPARA "----------------------------"	ENDL
	  DTAG(iRe(iM2)) DBLE(M_2_)
	  DTAG(iIm(iM2)) DIMAG(M_2_)
	  DTAG(iRe(iMUE)) DBLE(MUE_)
	  DTAG(iIm(iMUE)) DIMAG(MUE_)
	endif

	M_2 = M_2_
	MUE = MUE_

	X(1,1) = M_2
	X(1,2) = sqrt2*MW*SB
	X(2,1) = sqrt2*MW*CB
	X(2,2) = MUE

	call SVD(2, 2, X,2, MCha, UCha,2, VCha,2, 1)

	MCha2(1) = MCha(1)**2
	MCha2(2) = MCha(2)**2

	MUETB(3) = DCONJG(MUE)/TB
	MUETB(2) = DCONJG(MUE)*TB
	MUETB(4) = MUETB(2)
	end


************************************************************************

	subroutine CharginoPara(error, MCha_, UCha_, VCha_)
	implicit none
	integer error
	double complex MCha_(2), UCha_(2,2), VCha_(2,2)

#include "FH.h"

	double complex X(2,2)

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "CharginoPara input:"		ENDL
	  DPARA "----------------------------"	ENDL
	  DPARA "MCha(1) =", DBLE(MCha_(1))	ENDL
	  DPARA "    (2) =", DBLE(MCha_(2))	ENDL
	  DPARA "UCha(1,1) =", UCha_(1,1)	ENDL
	  DPARA "    (1,2) =", UCha_(1,2)	ENDL
	  DPARA "    (2,1) =", UCha_(2,1)	ENDL
	  DPARA "    (2,2) =", UCha_(2,2)	ENDL
	  DPARA "VCha(1,1) =", VCha_(1,1)	ENDL
	  DPARA "    (1,2) =", VCha_(1,2)	ENDL
	  DPARA "    (2,1) =", VCha_(2,1)	ENDL
	  DPARA "    (2,2) =", VCha_(2,2)	ENDL
	endif

	MCha(1) = DBLE(MCha_(1))
	MCha2(1) = MCha(1)**2
	MCha(2) = DBLE(MCha_(2))
	MCha2(2) = MCha(2)**2

	UCha(1,1) = UCha_(1,1)
	UCha(2,1) = UCha_(2,1)
	UCha(1,2) = UCha_(1,2)
	UCha(2,2) = UCha_(2,2)

	VCha(1,1) = VCha_(1,1)
	VCha(2,1) = VCha_(2,1)
	VCha(1,2) = VCha_(1,2)
	VCha(2,2) = VCha_(2,2)

	call ReverseSVD(2, X,2, MCha, UCha,2, VCha,2)

	M_2 = X(1,1)
	MUE = X(2,2)

	MUETB(3) = DCONJG(MUE)/TB
	MUETB(2) = DCONJG(MUE)*TB
	MUETB(4) = MUETB(2)
	end


************************************************************************

	subroutine NeutralinoMasses(error, M_1_)
	implicit none
	integer error
	double complex M_1_

#include "FH.h"
#include "FHRecord.h"

	double complex Y(4,4)

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "NeutralinoMasses input:"	ENDL
	  DPARA "----------------------------"	ENDL
	  DTAG(iRe(iM1)) DBLE(M_1_)
	  DTAG(iIm(iM1)) DIMAG(M_1_)
	endif

	M_1 = M_1_
* We use the SUSY-GUT relation here:
	if( abs(M_1) .lt. 1D-5 ) M_1 = 5/3D0*SW2/CW2*M_2

* Calculation of Neutralino masses and mixing matrix:
	Y(1,1) = M_1
	Y(2,2) = M_2
	Y(3,3) = 0
	Y(4,4) = 0
	Y(1,2) = 0
	Y(1,3) = -MZ*SW*CB
	Y(1,4) = MZ*SW*SB
	Y(2,3) = MZ*CW*CB
	Y(2,4) = -MZ*CW*SB
	Y(3,4) = -MUE

	call TakagiFactor(4, Y,4, MNeu, ZNeu,4, 1)

	MNeu2(1) = MNeu(1)**2
	MNeu2(2) = MNeu(2)**2
	MNeu2(3) = MNeu(3)**2
	MNeu2(4) = MNeu(4)**2
	end


************************************************************************

	subroutine NeutralinoPara(error, MNeu_, ZNeu_)
	implicit none
	integer error
	double complex MNeu_(4), ZNeu_(4,4)

#include "FH.h"

	integer i, j
	double complex Y(4,4)

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "NeutralinoPara input:"		ENDL
	  DPARA "----------------------------"	ENDL
	  DPARA "MNeu(1) =", DBLE(MNeu_(1))	ENDL
	  DPARA "    (2) =", DBLE(MNeu_(2))	ENDL
	  DPARA "    (3) =", DBLE(MNeu_(3))	ENDL
	  DPARA "    (4) =", DBLE(MNeu_(4))	ENDL
	  DPARA "ZNeu(1,1) =", ZNeu_(1,1)	ENDL
	  DPARA "    (1,2) =", ZNeu_(1,2)	ENDL
	  DPARA "    (1,3) =", ZNeu_(1,3)	ENDL
	  DPARA "    (1,4) =", ZNeu_(1,4)	ENDL
	  DPARA "    (2,1) =", ZNeu_(2,1)	ENDL
	  DPARA "    (2,2) =", ZNeu_(2,2)	ENDL
	  DPARA "    (2,3) =", ZNeu_(2,3)	ENDL
	  DPARA "    (2,4) =", ZNeu_(2,4)	ENDL
	  DPARA "    (3,1) =", ZNeu_(3,1)	ENDL
	  DPARA "    (3,2) =", ZNeu_(3,2)	ENDL
	  DPARA "    (3,3) =", ZNeu_(3,3)	ENDL
	  DPARA "    (3,4) =", ZNeu_(3,4)	ENDL
	  DPARA "    (4,1) =", ZNeu_(4,1)	ENDL
	  DPARA "    (4,2) =", ZNeu_(4,2)	ENDL
	  DPARA "    (4,3) =", ZNeu_(4,3)	ENDL
	  DPARA "    (4,4) =", ZNeu_(4,4)	ENDL
	endif

	do i = 1, 4
	  MNeu(i) = DBLE(MNeu_(i))
	  MNeu2(i) = MNeu(i)**2
	  if( MNeu(i) .ge. 0 ) then
	    do j = 1, 4
	      ZNeu(i,j) = ZNeu_(i,j)
	    enddo
	  else
	    MNeu(i) = -MNeu(i)
	    do j = 1, 4
	      ZNeu(i,j) = -cI*ZNeu_(i,j)
	    enddo
	  endif
	enddo

	call ReverseTakagi(4, Y,4, MNeu, ZNeu,4)

	M_1 = Y(1,1)
	end


************************************************************************

	subroutine SfermionMasses(error, g,
     &    MSL2_, MSE2_, MSQ2_, MSU2_, MSD2_,
     &    Ae_, Au_, Ad_,
     &    Qtau_, Qt_, Qb_)
	implicit none
	integer error, g
	double precision MSL2_, MSE2_, MSQ2_, MSU2_, MSD2_
	double complex Ae_, Au_, Ad_
	double precision Qtau_, Qt_, Qb_

#include "FH.h"
#include "FHRecord.h"

	logical fail
	integer t, sort, r
	double precision m, Q2, m1, m2
	double complex c, s
	double precision dMSf2(2,2:4), dtSf(2:4)
	double complex MSferm(2,2:4), USferm(2,2,2:4)

	double precision sincorr, dth
	double complex th
	sincorr(th, dth) = sin(asin(DBLE(th)) - dth)

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "SfermionMasses input:"		ENDL
	  DPARA "----------------------------"	ENDL
	  DTAG(iM1SL+(g-1)*(iM2SL-iM1SL)) SignSqrt(MSL2_)
	  DTAG(iM1SE+(g-1)*(iM2SE-iM1SE)) SignSqrt(MSE2_)
	  DTAG(iM1SQ+(g-1)*(iM2SQ-iM1SQ)) SignSqrt(MSQ2_)
	  DTAG(iM1SU+(g-1)*(iM2SU-iM1SU)) SignSqrt(MSU2_)
	  DTAG(iM1SD+(g-1)*(iM2SD-iM1SD)) SignSqrt(MSD2_)
	  DTAG(iRe(iAe+(g-1)*(iAmu-iAe))) DBLE(Ae_)
	  DTAG(iIm(iAe+(g-1)*(iAmu-iAe))) DIMAG(Ae_)
	  DTAG(iRe(iAu+(g-1)*(iAc-iAu))) DBLE(Au_)
	  DTAG(iIm(iAu+(g-1)*(iAc-iAu))) DIMAG(Au_)
	  DTAG(iRe(iAd+(g-1)*(iAs-iAd))) DBLE(Ad_)
	  DTAG(iIm(iAd+(g-1)*(iAs-iAd))) DIMAG(Ad_)
	  if( g .eq. 3 ) then
	    DTAG(iQtau) Qtau_
	    DTAG(iQt) Qt_
	    DTAG(iQb) Qb_
	  endif
	endif

	MSS2(1,2,g) = MSL2_
	MSS2(2,2,g) = MSE2_
	MSS2(1,3,g) = MSQ2_
	MSS2(1,4,g) = MSS2(1,3,g)
	MSS2(2,3,g) = MSU2_
	MSS2(2,4,g) = MSD2_

	DSS2(1,2,g) = (MSS2(1,2,g) - MSS2(2,2,g)) +
     &    (DSf(1,2) - DSf(2,2))
	DSS2(2,2,g) = MSS2(2,2,g) + DSf(2,2)
	DSS2(1,3,g) = (MSS2(1,3,g) - MSS2(2,3,g)) +
     &    (DSf(1,3) - DSf(2,3))
	DSS2(2,3,g) = MSS2(2,3,g) + DSf(2,3)
	DSS2(1,5,g) = (MSS2(1,4,g) - MSS2(2,4,g)) +
     &    (DSf(1,4) - DSf(2,4))
	DSS2(2,5,g) = MSS2(2,4,g) + DSf(2,4)

	Af(2,g,g) = Ae_
	Xf(2,g,g) = Af(2,g,g) - MUETB(2)
	Af(3,g,g) = Au_
	Xf(3,g,g) = Af(3,g,g) - MUETB(3)
	Af(4,g,g) = Ad_
	Xf(4,g,g) = Af(4,g,g) - MUETB(4)

* Sneutrino masses:
	MSf2(1,1,g) = MSS2(1,2,g) + DSf(1,1)
	MSf(1,1,g) = sqrt(MSf2(1,1,g))
	MSf2(2,1,g) = 1D123
	MSf(2,1,g) = 1D123
	USf(1,1,1,g) = 1
	USf(2,2,1,g) = 1
	USf(2,1,1,g) = 0
	USf(1,2,1,g) = 0

	entry SfermionIntern(error, g, Qtau_, Qt_, Qb_)

	DSS2(1,4,g) = DSS2(1,5,g)
	DSS2(2,4,g) = DSS2(2,5,g)

	sort = 1
	if( g .eq. 3 .and.
     &      abs(Qtau_) + abs(Qt_) + abs(Qb_) .ne. 0 ) sort = 0

	do t = 2, 4
	  m = Mf(t,g)
	  if( t .eq. 4 .and. g .eq. 3 ) then
	    call MBfromPara(error)
	    m = Mdy(3,1)
	  endif
	  call EigenSf(fail, MSf(1,t,g), MSf2(1,t,g), USf(1,1,t,g),
     &      DSS2(1,t,g), m, Xf(t,g,g), sort)
	  if( fail ) Error("negative sfermion mass squares")
	enddo

	if( sort .eq. 1 ) return

	Qtau = Qtau_
	Qt = Qt_
	Qb = Qb_

	dtSf(2) = -999
	dtSf(3) = -999
	dtSf(4) = -999

	if( Qt .ne. 0 ) then
	  if( Qt .gt. 0 ) then
	    Q2 = Qt**2
	  else
	    Q2 = sqrt(abs(MSS2(1,3,3)*MSS2(2,3,3)))
	  endif

	  if( debuglevel .ge. 2 ) then
	    DPARA "DRbar scale:  ", sqrt(Q2) ENDL
	    DPARA "MStop(DRbar) =", MSf(1,3,3), MSf(2,3,3) ENDL
	    DPARA "sstop(DRbar) =", DBLE(USf(1,2,3,3)) ENDL
	  endif

	  call DRbartoOS(dMSf2(1,3), dtSf(3), Q2, Q2, Q2,
     &      MSf2(1,3,3), USf(1,1,3,3),
     &      MSf2(1,4,3), USf(1,1,4,3),
     &      Xf(3,3,3), Xf(4,3,3), TB, MB, MT)

	  if( debuglevel .ge. 2 ) then
	    DPARA "MStop(OS)    =", sqrt(MSf2(1,3,3) - dMSf2(1,3)),
     &                              sqrt(MSf2(2,3,3) - dMSf2(2,3)) ENDL
	    DPARA "sstop(OS)    =", sincorr(USf(1,2,3,3), dtSf(3)) ENDL
	  endif
	endif

	if( Qb .ne. 0 ) then
	  if( Qb .gt. 0 ) then
	    Q2 = Qb**2
	  else
	    Q2 = sqrt(abs(MSS2(1,3,3)*MSS2(2,3,3)))
	  endif

	  if( debuglevel .ge. 2 ) then
	    DPARA "DRbar scale:  ", sqrt(Q2) ENDL
	    DPARA "MSbot(DRbar) =", MSf(1,4,3), MSf(2,4,3) ENDL
	    DPARA "ssbot(DRbar) =", DBLE(USf(1,2,4,3)) ENDL
	  endif

	  call DRbartoOS(dMSf2(1,4), dtSf(4), Q2, Q2, Q2,
     &      MSf2(1,4,3), USf(1,1,4,3),
     &      MSf2(1,3,3), USf(1,1,3,3),
     &      Xf(4,3,3), Xf(3,3,3), 1/TB, MT, MB)

	  if( debuglevel .ge. 2 ) then
	    DPARA "MSbot(OS)    =", sqrt(MSf2(1,4,3) - dMSf2(1,4)),
     &                              sqrt(MSf2(2,4,3) - dMSf2(2,4)) ENDL
	    DPARA "ssbot(OS)    =", sincorr(USf(1,2,4,3), dtSf(4)) ENDL
	  endif
	endif

	do t = 2, 4
	  m1 = MSf2(1,t,g)
	  m2 = MSf2(2,t,g)
	  c = USf(1,1,t,g)
	  s = USf(1,2,t,g)

	  if( dtSf(t) .ne. -999 ) then
	    m1 = m1 - dMSf2(1,t)
	    m2 = m2 - dMSf2(2,t)
	    s = sincorr(s, dtSf(t))
	    c = sqrt((1 - DBLE(s))*(1 + DBLE(s)))
	  endif

	  r = 1
	  if( m1 .gt. m2 ) r = 2

	  MSferm(r,t) = sqrt(m1)
	  MSferm(3-r,t) = sqrt(m2)

	  USferm(r,1,t) = c
	  USferm(3-r,2,t) = c
	  USferm(r,2,t) = s
	  USferm(3-r,1,t) = -DCONJG(s)
	enddo

	call SfermionPara(error, g,
     &    MSferm(1,2), MSferm(1,3), MSferm(1,4),
     &    USferm(1,1,2), USferm(1,1,3), USferm(1,1,4))
	end


************************************************************************

	subroutine SfermionPara(error, g,
     &    MSe_, MSu_, MSd_, UStau_, USt_, USb_)
	implicit none
	integer error, g
	double complex MSe_(2), MSu_(2), MSd_(2)
	double complex UStau_(2,2), USt_(2,2), USb_(2,2)

#include "FH.h"

	logical fail
	integer i, t
	double precision m, X11, X22
	double complex X(2,2), X12

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "SfermionPara input:"		ENDL
	  DPARA "----------------------------"	ENDL
	  DPARA "MSe("//Digit(g)//") =", DBLE(MSe_(1)), DBLE(MSe_(2)) ENDL
	  DPARA "MSu("//Digit(g)//") =", DBLE(MSu_(1)), DBLE(MSu_(2)) ENDL
	  DPARA "MSd("//Digit(g)//") =", DBLE(MSd_(1)), DBLE(MSd_(2)) ENDL
	  if( g .eq. 3 ) then
	    DPARA "UStau(1,1) =", UStau_(1,1)	ENDL
	    DPARA "     (1,2) =", UStau_(1,2)	ENDL
	    DPARA "     (2,1) =", UStau_(2,1)	ENDL
	    DPARA "     (2,2) =", UStau_(2,2)	ENDL
	    DPARA "UStop(1,1) =", USt_(1,1)	ENDL
	    DPARA "     (1,2) =", USt_(1,2)	ENDL
	    DPARA "     (2,1) =", USt_(2,1)	ENDL
	    DPARA "     (2,2) =", USt_(2,2)	ENDL
	    DPARA "USbot(1,1) =", USb_(1,1)	ENDL
	    DPARA "     (1,2) =", USb_(1,2)	ENDL
	    DPARA "     (2,1) =", USb_(2,1)	ENDL
	    DPARA "     (2,2) =", USb_(2,2)	ENDL
	  endif
	endif

	do i = 1, 2
	  MSf(i,2,g) = DBLE(MSe_(i))
	  MSf2(i,2,g) = MSf(i,2,g)**2
	  MSf(i,3,g) = DBLE(MSu_(i))
	  MSf2(i,3,g) = MSf(i,3,g)**2
	  MSf(i,4,g) = DBLE(MSd_(i))
	  MSf2(i,4,g) = MSf(i,4,g)**2
	enddo

	if( g .eq. 3 ) then
	  USf(1,1,2,3) = UStau_(1,1)
	  USf(2,1,2,3) = UStau_(2,1)
	  USf(1,2,2,3) = UStau_(1,2)
	  USf(2,2,2,3) = UStau_(2,2)

	  USf(1,1,3,3) = USt_(1,1)
	  USf(2,1,3,3) = USt_(2,1)
	  USf(1,2,3,3) = USt_(1,2)
	  USf(2,2,3,3) = USt_(2,2)

	  USf(1,1,4,3) = USb_(1,1)
	  USf(2,1,4,3) = USb_(2,1)
	  USf(1,2,4,3) = USb_(1,2)
	  USf(2,2,4,3) = USb_(2,2)

	  do t = 2, 4
	    call ReverseEigen(2, X,2, MSf2(1,t,3), USf(1,1,t,3),2)
	    if( t .eq. 4 ) then
c FIXME: Xb should be taken from bottom sector, but gives weird
c results sometimes, so for the moment we take Xt or Xtau instead.
c Lines computing Xb are currently commented out in DeltaMB.F.
c	      Xf(4,3,3) = Xf(2,3,3)
	      Xf(4,3,3) = Xf(3,3,3) + MUETB(3) - MUETB(4)
	      call MBfromMasses(error, X)
	    else
	      DSS2(1,t,3) = DBLE(X(1,1)) - DBLE(X(2,2))
	      DSS2(2,t,3) = DBLE(X(2,2)) - Mf2(t,3)
	      Xf(t,3,3) = DCONJG(X(1,2))/Mf(t,3)
	    endif
	  enddo
	else
	  do t = 2, 4
* we assume 3rd-generation mixing in generation 1 & 2 also
* (at least the SLHA doesn't contain any explicit 1st and 2nd
* generation mixing parameters)
	    Xf(t,g,g) = Xf(t,3,3)
	    X12 = Mf(t,g)*DCONJG(Xf(t,g,g))

* to get the MSS right, we first figure out the entries of the
* mixing matrix X using the trace and det relation:
*   X11 + X22 = MSf1^2 + MSf2^2
*   X11 X22 - |X12|^2 = MSf1^2 MSf2^2
	    m = MSf2(1,t,g) + MSf2(2,t,g)
	    X11 = .5D0*(m + sqrt((MSf2(1,t,g) - MSf2(2,t,g))**2 -
     &        4*Sq(X12)))
	    X22 = m - X11
	    DSS2(1,t,g) = X11 - X22
	    DSS2(2,t,g) = X11 - Mf2(t,g)

	    call EigenSf(fail, MSf(1,t,g), MSf2(1,t,g), USf(1,1,t,g),
     &        DSS2(1,t,g), Mf(t,g), Xf(t,g,g), 1)
	  enddo
	  DSS2(1,5,g) = DSS2(1,4,g)
	  DSS2(2,5,g) = DSS2(2,4,g)
	endif

	Af(2,g,g) = Xf(2,g,g) + MUETB(2)
	Af(3,g,g) = Xf(3,g,g) + MUETB(3)
	Af(4,g,g) = Xf(4,g,g) + MUETB(4)

	MSS2(2,2,g) = DSS2(2,2,g) - DSf(2,2)
	MSS2(1,2,g) = MSS2(2,2,g) + DSS2(1,2,g) - DSf(1,2)
	MSS2(2,3,g) = DSS2(2,3,g) - DSf(2,3)
	MSS2(1,3,g) = MSS2(2,3,g) + DSS2(1,3,g) - DSf(1,3)
	MSS2(2,4,g) = DSS2(2,4,g) - DSf(2,4)
	MSS2(1,4,g) = MSS2(2,4,g) + DSS2(1,5,g) - DSf(2,4)

* Sneutrino masses:
	MSf2(1,1,g) = MSS2(1,2,g) + DSf(1,1)
	MSf(1,1,g) = sqrt(MSf2(1,1,g))
	MSf2(2,1,g) = 1D123
	MSf(2,1,g) = 1D123
	USf(1,1,1,g) = 1
	USf(2,2,1,g) = 1
	USf(2,1,1,g) = 0
	USf(1,2,1,g) = 0
	end


************************************************************************

	subroutine EndPara(error)
	implicit none
	integer error

#include "FH.h"
#include "looptools.h"

	integer t, g, s1, s2

	character*(*) msg
	parameter (msg =
     &    " mixing selected in the presence of complex parameters")

	para_valid = 0
	if( error .eq. 0 ) para_valid = valid

	if( higgsmix .lt. 3 .and.
     &    abs(DIMAG(Af(2,1,1))) +
     &    abs(DIMAG(Af(3,1,1))) +
     &    abs(DIMAG(Af(4,1,1))) +
     &    abs(DIMAG(Af(2,2,2))) +
     &    abs(DIMAG(Af(3,2,2))) +
     &    abs(DIMAG(Af(4,2,2))) +
     &    abs(DIMAG(Af(2,3,3))) +
     &    abs(DIMAG(Af(3,3,3))) +
     &    abs(DIMAG(Af(4,3,3))) +
     &    abs(DIMAG(MUE)) +
     &    abs(DIMAG(M_1)) +
     &    abs(DIMAG(M_2)) +
     &    abs(DIMAG(M_3)) .ne. 0 )
     &    Warning(Digit(higgsmix)//"x"//Digit(higgsmix)//msg)

	do t = 3, 4
	  do g = 1, 3
	    do s1 = 1, 2
	      MASf(g+3*(s1-1),t) = MSf(s1,t,g)
	      MASf2(g+3*(s1-1),t) = MSf2(s1,t,g)
	      do s2 = 1, 2
	        UASf(g+3*(s1-1),g+3*(s2-1),t) = USf(s1,s2,t,g)
	      enddo
	    enddo
	  enddo
	enddo

	if( debuglevel .lt. 1 ) return

	DPARA "============================"	ENDL
	DPARA "parameters set by FHSetPara"	ENDL
	DPARA "----------------------------"	ENDL
	DPARA "mudim =", sqrt(getmudim())	ENDL
	DPARA "MT =", MT			ENDL
	DPARA "Mdy(3,1) =", Mdy(3,1)		ENDL
	DPARA "Mdy(3,2) =", Mdy(3,2)		ENDL
	DPARA "DeltaMBc(1) =", DeltaMBc(1)	ENDL
	DPARA "DeltaMBc(2) =", DeltaMBc(2)	ENDL
	DPARA "GS =", GS			ENDL
	DPARA "TB =", TB			ENDL
	DPARA "M_1 =", M_1			ENDL
	DPARA "M_2 =", M_2			ENDL
	DPARA "M_3 =", M_3			ENDL
	DPARA "MUE =", MUE			ENDL
	DPARA "----------------------------"	ENDL
	DPARA "M1SL =", SignSqrt(MSS2(1,2,1))	ENDL
	DPARA "M1SE =", SignSqrt(MSS2(2,2,1))	ENDL
	DPARA "M1SQ =", SignSqrt(MSS2(1,3,1))	ENDL
	DPARA "M1SU =", SignSqrt(MSS2(2,3,1))	ENDL
	DPARA "M1SD =", SignSqrt(MSS2(2,4,1))	ENDL
	DPARA "Ae   =", Af(2,1,1)		ENDL
	DPARA "Au   =", Af(3,1,1)		ENDL
	DPARA "Ad   =", Af(4,1,1)		ENDL
	DPARA "----------------------------"	ENDL
	DPARA "M2SL =", SignSqrt(MSS2(1,2,2))	ENDL
	DPARA "M2SE =", SignSqrt(MSS2(2,2,2))	ENDL
	DPARA "M2SQ =", SignSqrt(MSS2(1,3,2))	ENDL
	DPARA "M2SU =", SignSqrt(MSS2(2,3,2))	ENDL
	DPARA "M2SD =", SignSqrt(MSS2(2,4,2))	ENDL
	DPARA "Amu  =", Af(2,2,2)		ENDL
	DPARA "Ac   =", Af(3,2,2)		ENDL
	DPARA "As   =", Af(4,2,2)		ENDL
	DPARA "----------------------------"	ENDL
	DPARA "M3SL =", SignSqrt(MSS2(1,2,3))	ENDL
	DPARA "M3SE =", SignSqrt(MSS2(2,2,3))	ENDL
	DPARA "M3SQ =", SignSqrt(MSS2(1,3,3))	ENDL
	DPARA "M3SU =", SignSqrt(MSS2(2,3,3))	ENDL
	DPARA "M3SD =", SignSqrt(MSS2(2,4,3))	ENDL
	DPARA "Atau =", Af(2,3,3)		ENDL
	DPARA "At   =", Af(3,3,3)		ENDL
	DPARA "Ab   =", Af(4,3,3)		ENDL
	DPARA "----------------------------"	ENDL
	DPARA "Higgs masses (tree-level):"	ENDL
	DPARA "Mh0 =", Mh0			ENDL
	DPARA "MHH =", MHH			ENDL
	DPARA "MA0 =", MA0			ENDL
	DPARA "MHp =", MHp			ENDL
	DPARA "SA =", SA			ENDL
	DPARA "----------------------------"	ENDL
	DPARA "Sfermion masses:"		ENDL
	DPARA "Sneutrino e:", MSf(1,1,1)	ENDL
	DPARA "         mu:", MSf(1,1,2)	ENDL
	DPARA "        tau:", MSf(1,1,3)	ENDL
	DPARA "Slepton   e:", MSf(1,2,1), MSf(2,2,1) ENDL
	DPARA "         mu:", MSf(1,2,2), MSf(2,2,2) ENDL
	DPARA "        tau:", MSf(1,2,3), MSf(2,2,3) ENDL
	DPARA "u-Squark  u:", MSf(1,3,1), MSf(2,3,1) ENDL
	DPARA "          c:", MSf(1,3,2), MSf(2,3,2) ENDL
	DPARA "          t:", MSf(1,3,3), MSf(2,3,3) ENDL
	DPARA "d-Squark  d:", MSf(1,4,1), MSf(2,4,1) ENDL
	DPARA "          s:", MSf(1,4,2), MSf(2,4,2) ENDL
	DPARA "          b:", MSf(1,4,3), MSf(2,4,3) ENDL
	DPARA "----------------------------"	ENDL
	DPARA "Stau mixing:"			ENDL
	DPARA "USf(1,1,2,3) = ", USf(1,1,2,3)	ENDL
	DPARA "USf(1,2,2,3) = ", USf(1,2,2,3)	ENDL
	DPARA "USf(2,1,2,3) = ", USf(2,1,2,3)	ENDL
	DPARA "USf(2,2,2,3) = ", USf(2,2,2,3)	ENDL
	DPARA "----------------------------"	ENDL
	DPARA "Stop mixing:"			ENDL
	DPARA "USf(1,1,3,3) = ", USf(1,1,3,3)	ENDL
	DPARA "USf(1,2,3,3) = ", USf(1,2,3,3)	ENDL
	DPARA "USf(2,1,3,3) = ", USf(2,1,3,3)	ENDL
	DPARA "USf(2,2,3,3) = ", USf(2,2,3,3)	ENDL
	DPARA "----------------------------"	ENDL
	DPARA "Sbottom mixing:"			ENDL
	DPARA "USf(1,1,4,3) = ", USf(1,1,4,3)	ENDL
	DPARA "USf(1,2,4,3) = ", USf(1,2,4,3)	ENDL
	DPARA "USf(2,1,4,3) = ", USf(2,1,4,3)	ENDL
	DPARA "USf(2,2,4,3) = ", USf(2,2,4,3)	ENDL
	DPARA "----------------------------"	ENDL
	DPARA "Chargino masses:"		ENDL
	DPARA "MCha(1) =", MCha(1)		ENDL
	DPARA "MCha(2) =", MCha(2)		ENDL
	DPARA "----------------------------"	ENDL
	DPARA "Neutralino masses:"		ENDL
	DPARA "MNeu(1) =", MNeu(1)		ENDL
	DPARA "MNeu(2) =", MNeu(2)		ENDL
	DPARA "MNeu(3) =", MNeu(3)		ENDL
	DPARA "MNeu(4) =", MNeu(4)		ENDL
	end

