* DeltaMs.F
* computes the rate for Delta M_{B_s}
* this file is part of FeynHiggs
* last modified 1 Sep 08 th

#include "externals.h"
#include "debug.h"


#define CVLLSM coeff(1)
#define CVLL coeff(2)
#define CVRR coeff(3)
#define CLR1 coeff(4)
#define CLR2 coeff(5)
#define CSLL1 coeff(6)
#define CSLL2 coeff(7)
#define CSRR1 coeff(8)
#define CSRR2 coeff(9)
#define NCOEFF 9


	subroutine DeltaMs(deltaMsMSSM, deltaMsSM)
	implicit none
	double precision deltaMsMSSM, deltaMsSM

#include "FH.h"

	double complex coeff(NCOEFF)

	integer nf
	double precision asMB, asMT, eta5, eff
	double precision etaVLL, etaLR(2,2), etaSLL(2,2)
	double precision BVLL, BLR1, BLR2, BSLL1, BSLL2
	double precision PVLL, PLR1, PLR2, PSLL1, PSLL2

	double precision MBs, FBs
	parameter (MBs = 5367.5D-3)
	parameter (FBs = 240D-3)

	double precision AlphaS
	external AlphaS

	if( abs(CKM(2,3)) .lt. 1D-14 ) then
	  deltaMsMSSM = 0
	  deltaMsSM = 0
	  return
	endif

	call Gauss(NCOEFF, coeff, DeltaMsCoeff, -1D0, 1D0, 16)

* the implementation follows closely hep-ph/0102316

	asMB = AlphaS(MB, nf)
	asMT = AlphaS(MT - 1D-14, nf)
	eta5 = asMT/asMB

	etaVLL = 1.6273D0*(1 - eta5)*eta5**(6/23D0)

	etaLR(1,1) = .9250D0*eta5**(-24/23D0) +
     &    eta5**(3/23D0)*(-2.0994D0 + 1.1744D0*eta5)
	etaLR(2,1) = (-11.7329D0 + .7829D0*eta5)*eta5**(3/23D0) +
     &    eta5**(-24/23D0)*(-5.3048D0 + 16.2548D0*eta5)
	etaLR(1,2) = 1.3875D0*(eta5**(26/23D0) - eta5**(-24/23D0))
	etaLR(2,2) = (7.9572D0 - 8.8822D0*eta5)*eta5**(-24/23D0) +
     &    .9250D0*eta5**(26/23D0)

	etaSLL(1,1) = (4.8177D0 - 5.2272D0*eta5)*eta5**(-.6315D0) +
     &    (.3371D0 + .0724D0*eta5)*eta5**.7184D0
	etaSLL(2,1) = (.0531D0 + .0415D0*eta5)*eta5**(-.6315D0) -
     &    (.0566D0 + .0380D0*eta5)*eta5**.7184D0
	etaSLL(1,2) = (9.1696D0 - 38.8778D0*eta5)*eta5**(-.6315D0) +
     &    (42.5021D0 - 12.7939D0*eta5)*eta5**.7184D0
	etaSLL(2,2) = (.1011D0 + .3083D0*eta5)*eta5**(-.6315D0) +
     &    (-7.1314D0 + 6.7219D0*eta5)*eta5**.7184D0

	eff = 1.44D0*(4.4D0/(MB + MS))**2
	BVLL = .61D0*eff
	BLR1 = .96D0*eff
	BLR2 = 1.30D0*eff
	BSLL1 = .76D0*eff
	BSLL2 = .51D0*eff

	PVLL = etaVLL*BVLL
	PLR1 = -1/2D0*etaLR(1,1)*BLR1 + 3/4D0*etaLR(2,1)*BLR2
	PLR2 = -1/2D0*etaLR(1,2)*BLR1 + 3/4D0*etaLR(2,2)*BLR2
	PSLL1 = -5/8D0*etaSLL(1,1)*BSLL1 - 3/2D0*etaSLL(2,1)*BSLL2
	PSLL2 = -5/8D0*etaSLL(2,1)*BSLL1 - 3/2D0*etaSLL(2,2)*BSLL2

	if( debuglevel .ge. 2 ) then
	  DCONST "deltaMs Wilson coefficients before running" ENDL
	  DCONST "CVLLSM =", CVLLSM	ENDL
	  DCONST "CVLL =", CVLL		ENDL
	  DCONST "CVRR =", CVRR		ENDL
	  DCONST "CLR1 =", CLR1		ENDL
	  DCONST "CLR2 =", CLR2		ENDL
	  DCONST "CSLL1 =", CSLL1	ENDL
	  DCONST "CSLL2 =", CSLL2	ENDL
	  DCONST "CSRR1 =", CSRR1	ENDL
	  DCONST "CSRR2 =", CSRR2	ENDL
	endif

	deltaMsSM = GF**2/(24*pi**2)*MW2*MBs*FBs**2 *
     &    PVLL*CVLLSM

	deltaMsMSSM = GF**2/(24*pi**2)*MW2*MBs*FBs**2 * (
     &    PVLL*(CVLL + CVRR) +
     &    PLR1*CLR1 + PLR2*CLR2 +
     &    PSLL1*(CSLL1 + CSRR1) + PSLL2*(CSLL2 + CSRR2) )
	end


************************************************************************

	subroutine DeltaMsCoeff(coeff, costh)
	implicit none
	double complex coeff(NCOEFF)
	double precision costh

#include "FH.h"
#include "looptools.h"

	double complex CVLLHp, CVLLCha, CVLLNeu, CVLLGlu
	double complex CVRRNeu, CVRRGlu
	double complex CLR1Neu, CLR1Glu
	double complex CLR2Neu, CLR2Glu
	double complex CSLL1Neu, CSLL1Glu
	double complex CSLL2Neu, CSLL2Glu
	double complex CSRR1Neu, CSRR1Glu
	double complex CSRR2Neu, CSRR2Glu

	integer Cha5, Cha6, Gen5, Gen6, Neu5, Neu6
	integer All5, All6, Ind1, Ind2, Ind3, Ind4

#include "deltaMs.h"

	CVLL = CVLLSM + CVLLHp + CVLLCha + CVLLNeu + CVLLGlu
	CVRR = CVRRNeu + CVRRGlu
	CLR1 = CLR1Neu + CLR1Glu
	CLR2 = CLR1Neu + CLR1Glu
	CSLL1 = CSLL1Neu + CSLL1Glu
	CSLL2 = CSLL2Neu + CSLL2Glu
	CSRR1 = CSRR1Neu + CSRR1Glu
	CSRR2 = CSRR2Neu + CSRR2Glu
	end

