* FHMode.F
* the FeynHiggs front end for native FH input files
* this file is part of FeynHiggs
* last modified 20 Dec 08 th

#include "FeynHiggs.h"
#include "debug.h"


	subroutine FHMode(error, inputfile, scalefactor, higgsmix)
	implicit none
	integer error, higgsmix
	character*(*) inputfile
	double precision scalefactor

#include "FHMode.h"
#include "FHRecord.h"

	integer v, r, c, nmfv
	RecordDecl(record)

	double precision MHiggs(4)
	double complex SAeff, UHiggs(3,3), ZHiggs(3,3)

	double precision MSf(2,4,3), MASf(6,3:4), MCha(2), MNeu(4)
	double complex USf(2,2,4,3), UASf(6,6,3:4)
	double complex UCha(2,2), VCha(2,2), ZNeu(4,4)
	double complex DeltaMB
	double precision MGl

	integer h, hv, h1, h2, t, g, vv, p
	integer c1, c2, n1, n2, s1, s2, excl
	double complex couplings(ncouplings)
	double complex couplingsms(ncouplingsms)
	double precision gammas(ngammas)
	double precision gammasms(ngammasms)
	double precision prodxs(nprodxs)

	double precision gm2
	double precision DeltaRho, MWMSSM, MWSM, SW2MSSM, SW2SM
	double precision edmeTh, edmn, edmHg
	double precision bsgMSSM, bsgSM

#if U77EXT
	integer lnblnk
	external lnblnk
#endif

	character*2 higgs(4)

	character*7 f_f(2:4,3)
	data f_f /
     &    "e-e", "u-u", "d-d",
     &    "mu-mu", "c-c", "s-s",
     &    "tau-tau", "t-t", "b-b" /

	character*10 f_fprime(2,3)
	data f_fprime /
     &    "nu_e-e", "u-d",
     &    "nu_mu-mu", "c-s",
     &    "nu_tau-tau", "t-b" /

	character*7 sf(4,3)
	data sf /
     &    "snu_e", "se", "su", "sd",
     &    "snu_mu", "smu", "sc", "ss",
     &    "snu_tau", "stau", "stop", "sbot" /

	character*12 v_v(5)
	data v_v / "gamma-gamma", "gamma-Z", "Z-Z", "W-W", "g-g" /

	if( higgsmix .lt. 3 ) then
	  higgs(1) = "h0"
	  higgs(2) = "HH"
	  higgs(3) = "A0"
	else
	  higgs(1) = "h1"
	  higgs(2) = "h2"
	  higgs(3) = "h3"
	endif
	higgs(4) = "Hp"


	call FHReadRecord(error, record, inputfile)
	if( error .ne. 0 ) return

* turn on verbose mode:
	record(iAdmin,iStep) = 1

	call FHLoopRecord(error, record)
	if( error .ne. 0 ) return

* this is the main loop
1	continue

110	format(/"---------------------- PARAMETERS ----------------------")
	print 110

	call FHSetRecord(error, record, scalefactor)

111	format(TAG, A, EQU, 4 MASS_FMT)
	do v = FHRangeC
	  if( record(iRe(v),iVar) .ne. unset )
     &      print 111, FHNameC(v),
     &        record(iRe(v),iVar), record(iIm(v),iVar)
	enddo

	do v = FHRangeR
	  if( record(v,iVar) .ne. unset )
     &      print 111, FHName(v), record(v,iVar)
	enddo

	print 111, "scalefactor", scalefactor

	if( error .ne. 0 ) return

	call FHGetPara(error, nmfv, MSf, USf, MASf, UASf,
     &    MCha, UCha, VCha, MNeu, ZNeu, DeltaMB, MGl)
	if( error .ne. 0 ) return

112	format(TAG, A, EQU, 2 TRIG_FMT)
	print 111, "MStop", MSf(1,3,3), MSf(2,3,3)
	print 112, "cstop", USf(1,1,3,3)
	print 112, "sstop", USf(1,2,3,3)
	print 111, "MSbot", MSf(1,4,3), MSf(2,4,3)
	print 112, "csbot", USf(1,1,4,3)
	print 112, "ssbot", USf(1,2,4,3)
	print 111, "MStau", MSf(1,2,3), MSf(2,2,3)
	print 112, "cstau", USf(1,1,2,3)
	print 112, "sstau", USf(1,2,2,3)
	print 111, "MSnu_tau", MSf(1,1,3)

	print 111, "MCha", MCha
	print 111, "MNeu", MNeu
	print 111, "DeltaMB", DeltaMB
	print 111, "MGl", MGl


120	format(/"--------------------- HIGGS MASSES ---------------------")
	print 120

	call FHHiggsCorr(error, MHiggs, SAeff, UHiggs, ZHiggs)
	if( error .ne. 0 ) return

121	format(TAG, A,A, EQU, MASS_FMT)
	print 121, ("M", higgs(h), MHiggs(h), h = 1, 4)

	print 112, "SAeff", SAeff

122	format(TAG, A, EQU, 3 TRIG_FMT,
     &                CONT, 3 TRIG_FMT,
     &                CONT, 3 TRIG_FMT)
	print 122, "UHiggsRe",
     &    ((DBLE(UHiggs(r,c)), c = 1, 3), r = 1, 3)
	print 122, "UHiggsIm",
     &    ((DIMAG(UHiggs(r,c)), c = 1, 3), r = 1, 3)

	print 122, "ZHiggsRe",
     &    ((DBLE(ZHiggs(r,c)), c = 1, 3), r = 1, 3)
	print 122, "ZHiggsIm",
     &    ((DIMAG(ZHiggs(r,c)), c = 1, 3), r = 1, 3)


123	format(/"---------------- ESTIMATED UNCERTAINTIES ---------------")
	print 123

	call FHUncertainties(error, MHiggs, SAeff, UHiggs, ZHiggs)
	if( error .ne. 0 ) return

	print 121, ("DeltaM", higgs(h), MHiggs(h), h = 1, 4)

	print 112, "DeltaSAeff", SAeff

	print 122, "DeltaUHiggsRe",
     &    ((DBLE(UHiggs(r,c)), c = 1, 3), r = 1, 3)
	print 122, "DeltaUHiggsIm",
     &    ((DIMAG(UHiggs(r,c)), c = 1, 3), r = 1, 3)

	print 122, "DeltaZHiggsRe",
     &    ((DBLE(ZHiggs(r,c)), c = 1, 3), r = 1, 3)
	print 122, "DeltaZHiggsIm",
     &    ((DIMAG(ZHiggs(r,c)), c = 1, 3), r = 1, 3)


130	format(/"---------- DECAY WIDTHS AND BRANCHING RATIOS -----------")
	print 130

	call FHCouplings(error,
     &    couplings, couplingsms, gammas, gammasms, excl)
	if( error .ne. 0 ) return

131	format(TAG, "GammaTot-",A, GAMMA_EQU, 1P, GAMMA_FMT)
	print 131, (higgs(h), GammaTot(h), h = 1, 4)
	print 131, ("SM"//higgs(h), GammaSMTot(h), h = 1, 3)

132	format(TAG, "LEP-excluded", GAMMA_EQU, I5)
	print 132, excl

#define GAMMA_BR_FMT GAMMA_EQU, 1P, 3 GAMMA_FMT

133	format(SKIP/SKIP,"  Channel", T29,A, T44,A,:,T59,A)
135	format(SKIP, TAG, A,"-",A, GAMMA_BR_FMT)
136	format(SKIP, TAG, A,"-",A,I1,"-",A,I1, GAMMA_BR_FMT)
137	format(SKIP, TAG, A,"-",A,"-",A, GAMMA_BR_FMT)

	do h = 1, 3
	  print 133, "Gamma", "BR", "BRSM"

	  do vv = 1, 5
	    if( Gamma(H0VV(h,vv)) .ne. 0 .or.
     &          GammaSM(H0VV(h,vv)) .ne. 0 )
     &        print 135, higgs(h), v_v(vv),
     &          Gamma(H0VV(h,vv)),
     &          BR(H0VV(h,vv)),
     &          BRSM(H0VV(h,vv))
	  enddo

	  do t = 2, 4
	    do g = 1, 3
	      if( Gamma(H0FF(h,t,g)) .ne. 0 .or.
     &            GammaSM(H0FF(h,t,g)) .ne. 0 )
     &          print 135, higgs(h), f_f(t,g),
     &            Gamma(H0FF(h,t,g)),
     &            BR(H0FF(h,t,g)),
     &            BRSM(H0FF(h,t,g))
	    enddo
	  enddo

	  do c1 = 1, 2
	    do c2 = 1, 2
	      if( Gamma(H0ChaCha(h,c1,c2)) .ne. 0 )
     &          print 136, higgs(h), "Cha",c1, "Cha",c2,
     &            Gamma(H0ChaCha(h,c1,c2)),
     &            BR(H0ChaCha(h,c1,c2))
	    enddo
	  enddo

	  do n1 = 1, 4
	    do n2 = n1, 4
	      if( Gamma(H0NeuNeu(h,n1,n2)) .ne. 0 )
     &          print 136, higgs(h), "Neu",n1, "Neu",n2,
     &            Gamma(H0NeuNeu(h,n1,n2)),
     &            BR(H0NeuNeu(h,n1,n2))
	    enddo
	  enddo

	  do hv = 1, 3
	    if( Gamma(H0HV(h,hv)) .ne. 0 )
     &        print 137, higgs(h), higgs(hv), "Z",
     &          Gamma(H0HV(h,hv)),
     &          BR(H0HV(h,hv))
	  enddo

	  do t = 2, 4
	    do g = 1, 3
	      do s1 = 1, 2
	        do s2 = s1, 2
	          if( Gamma(H0SfSf(h,s1,s2,t,g)) .ne. 0 )
     &              print 136,
     &                higgs(h), Strip(sf(t,g)),s1, Strip(sf(t,g)),s2,
     &                Gamma(H0SfSf(h,s1,s2,t,g)),
     &                BR(H0SfSf(h,s1,s2,t,g))
	        enddo
	      enddo
	    enddo
	  enddo

	  do h1 = 1, 4
	    do h2 = h1, 4
	      if( Gamma(H0HH(h,h1,h2)) .ne. 0 )
     &          print 137, higgs(h), higgs(h1), higgs(h2),
     &            Gamma(H0HH(h,h1,h2)), BR(H0HH(h,h1,h2))
	    enddo
	  enddo
	enddo

	print 133, "Gamma", "BR"

	do g = 1, 3
	  do p = 1, 2
	    if( Gamma(HpFF(p,g)) .ne. 0 )
     &        print 135, higgs(4), f_fprime(p,g),
     &          Gamma(HpFF(p,g)),
     &          BR(HpFF(p,g))
	  enddo
	enddo

	do c2 = 1, 2
	  do n1 = 1, 4
	    if( Gamma(HpNeuCha(n1,c2)) .ne. 0 )
     &        print 136, higgs(4), "Neu",n1, "Cha",c2,
     &          Gamma(HpNeuCha(n1,c2)),
     &          BR(HpNeuCha(n1,c2))
	  enddo
	enddo

	do hv = 1, 3
	  if( Gamma(HpHV(hv)) .ne. 0 )
     &      print 137, higgs(4), higgs(hv), "W",
     &        Gamma(HpHV(hv)),
     &        BR(HpHV(hv))
	enddo

	do t = 2, 4, 2
	  do g = 1, 3
	    do s1 = 1, 2
	      do s2 = 1, 2
	        if( Gamma(HpSfSf(s1,s2,t/2,g)) .ne. 0 )
     &            print 136,
     &              higgs(4), Strip(sf(t-1,g)),s1, Strip(sf(t,g)),s2,
     &              Gamma(HpSfSf(s1,s2,t/2,g)),
     &              BR(HpSfSf(s1,s2,t/2,g))
	      enddo
	    enddo
	  enddo
	enddo

	print 133, "Gamma", "BR"

	print 137, "t", "W", "b", Gamma(tBF(1)), BR(tBF(1))
	print 137, "t", higgs(4), "b", Gamma(tBF(2)), BR(tBF(2))

140	format(SKIP/SKIP,
     &    "------------------ SELECTED COUPLINGS ------------------")
	print 140

#define COUPLING_FMT GAMMA_EQU, 1P, 3 GAMMA_FMT

142	format(SKIP, TAG, A,":",A,"-",A, COUPLING_FMT)
143	format(SKIP, TAG, A,":",A,"-",A,I1,"-",A,I1, COUPLING_FMT)
144	format(SKIP, TAG, A,":",A,"-",A,"-",A, COUPLING_FMT)

#define SMRatio(c,mssm,sm) DIMAG(mssm(c))/DIMAG(sm(c))

	do h = 1, 3
	  print 133, "Re Coupling", "Im Coupling", "SM Ratio"

	  if( Coupling(H0VV(h,3)) .ne. 0 )
     &      print 142, "C", higgs(h), v_v(3),
     &        Coupling(H0VV(h,3)),
     &        SMRatio(H0VV(h,3), Coupling, CouplingSM)

	  if( Coupling(H0VV(h,4)) .ne. 0 )
     &      print 142, "C", higgs(h), v_v(4),
     &        Coupling(H0VV(h,4)),
     &        SMRatio(H0VV(h,4), Coupling, CouplingSM)

	  do t = 2, 4
	    do g = 3, 3
	      if( LCoupling(H0FF(h,t,g)) .ne. 0 .or.
     &            RCoupling(H0FF(h,t,g)) .ne. 0 ) then
	        print 142, "CL", higgs(h), f_f(t,g),
     &            LCoupling(H0FF(h,t,g)),
     &            SMRatio(H0FF(h,t,g), LCoupling, LCouplingSM)
	        print 142, "CR", higgs(h), f_f(t,g),
     &            RCoupling(H0FF(h,t,g)),
     &            SMRatio(H0FF(h,t,g), RCoupling, RCouplingSM)
	      endif
	    enddo
	  enddo

	  do c1 = 1, 2
	    do c2 = 1, 2
	      if( LCoupling(H0ChaCha(h,c1,c2)) .ne. 0 .or.
     &            RCoupling(H0ChaCha(h,c1,c2)) .ne. 0 ) then
	        print 143, "CL", higgs(h), "Cha",c1, "Cha",c2,
     &            LCoupling(H0ChaCha(h,c1,c2))
	        print 143, "CR", higgs(h), "Cha",c1, "Cha",c2,
     &            RCoupling(H0ChaCha(h,c1,c2))
	      endif
	    enddo
	  enddo

	  do n1 = 1, 4
	    do n2 = n1, 4
	      if( LCoupling(H0NeuNeu(h,n1,n2)) .ne. 0 .or.
     &            RCoupling(H0NeuNeu(h,n1,n2)) .ne. 0 ) then
	        print 143, "CL", higgs(h), "Neu",n1, "Neu",n2,
     &            LCoupling(H0NeuNeu(h,n1,n2))
	        print 143, "CR", higgs(h), "Neu",n1, "Neu",n2,
     &            RCoupling(H0NeuNeu(h,n1,n2))
	      endif
	    enddo
	  enddo

	  do hv = 1, 3
	    if( Coupling(H0HV(h,hv)) .ne. 0 )
     &        print 144, "C", higgs(h), higgs(hv), "Z",
     &          Coupling(H0HV(h,hv))
	  enddo

	  do t = 2, 4
	    do g = 3, 3
	      do s1 = 1, 2
	        do s2 = s1, 2
	          if( Coupling(H0SfSf(h,s1,s2,t,g)) .ne. 0 )
     &              print 143, "C", higgs(h),
     &                Strip(sf(t,g)),s1, Strip(sf(t,g)),s2,
     &                Coupling(H0SfSf(h,s1,s2,t,g))
	        enddo
	      enddo
	    enddo
	  enddo

	  do h1 = 1, 4
	    do h2 = h1, 4
	      if( Coupling(H0HH(h,h1,h2)) .ne. 0 )
     &          print 144, "C", higgs(h), higgs(h1), higgs(h2),
     &            Coupling(H0HH(h,h1,h2))
	    enddo
	  enddo
	enddo

	print 133, "Re Coupling", "Im Coupling"

	do g = 3, 3
	  do p = 1, 2
	    if( LCoupling(HpFF(p,g)) .ne. 0 .or.
     &          RCoupling(HpFF(p,g)) .ne. 0 ) then
	      print 142, "CL", higgs(4), f_fprime(p,g),
     &          LCoupling(HpFF(p,g))
	      print 142, "CR", higgs(4), f_fprime(p,g),
     &          RCoupling(HpFF(p,g))
	    endif
	  enddo
	enddo

	do c2 = 1, 2
	  do n1 = 1, 4
	    if( LCoupling(HpNeuCha(n1,c2)) .ne. 0 .or.
     &          RCoupling(HpNeuCha(n1,c2)) .ne. 0 ) then
	      print 143, "CL", higgs(4), "Neu",n1, "Cha",c2,
     &          LCoupling(HpNeuCha(n1,c2))
	      print 143, "CR", higgs(4), "Neu",n1, "Cha",c2,
     &          RCoupling(HpNeuCha(n1,c2))
	    endif
	  enddo
	enddo

	do hv = 1, 3
	  if( Coupling(HpHV(hv)) .ne. 0 )
     &      print 144, "C", higgs(4), higgs(hv), "W",
     &        Coupling(HpHV(hv))
	enddo

	do t = 2, 4, 2
	  do g = 3, 3
	    do s1 = 1, 2
	      do s2 = 1, 2
	        if( Coupling(HpSfSf(s1,s2,t/2,g)) .ne. 0 )
     &            print 143, "C", higgs(4),
     &              Strip(sf(t-1,g)),s1, Strip(sf(t,g)),s2,
     &              Coupling(HpSfSf(s1,s2,t/2,g))
	      enddo
	    enddo
	  enddo
	enddo

150	format(SKIP/SKIP,
     &    "-------------- PRODUCTION CROSS-SECTIONS ---------------")
	print 150

	call FHHiggsProd(error, prodxs)
	if( error .ne. 0 ) return

	do h = 1, 3
	  print 133, "xsection/fb", "SM xsection/fb"
	  print 135, "Tev:b-b",higgs(h), bbhTev(h), bbhTevSM(h)
	  print 135, "LHC:b-b",higgs(h), bbhLHC(h), bbhLHCSM(h)
	  print 135, "Tev:btag-b",higgs(h), btagbhTev(h), btagbhTevSM(h)
	  print 135, "LHC:btag-b",higgs(h), btagbhLHC(h), btagbhLHCSM(h)
	  print 135, "Tev:g-g",higgs(h), gghTev(h), gghTevSM(h)
	  print 135, "LHC:g-g",higgs(h), gghLHC(h), gghLHCSM(h)
	  print 135, "Tev:q-q",higgs(h), qqhTev(h), qqhTevSM(h)
	  print 135, "LHC:q-q",higgs(h), qqhLHC(h), qqhLHCSM(h)
	  print 135, "Tev:t-t",higgs(h), tthTev(h), tthTevSM(h)
	  print 135, "LHC:t-t",higgs(h), tthLHC(h), tthLHCSM(h)
	  print 135, "Tev:W",  higgs(h), WhTev(h),  WhTevSM(h)
	  print 135, "LHC:W",  higgs(h), WhLHC(h),  WhLHCSM(h)
	  print 135, "Tev:Z",  higgs(h), ZhTev(h),  ZhTevSM(h)
	  print 135, "LHC:Z",  higgs(h), ZhLHC(h),  ZhLHCSM(h)
	  if( h .eq. 1 ) print 135, "LHC:St-St", higgs(h), StSthLHC(h)
	enddo

	print 133, "xsection/fb"
	print 135, "LHC:t", higgs(4), tHmLHC

160	format(/"--------------------- CONSTRAINTS ----------------------"
     &    /"Note: MW, SW2, and the EDMs are not fully checked yet!")

	print 160

	call FHConstraints(error, gm2,
     &    DeltaRho, MWMSSM, MWSM, SW2MSSM, SW2SM,
     &    edmeTh, edmn, edmHg,
     &    bsgMSSM, bsgSM)
	if( error .ne. 0 ) return

161	format(TAG, A, EQU, 1P, GAMMA_FMT)
	print 161, "DeltaRho", DeltaRho
	print 161, "MW", MWMSSM
	print 161, "MWSM", MWSM
	print 161, "SW2eff", SW2MSSM
	print 161, "SW2effSM", SW2SM
	print 161, "g-2", gm2
	print 161, "EDMeTh", edmeTh
	print 161, "EDMn", edmn
	print 161, "EDMHg", edmHg
	print 161, "bsgamma", bsgMSSM
	print 161, "bsgammaSM", bsgSM

200	format(/"========================================================")
	print 200

* end main loop
2	call FHLoopRecord(error, record)
	if( error .eq. 0 ) goto 1
	if( error .eq. -2 ) goto 2
	end

