* SetNMFV.F
* set the non-minimal flavour-violation parameters
* this file is part of FeynHiggs
* last modified 13 Oct 08 th

#include "externals.h"
#include "debug.h"
#include "SLHADefs.h"


#define MX2_LL(i,j) MX2(i,j)
#define MX2_LR(i,j) MX2(i,j+3)
#define MX2_RL(i,j) DCONJG(MX2(j,i+3))
#define MX2_RR(i,j) MX2(i+3,j+3)


	subroutine FHSetNMFV(error,
     &    deltaLLuc_, deltaLRuc_, deltaRLuc_, deltaRRuc_,
     &    deltaLLct_, deltaLRct_, deltaRLct_, deltaRRct_,
     &    deltaLLut_, deltaLRut_, deltaRLut_, deltaRRut_,
     &    deltaLLds_, deltaLRds_, deltaRLds_, deltaRRds_,
     &    deltaLLsb_, deltaLRsb_, deltaRLsb_, deltaRRsb_,
     &    deltaLLdb_, deltaLRdb_, deltaRLdb_, deltaRRdb_)
	implicit none
	integer error
	double complex deltaLLuc_, deltaLRuc_
	double complex deltaRLuc_, deltaRRuc_
	double complex deltaLLct_, deltaLRct_
	double complex deltaRLct_, deltaRRct_
	double complex deltaLLut_, deltaLRut_
	double complex deltaRLut_, deltaRRut_
	double complex deltaLLds_, deltaLRds_
	double complex deltaRLds_, deltaRRds_
	double complex deltaLLsb_, deltaLRsb_
	double complex deltaRLsb_, deltaRRsb_
	double complex deltaLLdb_, deltaLRdb_
	double complex deltaRLdb_, deltaRRdb_

#include "FH.h"
#define __SUBROUTINE__ "FHSetNMFV"

	integer t, g, g2
	double complex MX2(6,6), m2

	double complex MX2_flat(6*6)
	equivalence (MX2, MX2_flat)

	error = 0

	CheckPara()

	if( error .ne. 0 ) return

	do g = 1, 2*6*6
	  deltaSf_flat(g) = 0
	enddo

	deltaSf_LL(1,2,3) = deltaLLuc_
	deltaSf_LR(1,2,3) = deltaLRuc_
	deltaSf_RL(1,2,3) = deltaRLuc_
	deltaSf_RR(1,2,3) = deltaRRuc_

	deltaSf_LL(2,3,3) = deltaLLct_
	deltaSf_LR(2,3,3) = deltaLRct_
	deltaSf_RL(2,3,3) = deltaRLct_
	deltaSf_RR(2,3,3) = deltaRRct_

	deltaSf_LL(1,3,3) = deltaLLut_
	deltaSf_LR(1,3,3) = deltaLRut_
	deltaSf_RL(1,3,3) = deltaRLut_
	deltaSf_RR(1,3,3) = deltaRRut_

	deltaSf_LL(1,2,4) = deltaLLds_
	deltaSf_LR(1,2,4) = deltaLRds_
	deltaSf_RL(1,2,4) = deltaRLds_
	deltaSf_RR(1,2,4) = deltaRRds_

	deltaSf_LL(2,3,4) = deltaLLsb_
	deltaSf_LR(2,3,4) = deltaLRsb_
	deltaSf_RL(2,3,4) = deltaRLsb_
	deltaSf_RR(2,3,4) = deltaRRsb_

	deltaSf_LL(1,3,4) = deltaLLdb_
	deltaSf_LR(1,3,4) = deltaLRdb_
	deltaSf_RL(1,3,4) = deltaRLdb_
	deltaSf_RR(1,3,4) = deltaRRdb_

	do t = 3, 4
	  do g = 1, 6*6
	    MX2_flat(g) = 0
	  enddo
	  do g = 1, 3
	    MX2_LL(g,g) = DSS2(1,t,g) + DSS2(2,t,g) + Mf2(t,g)
	    MX2_RR(g,g) = DSS2(2,t,g) + Mf2(t,g)
	    MX2_LR(g,g) = Mfy(t,g)*DCONJG(Xf(t,g,g))
	  enddo

	  do g2 = 6, 2, -1
	    m2 = MX2(g2,g2)
	    do g = 1, g2 - 1
	      MX2(g,g2) = MX2(g,g2) + deltaSf(g,g2,t)*sqrt(MX2(g,g)*m2)
	    enddo
	  enddo

	  call NMFVMasses(error, t, MX2)
	enddo
	end

#undef __SUBROUTINE__


************************************************************************

	subroutine SetNMFVSLHA(error, t, MSQ2_, MSU2_, TU_, XB_)
	implicit none
	integer error, t
	double complex MSQ2_(3,3), MSU2_(3,3), TU_(3,3)
	double precision XB_

#include "FH.h"

	integer g, g2
	double complex MX2(6,6), m2

	double complex MX2_flat(6*6)
	equivalence (MX2, MX2_flat)

	do g = 1, 6*6
	  MX2_flat(g) = 0
	enddo
	do g = 1, 3
	  MX2_LL(g,g) = MSS2(1,t,g)
	  MX2_RR(g,g) = MSS2(2,t,g)
	  MX2_LR(g,g) = Mfy(t,g)*DCONJG(Af(t,g,g))
	enddo

	do g2 = 1, 3
	  do g = g2, 3
	    if( MSQ2_(g,g2) .ne. invalid ) MX2_LL(g,g2) = MSQ2_(g,g2)
	    if( MSU2_(g,g2) .ne. invalid ) MX2_RR(g,g2) = MSU2_(g,g2)
	    if( TU_(g2,g) .ne. invalid )
     &        MX2_LR(g2,g) = XB_*vev*TU_(g2,g)
	    if( TU_(g,g2) .ne. invalid )
     &        MX2_LR(g,g2) = XB_*vev*DCONJG(TU_(g,g2))
	  enddo
	  MX2_LL(g2,g2) = MX2_LL(g2,g2) + DSf(1,t) + Mfy2(t,g2)
	  MX2_RR(g2,g2) = MX2_RR(g2,g2) + DSf(2,t) + Mfy2(t,g2)
	  MX2_LR(g2,g2) = MX2_LR(g2,g2) - Mfy(t,g)*DCONJG(MUETB(t))
	enddo

	do g = 1, 6*6
	  deltaSf_flat((t-3)*(6*6)+g) = 0
	enddo
	do g2 = 6, 2, -1
	  m2 = MX2(g2,g2)
	  do g = 1, g2 - 1
	    deltaSf(g,g2,t) = MX2(g,g2)/sqrt(MX2(g,g)*m2)
	  enddo
	enddo

	call NMFVMasses(error, t, MX2)
	end


************************************************************************

	subroutine NMFVMasses(error, t, MX2)
	implicit none
	integer error, t
	double complex MX2(6,6)

#include "FH.h"
#include "FHRecord.h"
#define __SUBROUTINE__ "NMFVMasses"

	integer as1, o

	MX2_LR(2,1) = DCONJG(MX2_LR(2,1))
	MX2_LR(3,1) = DCONJG(MX2_LR(3,1))
	MX2_LR(3,2) = DCONJG(MX2_LR(3,2))

	Af(t,1,2) = MX2_LR(1,2)/Mf(t,1)
	Af(t,2,1) = MX2_LR(2,1)/Mf(t,2)
	Af(t,1,3) = MX2_LR(1,3)/Mf(t,1)
	Af(t,3,1) = MX2_LR(3,1)/Mf(t,3)
	Af(t,2,3) = MX2_LR(2,3)/Mf(t,2)
	Af(t,3,2) = MX2_LR(3,2)/Mf(t,3)

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "FHSetNMFV input parameters:"	ENDL
	  DPARA "----------------------------"	ENDL

	  o = (t - 3)*(ideltaLLds - ideltaLLuc)

	  DTAG(iRe(ideltaLLuc+o)) DBLE(deltaSf_LL(1,2,t))	ENDL
	  DTAG(iIm(ideltaLLuc+o)) DIMAG(deltaSf_LL(1,2,t))	ENDL
	  DTAG(iRe(ideltaLRuc+o)) DBLE(deltaSf_LR(1,2,t))	ENDL
	  DTAG(iIm(ideltaLRuc+o)) DIMAG(deltaSf_LR(1,2,t))	ENDL
	  DTAG(iRe(ideltaRLuc+o)) DBLE(deltaSf_RL(1,2,t))	ENDL
	  DTAG(iIm(ideltaRLuc+o)) DIMAG(deltaSf_RL(1,2,t))	ENDL
	  DTAG(iRe(ideltaRRuc+o)) DBLE(deltaSf_RR(1,2,t))	ENDL
	  DTAG(iIm(ideltaRRuc+o)) DIMAG(deltaSf_RR(1,2,t))	ENDL

	  DTAG(iRe(ideltaLLct+o)) DBLE(deltaSf_LL(2,3,t))	ENDL
	  DTAG(iIm(ideltaLLct+o)) DIMAG(deltaSf_LL(2,3,t))	ENDL
	  DTAG(iRe(ideltaLRct+o)) DBLE(deltaSf_LR(2,3,t))	ENDL
	  DTAG(iIm(ideltaLRct+o)) DIMAG(deltaSf_LR(2,3,t))	ENDL
	  DTAG(iRe(ideltaRLct+o)) DBLE(deltaSf_RL(2,3,t))	ENDL
	  DTAG(iIm(ideltaRLct+o)) DIMAG(deltaSf_RL(2,3,t))	ENDL
	  DTAG(iRe(ideltaRRct+o)) DBLE(deltaSf_RR(2,3,t))	ENDL
	  DTAG(iIm(ideltaRRct+o)) DIMAG(deltaSf_RR(2,3,t))	ENDL

	  DTAG(iRe(ideltaLLut+o)) DBLE(deltaSf_LL(1,3,t))	ENDL
	  DTAG(iIm(ideltaLLut+o)) DIMAG(deltaSf_LL(1,3,t))	ENDL
	  DTAG(iRe(ideltaLRut+o)) DBLE(deltaSf_LR(1,3,t))	ENDL
	  DTAG(iIm(ideltaLRut+o)) DIMAG(deltaSf_LR(1,3,t))	ENDL
	  DTAG(iRe(ideltaRLut+o)) DBLE(deltaSf_RL(1,3,t))	ENDL
	  DTAG(iIm(ideltaRLut+o)) DIMAG(deltaSf_RL(1,3,t))	ENDL
	  DTAG(iRe(ideltaRRut+o)) DBLE(deltaSf_RR(1,3,t))	ENDL
	  DTAG(iIm(ideltaRRut+o)) DIMAG(deltaSf_RR(1,3,t))	ENDL
	endif

	if( debuglevel .ge. 3 ) then
	  DPARA "========================"	ENDL
	  DPARA "MX2_LL(1,1) =", MX2_LL(1,1)	ENDL
	  DPARA "      (1,2) =", MX2_LL(1,2)	ENDL
	  DPARA "      (1,3) =", MX2_LL(1,3)	ENDL
	  DPARA "      (2,2) =", MX2_LL(2,2)	ENDL
	  DPARA "      (2,3) =", MX2_LL(2,3)	ENDL
	  DPARA "      (3,3) =", MX2_LL(3,3)	ENDL
	  DPARA "------------------------"	ENDL
	  DPARA "MX2_LR(1,1) =", MX2_LR(1,1)	ENDL
	  DPARA "      (1,2) =", MX2_LR(1,2)	ENDL
	  DPARA "      (1,3) =", MX2_LR(1,3)	ENDL
	  DPARA "      (2,2) =", MX2_LR(2,2)	ENDL
	  DPARA "      (2,3) =", MX2_LR(2,3)	ENDL
	  DPARA "      (3,3) =", MX2_LR(3,3)	ENDL
	  DPARA "------------------------"	ENDL
	  DPARA "MX2_RL(1,1) =", MX2_RL(1,1)	ENDL
	  DPARA "      (1,2) =", MX2_RL(2,1)	ENDL
	  DPARA "      (1,3) =", MX2_RL(3,1)	ENDL
	  DPARA "      (2,2) =", MX2_RL(2,2)	ENDL
	  DPARA "      (2,3) =", MX2_RL(3,2)	ENDL
	  DPARA "      (3,3) =", MX2_RL(3,3)	ENDL
	  DPARA "------------------------"	ENDL
	  DPARA "MX2_RR(1,1) =", MX2_RR(1,1)	ENDL
	  DPARA "      (1,2) =", MX2_RR(1,2)	ENDL
	  DPARA "      (1,3) =", MX2_RR(1,3)	ENDL
	  DPARA "      (2,2) =", MX2_RR(2,2)	ENDL
	  DPARA "      (2,3) =", MX2_RR(2,3)	ENDL
	  DPARA "      (3,3) =", MX2_RR(3,3)	ENDL
        endif

	if( abs(MX2_LL(1,2)) + abs(MX2_LL(1,3)) + abs(MX2_LL(2,3)) +
     &      abs(MX2_LR(1,2)) + abs(MX2_LR(1,3)) + abs(MX2_LR(2,3)) +
     &      abs(MX2_LR(2,1)) + abs(MX2_LR(3,1)) + abs(MX2_LR(3,2)) +
     &      abs(MX2_RR(1,2)) + abs(MX2_RR(1,3)) + abs(MX2_RR(2,3)) .gt.
     &      1D-13 ) nmfv = 1

	call HEigensystem(6, MX2,6, MASf2(1,t), UASf(1,1,t),6, 1)

	if( MASf2(1,t) .lt. 0 ) then
	  Error("Negative MASf2")
	  return
	endif

	do as1 = 1, 6
	  MASf(as1,t) = sqrt(MASf2(as1,t))
	enddo

	if( debuglevel .lt. 1 ) return

	DPARA "============================"	ENDL
	DPARA "parameters set by NMFVMasses"	ENDL
	DPARA "----------------------------"	ENDL
	DPARA "nmfv = ", nmfv			ENDL
	DPARA "MASf("//Digit(t)//") =", MASf(1,t), MASf(2,t) ENDL
	DPARA "         ", MASf(3,t), MASf(4,t) ENDL
	DPARA "         ", MASf(5,t), MASf(6,t) ENDL
	end

