* Uncertainties.F
* Compute (actually, not completely invent) an error estimate
* for the Higgs masses and mixings
* this file is part of FeynHiggs
* last modified 20 Dec 08 th

#include "externals.h"
#include "debug.h"


	subroutine FHUncertainties(error,
     &    DeltaMHiggs_, DeltaSAeff_, DeltaUHiggs_, DeltaZHiggs_)
	implicit none

#include "FH.h"
#include "looptools.h"
#define __SUBROUTINE__ "FHUncertainties"

	integer error
	double precision DeltaMHiggs_(NHiggs)
	double complex DeltaSAeff_
	double complex DeltaUHiggs_(NNeutral,NNeutral)
	double complex DeltaZHiggs_(NNeutral,NNeutral)

	double precision HiggsErr(NHiggsErr)
	double precision HiggsSave(NHiggsData)
	double precision SfermionSave(NSfermionData)
	double precision oldmudim
	integer i, j, err

	double precision DeltaMHiggs(NHiggs)
	double complex DeltaSAeff
	double complex DeltaXHiggs(0:NNeutral,0:NNeutral,2)
	equivalence (HiggsErr(1), DeltaMHiggs)
	equivalence (HiggsErr(1+NHiggs), DeltaSAeff)
	equivalence (HiggsErr(1+NHiggs+2), DeltaXHiggs)

	call CheckHiggsCorr(error)
	if( error .ne. 0 ) return

* store the central value

	do i = 1, NHiggsData
	  HiggsSave(i) = HiggsData(i)
	enddo

	do i = 1, NHiggsErr
	  HiggsErr(i) = uncomputable
	enddo

	err = 0

* a) variation of mudim from MT/2 to 2 MT

	if( debuglevel .ge. 2 )
     &    DHIGGS "Uncertainties: mudim = 1/2 MT" ENDL

	oldmudim = getmudim()

	call setmudim(.25D0*MT2)
	call CalcHiggsCorr(err)

	if( err .ne. 0 ) then
	  Warning("Could not evaluate corrections at mudim = MT/2")
	  call setmudim(oldmudim)
	  goto 999
	endif

	do i = 1, NHiggsErr
	  if( HiggsData(i) .ne. uncomputable .and.
     &        HiggsSave(i) .ne. uncomputable )
     &      HiggsErr(i) = abs(HiggsData(i) - HiggsSave(i))
	enddo

	if( debuglevel .ge. 2 )
     &    DHIGGS "Uncertainties: mudim = 2 MT" ENDL

	call setmudim(4*MT2)
	call CalcHiggsCorr(err)

	call setmudim(oldmudim)

	if( err .ne. 0 ) then
	  Warning("Could not evaluate corrections at mudim = 2 MT")
	  goto 999
	endif

	do i = 1, NHiggsErr
	  if( HiggsData(i) .ne. uncomputable .and.
     &        HiggsErr(i) .ne. uncomputable )
     &      HiggsErr(i) = max(HiggsErr(i),
     &        abs(HiggsData(i) - HiggsSave(i)))
	enddo

	do i = 1, NSfermionData
	  SfermionSave(i) = SfermionData(i)
	enddo

* b) use MTpole instead of MTrun in the two-loop corrections

	if( MTrun .ne. MT ) then
	  if( debuglevel .ge. 2 )
     &      DHIGGS "Uncertainties: MTpole in 2L corrections" ENDL

	  runningMT = ieor(runningMT, 1)
	  MTy = MTresum(ibits(runningMT, 0, 1))
	  MTy2 = MTy**2

	  call TLShifts(err)
	  call CalcRenSETL(err)
	  call CalcHiggsCorr(err)

	  runningMT = ieor(runningMT, 1)
	  MTy = MTresum(ibits(runningMT, 0, 1))
	  MTy2 = MTy**2

	  if( err .ne. 0 ) then
	    Warning("Could not evaluate corrections for MTrun = MTpole")
	    goto 998
	  endif

	  do i = 1, NHiggsErr
	    if( HiggsData(i) .ne. uncomputable .and.
     &          HiggsErr(i) .ne. uncomputable )
     &        HiggsErr(i) = HiggsErr(i) +
     &          abs(HiggsData(i) - HiggsSave(i))
	  enddo
	endif

* c) quit MB resummation loop after one pass
*    (thus excluding higher-order resummation effects)

	if( debuglevel .ge. 2 )
     &    DHIGGS "Uncertainties: no higher-order MB resummation" ENDL

	DeltaMB_maxpass = 1
	call SfermionIntern(err, 3, 0D0, 0D0, 0D0)
	call CalcRenSETL(err)
	call CalcHiggsCorr(err)

	if( err .ne. 0 ) then
	  Warning("Could not evaluate corrections for unresummed MB")
	  goto 998
	endif

	do i = 1, NHiggsErr
	  if( HiggsData(i) .ne. uncomputable .and.
     &        HiggsErr(i) .ne. uncomputable )
     &      HiggsErr(i) = HiggsErr(i) +
     &        abs(HiggsData(i) - HiggsSave(i))
	enddo

998	do i = 1, NSfermionData
	  SfermionData(i) = SfermionSave(i)
	enddo

999	do i = 1, NHiggsData
	  HiggsData(i) = HiggsSave(i)
	enddo
	higgs_valid = valid

	if( err .ne. 0 ) then
	  do i = 1, NHiggs
	    DeltaMHiggs_(i) = 100000
	  enddo
	  DeltaSAeff_ = uncomputable
	  do i = 1, NNeutral
	    do j = 1, NNeutral
	      DeltaUHiggs_(j,i) = uncomputable
	      DeltaZHiggs_(j,i) = uncomputable
	    enddo
	  enddo
	else
	  do i = 1, NHiggs
	    DeltaMHiggs_(i) = DeltaMHiggs(i)
	  enddo
	  DeltaSAeff_ = DeltaSAeff
	  do i = 1, NNeutral
	    do j = 1, NNeutral
	      DeltaUHiggs_(j,i) = DeltaXHiggs(j,i,1)
	      DeltaZHiggs_(j,i) = DeltaXHiggs(j,i,2)
	    enddo
	  enddo
	endif
	end
