* DeltaMB.F
* calculation of the Delta-MB corrections
* this file is part of FeynHiggs
* last modified 20 Dec 08 th

#include "externals.h"
#include "debug.h"

* Note:
* Mfy(4,3) and DeltaMBc(4) are the values used in the 1L corrections
*   (and must `fit' to the 2L part)
* Mfy(5,3) and DeltaMBc(5) are the values used in decays etc.


	subroutine MBfromPara(error)
	implicit none
	integer error

#include "FH.h"
#define __SUBROUTINE__ "MBfromPara"

	double precision old
	integer i
	logical fail

	if( tlBotResum .ne. 0 ) then
	  do i = 1, DeltaMB_maxpass
	    call EigenSf(fail, MSf(1,4,3), MSf2(1,4,3), USf(1,1,4,3),
     &        DSS2(1,5,3), Mfy(5,3), Xf(4,3,3), 1)

	    old = Mfy(5,3)
	    call ResumMB

	    if( debuglevel .ge. 2 )
     &        DPARA "MBfromPara: Mfy(5,3) =", Mfy(5,3) ENDL

	    if( abs(Mfy(5,3) - old) .lt. 1D-8 ) goto 1
	  enddo

	  if( DeltaMB_maxpass .gt. 1 )
     &      Warning("resummation of MB did not converge")

1	  continue
	endif

	if( looplevel .ge. 2 ) then
	  call TLShifts(error)
	  if( debuglevel .ge. 2 )
     &      DPARA "MBfromPara: Mfy(4,3) =", Mfy(4,3) ENDL
	endif

	call BUpdate
	end

#undef __SUBROUTINE__


************************************************************************

	subroutine MBfromMasses(error, X)
	implicit none
	integer error
	double complex X(2,2)

#include "FH.h"
#define __SUBROUTINE__ "MBfromMasses"

	double precision old
	integer i, err
	logical fail

	if( tlBotResum .ne. 0 ) call ResumMB

	if( debuglevel .ge. 2 )
     &    DPARA "MBfromMasses: Mfy(5,3) =", Mfy(5,3) ENDL

	DSS2(1,4,3) = DBLE(X(1,1)) - DBLE(X(2,2))
	DSS2(1,5,3) = DSS2(1,4,3)
	DSS2(2,4,3) = DBLE(X(2,2)) - Mfy2(4,3)
	DSS2(2,5,3) = DSS2(2,4,3)
	if( xb_valid .eq. 0 ) Xf(4,3,3) = DCONJG(X(1,2))/Mfy(4,3)

	if( looplevel .ge. 2 ) then
	  do i = 1, DeltaMB_maxpass
	    old = Mfy(4,3)
	    err = error
	    call TLShifts(err)
	    if( debuglevel .ge. 2 )
     &        DPARA "MBfromMasses: Mfy(4,3) =", Mfy(4,3) ENDL
	    if( abs(Mfy(4,3) - old) .lt. 1D-8 ) goto 1

	    DSS2(1,5,3) = DBLE(X(1,1)) - DBLE(X(2,2))
	    DSS2(2,5,3) = DBLE(X(2,2)) - Mfy(4,3)**2
	    if( xb_valid .eq. 0 ) Xf(4,3,3) = DCONJG(X(1,2))/Mfy(4,3)
	  enddo

	  if( DeltaMB_maxpass .gt. 1 )
     &      Warning("resummation of MB did not converge")

1	  error = err
	  call EigenSf(fail, MSf(1,4,3), MSf2(1,4,3), USf(1,1,4,3),
     &      DSS2(1,4,3), Mfy(4,3), Xf(4,3,3), 1)
	  if( fail ) Error("negative sbottom mass squares")
	endif

	call BUpdate
	end


************************************************************************

	subroutine ResumMB
	implicit none

#include "FH.h"

	double precision Tdmb
	external Tdmb

*** 264
#if 1
	double precision as
	integer nf

	double precision AlphaS
	external AlphaS

	as = AlphaS(sqrt(MSf(1,4,3)*MSf(2,4,3)), nf)
#else
#define as Alfas
#endif

	DeltaMBc(5) = TB*DCONJG(MUE)*(
     &    2/(3*pi)*as*DCONJG(M_3)*
     &      Tdmb(MSf2(1,4,3), MSf2(2,4,3), Sq(M_3)) +
     &    Alfat**2*DCONJG(Af(3,3,3))*
     &      Tdmb(MSf2(1,3,3), MSf2(2,3,3), Sq(MUE)) )

	Mfy(5,3) = MBrun/abs(1 + DeltaMBc(5))

#if 0
	if( looplevel .eq. 2 .and. tlCplxApprox .eq. 1 ) then
	  DeltaMBc(4) = DeltaMBc(5)
	  Mfy(4,3) = Mfy(5,3)
	endif
#endif
	end


************************************************************************
	
	double precision function Tdmb(x, y, z)
	implicit none
	double precision x, y, z

	double precision n, d

	n = 0
	d = 1

	if( x .ne. y ) then
	  if( x*y .ne. 0 ) n = n + x*y*log(x/y)
	  d = d*(x - y)
	endif
	if( x .ne. z ) then
	  if( x*z .ne. 0 ) n = n + x*z*log(z/x)
	  d = d*(x - z)
	endif
	if( y .ne. z ) then
	  if( y*z .ne. 0 ) n = n + y*z*log(y/z)
	  d = d*(y - z)
	endif

	Tdmb = n/d
	end       


************************************************************************
	
	subroutine BUpdate
	implicit none

#include "FH.h"

	Mfy2(4,3) = Mfy(4,3)**2

	Mfy(5,3) = MBrun/abs(1 + DeltaMBc(5))
	Mfy2(5,3) = Mfy(5,3)**2

	Hff(1,3,4) = 1 - CA/(SA*TB)*DeltaMBc(4)
	Hff(2,3,4) = 1 + SA/(CA*TB)*DeltaMBc(4)

	Hff(1,3,5) = 1 - CA/(SA*TB)*DeltaMBc(5)
	Hff(2,3,5) = 1 + SA/(CA*TB)*DeltaMBc(5)
	end

