* SetSLHA.F
* initialize parameters from the SLHA data structure
* this file is part of FeynHiggs
* last modified 29 Sep 08 th

#include "externals.h"
#include "debug.h"


	subroutine FHSetSLHA(error, slhadata, scalefactor_)
	implicit none
	integer error
	double complex slhadata(*)
	double precision scalefactor_

#include "SLHA.h"
#include "FH.h"
#define __SUBROUTINE__ "FHSetSLHA"

#define Missing(p) Error("cannot determine "//p//" from SLHA data")
#define Origin(p) if(debuglevel.ge.1) DSLHA p ENDL

	double complex M_1_, M_2_, M_3_, MUE_, Af_(2:4)
	double precision TB_, MA0_, QA0_, MSS2_(5), Q_(2:4)
	double precision CKMlambda_, CKMA_, CKMrho_, CKMeta_
	double complex MSQ2_CKM(3,3)
	integer g, t, n

	character*4 mssname(5)
	data mssname /"MSL(", "MSE(", "MSQ(", "MSU(", "MSD("/

	if( SLHAValid(BlockVCKMIn(1), LengthVCKMIn) .ne. 0 ) then
	  Origin("CKM from VCKMIn")
	  CKMlambda_ = VCKMIn_lambda
	  CKMA_ = VCKMIn_A
	  CKMrho_ = VCKMIn_rhobar
	  CKMeta_ = VCKMIn_etabar
	else
	  CKMlambda_ = -1
	  CKMA_ = -1
	  CKMrho_ = -1
	  CKMeta_ = -1
	endif

	call FHSetSMPara(error,
     &    SMInputs_invAlfaMZ, SMInputs_AlfasMZ, SMInputs_GF,
     &    SMInputs_Ms, SMInputs_Mc, SMInputs_Mb,
     &    -1D0, SMInputs_MZ,
     &    CKMlambda_, CKMA_, CKMrho_, CKMeta_)


	if( int(ModSel_Content) .gt. 0 )
     &    Warning("SLHA record indicates model other than MSSM")

	if( ExtPar_TB .ne. invalid ) then
	  Origin("TB from ExtPar_TB")
	  TB_ = ExtPar_TB
	else if( HMix_TB .ne. invalid ) then
	  Origin("TB from HMix_TB")
	  TB_ = HMix_TB
	else if( MinPar_TB .ne. invalid ) then
	  Origin("TB from MinPar_TB")
	  TB_ = MinPar_TB
	else
	  Missing("TB")
	  TB_ = -1
	endif

	if( ExtPar_M3 .ne. invalid ) then
	  Origin("M_3 from ExtPar_M3")
	  M_3_ = ExtPar_M3
	else if( Mass_MGl .ne. invalid ) then
	  Origin("M_3 from Mass_MGl")
	  M_3_ = Mass_MGl
	else if( MSoft_M3 .ne. invalid ) then
	  Origin("M_3 from MSoft_M3")
	  M_3_ = MSoft_M3
	else
	  Missing("M_3")
	  M_3_ = 0
	endif

	call BeginPara(error, scalefactor_, SMInputs_Mt, TB_, M_3_)

* Higgs sector

	if( ExtPar_MA02 .ne. invalid ) then
	  Origin("MA0 from ExtPar_MA02")
	  MA0_ = sqrt(ExtPar_MA02)
	  QA0_ = ExtPar_Q
	else if( ExtPar_MA0 .ne. invalid ) then
	  Origin("MA0 from Mass_MA0")
	  MA0_ = ExtPar_MA0
	  QA0_ = ExtPar_Q
	else if( Mass_MA0 .ne. invalid ) then
	  Origin("MA0 from Mass_MA0")
	  MA0_ = Mass_MA0
	  QA0_ = 0
	else
	  Missing("MA0")
	  MA0_ = -1
	endif

	call HiggsMasses(error, MA0_, Mass_MHp, QA0_)

* Gaugino sector

	if( ExtPar_MUE .ne. invalid ) then
	  Origin("MUE from ExtPar_MUE")
	  MUE_ = ExtPar_MUE
	else if( HMix_MUE .ne. invalid ) then
	  Origin("MUE from HMix_MUE")
	  MUE_ = HMix_MUE
	else
	  MUE_ = invalid
	endif

	if( ExtPar_M2 .ne. invalid ) then
	  Origin("M_2 from ExtPar_M2")
	  M_2_ = ExtPar_M2
	else if( MSoft_M2 .ne. invalid ) then
	  Origin("M_2 from MSoft_M2")
	  M_2_ = MSoft_M2
	else
	  M_2_ = invalid
	endif

	if( MUE_ .ne. invalid .and. M_2_ .ne. invalid ) then
	  call CharginoMasses(error, M_2_, MUE_)
	else if( SLHAValid(Mass_MCha(1), 2) .ne. 0 .and.
     &           SLHAValid(UMix_UCha(1,1), 4) .ne. 0 .and.
     &           SLHAValid(VMix_VCha(1,1), 4) .ne. 0 ) then
	  call CharginoPara(error,
     &      Mass_MCha(1), UMix_UCha(1,1), VMix_VCha(1,1))
	else
	  Missing("M_2 and MUE")
	endif

	if( ExtPar_M1 .ne. invalid ) then
	  Origin("M_1 from ExtPar_M1")
	  M_1_ = ExtPar_M1
	else if( MSoft_M1 .ne. invalid ) then
	  Origin("M_1 from MSoft_M1")
	  M_1_ = MSoft_M1
	else
	  M_1_ = invalid
	endif

	if( M_1_ .ne. invalid ) then
	  call NeutralinoMasses(error, M_1_)
	else if( SLHAValid(Mass_MNeu(1), 4) .ne. 0 .and.
     &           SLHAValid(NMix_ZNeu(1,1), 16) .ne. 0 ) then
	  call NeutralinoPara(error,
     &      Mass_MNeu(1), NMix_ZNeu(1,1))
	else
	  Missing("M_1")
	endif

* Sfermion sector

	do g = 3, 1, -1
	  do t = 2, 4
	    Q_(t) = 0
	    if( Tf_Tf(g,g,t) .ne. invalid ) then
	      Origin("Af("//Digit(t)//",3) from Tf_Tf")
	      Af_(t) = Tf_Tf(g,g,t)
	      Q_(t) = Tf_Q(t)
	    else if( TfIn_Tf(g,g,t) .ne. invalid ) then
	      Origin("Af("//Digit(t)//",3) from Tf_TfIn")
	      Af_(t) = TfIn_Tf(g,g,t)
	    else if( Af_Af(g,g,t) .ne. invalid ) then
	      Origin("Af("//Digit(t)//",3) from Af_Af")
	      Af_(t) = Af_Af(g,g,t)
	      Q_(t) = Af_Q(t)
	    else if( ExtPar_Af(t) .ne. invalid ) then
	      Origin("Af("//Digit(t)//",3) from ExtPar_Af")
	      Af_(t) = ExtPar_Af(t)
	      Q_(t) = ExtPar_Q
	    else
	      if( g .eq. 3 ) then
	        Af_(t) = invalid
	      else
	        Af_(t) = Af(t,g+1,g+1)
	      endif
	    endif
	  enddo

	  do n = 1, 5
	    if( MSS2_MSS2(g,g,n) .ne. invalid ) then
	      Origin(mssname(n)//Digit(g)//") from MSS_MSS2")
	      MSS2_(n) = DBLE(MSS2_MSS2(g,g,n))
	    else if( MSS2In_MSS2(g,g,n) .ne. invalid ) then
	      Origin(mssname(n)//Digit(g)//") from MSSIn_MSS2")
	      MSS2_(n) = DBLE(MSS2In_MSS2(g,g,n))
	    else if( MSoft_MSS(g,n) .ne. invalid ) then
	      Origin(mssname(n)//Digit(g)//") from MSoft_MSS")
	      MSS2_(n) = DBLE(MSoft_MSS(g,n))**2
	    else if( ExtPar_MSS(g,n) .ne. invalid ) then
	      Origin(mssname(n)//Digit(g)//") from ExtPar_MSS")
	      MSS2_(n) = DBLE(ExtPar_MSS(g,n))**2
	    else
	      MSS2_(n) = invalid
	    endif
	  enddo

	  if( MSS2_(1) .ne. invalid .and.
     &        MSS2_(2) .ne. invalid .and.
     &        MSS2_(3) .ne. invalid .and.
     &        MSS2_(4) .ne. invalid .and.
     &        MSS2_(5) .ne. invalid .and.
     &        Af_(2) .ne. invalid .and.
     &        Af_(3) .ne. invalid .and.
     &        Af_(4) .ne. invalid ) then
	    call SfermionMasses(error, g,
     &        MSS2_(1), MSS2_(2), MSS2_(3), MSS2_(4), MSS2_(5),
     &        Af_(2), Af_(3), Af_(4), Q_(2), Q_(3), Q_(4))
	  else if( SLHAValid(Mass_MSf(1,2,g), 3*2) .ne. 0 .and.
     &             SLHAValid(SfMix_USf(1,1,2), 3*4) .ne. 0 ) then
	    call SfermionPara(error, g,
     &        Mass_MSf(1,2,g), Mass_MSf(1,3,g), Mass_MSf(1,4,g),
     &        StauMix_USf(1,1), StopMix_USf(1,1), SbotMix_USf(1,1))
	  else
	    Missing("MSS and Af("//Digit(g)//")")
	  endif
	enddo

	call EndPara(error)

	if( SLHAExist(BlockMSQ2(1), LengthMSQ2) .ne. 0 .or.
     &      SLHAExist(BlockMSU2(1), LengthMSU2) .ne. 0 .or.
     &      SLHAExist(BlockMSD2(1), LengthMSD2) .ne. 0 ) then
	  Origin("NMFV from MSS2 and Tf")
	  call CKMRotate(MSQ2_MSQ2(1,1), MSQ2_CKM)
	  call SetNMFVSLHA(error, 3,
     &      MSQ2_CKM, MSU2_MSU2(1,1), Tu_Tf(1,1), CB)
	  call SetNMFVSLHA(error, 4,
     &      MSQ2_MSQ2(1,1), MSD2_MSD2(1,1), Td_Tf(1,1), SB)
	else if( SLHAExist(BlockMSQ2In(1), LengthMSQ2In) .ne. 0 .or.
     &           SLHAExist(BlockMSU2In(1), LengthMSU2In) .ne. 0 .or.
     &           SLHAExist(BlockMSD2In(1), LengthMSD2In) .ne. 0 ) then
	  Origin("NMFV from MSS2In and TfIn")
	  call CKMRotate(MSQ2In_MSQ2(1,1), MSQ2_CKM)
	  call SetNMFVSLHA(error, 3,
     &      MSQ2_CKM, MSU2In_MSU2(1,1), TuIn_Tf(1,1), CB)
	  call SetNMFVSLHA(error, 4,
     &      MSQ2In_MSQ2(1,1), MSD2In_MSD2(1,1), TdIn_Tf(1,1), SB)
	endif
	end


************************************************************************

	subroutine CKMRotate(in, out)
	implicit none
	double complex in(3,3), out(3,3)

#include "FH.h"

	integer i, j, k
	double complex tmp(3,3), s

	do j = 1, 3
	  do i = 1, 3
	    s = 0
	    do k = 1, 3
	      s = s + in(i,k)*CKMC(j,k)
	    enddo
	    tmp(i,j) = s
	  enddo
	enddo

	do j = 1, 3
	  do i = 1, 3
	    s = 0
	    do k = 1, 3
	      s = s + CKM(i,k)*tmp(k,j)
	    enddo
	    out(i,j) = s
	  enddo
	enddo
	end
