* TLShifts.F
* computes the two-loop corrections
* this file is part of FeynHiggs
* last modified 20 Dec 08 th

#include "externals.h"
#include "debug.h"


************************************************************************
* calculate the two-loop self-energies
* Note: this subroutine is actually called from DeltaMB.F since it also
* computes the DeltaMB compatible with the two-loop corrections.

	subroutine CalcRenSETL(error)
	implicit none
	integer error

#include "TLps.h"

	double precision mudim, dMTfin
	double precision asat(3), asab(3), atat(3)
	double precision atab(3), atat_largeTB(3)
	double precision dPhi(3,MaxSlots)
	integer i

	double precision TLInterpolate
	external TLInterpolate

	logical blurb
	data blurb /.TRUE./

	mudim = getmudim()

	do i = 1, semax
	  se2R(i) = 0
	enddo

	if( tlCplxApprox .ne. 0 ) call TLhrSE(error, mudim)

	if( tlCplxApprox .eq. 1 ) return

	if( blurb ) then
	  print *, "FHHiggsCorr contains code by:"
	  print *, "P. Slavich et al. (2-loop rMSSM Higgs self-energies)"
	  blurb = .FALSE.
	endif

	do vs = 1, 2**nvars
	  Q = mudim
	  call TLNormalTB(error, MBy)
	  if( error .ne. 0 ) return

	  if( tlCplxApprox .eq. 0 ) then
	    call TLasat(asat)
	  else
	    asat(1) = 0
	    asat(2) = 0
	    asat(3) = 0
	  endif

	  call TLatat(atat)

	  call TLLargeTB(error)
	  if( error .ne. 0 ) return

	  call TLasab(asab)
	  call TLatab(atab)

* the O(at^2) part at large TB will be subtracted:
	  call TLatat(atat_largeTB)

	  do i = 1, 3
	    dPhi(i,vs) = atat_largeTB(i) - atab(i) -
     &        atat(i) - asat(i) - asab(i)
	  enddo

	  if( debuglevel .ge. 4 ) then
	    DSELF "point ", vs ENDL
	    DSELF "Sigma11_asat =", asat(1) ENDL
	    DSELF "        atat =", atat(1) ENDL
	    DSELF "        asab =", asab(1) ENDL
	    DSELF "        atab =", atab(1) ENDL
	    DSELF "atat_largeTB =", atat_largeTB(1) ENDL
	    DSELF "Sigma12_asat =", asat(2) ENDL
	    DSELF "        atat =", atat(2) ENDL
	    DSELF "        asab =", asab(2) ENDL
	    DSELF "        atab =", atab(2) ENDL
	    DSELF "atat_largeTB =", atat_largeTB(2) ENDL
	    DSELF "Sigma22_asat =", asat(3) ENDL
	    DSELF "        atat =", atat(3) ENDL
	    DSELF "        asab =", asab(3) ENDL
	    DSELF "        atab =", atab(3) ENDL
	    DSELF "atat_largeTB =", atat_largeTB(3) ENDL
	  endif
	enddo

	do i = 1, 3
	  if( debuglevel .ge. 3 )
     &      DSELF "interpolating self-energy ", Digit(i) ENDL
	  dPhi(i,1) = TLInterpolate(dPhi(i,1), 3)
	enddo

	se2R(h0h0) = se2R(h0h0) +
     &    SA2*dPhi(1,1) + CA2*dPhi(3,1) - S2A*dPhi(2,1)
	se2R(HHHH) = se2R(HHHH) +
     &    CA2*dPhi(1,1) + SA2*dPhi(3,1) + S2A*dPhi(2,1)
	se2R(h0HH) = se2R(h0HH) +
     &    C2A*dPhi(2,1) + SA*CA*(dPhi(3,1) - dPhi(1,1))

	if( btest(runningMT, 0) ) then
	  dMTfin = -Alfat/(2*pi)*MT
	  if( tlCplxApprox .eq. 0 )
     &      dMTfin = dMTfin + 4/(3*pi)*Alfas*MT
	  call dMTcorr(error, dMTfin, mudim)
	endif

	if( debuglevel .lt. 2 ) return

	DSELF "========================"	ENDL
	DSELF "two-loop self-energies"		ENDL
	DSELF "------------------------"	ENDL
	DSELF "Sigma_11 =", dPhi(1,1)		ENDL
	DSELF "Sigma_12 =", dPhi(2,1)		ENDL
	DSELF "Sigma_22 =", dPhi(3,1)		ENDL
	DSELF "se2R(h0h0) =", se2R(h0h0)	ENDL
	DSELF "se2R(HHHH) =", se2R(HHHH)	ENDL
	DSELF "se2R(h0HH) =", se2R(h0HH)	ENDL
	DSELF "se2R(A0A0) =", se2R(A0A0)	ENDL
	DSELF "se2R(h0A0) =", se2R(h0A0)	ENDL
	DSELF "se2R(HHA0) =", se2R(HHA0)	ENDL
	DSELF "se2R(HmHp) =", se2R(HmHp)	ENDL
	end


************************************************************************

	subroutine TLShifts(error)
	implicit none
	integer error

#include "TLps.h"

	integer i, j, n, bit(MaxVars)
	double precision phase

	double precision TLInterpolate
	external TLInterpolate

	character*3 name(4)
	data name /"M_3", "MUE", "Xt", "Xb"/

	if( tlCplxApprox .eq. 1 ) return

	var(1) = M_3
	var(2) = MUE
	var(3) = Xf(3,3,3)
	var(4) = Xf(4,3,3)
	nvars = 0

	if( tlCplxApprox .eq. 3 ) then
	  monomial(1) = 1
	  do i = 1, MaxVars
	    bit(i) = MaxVars
	    if( DIMAG(var(i)) .ne. 0 ) then
	      phase = atan2(abs(DIMAG(var(i))), DBLE(var(i)))/pi
	      if( debuglevel .ge. 3 )
     &          DSELF "scaled phase ", name(i), phase ENDL
	      n = 2**nvars
	      do j = 1, n
	        monomial(j+n) = monomial(j)*phase
	      enddo
	      bit(i) = nvars
	      nvars = nvars + 1
	      var(i) = abs(var(i))
	    endif
	  enddo
	endif

#define SignedVar(i) (1-2*ibits(vs-1,bit(i),1))*var(i)

	do vs = 1, 2**nvars
	  pMGl = DBLE(SignedVar(1))
	  pMGl2 = max(pMGl**2, 1D-14)

* note that {MUEr,Atr,Abr} \equiv Re({MUEc,Atc,Abc}) (see TLps.m)

	  MUEc = SignedVar(2)
	  if( abs(MUEc) .lt. 1D-7 ) MUEc = 1D-7
	  MUEr2 = MUEr**2

	  Xtc = SignedVar(3)
	  Atc = Xtc + MUEc/TB
	  Xbc = SignedVar(4)
	  Abc = Xbc + MUEc*TB

	  if( debuglevel .gt. 3 ) then
	    DSELF "point ", vs ENDL
	    DSELF name(1), pMGl ENDL
	    DSELF name(2), MUEc ENDL
	    DSELF name(3), Xtc ENDL
	    DSELF name(4), Xbc ENDL
	  endif

	  BSS2(1) = DSS2(1,5,3)
	  BSS2(2) = DSS2(2,5,3)
	  DeltaMB = 0
	  MBy = Mfy(4,3)

	  if( tlBotResum .ne. 0 ) call TLDeltaMB(error)

	  if( debuglevel .gt. 3 ) then
	    DSELF "BSS2(1) =", BSS2(1)
	    DSELF "DeltaMB =", DeltaMB
	    DSELF "MBy     =", MBy
	  endif
	enddo

	vs = 1

	if( debuglevel .ge. 3 )
     &    DSELF "interpolating BSS" ENDL
	DSS2(1,4,3) = TLInterpolate(BSS2(1), Nvr)
	DSS2(2,4,3) = DSS2(2,5,3)

	if( debuglevel .ge. 3 )
     &    DSELF "interpolating DeltaMB" ENDL
	DeltaMBc(4) = TLInterpolate(DeltaMB, Nvr)

	if( debuglevel .ge. 3 )
     &    DSELF "interpolating MBy" ENDL
	Mfy(4,3) = TLInterpolate(MBy, Nvr)
	end


************************************************************************

	double precision function TLInterpolate(data, lddata)
	implicit none
	integer lddata
	double precision data(lddata,*)

#include "TLps.h"

	double precision val(MaxSlots), res
	integer n, i, j, k, stride

	if( nvars .eq. 0 ) then
	  TLInterpolate = data(1,1)
	  return
	endif

	n = 2**nvars

	do j = 1, n, 2
	  val(j) = data(1,j)
	  val(j+1) = data(1,j+1) - val(j)
	enddo

	stride = 1
	do i = 2, nvars
	  stride = 2*stride
	  do j = 1, n, 2*stride
	    do k = j, j + stride - 1
	      val(k+stride) = val(k+stride) - val(k)
	    enddo
	  enddo
	enddo

	res = 0
	do i = 1, n
	  res = res + val(i)*monomial(i)
	enddo

	TLInterpolate = res

	if( debuglevel .lt. 3 ) return

	do i = 1, n
	  DSELF "  data =", data(1,i) ENDL
	enddo
	DSELF "  result =", res ENDL
	end


************************************************************************

	double precision function mld(m1, m2)
	implicit none
	double precision m1, m2

	if( abs(m1 - m2) .lt. 1D-12 ) then
	  mld = 1
	else
	  mld = m1/(m1 - m2)*log(m1/m2)
	endif
	end

************************************************************************

	subroutine TLDeltaMB(error)
	implicit none
	integer error

#include "TLps.h"

	double precision db(2), eb(2), dhbhb(2), dMSQ2(2), dvevvev

	double precision mld
	external mld

	call TLNormalTB(error, TLMBrun)
	if( error .ne. 0 ) return

	Q = TLMBScale**2

* For computing the bottom shifts, we use MTrun only in ht.
* In all other places, we use MT proper.
	ht = ht*MTy/MT
	ht2 = ht**2

	call TLStrongShifts(db(1), dhbhb(1), dMSQ2(1))

	eb(1) = CF/(4*pi)*Alfas*2*MUEr*pMGl*TB*
     &    (mld(MSb22, pMGl2) - mld(MSb12, pMGl2))/MSb2diff

	eb(2) = 1/(16*pi**2)*ht2*Atr*MUEr*TB*
     &    (mld(MSt22, MUEr2) - mld(MSt12, MUEr2))/MSt2diff

	call TLLargeTB(error)
	if( error .ne. 0 ) return

	call TLYukawaShifts(db(2), dhbhb(2), dMSQ2(2), dvevvev)

	BSS2(1) = BSS2(1) + dMSQ2(1) + dMSQ2(2)

*** 264
	MBy = TLMBrun*((1 + db(1) + db(2))/
c	MBy = MBrun*((1 + db(1) + db(2))/
     &      ((1 + dvevvev)*abs(1 + eb(1) + eb(2))) -
     &    dhbhb(1) - dhbhb(2))
*** 264
	DeltaMB = TLMBrun/MBy - 1
c	DeltaMB = MBrun/MBy - 1

c	DeltaMBc(2) = eb(1) + eb(2)
c	call BUpdate
	end


************************************************************************

	subroutine TLNormalTB(error, MB_)
	implicit none
	integer error
	double precision MB_

#include "TLps.h"
#define __SUBROUTINE__ "TLNormalTB"

	logical fail

	pCB2 = CB2
	pSB2 = SB2

	pMT = MTy
	pMT2 = pMT**2

	ht = sqrt2/vev*pMT/SB
	ht2 = ht**2

	call TLMSf(fail, StopPara(1), DSS2(1,3,3),
     &    MTresum(ibits(runningMT, 1, 1)), Xtc)
	if( fail ) Error("negative stop mass squares")

	Xtr = -.5D0*MSt2diff*s2tt/pMT
	Ytr = Atr + MUEr*TB

	pMB = MB_
	pMB2 = pMB**2

	hb = sqrt2/vev*pMB/CB
	hb2 = hb**2

	call TLMSf(fail, SbotPara(1), BSS2(1), pMB, Xbc)
	if( fail ) Error("negative sbottom mass squares")

	if( debuglevel .ge. 4 ) then
	  DSELF "normalTB:"			ENDL
	  DSELF "MStop1 =", SignSqrt(MSt12)	ENDL
	  DSELF "MStop2 =", SignSqrt(MSt22)	ENDL
	  DSELF "ctt    =", ctt			ENDL
	  DSELF "stt    =", stt			ENDL
	  DSELF "MSbot1 =", SignSqrt(MSb12)	ENDL
	  DSELF "MSbot2 =", SignSqrt(MSb22)	ENDL
	  DSELF "ctb    =", ctb			ENDL
	  DSELF "stb    =", stb			ENDL
	endif
	end

#undef __SUBROUTINE__


************************************************************************

	subroutine TLLargeTB(error)
	implicit none
	integer error

#include "TLps.h"
#define __SUBROUTINE__ "TLLargeTB"

	double precision TBlarge
	parameter (TBlarge = 1000)

	double precision x, DSS2large(2)
	logical fail

	pCB2 = 1/(1 + TBlarge**2)
	pSB2 = 1

* Note: for convenience, SB, CB, and C2B keep their old
* (normal TB) values -- they are not used in the TLa?a? terms.

	Xtr = Atr
	Ytr = Atr + MUEr*TBlarge

	ht = sqrt2/vev*pMT
	ht2 = ht**2

	x = 1 + 1/C2B
	DSS2large(1) = DSS2(1,3,3) - x*(DSf(1,3) - DSf(2,3))
	DSS2large(2) = DSS2(2,3,3) - x*DSf(2,3)

c	call TLMSf(fail, StopPara(1), DSS2large, pMT, Atc)
	call TLMSf(fail, StopPara(1), DSS2large, MT, Atc)
	if( fail ) Error("negative stop mass squares")

	DSS2large(1) = BSS2(1) - x*(DSf(1,4) - DSf(2,4))
	DSS2large(2) = BSS2(2) - x*DSf(2,4)

	call TLMSf(fail, SbotPara(1), DSS2large, pMB, -MUEc/CB)
	if( fail ) Error("negative sbottom mass squares")

	if( debuglevel .ge. 4 ) then
	  DSELF "largeTB:"			ENDL
	  DSELF "MStop1 =", SignSqrt(MSt12)	ENDL
	  DSELF "MStop2 =", SignSqrt(MSt22)	ENDL
	  DSELF "ctt    =", ctt			ENDL
	  DSELF "stt    =", stt			ENDL
	  DSELF "MSbot1 =", SignSqrt(MSb12)	ENDL
	  DSELF "MSbot2 =", SignSqrt(MSb22)	ENDL
	  DSELF "ctb    =", ctb			ENDL
	  DSELF "stb    =", stb			ENDL
	endif
	end


************************************************************************

	subroutine TLStrongShifts(db, dhbhb, dMSQ2)
	implicit none
	double precision db, dhbhb, dMSQ2

#include "TLps.h"

	double precision logMT2, logMGl2
	double precision logMSt12, logMSt22, logMSb12, logMSb22
	double precision b0MSt12, b0MSt22
	double precision stop11, stop12, stop22
	double precision sbot11, sbot12, sbot22, top

	logMT2 = log(pMT2/Q)
	logMGl2 = log(pMGl2/Q)
	logMSt12 = log(MSt12/Q)
	logMSt22 = log(MSt22/Q)
	logMSb12 = log(MSb12/Q)
	logMSb22 = log(MSb22/Q)
	b0MSt12 = myB0(MSt12, pMT2, pMGl2, Q)
	b0MSt22 = myB0(MSt22, pMT2, pMGl2, Q)

	db = CF/(4*pi)*Alfas*(3/2D0 - logMGl2 + .5D0*(
     &    MSb12/(pMGl2 - MSb12)*(1 - (logMGl2 - logMSb12)*
     &      (1 + pMGl2/(pMGl2 - MSb12) + 4*pMGl*Abr/MSb2diff)) +
     &    MSb22/(pMGl2 - MSb22)*(1 - (logMGl2 - logMSb22)*
     &      (1 + pMGl2/(pMGl2 - MSb22) - 4*pMGl*Abr/MSb2diff)) ))

	dhbhb = CF/(4*pi)*Alfas*(-4 - 2*logMGl2 - 2/MSb2diff*(
     &    MSb12*(2*logMSb12 -
     &      (1 - pMGl2/MSb12)**2*log(abs(1 - MSb12/pMGl2))) -
     &    MSb22*(2*logMSb22 -
     &      (1 - pMGl2/MSb22)**2*log(abs(1 - MSb22/pMGl2))) ))

	top = pMT*(3*logMT2 + MSDR) +
     &    .5D0/pMT*(2*pMGl2*(logMGl2 - 1) -
     &      MSt12*(logMSt12 - 1) - MSt22*(logMSt22 - 1) +
     &      (pMGl2 + pMT2 - MSt12 - 2*s2tt*pMT*pMGl)*
     &        myB0(pMT2, pMGl2, MSt12, Q) +
     &      (pMGl2 + pMT2 - MSt22 + 2*s2tt*pMT*pMGl)*
     &        myB0(pMT2, pMGl2, MSt22, Q))

	stop11 = MSt12*(3*logMSt12 - 7 - c2tt2*(logMSt12 - 1)) -
     &    s2tt2*MSt22*(logMSt22 - 1) +
     &    2*(pMGl2*(logMGl2 - 1) + pMT2*(logMT2 - 1) +
     &      (MSt12 - pMGl2 - pMT2 + 2*s2tt*pMT*pMGl)*b0MSt12)

	stop22 = MSt22*(3*logMSt22 - 7 - c2tt2*(logMSt22 - 1)) -
     &    s2tt2*MSt12*(logMSt12 - 1) +
     &    2*(pMGl2*(logMGl2 - 1) + pMT2*(logMT2 - 1) +
     &      (MSt22 - pMGl2 - pMT2 - 2*s2tt*pMT*pMGl)*b0MSt22)

	stop12 =
     &    c2tt*s2tt*(MSt22*(1 - logMSt22) - MSt12*(1 - logMSt12)) +
     &    2*pMT*pMGl*c2tt*(b0MSt12 + b0MSt22)

	sbot11 = MSb12*(3*logMSb12 - 3 - c2tb2*(logMSb12 - 1)) -
     &    s2tb2*MSb22*(logMSb22 - 1) - 6*pMGl2 -
     &    2*(MSb12 - 2*pMGl2)*logMGl2 -
     &    2*(MSb12 - pMGl2)**2/MSb12*log(abs(1 - MSb12/pMGl2))

	sbot22 = MSb22*(3*logMSb22 - 3 - c2tb2*(logMSb22 - 1)) -
     &    s2tb2*MSb12*(logMSb12 - 1) - 6*pMGl2 -
     &    2*(MSb22 - 2*pMGl2)*logMGl2 -
     &    2*(MSb22 - pMGl2)**2/MSb22*log(abs(1 - MSb22/pMGl2))

	sbot12 =
     &    c2tb*s2tb*(MSb22*(1 - logMSb22) - MSb12*(1 - logMSb12))

	dMSQ2 = CF/(4*pi)*Alfas*(
     &    ctt2*stop11 + stt2*stop22 - 2*stt*ctt*stop12 -
     &    ctb2*sbot11 - stb2*sbot22 + 2*stb*ctb*sbot12 -
     &    2*pMT*top )
	end


************************************************************************

	subroutine TLYukawaShifts(db, dhbhb, dMSQ2, dvevvev)
	implicit none
	double precision db, dhbhb, dMSQ2, dvevvev

#include "TLps.h"

	double precision logMT2, logMA02, top, deltaM, t1, t2
	double precision stop11, stop12, stop22
	double precision sbot11, sbot12, sbot22

	double precision Pi11t, Pi22t, Pi12t
	double precision Pi11b, Pi22b, Pi12b, Pitt
	external Pi11t, Pi22t, Pi12t, Pi11b, Pi22b, Pi12b, Pitt

	double precision x, y, fW
	fW(x, y) = .25D0*(x + y) - .5D0*x*y/(x - y)*log(x/y)

	logMT2 = log(pMT2/Q)
	logMA02 = log(MA02/Q)

	top = Pitt(pMT2)

	stop11 = Pi11t(MSt12)
	stop22 = Pi22t(MSt22)
	stop12 = .5D0*(Pi12t(MSt12) + Pi12t(MSt22))

	sbot11 = Pi11b(MSb12)
	sbot22 = Pi22b(MSb22)
	sbot12 = .5D0*(Pi12b(MSb12) + Pi12b(MSb22))

	deltaM = pMT2 - MA02
	if( abs(deltaM) .lt. 1D-13 ) then
	  t1 = -logMT2
	  t2 = -.5D0
	else
	  t1 = 1 - (pMT2*logMT2 - MA02*logMA02)/deltaM
	  t2 = pMT2*(deltaM - pMT2*(logMT2 - logMA02))/deltaM**2
	endif

	db = 1/(32*pi**2)*(
     &    ht2*(6/4D0*logMT2 - 5/4D0 + 2*t1 +
     &         2*MUEr2/MSt2diff*(myB0(0D0, MUEr2, MSt12, Q) -
     &                           myB0(0D0, MUEr2, MSt22, Q))) +
     &    hb2*(.5D0*t2 + 3/4D0 - 3/2D0*logMA02 +
     &         myB1(0D0, MUEr2, MSb12, Q) +
     &         myB1(0D0, MUEr2, MSb22, Q)) +
     &    (hb2*ctt2 + ht2*stt2)*myB1(0D0, MUEr2, MSt12, Q) +
     &    (hb2*stt2 + ht2*ctt2)*myB1(0D0, MUEr2, MSt22, Q) )

	dvevvev = Nc/(16*pi**2)*ht2*(.5D0*logMT2 - .25D0 -
     &    (ctt2*ctb2*fW(MSt12, MSb12) + 
     &     ctt2*stb2*fW(MSt12, MSb22) +
     &     stt2*ctb2*fW(MSt22, MSb12) +
     &     stt2*stb2*fW(MSt22, MSb22))/pMT2)

	dhbhb = (sbot22 - sbot11 - 2*c2tb/s2tb*sbot12)/
     &      (16*pi**2*MSb2diff) -
     &    dvevvev

	dMSQ2 = 1/(16*pi**2)*(
     &    ctt2*stop11 + stt2*stop22 - s2tt*stop12 -
     &    ctb2*sbot11 - stb2*sbot22 + s2tb*sbot12 -
     &    2*pMT*top )
	end


************************************************************************

#undef StopPara

	subroutine TLMSf(fail, StopPara, M11, Mf, Xf)
	implicit none
	logical fail
	double precision StopPara(*), M11(2), Mf
	double complex Xf

	double precision MSf(2)
	double complex USf(2,2)

	call EigenSf(fail, MSf, MSt12, USf, M11, Mf, Xf, 0)

	stt = DBLE(USf(1,2))
	stt2 = stt**2
	ctt2 = (1 - stt)*(1 + stt)
	ctt = sqrt(ctt2)

	s2tt = 2*stt*ctt
	s2tt2 = s2tt**2
	c2tt = (ctt - stt)*(ctt + stt)
	c2tt2 = c2tt**2
	end

