* AlphaS.F
* AlphaS computes the strong coupling constant at a given scale.
* Code is transcribed from I. Hinchliffe's Java applet at
* http://www-theory.lbl.gov/~ianh/alpha/alpha.html.
* this file is part of FeynHiggs
* last modified 24 Jun 09 th
* Modified by Hamish Silverwood, October 2011, to fix the segmentation
* fault which arises when nf_to is called without first being set, which
* occurs if scale is less than 5D0

#include "externals.h"


	double precision function AlphaS(scale, nf)
	implicit none
	double precision scale
	integer nf

#include "FH.h"

* the betas and deltas are calculated as follows:
*
* beta0[nf_] := 11 - 2/3 nf
* beta1[nf_] := 51 - 19/3 nf
* beta2[nf_] := 2857 - 5033/9 nf + 325/27 nf^2
*
* delta0[nf_] := (beta0[nf] - beta0[nf + 1])/(2 beta0[nf])
* delta10[nf_] := beta1[nf]/beta0[nf]^2
* delta11[nf_] := (beta1[nf]/beta0[nf] -
*   beta1[nf + 1]/beta0[nf + 1])/beta0[nf]
* delta20[nf_] := (28/9 -
*   8 (beta1[nf + 1]/beta0[nf + 1])^2 + beta2[nf + 1]/beta0[nf + 1] +
*   8 (beta1[nf]/beta0[nf])^2 - beta2[nf]/beta0[nf])/
*   (4 beta0[nf + 1] beta0[nf])
* delta21[nf_] := 2 beta1[nf + 1]/beta0[nf + 1]^2 delta11[nf]

	double precision beta3_0, beta3_1, beta3_2
	parameter (beta3_0 = 9)
	parameter (beta3_1 = 32)
	parameter (beta3_2 = 3863/3D0)

	double precision beta4_0, beta4_1, beta4_2
	parameter (beta4_0 = 25/3D0)
	parameter (beta4_1 = 77/3D0)
	parameter (beta4_2 = 21943/27D0)

	double precision beta5_0, beta5_1, beta5_2
	parameter (beta5_0 = 23/3D0)
	parameter (beta5_1 = 58/3D0)
	parameter (beta5_2 = 9769/27D0)

	double precision beta6_0, beta6_1, beta6_2
	parameter (beta6_0 = 7)
	parameter (beta6_1 = 13)
	parameter (beta6_2 = -65)

	double precision delta54_0, delta54_10, delta54_11
	double precision delta54_20, delta54_21
	parameter (delta54_0  =  .040000000000000000000D0)
	parameter (delta54_10 = -.030817842663873271183D0)
	parameter (delta54_11 =  .066991304347826086957D0)
	parameter (delta54_20 = -.086878340429029341662D0)
	parameter (delta54_21 =  .044069893975507520342D0)

	double precision delta43_0, delta43_10, delta43_11
	double precision delta43_20, delta43_21
	parameter (delta43_0  =  .037037037037037037037D0)
	parameter (delta43_10 = -.030404361930322301228D0)
	parameter (delta43_11 =  .052839506172839506173D0)
	parameter (delta43_20 = -.057313053497942386831D0)
	parameter (delta43_21 =  .039058962962962962963D0)

	double precision lam5
	parameter (lam5 = 204D-3)
* The 5-flavour scale lam5 is chosen to reproduce the
* "world average" alpha_s(MZ) = .1176.

	double precision b0, b1, b2, lam, t, logt, c

	if( scale .lt. .5D0 ) then
	  AlphaS = 1D10
	  write(*,*) 'In AlphaS less than 5, scale = ', scale
	  return
	endif

	if( scale .gt. MT ) then
	  nf = 6
	  b0 = beta6_0
	  b1 = beta6_1
	  b2 = beta6_2
	  t = MT/lam5
	  lam = lam5*t**(-2/21D0)*(2*log(t))**(-107/1127D0)
	else if( scale .gt. MB ) then
	  nf = 5
	  b0 = beta5_0
	  b1 = beta5_1
	  b2 = beta5_2
	  lam = lam5
	else
	  t = 2*log(MB/lam5)
	  logt = log(t)
	  lam = lam5*exp(delta54_0*t +
     &      delta54_11*logt + delta54_10 +
     &      (delta54_21*logt + delta54_20)/t)
	  if( scale .gt. MC ) then
	    nf = 4
	    b0 = beta4_0
	    b1 = beta4_1
	    b2 = beta4_2
	  else
	    nf = 3
	    b0 = beta3_0
	    b1 = beta3_1
	    b2 = beta3_2
	    t = 2*log(MC/lam)
	    logt = log(t)
	    lam = lam*exp(delta43_0*t +
     &        delta43_11*logt + delta43_10 +
     &        (delta43_21*logt + delta43_20)/t)
	  endif
	endif 

	t = 2*log(scale/lam)
	logt = log(t)
	c = 2*b1/b0**2/t

	AlphaS = 4*pi/(t*b0)*(1 - logt*c +
     &    ((logt - .5D0)**2 + b2*b0/(8*b1**2) - 5/4D0)*c**2)
	end


************************************************************************

	double precision function Run(Q_from, Q_to)
	implicit none
	double precision Q_from, Q_to

#include "FH.h"

	double precision scale(3:5), as1, as2
	integer nf, nf_from, nf_to, nf1, nf2

	double precision g0, eps
	parameter (g0 = -8, eps = 1D-12)

	double precision AlphaS
	external AlphaS

	double precision b0, cfn, as

* for the coefficients see running.m
	double precision c1(3:6), c2(3:6)
	data c1 / .2849069968928990578578783D0,
     &            .3228086659085215796955013D0,
     &            .3741695606023071819494047D0,
     &            .4449842286446869591905526D0 /
	data c2 / .1389552059662087337299132D0,
     &            .1407560845491501961210594D0,
     &            .1520533792527639666058513D0,
     &            .1817174007626149992000586D0 /

	b0(nf) = 11 - 2/3D0*nf
	cfn(as, nf) = (b0(nf)*as/(2*pi))**(-g0/(2*b0(nf))) *
     &    (1 + as*(c1(nf) + as*c2(nf)))

	scale(3) = MC
	scale(4) = MB
	scale(5) = MT

	as1 = AlphaS(Q_to, nf_to)
	as2 = AlphaS(Q_from, nf_from)
* NB: 1/(3*pi)*Alfas is the MSbar -> DRbar correction


	if (abs(nf_to).gt.16) then
	   write(*,*) 'nf_to is out of bounds & will cause segfault,
     & resetting to 15'
	   nf_to = 15
	endif

	Run = cfn(as1, nf_to)/cfn(as2, nf_from)*(1 - 1/(3*pi)*as2)
* nf_to > nf_from
	do nf = nf_from, nf_to - 1
	  as1 = AlphaS(scale(nf), nf1)
	  as2 = AlphaS(scale(nf) + eps, nf2)
	  Run = cfn(as1, nf1)/cfn(as2, nf2)*Run
	enddo

* nf_to < nf_from
	do nf = nf_from - 1, nf_to, -1
	  as1 = AlphaS(scale(nf), nf1)
	  as2 = AlphaS(scale(nf) + eps, nf2)
	  Run = cfn(as2, nf2)/cfn(as1, nf1)*Run
	enddo
	end

