* CalcSelf.F
* Calculation of the renormalized Higgs self-energies
* this file is part of FeynHiggs
* last modified 26 Aug 10 th

#include "externals.h"
#include "debug.h"


	subroutine CalcSelf(k2, key, dkey)
	implicit none
	double precision k2
	integer key, dkey

#include "FH.h"
#include "OneLoop.h"

#define SelfEnergy(i,f,ty) if( ibits(key,i-1,1) .eq. 1 ) \
seR(i) = f(p2,ty)+dZ(i)*(p2-Msq(i))-dMsq(i)+se2R(i)+seAdd(i)

#define DSelfEnergy(i,f,ty) if( ibits(dkey,i-1,1) .eq. 1 ) \
dseR(i) = f(p2,ty) + dZ(i)

	double precision p2
	logical onshell
	integer i

* p2approx decides at which p^2 the self-energies are to be evaluated
	p2 = k2
	onshell = p2approx .eq. 2 .or.
     &    ((p2approx .eq. 0 .or. p2approx .eq. 3) .and. p2 .eq. -1)
	if( p2approx .eq. 1 .or. p2 .eq. -1 ) p2 = 0

	do i = 1, semax
	  seR(i) = 0
	  dseR(i) = 0
	enddo

* 1) the CP-even bosons h & H + the mixing h-H

	if( iand(key, cpeven) .ne. 0 .or.
     &      iand(dkey, cpeven) .ne. 0 ) then
	  if( mssmpart .eq. 0 ) then
	    call CPevenSEs_MTapprox
	  else
	    if( onshell ) p2 = Msq(h0h0)
	    SelfEnergy(h0h0, se_h0h0, 5)
	    DSelfEnergy(h0h0, dse_h0h0, 5)

	    if( onshell ) p2 = Msq(HHHH)
	    SelfEnergy(HHHH, se_HHHH, 5)
	    DSelfEnergy(HHHH, dse_HHHH, 5)

	    if( onshell ) p2 = Msq(h0HH)
	    SelfEnergy(h0HH, se_h0HH, 5)
	    DSelfEnergy(h0HH, dse_h0HH, 5)
	  endif
	endif

* 2) the CP-odd boson A

	if( onshell ) p2 = Msq(A0A0)
	SelfEnergy(A0A0, se_A0A0, 4)
	DSelfEnergy(A0A0, dse_A0A0, 4)

* 3) the CP-violating h-A & H-A mixing

	if( onshell ) p2 = Msq(h0A0)
	SelfEnergy(h0A0, se_h0A0, 4)
	DSelfEnergy(h0A0, dse_h0A0, 4)

	if( onshell ) p2 = Msq(HHA0)
	SelfEnergy(HHA0, se_HHA0, 4)
	DSelfEnergy(HHA0, dse_HHA0, 4)

* 4) the charged boson H+

#if 1
	if( iand(key, Key(HmHp)) .ne. 0 .or.
     &      iand(dkey, Key(HmHp)) .ne. 0 ) then
	  if( mssmpart .eq. 0 ) then
	    call HpSEs_MTapprox
	  else
	    if( onshell ) p2 = Msq(HmHp)
	    SelfEnergy(HmHp, se_HmHp, 4)
	    DSelfEnergy(HmHp, dse_HmHp, 4)
	  endif
	endif
#else
	if( onshell ) p2 = Msq(HmHp)
	SelfEnergy(HmHp, se_HmHp, 4)
	DSelfEnergy(HmHp, dse_HmHp, 4)
#endif

* 5) the Goldstones

	if( onshell ) p2 = Msq(G0G0)
	SelfEnergy(G0G0, se_G0G0, 4)
	DSelfEnergy(G0G0, dse_G0G0, 4)

	if( onshell ) p2 = Msq(h0G0)
	SelfEnergy(h0G0, se_h0G0, 4)
	DSelfEnergy(h0G0, dse_h0G0, 4)

	if( onshell ) p2 = Msq(HHG0)
	SelfEnergy(HHG0, se_HHG0, 4)
	DSelfEnergy(HHG0, dse_HHG0, 4)

	if( onshell ) p2 = Msq(A0G0)
	SelfEnergy(A0G0, se_A0G0, 4)
	DSelfEnergy(A0G0, dse_A0G0, 4)

	if( onshell ) p2 = Msq(GmGp)
	SelfEnergy(GmGp, se_GmGp, 4)
	DSelfEnergy(GmGp, dse_GmGp, 4)

	if( onshell ) p2 = Msq(HmGp)
	SelfEnergy(HmGp, se_HmGp, 4)
	DSelfEnergy(HmGp, dse_HmGp, 4)


	if( p2approx .eq. 3 ) then
	  do i = 1, semax
	    seR(i) = DBLE(seR(i))
	    dseR(i) = DBLE(dseR(i))
	  enddo
	endif
	end


************************************************************************
* CPevenSEs_MTapprox: the one-loop CP-even renormalized self-energies
* in the MT^4-approximation as specified in Sven Heinemeyer's PhD
* thesis, p. 74.

	subroutine CPevenSEs_MTapprox
	implicit none

#include "FH.h"

	double precision mlog, x, x2, c
	double precision sigma_t, lam_t, omega_t

	if( MSf2(3,3,3) .lt. 1D-12 ) then
	  x = DBLE(Xf(3,3))/MSf2(1,3,3)
	  x2 = -1/12D0*x**2
	else
	  mlog = log(MSf2(2,3,3)/MSf2(1,3,3))
	  x = DBLE(Xf(3,3))/MSf2(3,3,3)
	  x2 = x**2*(1 - .5D0*MSf2(4,3,3)/MSf2(3,3,3)*mlog)
	  x = x*mlog
	endif

	c = 3*GF*MTy2**2/(sqrt2*pi**2*SB2)

	sigma_t = c*MUE**2*x2

	lam_t = c*MUE*(-.5D0*x - Af(3,3)*x2)

	omega_t = c*(log(MSf(1,3,3)*MSf(2,3,3)/MT2) +
     &    Af(3,3)*(x + Af(3,3)*x2))

	seR(h0h0) = -(SA2*sigma_t + CA2*omega_t - 2*SA*CA*lam_t)
	seR(HHHH) = -(CA2*sigma_t + SA2*omega_t + 2*SA*CA*lam_t)
	seR(h0HH) = -((CA2 - SA2)*lam_t + SA*CA*(omega_t - sigma_t))
	end


************************************************************************
* HpSEs_MTapprox: the one-loop renormalized charged-Higgs self-energy
* in the MT^4-approximation

#define __SUBROUTINE__ "HpSEs_MTapprox"

	subroutine HpSEs_MTapprox
	implicit none

#include "FH.h"

	double precision DSStop2(2), MStop2(4), MStop(2)
	double complex UStop(2,2)
	double precision d, MSbot12, mlog1, dmlog
	integer fail

	double precision ctt, stt
	equivalence (UStop(1,1), ctt)
	equivalence (UStop(1,2), stt)

* use squark masses w/o D-terms
	DSStop2(1) = DSS2(1,3,3) - (DSf(1,3) - DSf(2,3))
	DSStop2(2) = DSS2(2,3,3) - DSf(2,3)
	if( DSStop2(1)**2 + Sq(Xf(3,3)) .lt. .1D0 )
     &    DSStop2(1) = -.2D0*sqrt(DSStop2(2))
	fail = 0
	call EigenSf(fail, 1, MStop, MStop2, UStop,
     &    DSStop2, MTy, Xf(3,3), 0)
	if( fail .ne. 0 ) Warning("negative stop mass squares")

	d = MTy2 - stt**2*MStop2(3)
	MSbot12 = MStop2(1) - d

	dmlog = log(MStop2(1))
	mlog1 = MStop2(1)*(dmlog - 1)
	if( abs(MStop2(3)) .gt. 1D-12 )
     &    dmlog = (MStop2(2)*(log(MStop2(2)) - 1) - mlog1)/MStop2(3)

	seR(HmHp) = 3/(8.D0*pi)*AlfaGF*MTy2/
     &    (MW2*SW2*SB2*(d + MStop2(3)))*
     &    (MTy*(Af(3,3)*CB + MUE*SB) + CB*ctt*abs(stt)*MStop2(3))**2*
     &    (dmlog + (MSbot12*(log(MSbot12) - 1) - mlog1)/d)
	end


************************************************************************
* CalcCTs: calculate the relevant counterterms in the Higgs sector.

	subroutine CalcCTs
	implicit none

#include "FH.h"
#include "OneLoop.h"

	double precision d1, d2, c
	double precision dZH(2,4:5), dTB(4:5)
	double precision dMZ2, dMA, dZA0G0, dTHH, dTh0, dTA0

	Msq(h0h0) = Mh02
	Msq(HHHH) = MHH2
	Msq(A0A0) = MA02
	Msq(HmHp) = MHp2
	Msq(h0HH) = .5D0*(Msq(h0h0) + Msq(HHHH))
	Msq(h0A0) = .5D0*(Msq(h0h0) + Msq(A0A0))
	Msq(HHA0) = .5D0*(Msq(HHHH) + Msq(A0A0))
	Msq(G0G0) = 0
	Msq(h0G0) = .5D0*(Msq(h0h0) + Msq(G0G0))
	Msq(HHG0) = .5D0*(Msq(HHHH) + Msq(G0G0))
	Msq(A0G0) = .5D0*(Msq(A0A0) + Msq(G0G0))
	Msq(GmGp) = 0
	Msq(HmGp) = .5D0*(Msq(HmHp) + Msq(GmGp))

* choose the field renormalization:
	if( fieldren .eq. 4 ) then
* on-shell field renormalization IV goes like this...
	  dZ(h0h0) = -DBLE(dse_h0h0(0D0, 5))
	  dZ(HHHH) = -DBLE(dse_HHHH(0D0, 5))
	  dZ(h0HH) = -DBLE(dse_h0HH(0D0, 5))

	  dZ(A0A0) = -DBLE(dse_A0A0(0D0, 4))
	  dZ(HmHp) = -DBLE(dse_HmHp(0D0, 4))
	  dZ(h0A0) = -DBLE(dse_h0A0(0D0, 4))
	  dZ(HHA0) = -DBLE(dse_HHA0(0D0, 4))
	  dZ(G0G0) = -DBLE(dse_G0G0(0D0, 4))
	  dZ(h0G0) = -DBLE(dse_h0G0(0D0, 4))
	  dZ(HHG0) = -DBLE(dse_HHG0(0D0, 4))
	  dZ(A0G0) = -DBLE(dse_A0G0(0D0, 4))
	  dZA0G0 = -DBLE(dse_A0G0(0D0, 5))
	  dZ(GmGp) = -DBLE(dse_GmGp(0D0, 4))
	  dZ(HmGp) = -DBLE(dse_HmGp(0D0, 4))
	else
* ... all others use these relations:
	  if( fieldren .eq. 0 ) then
	    dZH(1,4) = 0
	    dZH(2,4) = 0
	    dZH(1,5) = 0
	    dZH(2,5) = 0
	  else if( fieldren .eq. 1 ) then
	    d1 = DBLE(dse_A0A0(MA02, 4))
	    d2 = DBLE(se_A0Z(MA02, 4))
	    dZH(1,4) = -d1 - d2/(MZ*TB)
	    dZH(2,4) = -d1 + d2*TB/MZ
	    d1 = DBLE(dse_A0A0(MA02, 5))
	    d2 = DBLE(se_A0Z(MA02, 5))
	    dZH(1,5) = -d1 - d2/(MZ*TB)
	    dZH(2,5) = -d1 + d2*TB/MZ
	  else if( fieldren .eq. 2 ) then
	    d1 = DBLE(dse_G0G0(MZ2, 4))
	    d2 = DBLE(dse_A0A0(MA02, 4))
	    dZH(1,4) = -( CB2*d1 - SB2*d2)/C2B
	    dZH(2,4) = -(-SB2*d1 + CB2*d2)/C2B
	    d1 = DBLE(dse_G0G0(MZ2, 5))
	    d2 = DBLE(dse_A0A0(MA02, 5))
	    dZH(1,5) = -( CB2*d1 - SB2*d2)/C2B
	    dZH(2,5) = -(-SB2*d1 + CB2*d2)/C2B
	  else
	    d1 = DBLE(dse_HHHH(MHH2, 4))
	    d2 = DBLE(dse_h0h0(Mh02, 4))
	    dZH(1,4) = -( CA2*d1 - SA2*d2)/C2A
	    dZH(2,4) = -(-SA2*d1 + CA2*d2)/C2A
	    d1 = DBLE(dse_HHHH(MHH2, 5))
	    d2 = DBLE(dse_h0h0(Mh02, 5))
	    dZH(1,5) = -( CA2*d1 - SA2*d2)/C2A
	    dZH(2,5) = -(-SA2*d1 + CA2*d2)/C2A
	  endif

	  dZ(h0h0) = SA2*dZH(1,5) + CA2*dZH(2,5)
	  dZ(HHHH) = CA2*dZH(1,5) + SA2*dZH(2,5)
	  dZ(h0HH) = SA*CA*(dZH(2,5) - dZH(1,5))

	  dZ(A0A0) = SB2*dZH(1,4) + CB2*dZH(2,4)
	  dZ(HmHp) = dZ(A0A0)
	  dZ(h0A0) = 0
	  dZ(HHA0) = 0
	  dZ(G0G0) = CB2*dZH(1,4) + SB2*dZH(2,4)
	  dZ(GmGp) = dZ(G0G0)
	  dZ(h0G0) = 0
	  dZ(HHG0) = 0
	  dZ(A0G0) = SB*CB*(dZH(2,4) - dZH(1,4))
	  dZA0G0 = SB*CB*(dZH(2,5) - dZH(1,5))
	  dZ(HmGp) = dZ(A0G0)
	endif

* renormalization of TB:
	if( tanbren .eq. 0 ) then
	  dTB(4) = 0
	  dTB(5) = 0
	else if( tanbren .eq. 1 ) then
	  dTB(4) = .5D0*(dZH(2,4) - dZH(1,4))
	  dTB(5) = .5D0*(dZH(2,5) - dZH(1,5))
	else
	  dTB(4) = (DBLE(se_A0Z(MA02, 4)) + .5D0*dZ(A0G0)*MZ)/(SB*CB*MZ)
	  dTB(5) = (DBLE(se_A0Z(MA02, 5)) + .5D0*dZA0G0*MZ)/(SB*CB*MZ)
	endif

* depending on `inputmass', either MA0 or MHp is defined on-shell:
	if( inputmass .eq. 3 ) then
	  dMsq(A0A0) = DBLE(se_A0A0(MA02, 4))
	  dMsq(HmHp) = dMsq(A0A0) + DBLE(se_WW(MW2, 4))
	  dMA = DBLE(se_A0A0(MA02, 5))
	else
	  dMsq(HmHp) = DBLE(se_HmHp(MHp2, 4))
	  dMsq(A0A0) = dMsq(HmHp) - DBLE(se_WW(MW2, 4))
	  dMA = DBLE(se_HmHp(MHp2, 5)) - DBLE(se_WW(MW2, 5))
	endif

	c = ELGF/(2*MZ*SW*CW)

* and finally the Higgs mass counterterms:
	dMZ2 = DBLE(se_ZZ(MZ2, 5))
	dTh0 = -DBLE(td_h0(5))
	dTHH = -DBLE(td_HH(5))
	dTA0 = -DBLE(td_A0(5))

	dMsq(h0h0) = dMA*CBA2 + dMZ2*SAB**2 +
     &    c*SBA*(dTHH*CBA*SBA - dTh0*(1 + CBA2)) -
     &    dTB(5)*S2B*(MA02*SBA*CBA - MZ2*SAB*CAB)

	dMsq(HHHH) = dMA*SBA2 + dMZ2*CAB**2 -
     &    c*CBA*(dTHH*(1 + SBA2) - dTh0*SBA*CBA) +
     &    dTB(5)*S2B*(MA02*SBA*CBA - MZ2*SAB*CAB)

	dMsq(h0HH) = -dMA*SBA*CBA - dMZ2*SAB*CAB -
     &    c*(dTHH*SBA**3 + dTh0*CBA**3) -
     &    dTB(5)*SB*CB*(MA02*(CBA - SBA)*(CBA + SBA) +
     &      MZ2*(CAB - SAB)*(CAB + SAB))

	dTh0 = -DBLE(td_h0(4))
	dTHH = -DBLE(td_HH(4))
	dTA0 = -DBLE(td_A0(4))

	dMsq(h0A0) = -c*dTA0*SBA
	dMsq(HHG0) = dMsq(h0A0)
	dMsq(h0G0) = c*dTA0*CBA
	dMsq(HHA0) = -dMsq(h0G0)

	dMsq(G0G0) = -c*(dTHH*CBA + dTh0*SBA)
	dMsq(GmGp) = dMsq(G0G0)
	dMsq(A0G0) = c*(dTHH*SBA - dTh0*CBA) - dTB(4)*SB*CB*MA02
	dMsq(HmGp) = c*(dTHH*SBA - dTh0*CBA - cI*dTA0) -
     &    dTB(4)*SB*CB*MHp2

	if( debuglevel .lt. 2 ) return

	DSELF "========================"	ENDL
	DSELF "counter terms by CalcCTs"	ENDL
	DSELF "------------------------"	ENDL
	DSELF "dMsq(h0h0) =", dMsq(h0h0)	ENDL
	DSELF "dZ(h0h0)   =", dZ(h0h0)		ENDL
	DSELF "dMsq(HHHH) =", dMsq(HHHH)	ENDL
	DSELF "dZ(HHHH)   =", dZ(HHHH)		ENDL
	DSELF "dMsq(A0A0) =", dMsq(A0A0)	ENDL
	DSELF "dZ(A0A0)   =", dZ(A0A0)		ENDL
	DSELF "dMsq(HmHp) =", dMsq(HmHp)	ENDL
	DSELF "dZ(HmHp)   =", dZ(HmHp)		ENDL
	DSELF "dMsq(h0HH) =", dMsq(h0HH)	ENDL
	DSELF "dZ(h0HH)   =", dZ(h0HH)		ENDL
	DSELF "dMsq(h0A0) =", dMsq(h0A0)	ENDL
	DSELF "dZ(h0A0)   =", dZ(h0A0)		ENDL
	DSELF "dMsq(HHA0) =", dMsq(HHA0)	ENDL
	DSELF "dZ(HHA0)   =", dZ(HHA0)		ENDL
	DSELF "dMsq(G0G0) =", dMsq(G0G0)	ENDL
	DSELF "dZ(G0G0)   =", dZ(G0G0)		ENDL
	DSELF "dMsq(h0G0) =", dMsq(h0G0)	ENDL
	DSELF "dZ(h0G0)   =", dZ(h0G0)		ENDL
	DSELF "dMsq(HHG0) =", dMsq(HHG0)	ENDL
	DSELF "dZ(HHG0)   =", dZ(HHG0)		ENDL
	DSELF "dMsq(A0G0) =", dMsq(A0G0)	ENDL
	DSELF "dZ(A0G0)   =", dZ(A0G0)		ENDL
	DSELF "dMsq(GmGp) =", dMsq(GmGp)	ENDL
	DSELF "dZ(GmGp)   =", dZ(GmGp)		ENDL
	DSELF "dMsq(HmGp) =", dMsq(HmGp)	ENDL
	DSELF "dZ(HmGp)   =", dZ(HmGp)		ENDL
	end

