* Couplings.F
* evaluation of Higgs couplings
* this file is part of FeynHiggs
* last modified 30 Jul 10 th

#include "externals.h"
#include "debug.h"
#include "FHCouplings.h"


	subroutine FHCouplings(error,
     &    couplings_, couplingsms_, gammas_, gammasms_, fast)
	implicit none
	integer error, fast
	double complex couplings_(ncouplings)
	double complex couplingsms_(ncouplingsms)
	double precision gammas_(ngammas)
	double precision gammasms_(ngammasms)

#include "FH.h"
#define __SUBROUTINE__ "FHCouplings"

	integer i
	call CheckCouplings(error, fast)

	if( error .gt. 0 ) return

	do i = 1, ncouplings
	  couplings_(i) = couplings(i)
	enddo
	do i = 1, ncouplingsms
	  couplingsms_(i) = couplingsms(i)
	enddo
	do i = 1, ngammas
	  gammas_(i) = gammas(i)
	enddo
	do i = 1, ngammasms
	  gammasms_(i) = gammasms(i)
	enddo
	end


************************************************************************

	subroutine CheckCouplings(error, fast)
	implicit none
	integer error, fast

#include "Decay.h"

	integer i, f, h, h1, h2, hv, t, ty, g, g1, g2
	integer s1, s2, c1, c2, n1, n2, uz
	integer Nc, fail
	double precision fac, sqme(2), kEWNLO
	double precision resummedDeltab, kw, kt, kx
	double precision r, rr, r2, logrr, logr2, mhi, mlo, m, gam
	double complex DD(2,3,3), Hff(3,2:4,3), dBhttL, dBhttR
	double complex cL(2), cR(2), chffL(2,0:1), chffR(2,0:1)
	double complex chhv(2,3), chhh(2,3,4,4)
	double complex ch0ss(2,3,2,2), ch0ssy(2,3,2,2)
	double complex chpss(2,2), chpssy(2,2)
	double complex u1, u2, u3, u4, c, cx
	double precision ga(4), gasm(4), gatot_top

	double precision Kallen, KallenLR, HXVWidth, Li2
	external Kallen, KallenLR, HXVWidth, Li2
	external hee_SquaredME, heeSM_SquaredME
	external huu_SquaredME, huuSM_SquaredME
	external hdd_SquaredME, hddSM_SquaredME

	character*2 higgs(4)
	data higgs /"h0", "HH", "A0", "Hp"/

	double precision lambda, ratio
	common /KallenVars/ lambda, ratio

	error = 0
	if( coup_valid .eq. ieor(valid, fast) ) return

	call CheckHiggsCorr(error)
	if( error .gt. 0 ) return

	f = -ibits(ior(fast, -fast), 31, 1)

	do i = 1, ncouplings
	  couplings(i) = 0
	enddo
	do i = 1, ncouplingsms
	  couplingsms(i) = 0
	enddo
	do i = 1, ngammas
	  gammas(i) = 0
	enddo
	do i = 1, ngammasms
	  gammasms(i) = 0
	enddo
	do i = 1, H0FF(3,4,3,3)
	  ratios(i) = 0
	enddo

* We first compute DD(1,2) = (U,Z)Higgs.Y, where
* Y = {{CA, -SA, 0}, {SA, CA, 0}, {0, 0, 1}}
* Note that SA & CA are the sine and cosine of alpha_*tree*.

* for checking with the MSSM.mod couplings
#if 0
#define YHiggs(i,j,uz) Delta(i,j)
#else
#define YHiggs(i,j,uz) XHiggs(i,j,uz)
#endif

	do uz = 1, 2
	  DD(uz,1,1) = CA*YHiggs(1,1,uz) + SA*YHiggs(1,2,uz)
	  DD(uz,2,1) = CA*YHiggs(2,1,uz) + SA*YHiggs(2,2,uz)
	  DD(uz,3,1) = CA*YHiggs(3,1,uz) + SA*YHiggs(3,2,uz)

	  DD(uz,1,2) = CA*YHiggs(1,2,uz) - SA*YHiggs(1,1,uz)
	  DD(uz,2,2) = CA*YHiggs(2,2,uz) - SA*YHiggs(2,1,uz)
	  DD(uz,3,2) = CA*YHiggs(3,2,uz) - SA*YHiggs(3,1,uz)

	  DD(uz,1,3) = YHiggs(1,3,uz)
	  DD(uz,2,3) = YHiggs(2,3,uz)
	  DD(uz,3,3) = YHiggs(3,3,uz)
	enddo


*** COUPLINGS AND DECAY WIDTHS ***

* For couplings where the order is essential, the following
* conventions apply:
* - A charged Higgs at pos. 1 is always the particle, i.e. H-,
*   this fixes all other particles by charge conservation.
* - For the H0ChaCha coupling, the first Chargino is the
*   particle, chi-, and the second one the antiparticle, chi+.

	resummedDeltab = 1/abs(1 + Deltaf(4,3))

	do g = 1, 3
	  do t = 2, 4
	    Hff(1,t,g) = 1 - CA/(SA*TB)*Deltaf(t,g)
	    Hff(2,t,g) = 1 + SA/(CA*TB)*Deltaf(t,g)
	    Hff(3,t,g) = 1
	  enddo
	enddo

	do i = 1, 2*NSf
	  Sf(i,6) = Sf(i,3)
	enddo

	Divergence = getdelta()
	call clearcache

	do i = 1, 3*3
	  hggU_flat(i) = 1
	  hggDRe_flat(i) = 1
	  hggDIm_flat(i) = 1
	enddo

* Higgs -> Vector Vector
	do h = 1, 3 
	  m = MHiggs(h)
	  call SetDecayScale(fail, m)

* leading correction factors in O(alpha_s)
	  hgagaQ = 1 - AlfasMH/pi
	  hgagaSq = 1 + 8/(3*pi)*AlfasMH

	  hggU(1,3) = hggSq(1)
	  hggU(2,3) = hggSq(2)
	  hggU(3,3) = hggSq(3)

	  hggDRe(1,3) = hggSq(1)
	  hggDRe(2,3) = hggSq(2)
	  hggDRe(3,3) = hggSq(3)

	  hggDIm(1,3) = hggSq(1)
	  hggDIm(2,3) = hggSq(2)
	  hggDIm(3,3) = hggSq(3)

* Higgs -> gamma gamma
	  call Kin1to2(fac, h, 0D0, 0D0, .5D0)

	  call hgagaSM_SquaredME(sqme, O'155', -1)
	  GammaSM(H0VV(h,1)) = fac*sqme(2)
	  GammaSMTot(h) = GammaSMTot(h) + GammaSM(H0VV(h,1))

	  call hgaga_SquaredME(sqme, O'155', -1)
	  Gamma(H0VV(h,1)) = fac*sqme(2)
	  GammaTot(h) = GammaTot(h) + Gamma(H0VV(h,1))

* Higgs -> gamma Z

#if 0
	  call Kin1to2(fac, h, 0D0, MZ, .5D0)

	  call hgaZSM_SquaredME(sqme, O'157', -1)
	  GammaSM(H0VV(h,2)) = fac*sqme(2)
	  GammaSMTot(h) = GammaSMTot(h) + GammaSM(H0VV(h,2))

	  call hgaZ_SquaredME(sqme, O'155', -1)
	  Gamma(H0VV(h,2)) = fac*sqme(2)
	  GammaTot(h) = GammaTot(h) + Gamma(H0VV(h,2))
#endif

* Higgs -> Z Z
	  
	  do uz = 1, 2
	    cL(uz) = SB*DD(uz,h,1) + CB*DD(uz,h,2)
	  enddo

	  CouplingSM(H0VV(h,3)) = cI*EL0*MW/(SW*CW2)
	  Coupling(H0VV(h,3)) = CouplingSM(H0VV(h,3))*cL(uzint)

	  GammaSM(H0VV(h,3)) =
     &      .5D0*HXVWidth(m, MZ, MZ, GammaZ, .8D0, 1D0, 1)
	  GammaSMTot(h) = GammaSMTot(h) + GammaSM(H0VV(h,3))

	  ratios(H0VV(h,3)) = Sq(cL(uzext))
	  Gamma(H0VV(h,3)) = GammaSM(H0VV(h,3))*ratios(H0VV(h,3))
	  GammaTot(h) = GammaTot(h) + Gamma(H0VV(h,3))

* Higgs -> W W

	  CouplingSM(H0VV(h,4)) = cI*EL0*MW/SW
	  Coupling(H0VV(h,4)) = CouplingSM(H0VV(h,4))*cL(uzint)

	  GammaSM(H0VV(h,4)) =
     &      HXVWidth(m, MW, MW, GammaW, 1D0, 1D0, 1)
	  GammaSMTot(h) = GammaSMTot(h) + GammaSM(H0VV(h,4))

	  ratios(H0VV(h,4)) = Sq(cL(uzext))
	  Gamma(H0VV(h,4)) = GammaSM(H0VV(h,4))*ratios(H0VV(h,4))
	  GammaTot(h) = GammaTot(h) + Gamma(H0VV(h,4))

* Higgs -> gluon gluon

	  call Kin1to2(fac, h, 0D0, 0D0, .5D0)

*#include "kfactor-gghEW.F"

	  call hggSM_SquaredME(sqme, O'155', -1)
	  GammaSM(H0VV(h,5)) = fac*sqme(2)
	  GammaSMTot(h) = GammaSMTot(h) + GammaSM(H0VV(h,5))

	  call hgg_SquaredME(sqme, O'155', -1)
	  Gamma(H0VV(h,5)) = fac*sqme(2)
	  GammaTot(h) = GammaTot(h) + Gamma(H0VV(h,5))

* Higgs -> Fermion Fermion

	  do uz = 1, 2
	    chffL(uz,0) = DD(uz,h,2)/CB + cI*DD(uz,h,3)*TB
	    chffR(uz,0) = DD(uz,h,2)/CB - cI*DD(uz,h,3)*TB
	    chffL(uz,1) = DD(uz,h,1)/SB + cI*DD(uz,h,3)/TB
	    chffR(uz,1) = DD(uz,h,1)/SB - cI*DD(uz,h,3)/TB
	  enddo

	  do t = 2, 4
	    Nc = ior(t - 1, 1)
	    ty = t + 3*ibits(t + 1, 2, 1)
	    do uz = 1, 2
	      cL(uz) = chffL(uz,iand(t, 1))
	      cR(uz) = chffR(uz,iand(t, 1))
	    enddo

	    do g = 1, 3
	      if( t*8 + g .eq. O'43' ) then
	        do uz = 1, 2
* amplitude subtracted at 1L
	          dBhttL = DBLE(Deltaf(4,3))*chffR(uz,1) +
     &                     DIMAG(Deltaf(4,3))*chffL(uz,1)
	          dBhttR = DBLE(Deltaf(4,3))*chffL(uz,1) -
     &                     DIMAG(Deltaf(4,3))*chffR(uz,1)
	          c = Hff(h,4,3)*(resummedDeltab - 1) + 1 +
     &              DBLE(Deltaf(4,3))
	          cx = Hff(h,4,3)*resummedDeltab - 1
	          cL(uz) = c*cL(uz) + cx*dBhttL
	          cR(uz) = c*cR(uz) + cx*dBhttR
	        enddo
	      endif
c	      write(*,*) 'Point 2.3'
	      r = -.5D0/(SW*MW)*Mfy(ty,g)
	      c = EL0*r*cI
	      LCouplingSM(H0FF(h,t,g,g)) = c
	      RCouplingSM(H0FF(h,t,g,g)) = c
	      LCoupling(H0FF(h,t,g,g)) = c*cL(uzint)
	      RCoupling(H0FF(h,t,g,g)) = c*cR(uzint)

	      r = .5D0*Nc*AlfaMZ*m*r**2
	      Gamma(H0FF(h,t,g,g)) = r*KallenLR(m,
     &          Mf(t,g), Mf(t,g), cL(uzext), cR(uzext))
	      GammaSM(H0FF(h,t,g,g)) = r*lambda

	      ratios(H0FF(h,t,g,g)) = ratio
	    enddo
	  enddo

	  call hee_CalcRenConst
	  call heeSM_CalcRenConst
	  call huu_CalcRenConst
	  call huuSM_CalcRenConst
	  call hdd_CalcRenConst
	  call hddSM_CalcRenConst

	  do g2 = 1, 3
	    do g1 = 1 + iand(f, g2 - 1), 3 + iand(f, g2 - 3)
	      call HffDecays(h, 2, g1, g2,
     &          hee_SquaredME, heeSM_SquaredME)
	      call HffDecays(h, 3, g1, g2,
     &          huu_SquaredME, huuSM_SquaredME)
	      call HffDecays(h, 4, g1, g2,
     &          hdd_SquaredME, hddSM_SquaredME)
	    enddo
	  enddo
	enddo

	m = MHiggs(4)
	call SetDecayScale(fail, m)

	c = cI/sqrt2*EL0/(SW*MW)

	mlo = MT - .5D0
	mhi = MT + MB + .8D0

	do t = 2, 4, 2
	  Nc = ior(t - 1, 1)
	  ty = t + 3*ibits(t + 1, 2, 1)
c	  if( t .eq. 3 ) c = c*(1 + AlfasMH/pi*(CF*9/4D0 + 8/3D0))
	  do g = 1, 3
	    cL(1) = Mfy(ty,g)*TB*c
	    cR(1) = Mfy(ty-1,g)/TB*c

	    if( t*8 + g .eq. O'43' ) then
	      m = max(m, mhi)
	      cL(1) = cL(1)*resummedDeltab
	    endif

	    LCoupling(HpFF(t/2,g,g)) = cL(1)
	    RCoupling(HpFF(t/2,g,g)) = cR(1)

	    Gamma(HpFF(t/2,g,g)) = Nc/(8*pi)*m*
     &        KallenLR(m, Mf(t-1,g), Mf(t,g), cL(1), cR(1))
	    GammaTot(4) = GammaTot(4) + Gamma(HpFF(t/2,g,g))
	  enddo
	enddo

	if( m .lt. mhi ) then
	  GammaTot(4) = GammaTot(4) - Gamma(HpFF(2,3,3))
	  gam = 0
	  if( m .gt. MW + 2*MB ) then
	    m = min(m, mlo)
* off-shell decay to top* bottom -> W bb
* taken from hep-ph/9511342, Eq. (65)
	    kw = MW2/m**2
	    kt = MT2/m**2
	    kx = (kt - 1)/(kt - kw)
	    gam = 3/(64*pi**3)*(GF*MT2/TB)**2*m*
     &        ( -2.5D0 + (4 - 1.5D0*kw)*kw +
     &          (1 - kw)/kt**2*(2*kw**2*(2 - kt) + kt*(3*kt**2 - kw)) +
     &          (1 - 4*kt + 3*(kt - kw)*(kt + kw))*log(kx) +
     &          kw**2/kt**3*(3*kt + 4*kw*(kt - 1))*log(kx*kw) )*
     &        (1 - MB/MT)**3
*             ^^^^^^^^^^^^^^ fudge factor for non-zero bottom mass
* interpolate in the region MT - .5 GeV < MHp < MT + MB + .8 GeV
	    if( MHiggs(4) .gt. mlo ) gam = gam +
     &        (MHiggs(4) - mlo)/(mhi - mlo)*
     &        (Gamma(HpFF(2,3,3)) - gam)
	  endif
	  Gamma(HpFF(2,3,3)) = gam
	  GammaTot(4) = GammaTot(4) + Gamma(HpFF(2,3,3))
	endif

* Higgs -> Higgs Vector

	do uz = 1, 2
	  do hv = 1, 3
	    chhv(uz,hv) = CB*DD(uz,hv,1) - SB*DD(uz,hv,2)
	  enddo
	enddo

* Higgs -> Higgs Z

	do hv = 1, 3
	  do h = 1, 3
	    do uz = 1, 2
	      cL(uz) = DD(uz,h,3)*chhv(uz,hv) - DD(uz,hv,3)*chhv(uz,h)
	    enddo

	    Coupling(H0HV(h,hv)) = -EL0/(2*SW*CW)*cL(uzint)

	    Gamma(H0HV(h,hv)) =
     &        HXVWidth(MHiggs(h), MHiggs(hv), MZ, GammaZ,
     &          1.8D0, 2.4D0, 0)*
     &        Sq(cL(uzext))
	    GammaTot(h) = GammaTot(h) + Gamma(H0HV(h,hv))
	  enddo
	enddo

* Higgs -> Higgs W

	do hv = 1, 3
	  do uz = 1, 2
	    cL(uz) = chhv(uz,hv) + cI*DD(uz,hv,3)
	  enddo

	  Coupling(HpHV(hv)) = -cI*EL0/(2*SW)*cL(uzint)

	  Gamma(HpHV(hv)) =
     &      HXVWidth(MHiggs(4), MHiggs(hv), MW, GammaW,
     &        1.7D0, 2.2D0, 0)*
     &      Sq(cL(uzext))
	  GammaTot(4) = GammaTot(4) + Gamma(HpHV(hv))
	enddo

* Higgs -> Higgs Higgs

#if 0
	XHiggs(1,1,uzext) = 1
	XHiggs(2,2,uzext) = 1
	XHiggs(3,3,uzext) = 1
	XHiggs(1,2,uzext) = 0
	XHiggs(1,3,uzext) = 0
	XHiggs(2,1,uzext) = 0
	XHiggs(2,3,uzext) = 0
	XHiggs(3,1,uzext) = 0
	XHiggs(3,2,uzext) = 0
#endif
	call H0HHCouplings(chhh, DD)

	do h2 = 1, 4
	  do h1 = 1, 4
	    do h = 1, 3
	      Coupling(H0HH(h,h1,h2)) = chhh(uzint,h,h1,h2)
	      Gamma(H0HH(h,h1,h2)) = 1/(32*pi*MHiggs(h))*
     &          Kallen(MHiggs(h), MHiggs(h1), MHiggs(h2))*
     &          Sq(chhh(uzext,h,h1,h2))
#if 0
	if( h1 .lt. 4 .and. h2 .lt. 4 ) then
	  call Kin1to2(fac, h, MHiggs(h1), MHiggs(h2),
     &      .5D0*(2 - Delta(h1, h2)))
	if( abs(fac) .gt. 1D-12 ) then
	  hno2 = h1
	  hno3 = h2
	Divergence = 100D0
	call setdelta(Divergence)
	  call hhh_SquaredME(sqme, O'111', -1)
c	  GammaSM(H0HH(h,h1,h2)) = fac*sqme(2)
c	  GammaSMTot(h) = GammaSMTot(h) + GammaSM(H0HH(h,h1,h2))
	  PRINT *, "hhh:h,h1,h2=", h,h1,h2
	  PRINT *, "hhh:fac=", fac
	  PRINT *, "hhh:old,new=", Gamma(H0HH(h,h1,h2)), fac*(sqme(1)+sqme(2))
	  PRINT *, "hhh:sqme=", sqme(1), sqme(2)
	  PRINT *, "hhh:kallen=", Kallen(MHiggs(h), MHiggs(h1), MHiggs(h2))
	  PRINT *, "hhh:cpl=", (EL0*MZ/(2*CW*SW)*(C2A*CAB - 2*S2A*SAB))**2
	  PRINT *, "hhh:cpl=", Sq(Coupling(H0HH(h,h1,h2)))
	STOP
	endif
	endif
#endif
	      GammaTot(h) = GammaTot(h) + Gamma(H0HH(h,h1,h2))
	    enddo
	  enddo
	enddo

* Higgs -> Sfermion Sfermion

	do t = 2, 4
	  Nc = ior(t - 1, 1)
	  do g = 1, 3
	    call H0SfSfCouplings(ch0ss, t, g, Mf(t,g), DD)
	    call H0SfSfCouplings(ch0ssy, t, g, Mfy(t,g), DD)
	    do s2 = 1, 2
	      do s1 = 1, 2
	        do h = 1, 3
	          Coupling(H0SfSf(h,s1,s2,t,g)) = ch0ss(uzint,h,s1,s2)
	          Gamma(H0SfSf(h,s1,s2,t,g)) =
     &              Nc/(16*pi*MHiggs(h))*Hff(h,t,g)**2*
     &              Kallen(MHiggs(h), MSf(s1,t,g), MSf(s2,t,g))*
     &              Sq(ch0ssy(uzext,h,s1,s2))
	          GammaTot(h) = GammaTot(h) + Gamma(H0SfSf(h,s1,s2,t,g))
	        enddo
	      enddo
	    enddo
	  enddo
	enddo

	do t = 2, 4, 2
	  Nc = ior(t - 1, 1)
	  do g = 1, 3
	    call HpSfSfCouplings(chpss, t, g, Mf(t,g))
	    call HpSfSfCouplings(chpssy, t, g, Mfy(t,g))
	    do s2 = 1, 2
	      do s1 = 1, 2
	        Coupling(HpSfSf(s1,s2,t/2,g)) = chpss(s1,s2)
	        Gamma(HpSfSf(s1,s2,t/2,g)) = Nc/(16*pi*MHiggs(4))*
     &            Kallen(MHiggs(4), MSf(s1,t-1,g), MSf(s2,t,g))*
     &            Sq(chpssy(s1,s2))
	        GammaTot(4) = GammaTot(4) + Gamma(HpSfSf(s1,s2,t/2,g))
	      enddo
	    enddo
	  enddo
	enddo

* Higgs -> Chargino Chargino

	do c2 = 1, 2
          do c1 = 1, 2
	    u1 = UChaC(c1,2)*VChaC(c2,1)
	    u2 = UChaC(c1,1)*VChaC(c2,2)
	    u3 = VCha(c1,1)*UCha(c2,2)
	    u4 = VCha(c1,2)*UCha(c2,1)
	    do h = 1, 3
	      do uz = 1, 2
	        cL(uz) = 1/sqrt2*EL0/SW*(
     &            DD(uz,h,3)*(-u1*SB - u2*CB) +
     &            DD(uz,h,2)*(-cI*u1) +
     &            DD(uz,h,1)*(-cI*u2) )
	        cR(uz) = 1/sqrt2*EL0/SW*(
     &            DD(uz,h,3)*(u3*SB + u4*CB) +
     &            DD(uz,h,2)*(-cI*u3) +
     &            DD(uz,h,1)*(-cI*u4) )
	      enddo

	      LCoupling(H0ChaCha(h,c1,c2)) = cL(uzint)
	      RCoupling(H0ChaCha(h,c1,c2)) = cR(uzint)

	      Gamma(H0ChaCha(h,c1,c2)) = MHiggs(h)/(8*pi)*
     &          KallenLR(MHiggs(h), MCha(c1), MCha(c2),
     &            cL(uzext), cR(uzext))
	      GammaTot(h) = GammaTot(h) + Gamma(H0ChaCha(h,c1,c2))
	    enddo
	  enddo
	enddo

* Higgs -> Neutralino Neutralino

	do n2 = 1, 4
          do n1 = 1, 4
	    u1 = ZNeu(n1,3)*(CW*ZNeu(n2,2) - SW*ZNeu(n2,1)) +
     &           ZNeu(n2,3)*(CW*ZNeu(n1,2) - SW*ZNeu(n1,1))
	    u2 = ZNeu(n1,4)*(CW*ZNeu(n2,2) - SW*ZNeu(n2,1)) +
     &           ZNeu(n2,4)*(CW*ZNeu(n1,2) - SW*ZNeu(n1,1))
	    do h = 1, 3
	      do uz = 1, 2
	        cL(uz) = .5D0*EL0/(CW*SW)*(
     &            DD(uz,h,3)*(-DCONJG(u1)*SB + DCONJG(u2)*CB) +
     &            DD(uz,h,2)*(-cI*DCONJG(u1)) +
     &            DD(uz,h,1)*( cI*DCONJG(u2)) )
	        cR(uz) = .5D0*EL0/(CW*SW)*(
     &            DD(uz,h,3)*( u1*SB - u2*CB) +
     &            DD(uz,h,2)*(-cI*u1) +
     &            DD(uz,h,1)*( cI*u2) )
	      enddo

	      LCoupling(H0NeuNeu(h,n1,n2)) = cL(uzint)
	      RCoupling(H0NeuNeu(h,n1,n2)) = cR(uzint)

	      Gamma(H0NeuNeu(h,n1,n2)) = MHiggs(h)/(16*pi)*
     &          KallenLR(MHiggs(h), MNeu(n1), MNeu(n2),
     &            cL(uzext), cR(uzext))
	      GammaTot(h) = GammaTot(h) + Gamma(H0NeuNeu(h,n1,n2))
	      if( n1 .ne. n2 )
     &          Gamma(H0NeuNeu(h,n1,n2)) = 2*Gamma(H0NeuNeu(h,n1,n2))
	    enddo
	  enddo
	enddo

* Higgs -> Neutralino Chargino

	do c2 = 1, 2
	  do n1 = 1, 4
	    cL(1) = -cI*EL0/SW*CB*(ZNeuC(n1,4)*VChaC(c2,1) +
     &        1/sqrt2*(ZNeuC(n1,2) + ZNeuC(n1,1)*SW/CW)*VChaC(c2,2))
	    cR(1) = -cI*EL0/SW*SB*(ZNeu(n1,3)*UCha(c2,1) -
     &        1/sqrt2*(ZNeu(n1,2) + ZNeu(n1,1)*SW/CW)*UCha(c2,2))

	    LCoupling(HpNeuCha(n1,c2)) = cL(1)
	    RCoupling(HpNeuCha(n1,c2)) = cR(1)

	    Gamma(HpNeuCha(n1,c2)) = MHiggs(4)/(8*pi)*
     &        KallenLR(MHiggs(4), MNeu(n1), MCha(c2),
     &          cL(1), cR(1))
	    GammaTot(4) = GammaTot(4) + Gamma(HpNeuCha(n1,c2))
	  enddo
	enddo

* top -> W bottom

	r = MW/MT
	rr = (1 - r)*(1 + r)
	r2 = 1 + 2*r**2
	logrr = log(rr)
	logr2 = log(r**2)

* hep-ph/0408158, Eqs 6 + 26:
	Gamma(tBF(1)) = GF*MT*MT2/(8*pi*sqrt2)*rr**2*r2*(
     &    1 - CF/(2*pi)*AlfasMT*(
     &      2/3D0*pi**2 + 1/2D0 + 2*(logr2 + 1)*(logrr - 1) -
     &      4/(3*rr) + (22 - 34*r**2)/(9*rr**2)*logr2 +
     &      (3 + 27*logrr - 4*logr2)/(9*r2) + 4*Li2(r**2) ) )

* top -> H+ bottom

	if( MT .gt. MHiggs(4) + MB ) then
	  r = 1 - MHiggs2(4)/MT2

* these is the H+ -> t b coupling with MB(MT), MT(MT)
	  c = cI/sqrt2*EL0/(SW*MW)
	  cL(1) = Mfy(4,3)*resummedDeltab*TB*c
	  cR(1) = Mfy(3,3)/TB*c

* Corresponds to hep-ph/0211098, Eq 7a, but with Delta_b corrections
* included (via the couplings).
* Here only a factor of 1/MT is used due to different definition of
* the Kallen function.  Additional factor 1/2 from mismatch of 'c' in 
* Couplings.F and \tilde c.
* Finite O(alpha_s) corrections from hep-ph/9912516, Eq 29.
	  Gamma(tBF(2)) = 1/(32*pi*MT) *
     &      Kallen(MT, MHiggs(4), MB) * (
     &        ((MT - MHiggs(4))*(MT + MHiggs(4)) + MB2)*
     &          (Sq(cL(1)) + Sq(cR(1))) +
     &        4*MT*MB*abs(cL(1)*cR(1)) ) *
     &      ( 1 + AlfasMT/pi*(7 - 8/9D0*pi**2 +
     &          (2/3D0*r**2 - 2)*log(r) + r*(2 + 4/9D0*r)) )
	endif

*** BRANCHING RATIOS ***

	do h = 1, 4
	  if( GammaTot(h) .gt. 0 ) then
	    ga(h) = 1/GammaTot(h)
	  else
	    ga(h) = 0
	  endif
	  if( GammaSMTot(h) .gt. 0 ) then
	    gasm(h) = 1/GammaSMTot(h)
	  else
	    gasm(h) = 0
	  endif
	enddo

	h = 0

*#define Next(h) mod(h, 3) + 1
#define Next(h) (iand(h + 1, 3) + ishft(h + 1, -2))

	do i = H0VV(1,1), H0VV(3,5)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	  BRSM(i) = GammaSM(i)*gasm(h)
	enddo

	do i = H0FF(1,2,1,1), H0FF(3,4,3,3)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	  BRSM(i) = GammaSM(i)*gasm(h)
	enddo

	do i = H0ChaCha(1,1,1), H0ChaCha(3,2,2)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	enddo

	do i = H0NeuNeu(1,1,1), H0NeuNeu(3,4,4)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	enddo

	do i = H0HV(1,1), H0HV(3,3)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	enddo

	do i = H0HH(1,1,1), H0HH(3,4,4)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	enddo

	do i = H0SfSf(1,1,1,2,1), H0SfSf(3,2,2,4,3)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	enddo

	do i = HpFF(1,1,1), HpFF(2,3,3)
	  BR(i) = Gamma(i)*ga(4)
	enddo

	do i = HpNeuCha(1,1), HpNeuCha(4,2)
	  BR(i) = Gamma(i)*ga(4)
	enddo

	do i = HpHV(1), HpHV(3)
	  BR(i) = Gamma(i)*ga(4)
	enddo

	do i = HpSfSf(1,1,1,1), HpSfSf(2,2,2,3)
	  BR(i) = Gamma(i)*ga(4)
	enddo

	gatot_top = Gamma(tBF(1)) + Gamma(tBF(2))
	if( gatot_top .ne. 0 ) then
	  BR(tBF(1)) = Gamma(tBF(1))/gatot_top
	  BR(tBF(2)) = Gamma(tBF(2))/gatot_top
	endif

	coup_valid = ieor(valid, fast)

	if( debuglevel .lt. 1 ) return

#define Header(s) s, " decays    Width                    BR"
#define HiggsHeader(h) Header(higgs(h))

	DCOUP "============================" ENDL
	do h = 1, 3
	  DHEAD higgs(h), " total width =", GammaTot(h) ENDL
	  DHEAD HiggsHeader(h) ENDL
	  DCOUP "gamma-gamma", Gamma(H0VV(h,1)), BR(H0VV(h,1)) ENDL
	  DCOUP "gamma-Z    ", Gamma(H0VV(h,2)), BR(H0VV(h,2)) ENDL
	  DCOUP "Z-Z        ", Gamma(H0VV(h,3)), BR(H0VV(h,3)) ENDL
	  DCOUP "W-W        ", Gamma(H0VV(h,4)), BR(H0VV(h,4)) ENDL
	  DCOUP "gluon-gluon", Gamma(H0VV(h,5)), BR(H0VV(h,5)) ENDL
	  DCOUP " " ENDL
	  DCOUP "h0-Z       ", Gamma(H0HV(h,1)), BR(H0HV(h,1)) ENDL
	  DCOUP "HH-Z       ", Gamma(H0HV(h,2)), BR(H0HV(h,2)) ENDL
	  DCOUP "A0-Z       ", Gamma(H0HV(h,3)), BR(H0HV(h,3)) ENDL
	  DCOUP " " ENDL
	  DCOUP "h0-h0      ", Gamma(H0HH(h,1,1)), BR(H0HH(h,1,1)) ENDL
	  DCOUP "Hp-Hp      ", Gamma(H0HH(h,4,4)), BR(H0HH(h,4,4)) ENDL
	  DCOUP " " ENDL
	  DCOUP "f-f    e   ", Gamma(H0FF(h,2,1,1)),
     &                            BR(H0FF(h,2,1,1))	ENDL
	  DCOUP "      mu   ", Gamma(H0FF(h,2,2,2)),
     &                            BR(H0FF(h,2,2,2))	ENDL
	  DCOUP "     tau   ", Gamma(H0FF(h,2,3,3)),
     &                            BR(H0FF(h,2,3,3))	ENDL
	  DCOUP "       u   ", Gamma(H0FF(h,3,1,1)),
     &                            BR(H0FF(h,3,1,1))	ENDL
	  DCOUP "       c   ", Gamma(H0FF(h,3,2,2)),
     &                            BR(H0FF(h,3,2,2))	ENDL
	  DCOUP "       t   ", Gamma(H0FF(h,3,3,3)),
     &                            BR(H0FF(h,3,3,3))	ENDL
	  DCOUP "       d   ", Gamma(H0FF(h,4,1,1)),
     &                            BR(H0FF(h,4,1,1))	ENDL
	  DCOUP "       s   ", Gamma(H0FF(h,4,2,2)),
     &                            BR(H0FF(h,4,2,2))	ENDL
	  DCOUP "       b   ", Gamma(H0FF(h,4,3,3)),
     &                            BR(H0FF(h,4,3,3))	ENDL
	  DCOUP " " ENDL
	  DCOUP "Cha1-Cha1  ", Gamma(H0ChaCha(h,1,1)),
     &                            BR(H0ChaCha(h,1,1))	ENDL
	  DCOUP "Cha1-Cha2  ", Gamma(H0ChaCha(h,1,2)),
     &                            BR(H0ChaCha(h,1,2))	ENDL
	  DCOUP "Cha2-Cha1  ", Gamma(H0ChaCha(h,2,1)),
     &                            BR(H0ChaCha(h,2,1))	ENDL
	  DCOUP "Cha2-Cha2  ", Gamma(H0ChaCha(h,2,1)),
     &                            BR(H0ChaCha(h,2,2))	ENDL
	  DCOUP " " ENDL
	  DCOUP "Neu1-Neu1  ", Gamma(H0NeuNeu(h,1,1)),
     &                            BR(H0NeuNeu(h,1,1))	ENDL
	  DCOUP "Neu1-Neu2  ", Gamma(H0NeuNeu(h,1,2)),
     &                            BR(H0NeuNeu(h,1,2))	ENDL
	  DCOUP "Neu1-Neu3  ", Gamma(H0NeuNeu(h,1,3)),
     &                            BR(H0NeuNeu(h,1,3))	ENDL
	  DCOUP "Neu1-Neu4  ", Gamma(H0NeuNeu(h,1,4)),
     &                            BR(H0NeuNeu(h,1,4))	ENDL
	  DCOUP "Neu2-Neu2  ", Gamma(H0NeuNeu(h,2,2)),
     &                            BR(H0NeuNeu(h,2,2))	ENDL
	  DCOUP "Neu2-Neu3  ", Gamma(H0NeuNeu(h,2,3)),
     &                            BR(H0NeuNeu(h,2,3))	ENDL
	  DCOUP "Neu2-Neu4  ", Gamma(H0NeuNeu(h,2,4)),
     &                            BR(H0NeuNeu(h,2,4))	ENDL
	  DCOUP "Neu3-Neu3  ", Gamma(H0NeuNeu(h,3,3)),
     &                            BR(H0NeuNeu(h,3,3))	ENDL
	  DCOUP "Neu3-Neu4  ", Gamma(H0NeuNeu(h,3,4)),
     &                            BR(H0NeuNeu(h,3,4))	ENDL
	  DCOUP "Neu4-Neu4  ", Gamma(H0NeuNeu(h,4,4)),
     &                            BR(H0NeuNeu(h,4,4))	ENDL
	  DCOUP " " ENDL
	  DCOUP "sf-sf  e 11", Gamma(H0SfSf(h,1,1,2,1)),
     &                            BR(H0SfSf(h,1,1,2,1))	ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,2,1)),
     &                            BR(H0SfSf(h,1,2,2,1))	ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,2,1)),
     &                            BR(H0SfSf(h,2,2,2,1))	ENDL
	  DCOUP "      mu 11", Gamma(H0SfSf(h,1,1,2,2)),
     &                            BR(H0SfSf(h,1,1,2,2)) ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,2,2)),
     &                            BR(H0SfSf(h,1,2,2,2))	ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,2,2)),
     &                            BR(H0SfSf(h,2,2,2,2))	ENDL
	  DCOUP "     tau 11", Gamma(H0SfSf(h,1,1,2,3)),
     &                            BR(H0SfSf(h,1,1,2,3))	ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,2,3)),
     &                            BR(H0SfSf(h,1,2,2,3))	ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,2,3)),
     &                            BR(H0SfSf(h,2,2,2,3))	ENDL
	  DCOUP "       u 11", Gamma(H0SfSf(h,1,1,3,1)),
     &                            BR(H0SfSf(h,1,1,3,1))	ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,3,1)),
     &                            BR(H0SfSf(h,1,2,3,1))	ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,3,1)),
     &                            BR(H0SfSf(h,2,2,3,1))	ENDL
	  DCOUP "       c 11", Gamma(H0SfSf(h,1,1,3,2)),
     &                            BR(H0SfSf(h,1,1,3,2))	ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,3,2)),
     &                            BR(H0SfSf(h,1,2,3,2))	ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,3,2)),
     &                            BR(H0SfSf(h,2,2,3,2))	ENDL
	  DCOUP "       t 11", Gamma(H0SfSf(h,1,1,3,3)),
     &                            BR(H0SfSf(h,1,1,3,3))	ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,3,3)),
     &                            BR(H0SfSf(h,1,2,3,3))	ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,3,3)),
     &                            BR(H0SfSf(h,2,2,3,3))	ENDL
	  DCOUP "       d 11", Gamma(H0SfSf(h,1,1,4,1)),
     &                            BR(H0SfSf(h,1,1,4,1)) ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,4,1)),
     &                            BR(H0SfSf(h,1,2,4,1)) ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,4,1)),
     &                            BR(H0SfSf(h,2,2,4,1)) ENDL
	  DCOUP "       s 11", Gamma(H0SfSf(h,1,1,4,2)),
     &                            BR(H0SfSf(h,1,1,4,2)) ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,4,2)),
     &                            BR(H0SfSf(h,1,2,4,2)) ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,4,2)),
     &                            BR(H0SfSf(h,2,2,4,2)) ENDL
	  DCOUP "       b 11", Gamma(H0SfSf(h,1,1,4,3)),
     &                            BR(H0SfSf(h,1,1,4,3)) ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,4,3)),
     &                            BR(H0SfSf(h,1,2,4,3)) ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,4,3)),
     &                            BR(H0SfSf(h,2,2,4,3)) ENDL
	  DCOUP " " ENDL
	enddo

	DHEAD higgs(4), " total width =", GammaTot(4) ENDL
	DHEAD HiggsHeader(4) ENDL
	DCOUP "h0-W       ", Gamma(HpHV(1)), BR(HpHV(1)) ENDL
	DCOUP "HH-W       ", Gamma(HpHV(2)), BR(HpHV(2)) ENDL
	DCOUP "A0-W       ", Gamma(HpHV(3)), BR(HpHV(3)) ENDL
	DCOUP " " ENDL
	DCOUP "f-f'   e   ", Gamma(HpFF(1,1,1)), BR(HpFF(1,1,1)) ENDL
	DCOUP "      mu   ", Gamma(HpFF(1,2,2)), BR(HpFF(1,2,2)) ENDL
	DCOUP "     tau   ", Gamma(HpFF(1,3,3)), BR(HpFF(1,3,3)) ENDL
	DCOUP "     u-d   ", Gamma(HpFF(2,1,1)), BR(HpFF(2,1,1)) ENDL
	DCOUP "     c-s   ", Gamma(HpFF(2,2,2)), BR(HpFF(2,2,2)) ENDL
	DCOUP "     t-b   ", Gamma(HpFF(2,3,3)), BR(HpFF(2,3,3)) ENDL
	DCOUP " " ENDL
	DCOUP "Neu1-Cha1  ", Gamma(HpNeuCha(1,1)),
     &                          BR(HpNeuCha(1,1)) ENDL
	DCOUP "Neu1-Cha2  ", Gamma(HpNeuCha(1,2)),
     &                          BR(HpNeuCha(1,2)) ENDL
	DCOUP "Neu2-Cha1  ", Gamma(HpNeuCha(2,1)),
     &                          BR(HpNeuCha(2,1)) ENDL
	DCOUP "Neu2-Cha2  ", Gamma(HpNeuCha(2,2)),
     &                          BR(HpNeuCha(2,2)) ENDL
	DCOUP "Neu3-Cha1  ", Gamma(HpNeuCha(3,1)),
     &                          BR(HpNeuCha(3,1)) ENDL
	DCOUP "Neu3-Cha2  ", Gamma(HpNeuCha(3,2)),
     &                          BR(HpNeuCha(3,2)) ENDL
	DCOUP "Neu4-Cha1  ", Gamma(HpNeuCha(4,1)),
     &                          BR(HpNeuCha(4,1)) ENDL
	DCOUP "Neu4-Cha2  ", Gamma(HpNeuCha(4,2)),
     &                          BR(HpNeuCha(4,2)) ENDL
	DCOUP " " ENDL
	DCOUP "sf-sf' e 11", Gamma(HpSfSf(1,1,1,1)),
     &                          BR(HpSfSf(1,1,1,1)) ENDL
	DCOUP "         12", Gamma(HpSfSf(1,2,1,1)),
     &                          BR(HpSfSf(1,2,1,1)) ENDL
	DCOUP "         22", Gamma(HpSfSf(2,2,1,1)),
     &                          BR(HpSfSf(2,2,1,1)) ENDL
	DCOUP "      mu 11", Gamma(HpSfSf(1,1,1,2)),
     &                          BR(HpSfSf(1,1,1,2)) ENDL
	DCOUP "         12", Gamma(HpSfSf(1,2,1,2)),
     &                          BR(HpSfSf(1,2,1,2)) ENDL
	DCOUP "         22", Gamma(HpSfSf(2,2,1,2)),
     &                          BR(HpSfSf(2,2,1,2)) ENDL
	DCOUP "     tau 11", Gamma(HpSfSf(1,1,1,3)),
     &                          BR(HpSfSf(1,1,1,3)) ENDL
	DCOUP "         12", Gamma(HpSfSf(1,2,1,3)),
     &                          BR(HpSfSf(1,2,1,3)) ENDL
	DCOUP "         22", Gamma(HpSfSf(2,2,1,3)),
     &                          BR(HpSfSf(2,2,1,3)) ENDL
	DCOUP "     u-d 11", Gamma(HpSfSf(1,1,2,1)),
     &                          BR(HpSfSf(1,1,2,1)) ENDL
	DCOUP "         12", Gamma(HpSfSf(1,2,2,1)),
     &                          BR(HpSfSf(1,2,2,1)) ENDL
	DCOUP "         22", Gamma(HpSfSf(2,2,2,1)),
     &                          BR(HpSfSf(2,2,2,1)) ENDL
	DCOUP "     c-s 11", Gamma(HpSfSf(1,1,2,2)),
     &                          BR(HpSfSf(1,1,2,2)) ENDL
	DCOUP "         12", Gamma(HpSfSf(1,2,2,2)),
     &                          BR(HpSfSf(1,2,2,2)) ENDL
	DCOUP "         22", Gamma(HpSfSf(2,2,2,2)),
     &                          BR(HpSfSf(2,2,2,2)) ENDL
	DCOUP "     t-b 11", Gamma(HpSfSf(1,1,2,3)),
     &                          BR(HpSfSf(1,1,2,3)) ENDL
	DCOUP "         12", Gamma(HpSfSf(1,2,2,3)),
     &                          BR(HpSfSf(1,2,2,3)) ENDL
	DCOUP "         22", Gamma(HpSfSf(2,2,2,3)),
     &                          BR(HpSfSf(2,2,2,3)) ENDL
	DCOUP " " ENDL

	DHEAD "top total width =", gatot_top ENDL
	DHEAD Header("top") ENDL
	DCOUP "W-b        ", Gamma(tBF(1)), BR(tBF(1)) ENDL
	DCOUP "Hp-b       ", Gamma(tBF(2)), BR(tBF(2)) ENDL
	DCOUP "============================" ENDL
	end


************************************************************************

	subroutine H0HHCouplings(ghhh, DD)
	implicit none
	double complex ghhh(2,3,4,4), DD(2,3,3)

#include "FH.h"

	integer h, h1, h2, uz
	double complex omega_t
	double complex hc(6,6,6)
* 1 = Phi2, 2 = Phi1, 3 = A, 4 = H+-, 5 = G, 6 = G+-

	call CalcSelf(Mh02, cpeven, 0)
	omega_t = CA2*DBLE(seR(h0h0)) + SA2*DBLE(seR(HHHH)) +
     &    S2A*DBLE(seR(h0HH))

	hc(1,1,1) = -3*SB + 3/MZ2/SB*(omega_t +
     &    GF*sqrt2/(pi**2*SB2)*MT2**2)		! hep-ph/0108245, eq26
	hc(1,1,2) = CB 
	hc(1,2,2) = SB 
	hc(2,2,2) = -3*CB
	hc(1,3,3) = SB*(omega_t/(MZ2*TB2) - C2B)
	hc(2,3,3) = CB*C2B

	do h2 = 1, 3
	  do h1 = 1, 3
	    do h = 1, 3
	      do uz = 1, 2
	        ghhh(uz,h,h1,h2) = cI*EL0*MZ/(2*CW*SW)*(
     &            hc(1,1,1)* DD(uz,h,1)*DD(uz,h1,1)*DD(uz,h2,1) +
     &            hc(1,1,2)*(DD(uz,h,1)*DD(uz,h1,1)*DD(uz,h2,2) +
     &                       DD(uz,h,1)*DD(uz,h1,2)*DD(uz,h2,1) +
     &                       DD(uz,h,2)*DD(uz,h1,1)*DD(uz,h2,1)) +
     &            hc(1,2,2)*(DD(uz,h,1)*DD(uz,h1,2)*DD(uz,h2,2) +
     &                       DD(uz,h,2)*DD(uz,h1,1)*DD(uz,h2,2) +
     &                       DD(uz,h,2)*DD(uz,h1,2)*DD(uz,h2,1)) +
     &            hc(2,2,2)* DD(uz,h,2)*DD(uz,h1,2)*DD(uz,h2,2) +
     &            hc(1,3,3)*(DD(uz,h,1)*DD(uz,h1,3)*DD(uz,h2,3) +
     &                       DD(uz,h,3)*DD(uz,h1,1)*DD(uz,h2,3) +
     &                       DD(uz,h,3)*DD(uz,h1,3)*DD(uz,h2,1)) +
     &            hc(2,3,3)*(DD(uz,h,2)*DD(uz,h1,3)*DD(uz,h2,3) +
     &                       DD(uz,h,3)*DD(uz,h1,2)*DD(uz,h2,3) +
     &                       DD(uz,h,3)*DD(uz,h1,3)*DD(uz,h2,2)) )
	      enddo
	    enddo
	  enddo
	enddo
#if 0
	PRINT *, "uzext=", uzext
	PRINT *, "ghhh(1,2,1,1)=", ghhh(1,2,1,1)
	PRINT *, "ghhh(2,2,1,1)=", ghhh(2,2,1,1)
	PRINT *, "CA,SA=", CA,SA
	PRINT *, "DD(2,1,1)=", DD(2,1,1)
	PRINT *, "DD(2,1,2)=", DD(2,1,2)
	PRINT *, "DD(2,1,3)=", DD(2,1,3)
	PRINT *, "DD(2,2,1)=", DD(2,2,1)
	PRINT *, "DD(2,2,2)=", DD(2,2,2)
	PRINT *, "DD(2,2,3)=", DD(2,2,3)
	PRINT *, "DD(2,3,1)=", DD(2,3,1)
	PRINT *, "DD(2,3,2)=", DD(2,3,2)
	PRINT *, "DD(2,3,3)=", DD(2,3,3)
#endif

* hep-ph/9612362, eq16
	hc(1,4,4) = SB*(C2B - 2*CW2 - omega_t/(MZ2*TB2))
	hc(2,4,4) = CB*(C2B - 2*CW2)
	hc(3,4,4) = 0

	do h = 1, 3
	  do uz = 1, 2
	    ghhh(uz,h,4,1) = 0
	    ghhh(uz,h,4,2) = 0
	    ghhh(uz,h,4,3) = 0
	    ghhh(uz,h,1,4) = 0
	    ghhh(uz,h,2,4) = 0
	    ghhh(uz,h,3,4) = 0
	    ghhh(uz,h,4,4) = cI*EL0*MZ/(2*CW*SW)*(
     &        hc(1,4,4)*DD(uz,h,1) +
     &        hc(2,4,4)*DD(uz,h,2) +
     &        hc(3,4,4)*DD(uz,h,3) )
	  enddo
	enddo

c	hc(2,6,6) = -C2B*CB
c	hc(1,6,6) =  C2B*SB
c	hc(2,5,5) = -C2B*CB
c	hc(1,5,5) =  C2B*SB
c	hc(2,3,6) =  S2B*CB
c	hc(1,3,6) = -S2B*SB
c	hc(2,4,6) = -CW2*SB + S2B*CB
c	hc(1,4,6) =  CW2*CB - S2B*SB
c	hc(3,4,6) = CW2		! (-I) is missing for (AH+G-), (+I) for AH-G+
	end


************************************************************************

	subroutine H0SfSfCouplings(ghss, t, g, m, DD)
	implicit none
	double complex ghss(2,3,2,2), DD(2,3,3)
	double precision m
	integer t, g

#include "FH.h"

	integer h, uz
	double complex u1111, u1112, u1121, u1122, u1212
	double complex u1222, u2112, u2121, u2122, u2222
	double precision d1, d2, m11a, m22a, m11b, m22b, mx
	double complex m12a, m12b, m3, c
	double complex hc(3,3)

	u1111 = USf(1,1,t,g)*USfC(1,1,t,g)
	u1112 = USf(1,1,t,g)*USfC(1,2,t,g)
	u1121 = USf(1,1,t,g)*USfC(2,1,t,g)
	u1122 = USf(1,1,t,g)*USfC(2,2,t,g)
	u1212 = USf(1,2,t,g)*USfC(1,2,t,g)
	u1222 = USf(1,2,t,g)*USfC(2,2,t,g)
	u2112 = USf(2,1,t,g)*USfC(1,2,t,g)
	u2121 = USf(2,1,t,g)*USfC(2,1,t,g)
	u2122 = USf(2,1,t,g)*USfC(2,2,t,g)
	u2222 = USf(2,2,t,g)*USfC(2,2,t,g)

	d2 = Qf(t)*SW2*MZ2
	d1 = (iand(t, 1) - .5D0)*MZ2 - d2
	m11a = SB*d1
	m22a = SB*d2
	m11b = CB*d1
	m22b = CB*d2

	if( iand(t, 1) .eq. 1 ) then
	  mx = -m/SB
	  m11a = m11a + mx*m
	  m22a = m22a + mx*m
	  m12a = mx*DCONJG(Af(t,g))
	  m12b = mx*MUE
	  m3 = -m12a*CB - m12b*SB
	else
	  mx = m/CB
	  m11b = m11b + mx*m
	  m22b = m22b + mx*m
	  m12a = mx*MUE
	  m12b = mx*DCONJG(Af(t,g))
	  m3 = m12a*CB + m12b*SB
	endif

	hc(1,1) = m11a*u1111 + m22a*u1212 + DBLE(m12a*u1112)
	hc(2,1) = m11b*u1111 + m22b*u1212 + DBLE(m12b*u1112)
	hc(3,1) = DIMAG(m3*u1112)

	hc(1,2) = m11a*u2121 + m22a*u2222 + DBLE(m12a*u2122)
	hc(2,2) = m11b*u2121 + m22b*u2222 + DBLE(m12b*u2122)
	hc(3,2) = DIMAG(m3*u2122)

	hc(1,3) = m11a*u1121 + m22a*u1222 +
     &    .5D0*(m12a*u1122 + DCONJG(m12a*u2112))
	hc(2,3) = m11b*u1121 + m22b*u1222 +
     &    .5D0*(m12b*u1122 + DCONJG(m12b*u2112))
	hc(3,3) = -.5D0*cI*(m3*u1122 - DCONJG(m3*u2112))

	c = cI*EL0/(MW*SW)
	do h = 1, 3
	  do uz = 1, 2
	    ghss(uz,h,1,1) = c*(DD(uz,h,1)*hc(1,1) -
     &                          DD(uz,h,2)*hc(2,1) -
     &                          DD(uz,h,3)*hc(3,1))
	    ghss(uz,h,2,2) = c*(DD(uz,h,1)*hc(1,2) -
     &                          DD(uz,h,2)*hc(2,2) -
     &                          DD(uz,h,3)*hc(3,2))
	    ghss(uz,h,2,1) = c*(DD(uz,h,1)*hc(1,3) -
     &                          DD(uz,h,2)*hc(2,3) -
     &                          DD(uz,h,3)*hc(3,3))
	    ghss(uz,h,1,2) = -DCONJG(ghss(uz,h,2,1))
	  enddo
	enddo
	end


************************************************************************

	subroutine HpSfSfCouplings(ghpss, t, g, m)
	implicit none
	double complex ghpss(2,2)
	integer t, g
	double precision m

#include "FH.h"

	double precision m11, m22
	double complex m12, m21
	integer s1, s2

	m11 = Mf2(t-1,g)/TB + m**2*TB - MW2*S2B
	m22 = Mf(t-1,g)*m*(1/TB + TB)
	m12 = m*(DCONJG(MUE) + Af(t,g)*TB)
	m21 = Mf(t-1,g)*(MUE + DCONJG(Af(t-1,g))/TB)

	do s1 = 1, 2
	  do s2 = 1, 2
	    ghpss(s1,s2) = cI/sqrt2*EL0/(MW*SW)*(
     &        USfC(s1,1,t-1,g)*
     &          (m11*USf(s2,1,t,g) + m12*USf(s2,2,t,g)) +
     &        USfC(s1,2,t-1,g)*
     &          (m21*USf(s2,1,t,g) + m22*USf(s2,2,t,g)) )
	  enddo
	enddo
	end


************************************************************************
	
	double precision function Kallen(m, m1, m2)
	implicit none
	double precision m, m1, m2

	double precision km, kp

	if( m .gt. m1 + m2 ) then
	  km = (m1 - m2)/m
	  km = (1 - km)*(1 + km)
	  kp = (m1 + m2)/m
	  kp = (1 - kp)*(1 + kp)
	  Kallen = sqrt(km*kp)
	else
	  Kallen = 0
	endif
	end


************************************************************************
	
	double precision function KallenLR(m, m1, m2, cL, cR)
	implicit none
	double precision m, m1, m2
	double complex cL, cR

	double precision km, kp

	double precision lambda, ratio
	common /KallenVars/ lambda, ratio

	ratio = .5D0*(Sq(cL) + Sq(cR))

	kp = (m1 + m2)/m
	kp = (1 - kp)*(1 + kp)
	if( kp .gt. 0 ) then
	  km = (m1 - m2)/m
	  km = (1 - km)*(1 + km)
	  lambda = sqrt(km*kp)
	  KallenLR = .25D0*lambda*(km*Sq(cL - cR) + kp*Sq(cL + cR))
	else
	  lambda = 0
	  KallenLR = 0
	endif
	end


************************************************************************

	double precision function HXVWidth(mh, mx, mv, gammav,
     &    delta1, delta2, isvector)
	implicit none
	double precision mh, mx, mv, gammav, delta1, delta2
	integer isvector

#include "FH.h"

	double precision m, m1, g, c
	common /HXVVars/ m, m1, g, c

	double precision Kallen
	external Kallen, HXVIntegrand

	double precision lambda, d1, d2, w1, w2

	d1 = mx + mv - delta1
	d2 = mx + mv + delta2

	if( mh .gt. d2 ) then
* real decay above threshold
	  lambda = Kallen(mh, mx, mv)
	  HXVWidth = lambda*(lambda**2 + isvector*12*(mv/mh)**4)
	else if( mh .gt. d1 ) then
* interpolate inside [threshold - delta1, threshold + delta2]
	  lambda = Kallen(d2, mx, mv)
	  w2 = lambda*(lambda**2 + isvector*12*(mv/d2)**4)
	  m = d1/mv
	  m1 = mx/mv
	  g = gammav/mv
	  c = isvector*12
	  call Gauss(1, w1, HXVIntegrand, 0D0, m - m1, 16)
	  w1 = (isvector + 1)*w1
	  HXVWidth = w1 + (w2 - w1)*(mh - d1)/(delta1 + delta2)
	else if( mh .gt. mx + 1 ) then
* effects from finite width of gauge boson
	  m = mh/mv
	  m1 = mx/mv
	  g = gammav/mv
	  c = isvector*12
	  call Gauss(1, w1, HXVIntegrand, 0D0, m - m1, 16)
	  HXVWidth = (isvector + 1)*w1
	else
	  HXVWidth = 0
	endif

	HXVWidth = GF/(8*sqrt2*pi)*mh**3*HXVWidth
	end


************************************************************************

	subroutine HXVIntegrand(res, x)
	implicit none
	double precision res, x

#include "const.h"

	double precision Kallen
	external Kallen

	double precision m, m1, g, c
	common /HXVVars/ m, m1, g, c

	double precision lambda

	lambda = Kallen(m, m1, x)

	res = 2/pi*x*g*lambda*(lambda**2 + c*(x/m**2)**2)/
     &    (((x - 1)*(x + 1))**2 + g**2)
	end


************************************************************************

	subroutine SetDecayScale(fail, mh)
	implicit none
	integer fail
	double precision mh

#include "Decay.h"

	double precision qrun
	integer nf

	double precision AlphaS, Run
	external AlphaS, Run

	AlfasMH = AlphaS(mh, nf)

	qrun = Run(2D0, mh)

	Mfy(6,1) = MU*qrun
	Mfy2(6,1) = Mfy(6,1)**2
	Mfy(7,1) = MD*qrun
	Mfy2(7,1) = Mfy(7,1)**2
	Mfy(6,2) = MC*Run(MC, mh)
	Mfy2(6,2) = Mfy(6,2)**2
	Mfy(7,2) = MS*qrun
	Mfy2(7,2) = Mfy(7,2)**2
	Mfy(7,3) = MB*Run(MB, mh)
	Mfy2(7,3) = Mfy(7,3)**2

	call SfUpdate(fail, 6, 3)
	call SfUpdate(fail, 7, 4)

	hggSq(1) = sqrt(1 + AlfasMH/pi*(95/4D0 - 7/6D0*nf))
	hggSq(2) = hggSq(1)
	hggSq(3) = sqrt(1 + AlfasMH/pi*(97/4D0 - 7/6D0*nf))

	if( debuglevel .lt. 5 ) return

	DCOUP "Mfy(6,1) =", Mfy(6,1) ENDL
	DCOUP "   (6,2) =", Mfy(6,2) ENDL
	DCOUP "   (6,3) =", Mfy(6,3) ENDL
	DCOUP "Mfy(7,1) =", Mfy(7,1) ENDL
	DCOUP "   (7,2) =", Mfy(7,2) ENDL
	DCOUP "   (7,3) =", Mfy(7,3) ENDL
	end


************************************************************************

	subroutine Kin1to2(fac, h, m2, m3, symm)
	implicit none
	integer h
	double precision fac, m2, m3, symm

#include "Decay.h"

	double precision sqrtS, Pout

	sqrtS = MHiggs(h)
	if( sqrtS .le. m2 + m3 ) then
	  fac = 0
	  return
	endif

	Pout = .5D0*(sqrtS - (m2 - m3)*(m2 + m3)/sqrtS)
	Pout = (Pout - m3)*(Pout + m3)
	Pout = sqrt(Pout)

	call VecSet(1, sqrtS, 0D0, 0D0, 0D0, 1D0)
	call VecSet(2, m2, Pout, 1D0, 0D0, 0D0)
	call VecSet(3, m3, Pout, -1D0, 0D0, 0D0)

* 4 pi from the integration, symmetry factors, and flux
	fac = 4*pi/(2*(2*pi)**2*4)*Pout/sqrtS**2*symm

	hno = h

	if( debuglevel .lt. 5 ) return

	DCOUP "Kin1to2: h =", h	ENDL
	DCOUP "  m2 =", m2	ENDL
	DCOUP "  m3 =", m3	ENDL
	end


************************************************************************

	subroutine HffDecays(h, t, g1, g2, sq, sqSM)
	implicit none
	integer h, t, g1, g2
	external sq, sqSM

#include "Decay.h"

	double precision fac, sqmeSM(2), sqme(2)
	double precision m, b2, b, lb, d, ld, Abb, c, ir
	character*7 id

	double precision Li2, AlphaS
	external Li2, AlphaS

c	Divergence = 1000
c	call setdelta(Divergence)

	if( t .eq. 4 ) then
	  call Kin1to2(fac, h, Mfy(7,g1), Mfy(7,g2), 1D0)
	else
	  call Kin1to2(fac, h, Mf(t,g1), Mf(t,g2), 1D0)
	endif
	if( fac .eq. 0 ) return

	gno1 = g1
	gno2 = g2

	ir = 0
	if( g1 .eq. g2 ) then
* IR factor from Dabelstein, hep-ph/9503443, Eq. (4.4)
	  m = (mass(2) + mass(3))/mass(1)
	  b2 = (1 - m)*(1 + m)
	  b = sqrt(b2)
	  lb = log(b)
	  d = (1 - b)/(1 + b)
	  ld = log(d)
	  Abb = (1 + b2)/b*
     &        (4*Li2(d) + 2*Li2(-d) + (3*log(2/(1 + b)) + 2*lb)*ld) +
     &      6*log(.5D0*m) - 4*lb
	  if( h .lt. 3 ) then
	    c = Abb - (3/16D0 + b2*(34/16D0 - 13/16D0*b2))/(b*b2)*ld +
     &        21/8D0 - 3/8D0/b2
	  else
	    c = Abb - (19/16D0 + b2*(2/16D0 + 3/16D0*b2))/(b*b2)*ld +
     &        21/8D0 - 3/8D0*b2
	  endif
	  ir = Alfa0/pi*Qf(t)**2*c
	  if( t .gt. 2 )
     &      ir = ir + AlfasMH/pi*CF*(c + 3*log(mass(1)/mass(2)) + 2)
* 2 (= 8/4) is due to the MSbar shift, K Williams PhD Eq. (7.5)
	endif

	call sqSM(sqmeSM, O'155', -1)
	GammaSM(H0FF(h,t,g1,g2)) = fac*((1 + ir)*sqmeSM(1) + sqmeSM(2))
	GammaSMTot(h) = GammaSMTot(h) + GammaSM(H0FF(h,t,g1,g2))

	call sq(sqme, O'155', -1)
	if( t*64 + g1*8 + g2 .eq. O'433' ) then
	  Gamma(H0FF(h,t,g1,g2)) = Gamma(H0FF(h,t,g1,g2)) +
     &      fac*(ir*sqme(1) + sqme(2))
	else
	  Gamma(H0FF(h,t,g1,g2)) = fac*((1 + ir)*sqme(1) + sqme(2))
	endif
	GammaTot(h) = GammaTot(h) + Gamma(H0FF(h,t,g1,g2))

	if( debuglevel .lt. 3 ) return

	id = "Hff"//Digit(h)//Digit(t)//Digit(g1)//Digit(g2)

	DCOUP id, " SM:   sqme(tree) =", sqmeSM(1)	ENDL
	DCOUP id, " SM:   sqme(loop) =", sqmeSM(2)	ENDL
	DCOUP id, " MSSM: sqme(tree) =", sqme(1)	ENDL
	DCOUP id, " MSSM: sqme(loop) =", sqme(2)	ENDL
	DCOUP "  Mfy(t,g1) =", Mfy(t,g1)		ENDL
	DCOUP "  Mfy(t,g2) =", Mfy(t,g2)		ENDL
	DCOUP "  prefactor =", fac			ENDL
	DCOUP "  IR factor =", ir			ENDL
	end

