* Para.F
* initialization routines for the various parameter blocks
* this file is part of FeynHiggs
* last modified 26 Aug 10 th

* based on mssm_ini.F from FormCalc:
* Christian Schappacher, 17 Feb 99
* last modified 15 Jun 01 th
* (almost) completely rewritten to be independent
* of any external libraries and
* adapted to Higgs mass routines by Markus Frank.
* 01.10.2002 sh: inclusion of O(als alb) resummation

#include "externals.h"
#include "debug.h"

#define __SUBROUTINE__ "FHSetPara"


	subroutine BeginPara(error, scalefactor_, MT_, TB_, M_3_)
	implicit none
	integer error
	double precision scalefactor_, MT_, TB_
	double complex M_3_

#include "FH.h"
#include "FHRecord.h"
#include "looptools.h"

	external recordnamesini

	integer i, t
	double precision i3part, qpart, tmp

	double precision Run
	external Run

	para_valid = 0
	sf_valid = 0
	tl_valid = 0
	higgs_valid = 0
	coup_valid = 0

	error = 0

	CheckFlags()

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "BeginPara input parameters:"	ENDL
	  DPARA "----------------------------"	ENDL
	  DPARA "scalefactor =", scalefactor_	ENDL
	  DTAG(iMT, MT_)
	  DTAG(iTB, TB_)
	  DTAGre(iM3, M_3_)
	  DTAGimz(iM3, M_3_)
	endif

	nmfv = 0

	do i = 1, semax
	  se2R(i) = 0
	enddo
	do i = 1, semax
	  seAdd(i) = 0
	enddo

	Qf(1) = 0
	Qf(2) = Qe
	Qf(3) = Qu
	Qf(4) = Qd

	QSf(2) = 0
	QSf(3) = 0
	QSf(4) = 0

	do i = 1, NSf*7
	  Sf_flat(i) = 0
	enddo

	do t = 1, 4
	  Mfy(t,1) = Mf(t,1)
	  Mfy(t,2) = Mf(t,2)
	  Mfy(t,3) = Mf(t,3)
	  Mfy2(t,1) = Mf2(t,1)
	  Mfy2(t,2) = Mf2(t,2)
	  Mfy2(t,3) = Mf2(t,3)
	enddo

	MT = MT_
	if( MT .lt. 0 ) Error("invalid MT")
	MT2 = MT**2

	call ffini
	scalefactor = scalefactor_
	call setmudim(MT2*scalefactor**2)

	if( AlfaMZ .lt. 0 ) AlfaMZ = Alfa0/(1 - DeltaAlfa -
     &    4/(9*pi)*(1/3D0 - (2*MT2/MZ2 + 1)*
     &      (DBLE(B0(MZ2, MT2, MT2)) - DBLE(A0(MT2))/MT2 + 1)) )
* the last bit, 4/(9 pi)..., is the DeltaAlfa_top contribution
* we include it here (i.e. resummed) because AlfasMZ in the SLHA
* includes it, too
	ELMZ = sqrt(4*pi*AlfaMZ)

	AlfasMT = AlfasMZ/(1 + (11 - 10/3D0)/(4*pi)*AlfasMZ*log(MT2/MZ2))
c	AlfasMT = AlphaS(MT)
	GSMT = sqrt(4*pi*AlfasMT)

	MTrun(0) = MT
	MTrun(1) = MT
	if( looplevel .gt. 1 ) then
	  if( tlCplxApprox .eq. 1 ) then
	    MTrun(1) = MT/(1 + 4/(3*pi)*AlfasMT)
	    Alfat = (MTrun(1)/(vev/sqrt2))**2/(4*pi)
	  else
	    tmp = 0
	    do while( abs(MTrun(1) - tmp) .gt. 1D-5 )
	      tmp = MTrun(1)
	      Alfat = (MTrun(1)/(vev/sqrt2))**2/(4*pi)
	      MTrun(1) = MT/(1 - Alfat/(2*pi) + 4/(3*pi)*AlfasMT)
	    enddo
	  endif
	endif
	MTy = MTrun(runningMT)
	MTy2 = MTy**2

	MBrun = MB*Run(MB, MT)

	M_3 = M_3_
	MGl = abs(M_3)
	MGl2 = MGl**2

	TB = TB_
	if( TB .lt. 0 ) Error("invalid TB")
	TB2 = TB**2
	CB2 = 1/(1 + TB2)
	SB2 = CB2*TB2
	CB = sqrt(CB2)
	SB = CB*TB
	S2B = 2*TB*CB2
	C2B = (CB - SB)*(CB + SB)

	CSB(2) = SB
	CSB(3) = CB
	CSB(4) = SB

	i3part = .5D0*MZ2*C2B
	qpart = SW2*MZ2*C2B
	DSf(2,1) = 0
	DSf(1,1) = +i3part

	DSf(2,2) = Qe*qpart
	DSf(1,2) = -i3part - DSf(2,2)

	DSf(2,3) = Qu*qpart
	DSf(1,3) = +i3part - DSf(2,3)

	DSf(2,4) = Qd*qpart
	DSf(2,5) = DSf(2,4)
	DSf(1,4) = -i3part - DSf(2,4)
	DSf(1,5) = DSf(1,4)
	end


************************************************************************

	subroutine HiggsMasses(error, MA0_, MHp_, Q_)
	implicit none
	integer error
	double precision MA0_, MHp_, Q_

#include "FH.h"
#include "FHRecord.h"

	double precision avgMH2, deltaMH2

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "HiggsMasses input:"		ENDL
	  DPARA "----------------------------"	ENDL
	  DTAGm(iMA0, MA0_)
	  DTAGm(iMHp, MHp_)
	endif

	if( MA0_ .ge. 0 ) then
	  MA0 = max(MA0_, 1D-3)
	  MA02 = MA0**2
	  MHp2 = MA02 + MW2
	  MHp = sqrt(MHp2)
	  inputmass = 3
	else if( MHp_ .gt. MW ) then
	  MHp = MHp_
	  MHp2 = MHp**2
	  MA02 = (MHp - MW)*(MHp + MW)
	  MA0 = sqrt(MA02)
	  inputmass = 4
	else
	  Error("neither MA0 nor MHp given")
	  return
	endif

* these are the tree-level Higgs masses:
	avgMH2 = .5D0*(MA02 + MZ2)
	deltaMH2 = MA0*MZ*C2B
	deltaMH2 = sqrt((avgMH2 - deltaMH2)*(avgMH2 + deltaMH2))
* the 1D-6 is so we don't get killed by e.g. B0(0, Mh02, Mh02) later
	Mh02 = max(avgMH2 - deltaMH2, 1D-6)
	MHH2 = avgMH2 + deltaMH2
	Mh0 = -1
	if( Mh02 .ge. 0 ) Mh0 = sqrt(Mh02)
	MHH = -1
	if( MHH2 .ge. 0 ) MHH = sqrt(MHH2)

	CA2 = .5D0*(SB2*MA02 + CB2*MZ2 - Mh02)/deltaMH2
	CA  = sqrt(CA2)
	SA  = -SB*CB*avgMH2/(CA*deltaMH2)
	SA2 = SA**2
	S2A = 2*SA*CA
	C2A = (CA - SA)*(CA + SA)
	SAB = SA*CB + CA*SB
	CAB = CA*CB - SA*SB
	SBA = SB*CA - CB*SA
	SBA2 = SBA**2
	CBA = CB*CA + SB*SA
	CBA2 = CBA**2
	end


************************************************************************

	subroutine CharginoMasses(M_2_, MUE_)
	implicit none
	double complex M_2_, MUE_

#include "FH.h"
#include "FHRecord.h"

	double complex X(2,2)

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "CharginoMasses input:"		ENDL
	  DPARA "----------------------------"	ENDL
	  DTAGre(iM2, M_2_)
	  DTAGimz(iM2, M_2_)
	  DTAGre(iMUE, MUE_)
	  DTAGimz(iMUE, MUE_)
	endif

	M_2 = M_2_
	MUE = MUE_

	X(1,1) = M_2
	X(1,2) = sqrt2*MW*SB
	X(2,1) = sqrt2*MW*CB
	X(2,2) = MUE

	call SVD(2, 2, X,2, MCha, UCha,2, VCha,2, 1)

	MCha2(1) = MCha(1)**2
	MCha2(2) = MCha(2)**2

	MUETB(3) = -DCONJG(MUE)/TB
	MUETB(2) = -DCONJG(MUE)*TB
	MUETB(4) = MUETB(2)
	end


************************************************************************

	subroutine NeutralinoMasses(M_1_)
	implicit none
	double complex M_1_

#include "FH.h"
#include "FHRecord.h"

	double complex Y(4,4)

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "NeutralinoMasses input:"	ENDL
	  DPARA "----------------------------"	ENDL
	  DTAGrez(iM1, M_1_)
	  DTAGimz(iM1, M_1_)
	endif

	M_1 = M_1_
* We use the SUSY-GUT relation here:
	if( abs(M_1) .lt. 1D-5 ) M_1 = 5/3D0*SW2/CW2*M_2

* Calculation of Neutralino masses and mixing matrix:
	Y(1,1) = M_1
	Y(2,2) = M_2
	Y(3,3) = 0
	Y(4,4) = 0
	Y(1,2) = 0
	Y(1,3) = -MZ*SW*CB
	Y(1,4) = MZ*SW*SB
	Y(2,3) = MZ*CW*CB
	Y(2,4) = -MZ*CW*SB
	Y(3,4) = -MUE

	call TakagiFactor(4, Y,4, MNeu, ZNeu,4, 1)

	MNeu2(1) = MNeu(1)**2
	MNeu2(2) = MNeu(2)**2
	MNeu2(3) = MNeu(3)**2
	MNeu2(4) = MNeu(4)**2
	end


************************************************************************

	subroutine SfermionMasses(t,
     &    M1SL2_, M2SL2_, M3SL2_,
     &    M1SR2_, M2SR2_, M3SR2_,
     &    Af1_, Af2_, Af3_, Q_)
	implicit none
	integer t
	double precision M1SL2_, M2SL2_, M3SL2_
	double precision M1SR2_, M2SR2_, M3SR2_, Q_
	double complex Af1_, Af2_, Af3_

#include "FH.h"
#include "FHRecord.h"

	integer n

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "SfermionMasses input:"		ENDL
	  DPARA "----------------------------"	ENDL
	  n = ior(t - 1, 1) - 1
	  DTAG(iM1SL+n, SignSqrt(M1SL2_))
	  DTAG(iM2SL+n, SignSqrt(M2SL2_))
	  DTAG(iM3SL+n, SignSqrt(M3SL2_))
	  n = t + ibits(t + 1, 2, 1) - 1
	  DTAG(iM1SL+n, SignSqrt(M1SR2_))
	  DTAG(iM2SL+n, SignSqrt(M2SR2_))
	  DTAG(iM3SL+n, SignSqrt(M3SR2_))
	  n = (t - 2)*(iAu - iAe)
	  DTAGre(iAe+n, Af1_)
	  DTAGimz(iAe+n, Af1_)
	  DTAGre(iAmu+n, Af2_)
	  DTAGimz(iAmu+n, Af2_)
	  DTAGre(iAtau+n, Af3_)
	  DTAGimz(iAtau+n, Af3_)
	  DTAGz(iQtau+t-2, Q_)
	endif

	MSS2(1,t,1,1) = M1SL2_
	MSS2(1,t,2,2) = M2SL2_
	MSS2(1,t,3,3) = M3SL2_

	MSS2(2,t,1,1) = M1SR2_
	MSS2(2,t,2,2) = M2SR2_
	MSS2(2,t,3,3) = M3SR2_

	Af(t,1) = Af1_
	Af(t,2) = Af2_
	Af(t,3) = Af3_

	QSf(t) = Q_
	end


************************************************************************

	subroutine EndPara(error)
	implicit none
	integer error

#include "FH.h"
#include "looptools.h"

	double precision m1(2:4), Qsusy
	integer t, g, fail

	double precision Run
	external Run

	character*(*) msg
	parameter (msg =
     &    " mixing selected in the presence of complex parameters")

	double precision signsqrt
	double complex c
	signsqrt(c) = sign(sqrt(abs(DBLE(c))), DBLE(c))

	do g = 1, 3
	  m1(2) = DBLE(MSS2(1,2,g,g))
	  m1(3) = DBLE(MSS2(1,3,1,1))*Sq(CKMin(g,1)) +
     &            DBLE(MSS2(1,3,2,2))*Sq(CKMin(g,2)) +
     &            DBLE(MSS2(1,3,3,3))*Sq(CKMin(g,3))
	  m1(4) = DBLE(MSS2(1,4,g,g))
	  do t = 2, 4
	    DSS2(1,t,g) = (m1(t) - DBLE(MSS2(2,t,g,g))) +
     &        (DSf(1,t) - DSf(2,t))
	    DSS2(2,t,g) = DBLE(MSS2(2,t,g,g)) + DSf(2,t)
	    Xf(t,g) = Af(t,g) + MUETB(t)
	  enddo
	enddo

	if( abs(QSf(3)) + abs(QSf(4)) .gt. 1D-13 ) then
	  Qsusy = sqrt(abs(MSS2(1,3,3,3))*abs(MSS2(2,3,3,3)))
	  if( QSf(3) .lt. 0 ) QSf(3) = Qsusy
	  if( QSf(4) .lt. 0 ) QSf(4) = Qsusy
	  Qsusy = QSf(3) + QSf(4)
	  if( abs(QSf(3)*QSf(4)) .gt. 1D-13 ) Qsusy = .5D0*Qsusy

	  fail = 0
	  call EigenSf(fail, 3, MSf(1,3,3), MSf2(1,3,3), USf(1,1,3,3),
     &      DSS2(1,3,3), MT*Run(MT, Qsusy), Xf(3,3), 0)
	  call EigenSf(fail, 3, MSf(1,4,3), MSf2(1,4,3), USf(1,1,4,3),
     &      DSS2(1,4,3), MB*Run(MB, Qsusy), Xf(4,3), 0)

	  if( QSf(3) .ne. 0 ) call DRbartoOS(fail, 3)
	  if( QSf(4) .ne. 0 ) call DRbartoOS(fail, 4)
	  Af(3,3) = Xf(3,3) - MUETB(3)
	  Af(4,3) = Xf(4,3) - MUETB(4)

	  if( fail .ne. 0 ) Error("DRbartoOS transition failed")
	endif

	if( higgsmix .lt. 3 .and.
     &    abs(DIMAG(Af(2,1))) + abs(DIMAG(Af(3,1))) + abs(DIMAG(Af(4,1))) +
     &    abs(DIMAG(Af(2,2))) + abs(DIMAG(Af(3,2))) + abs(DIMAG(Af(4,2))) +
     &    abs(DIMAG(Af(2,3))) + abs(DIMAG(Af(3,3))) + abs(DIMAG(Af(4,3))) +
     &    abs(DIMAG(MUE)) +
     &    abs(DIMAG(M_1)) + abs(DIMAG(M_2)) + abs(DIMAG(M_3)) .ne. 0 )
     &    Warning(Digit(higgsmix)//"x"//Digit(higgsmix)//msg)

	if( error .eq. 0 ) para_valid = valid

	if( debuglevel .lt. 1 ) return

	DPARA "============================"	ENDL
	DPARA "parameters set by FHSetPara"	ENDL
	DPARA "----------------------------"	ENDL
	DPARA "mudim =", sqrt(getmudim())	ENDL
	DPARA "MT =", MT			ENDL
	DPARA "MTrun =", MTrun(1)		ENDL
	DPARA "MTy =", MTy			ENDL
	DPARA "1/Alfa(MZ) =", 1/AlfaMZ		ENDL
	DPARA "AlfasMT =", AlfasMT		ENDL
	DPARA "TB =", TB			ENDL
	DPARA "M_1 =", M_1			ENDL
	DPARA "M_2 =", M_2			ENDL
	DPARA "M_3 =", M_3			ENDL
	DPARA "MUE =", MUE			ENDL
	DPARA "----------------------------"	ENDL
	DPARA "M1SL =", signsqrt(MSS2(1,2,1,1))	ENDL
	DPARA "M1SE =", signsqrt(MSS2(2,2,1,1))	ENDL
	DPARA "M1SQ =", signsqrt(MSS2(1,3,1,1))	ENDL
	DPARA "M1SU =", signsqrt(MSS2(2,3,1,1))	ENDL
	DPARA "M1SD =", signsqrt(MSS2(2,4,1,1))	ENDL
	DPARA "Ae   =", Af(2,1)			ENDL
	DPARA "Au   =", Af(3,1)			ENDL
	DPARA "Ad   =", Af(4,1)			ENDL
	DPARA "----------------------------"	ENDL
	DPARA "M2SL =", signsqrt(MSS2(1,2,2,2))	ENDL
	DPARA "M2SE =", signsqrt(MSS2(2,2,2,2))	ENDL
	DPARA "M2SQ =", signsqrt(MSS2(1,3,2,2))	ENDL
	DPARA "M2SU =", signsqrt(MSS2(2,3,2,2))	ENDL
	DPARA "M2SD =", signsqrt(MSS2(2,4,2,2))	ENDL
	DPARA "Amu  =", Af(2,2)			ENDL
	DPARA "Ac   =", Af(3,2)			ENDL
	DPARA "As   =", Af(4,2)			ENDL
	DPARA "----------------------------"	ENDL
	DPARA "M3SL =", signsqrt(MSS2(1,2,3,3))	ENDL
	DPARA "M3SE =", signsqrt(MSS2(2,2,3,3))	ENDL
	DPARA "M3SQ =", signsqrt(MSS2(1,3,3,3))	ENDL
	DPARA "M3SU =", signsqrt(MSS2(2,3,3,3))	ENDL
	DPARA "M3SD =", signsqrt(MSS2(2,4,3,3))	ENDL
	DPARA "Atau =", Af(2,3)			ENDL
	DPARA "At   =", Af(3,3)			ENDL
	DPARA "Ab   =", Af(4,3)			ENDL
	DPARA "----------------------------"	ENDL
	DPARA "Higgs masses (tree-level):"	ENDL
	DPARA "Mh0 =", Mh0			ENDL
	DPARA "MHH =", MHH			ENDL
	DPARA "MA0 =", MA0			ENDL
	DPARA "MHp =", MHp			ENDL
	DPARA "SA =", SA			ENDL
	DPARA "----------------------------"	ENDL
	DPARA "Chargino masses:"		ENDL
	DPARA "MCha(1) =", MCha(1)		ENDL
	DPARA "MCha(2) =", MCha(2)		ENDL
	DPARA "----------------------------"	ENDL
	DPARA "Neutralino masses:"		ENDL
	DPARA "MNeu(1) =", MNeu(1)		ENDL
	DPARA "MNeu(2) =", MNeu(2)		ENDL
	DPARA "MNeu(3) =", MNeu(3)		ENDL
	DPARA "MNeu(4) =", MNeu(4)		ENDL
	end


************************************************************************

	block data recordnamesini

#include "FHRecord.h"

	data FHName(iinvAlfaMZ) /"invAlfaMZ"/
	data FHName(iAlfasMZ) /"AlfasMZ"/
	data FHName(iGF) /"GF"/

	data FHName(iMS) /"MS"/
	data FHName(iMC) /"MC"/
	data FHName(iMT) /"MT"/
	data FHName(iMB) /"MB"/
	data FHName(iMW) /"MW"/
	data FHName(iMZ) /"MZ"/

	data FHName(iCKMlambda) /"CKMlambda"/
	data FHName(iCKMA) /"CKMA"/
	data FHName(iCKMrho) /"CKMrho"/
	data FHName(iCKMeta) /"CKMeta"/

	data FHName(iTB) /"TB"/
	data FHName(iMA0) /"MA0"/
	data FHName(iMHp) /"MHp"/

	data FHName(iMSusy) /"MSusy"/
	data FHName(iM1SL) /"M1SL"/
	data FHName(iM2SL) /"M2SL"/
	data FHName(iM3SL) /"M3SL"/
	data FHName(iM1SE) /"M1SE"/
	data FHName(iM2SE) /"M2SE"/
	data FHName(iM3SE) /"M3SE"/
	data FHName(iM1SQ) /"M1SQ"/
	data FHName(iM2SQ) /"M2SQ"/
	data FHName(iM3SQ) /"M3SQ"/
	data FHName(iM1SU) /"M1SU"/
	data FHName(iM2SU) /"M2SU"/
	data FHName(iM3SU) /"M3SU"/
	data FHName(iM1SD) /"M1SD"/
	data FHName(iM2SD) /"M2SD"/
	data FHName(iM3SD) /"M3SD"/
	data FHName(iQtau) /"Qtau"/
	data FHName(iQt) /"Qt"/
	data FHName(iQb) /"Qb"/
	data FHName(iprodSqrts) /"prodSqrts"/

	data FHName(iRe(iAe)) /"Re(Ae)"/
	data FHName(iIm(iAe)) /"Im(Ae)"/
	data FHName(iAbs(iAe)) /"Abs(Ae)"/
	data FHName(iArg(iAe)) /"Arg(Ae)"/

	data FHName(iRe(iAu)) /"Re(Au)"/
	data FHName(iIm(iAu)) /"Im(Au)"/
	data FHName(iAbs(iAu)) /"Abs(Au)"/
	data FHName(iArg(iAu)) /"Arg(Au)"/

	data FHName(iRe(iAd)) /"Re(Ad)"/
	data FHName(iIm(iAd)) /"Im(Ad)"/
	data FHName(iAbs(iAd)) /"Abs(Ad)"/
	data FHName(iArg(iAd)) /"Arg(Ad)"/

	data FHName(iRe(iAmu)) /"Re(Amu)"/
	data FHName(iIm(iAmu)) /"Im(Amu)"/
	data FHName(iAbs(iAmu)) /"Abs(Amu)"/
	data FHName(iArg(iAmu)) /"Arg(Amu)"/

	data FHName(iRe(iAc)) /"Re(Ac)"/
	data FHName(iIm(iAc)) /"Im(Ac)"/
	data FHName(iAbs(iAc)) /"Abs(Ac)"/
	data FHName(iArg(iAc)) /"Arg(Ac)"/

	data FHName(iRe(iAs)) /"Re(As)"/
	data FHName(iIm(iAs)) /"Im(As)"/
	data FHName(iAbs(iAs)) /"Abs(As)"/
	data FHName(iArg(iAs)) /"Arg(As)"/

	data FHName(iRe(iAtau)) /"Re(Atau)"/
	data FHName(iIm(iAtau)) /"Im(Atau)"/
	data FHName(iAbs(iAtau)) /"Abs(Atau)"/
	data FHName(iArg(iAtau)) /"Arg(Atau)"/

	data FHName(iRe(iAt)) /"Re(At)"/
	data FHName(iIm(iAt)) /"Im(At)"/
	data FHName(iAbs(iAt)) /"Abs(At)"/
	data FHName(iArg(iAt)) /"Arg(At)"/

	data FHName(iRe(iAb)) /"Re(Ab)"/
	data FHName(iIm(iAb)) /"Im(Ab)"/
	data FHName(iAbs(iAb)) /"Abs(Ab)"/
	data FHName(iArg(iAb)) /"Arg(Ab)"/

	data FHName(iRe(iXtau)) /"Re(Xtau)"/
	data FHName(iIm(iXtau)) /"Im(Xtau)"/
	data FHName(iAbs(iXtau)) /"Abs(Xtau)"/
	data FHName(iArg(iXtau)) /"Arg(Xtau)"/

	data FHName(iRe(iXt)) /"Re(Xt)"/
	data FHName(iIm(iXt)) /"Im(Xt)"/
	data FHName(iAbs(iXt)) /"Abs(Xt)"/
	data FHName(iArg(iXt)) /"Arg(Xt)"/

	data FHName(iRe(iXb)) /"Re(Xb)"/
	data FHName(iIm(iXb)) /"Im(Xb)"/
	data FHName(iAbs(iXb)) /"Abs(Xb)"/
	data FHName(iArg(iXb)) /"Arg(Xb)"/

	data FHName(iRe(iMUE)) /"Re(MUE)"/
	data FHName(iIm(iMUE)) /"Im(MUE)"/
	data FHName(iAbs(iMUE)) /"Abs(MUE)"/
	data FHName(iArg(iMUE)) /"Arg(MUE)"/

	data FHName(iRe(iM1)) /"Re(M_1)"/
	data FHName(iIm(iM1)) /"Im(M_1)"/
	data FHName(iAbs(iM1)) /"Abs(M_1)"/
	data FHName(iArg(iM1)) /"Arg(M_1)"/

	data FHName(iRe(iM2)) /"Re(M_2)"/
	data FHName(iIm(iM2)) /"Im(M_2)"/
	data FHName(iAbs(iM2)) /"Abs(M_2)"/
	data FHName(iArg(iM2)) /"Arg(M_2)"/

	data FHName(iRe(iM3)) /"Re(M_3)"/
	data FHName(iIm(iM3)) /"Im(M_3)"/
	data FHName(iAbs(iM3)) /"Abs(M_3)"/
	data FHName(iArg(iM3)) /"Arg(M_3)"/

	data FHName(iRe(ideltaLL12)) /"Re(deltaLL12)"/
	data FHName(iIm(ideltaLL12)) /"Im(deltaLL12)"/
	data FHName(iAbs(ideltaLL12)) /"Abs(deltaLL12)"/
	data FHName(iArg(ideltaLL12)) /"Arg(deltaLL12)"/

	data FHName(iRe(ideltaLL23)) /"Re(deltaLL23)"/
	data FHName(iIm(ideltaLL23)) /"Im(deltaLL23)"/
	data FHName(iAbs(ideltaLL23)) /"Abs(deltaLL23)"/
	data FHName(iArg(ideltaLL23)) /"Arg(deltaLL23)"/

	data FHName(iRe(ideltaLL13)) /"Re(deltaLL13)"/
	data FHName(iIm(ideltaLL13)) /"Im(deltaLL13)"/
	data FHName(iAbs(ideltaLL13)) /"Abs(deltaLL13)"/
	data FHName(iArg(ideltaLL13)) /"Arg(deltaLL13)"/

	data FHName(iRe(ideltaLRuc)) /"Re(deltaLRuc)"/
	data FHName(iIm(ideltaLRuc)) /"Im(deltaLRuc)"/
	data FHName(iAbs(ideltaLRuc)) /"Abs(deltaLRuc)"/
	data FHName(iArg(ideltaLRuc)) /"Arg(deltaLRuc)"/

	data FHName(iRe(ideltaLRct)) /"Re(deltaLRct)"/
	data FHName(iIm(ideltaLRct)) /"Im(deltaLRct)"/
	data FHName(iAbs(ideltaLRct)) /"Abs(deltaLRct)"/
	data FHName(iArg(ideltaLRct)) /"Arg(deltaLRct)"/

	data FHName(iRe(ideltaLRut)) /"Re(deltaLRut)"/
	data FHName(iIm(ideltaLRut)) /"Im(deltaLRut)"/
	data FHName(iAbs(ideltaLRut)) /"Abs(deltaLRut)"/
	data FHName(iArg(ideltaLRut)) /"Arg(deltaLRut)"/

	data FHName(iRe(ideltaRLuc)) /"Re(deltaRLuc)"/
	data FHName(iIm(ideltaRLuc)) /"Im(deltaRLuc)"/
	data FHName(iAbs(ideltaRLuc)) /"Abs(deltaRLuc)"/
	data FHName(iArg(ideltaRLuc)) /"Arg(deltaRLuc)"/

	data FHName(iRe(ideltaRLct)) /"Re(deltaRLct)"/
	data FHName(iIm(ideltaRLct)) /"Im(deltaRLct)"/
	data FHName(iAbs(ideltaRLct)) /"Abs(deltaRLct)"/
	data FHName(iArg(ideltaRLct)) /"Arg(deltaRLct)"/

	data FHName(iRe(ideltaRLut)) /"Re(deltaRLut)"/
	data FHName(iIm(ideltaRLut)) /"Im(deltaRLut)"/
	data FHName(iAbs(ideltaRLut)) /"Abs(deltaRLut)"/
	data FHName(iArg(ideltaRLut)) /"Arg(deltaRLut)"/

	data FHName(iRe(ideltaRRuc)) /"Re(deltaRRuc)"/
	data FHName(iIm(ideltaRRuc)) /"Im(deltaRRuc)"/
	data FHName(iAbs(ideltaRRuc)) /"Abs(deltaRRuc)"/
	data FHName(iArg(ideltaRRuc)) /"Arg(deltaRRuc)"/

	data FHName(iRe(ideltaRRct)) /"Re(deltaRRct)"/
	data FHName(iIm(ideltaRRct)) /"Im(deltaRRct)"/
	data FHName(iAbs(ideltaRRct)) /"Abs(deltaRRct)"/
	data FHName(iArg(ideltaRRct)) /"Arg(deltaRRct)"/

	data FHName(iRe(ideltaRRut)) /"Re(deltaRRut)"/
	data FHName(iIm(ideltaRRut)) /"Im(deltaRRut)"/
	data FHName(iAbs(ideltaRRut)) /"Abs(deltaRRut)"/
	data FHName(iArg(ideltaRRut)) /"Arg(deltaRRut)"/

	data FHName(iRe(ideltaLRds)) /"Re(deltaLRds)"/
	data FHName(iIm(ideltaLRds)) /"Im(deltaLRds)"/
	data FHName(iAbs(ideltaLRds)) /"Abs(deltaLRds)"/
	data FHName(iArg(ideltaLRds)) /"Arg(deltaLRds)"/

	data FHName(iRe(ideltaLRsb)) /"Re(deltaLRsb)"/
	data FHName(iIm(ideltaLRsb)) /"Im(deltaLRsb)"/
	data FHName(iAbs(ideltaLRsb)) /"Abs(deltaLRsb)"/
	data FHName(iArg(ideltaLRsb)) /"Arg(deltaLRsb)"/

	data FHName(iRe(ideltaLRdb)) /"Re(deltaLRdb)"/
	data FHName(iIm(ideltaLRdb)) /"Im(deltaLRdb)"/
	data FHName(iAbs(ideltaLRdb)) /"Abs(deltaLRdb)"/
	data FHName(iArg(ideltaLRdb)) /"Arg(deltaLRdb)"/

	data FHName(iRe(ideltaRLds)) /"Re(deltaRLds)"/
	data FHName(iIm(ideltaRLds)) /"Im(deltaRLds)"/
	data FHName(iAbs(ideltaRLds)) /"Abs(deltaRLds)"/
	data FHName(iArg(ideltaRLds)) /"Arg(deltaRLds)"/

	data FHName(iRe(ideltaRLsb)) /"Re(deltaRLsb)"/
	data FHName(iIm(ideltaRLsb)) /"Im(deltaRLsb)"/
	data FHName(iAbs(ideltaRLsb)) /"Abs(deltaRLsb)"/
	data FHName(iArg(ideltaRLsb)) /"Arg(deltaRLsb)"/

	data FHName(iRe(ideltaRLdb)) /"Re(deltaRLdb)"/
	data FHName(iIm(ideltaRLdb)) /"Im(deltaRLdb)"/
	data FHName(iAbs(ideltaRLdb)) /"Abs(deltaRLdb)"/
	data FHName(iArg(ideltaRLdb)) /"Arg(deltaRLdb)"/

	data FHName(iRe(ideltaRRds)) /"Re(deltaRRds)"/
	data FHName(iIm(ideltaRRds)) /"Im(deltaRRds)"/
	data FHName(iAbs(ideltaRRds)) /"Abs(deltaRRds)"/
	data FHName(iArg(ideltaRRds)) /"Arg(deltaRRds)"/

	data FHName(iRe(ideltaRRsb)) /"Re(deltaRRsb)"/
	data FHName(iIm(ideltaRRsb)) /"Im(deltaRRsb)"/
	data FHName(iAbs(ideltaRRsb)) /"Abs(deltaRRsb)"/
	data FHName(iArg(ideltaRRsb)) /"Arg(deltaRRsb)"/

	data FHName(iRe(ideltaRRdb)) /"Re(deltaRRdb)"/
	data FHName(iIm(ideltaRRdb)) /"Im(deltaRRdb)"/
	data FHName(iAbs(ideltaRRdb)) /"Abs(deltaRRdb)"/
	data FHName(iArg(ideltaRRdb)) /"Arg(deltaRRdb)"/
	end

