* SetFlags.F
* set the flags for FeynHiggs
* this file is part of FeynHiggs
* last modified 12 Aug 10 th

#include "externals.h"
#include "debug.h"
#include "version.h"


	subroutine FHSetFlags(error,
     &    mssmpart_, fieldren_, tanbren_,
     &    higgsmix_, p2approx_, looplevel_,
     &    runningMT_, botResum_, tlCplxApprox_)
	implicit none
	integer error
	integer mssmpart_, fieldren_, tanbren_
	integer higgsmix_, p2approx_, looplevel_
	integer runningMT_, botResum_, tlCplxApprox_

#include "FH.h"
#define __SUBROUTINE__ "FHSetFlags"

#if U77EXT
	integer lnblnk
	external lnblnk
#endif

	character*128 env
	integer len

	logical blurb
	data blurb /.TRUE./

	if( blurb ) then
	  print *, "-----------------------------------------------------"
	  print *, "FeynHiggs "//FHVERSION
	  print *, "built on "//FHBUILDDATE
	  print *, "T. Hahn, S. Heinemeyer, W. Hollik, H. Rzehak, G. Weiglein"
	  print *, "http://www.feynhiggs.de"
	  print *, "-----------------------------------------------------"
	  blurb = .FALSE.
c	  call setfpu
	endif

	flags_valid = 0
	para_valid = 0
	sf_valid = 0
	tl_valid = 0
	higgs_valid = 0
	coup_valid = 0

	debuglevel = 0
	debugunit = 6
	uzint = 1
	uzext = 2

	call getenv("FHDEBUG", env)
	read(env, *, end=10, err=10) debuglevel
	DFLAGS "debug level ", debuglevel ENDL
10	continue

	call getenv("FHDEBUGFILE", env)
	len = lnblnk(env)
	if( len .gt. 0 ) then
	  open(66, file=env, status="unknown", err=11)
	  debugunit = 66
	  DFLAGS "debug file ", env(1:len) ENDL
	endif
11	continue

	call getenv("FHSELECTUZ", env)
	read(env, *, end=12, err=12) uzint, uzext
	DFLAGS "uzint, uzext ", uzint, uzext ENDL
12	continue

	error = 0

#define Check(flag, lo, hi, name) \
if(flag.lt.lo .or. flag.gt.hi) Error(name//" invalid")

	mssmpart = mssmpart_
	Check(mssmpart, 0, 4, "mssmpart")

	fieldren = fieldren_
	Check(fieldren, 0, 4, "fieldren")

	tanbren = tanbren_
	Check(tanbren, 0, 2, "tanbren")

	higgsmix = higgsmix_
	Check(higgsmix, 1, 4, "higgsmix")

	p2approx = p2approx_
	Check(p2approx, 0, 4, "p2approx")

	looplevel = looplevel_
	Check(looplevel, 0, 2, "looplevel")

	runningMT = runningMT_
	Check(runningMT, 0, 1, "runningMT")

	botResum = botResum_
	Check(botResum, 0, 1, "botResum")

	tlCplxApprox = tlCplxApprox_
	Check(tlCplxApprox, 0, 6, "tlCplxApprox")

	if( mssmpart .eq. 0 ) then
	  p2approx = 1
	  if( higgsmix .gt. 2 )
     &      Error("mssmpart = 0 with higgsmix > 2 not allowed")
	endif

	if( fieldren .eq. 4 .and. tanbren .eq. 1 )
     &    Error("fieldren = 4 not compatible with tanbren = 1")

	if( error .eq. 0 ) flags_valid = valid

	flags_valid = valid

	if( debuglevel .lt. 1 ) return

	DFLAGS "============================"	ENDL
	DFLAGS "flags set by FHSetFlags"	ENDL
	DFLAGS "----------------------------"	ENDL
	DFLAGS "mssmpart     =", mssmpart	ENDL
	DFLAGS "fieldren     =", fieldren	ENDL
	DFLAGS "tanbren      =", tanbren	ENDL
	DFLAGS "higgsmix     =", higgsmix	ENDL
	DFLAGS "p2approx     =", p2approx	ENDL
	DFLAGS "looplevel    =", looplevel	ENDL
	DFLAGS "runningMT    =", runningMT	ENDL
	DFLAGS "botResum     =", botResum	ENDL
	DFLAGS "tlCplxApprox =", tlCplxApprox	ENDL
	end

