* SetSLHA.F
* initialize parameters from the SLHA data structure
* this file is part of FeynHiggs
* last modified 11 Aug 10 th

#include "externals.h"
#include "debug.h"


	subroutine FHSetSLHA(error, slhadata, scalefactor_)
	implicit none
	integer error
	double complex slhadata(*)
	double precision scalefactor_

#include "SLHA.h"
#include "FH.h"
#define __SUBROUTINE__ "FHSetSLHA"

#define Missing(p) Error("cannot determine "//p//" from SLHA data")
#define Origin(p) if(debuglevel.ge.1) DSLHA p ENDL

c#define SLHA_STRICT

	double complex M_1_, M_2_, M_3_, MUE_
	double precision TB_, MA0_, MHp_, Q_
	double precision CKMlambda_, CKMA_, CKMrho_, CKMeta_

	if( SLHAValid(BlockVCKMIn(1), LengthVCKMIn) .ne. 0 ) then
	  Origin("CKM from VCKMIn")
	  CKMlambda_ = VCKMIn_lambda
	  CKMA_ = VCKMIn_A
	  CKMrho_ = VCKMIn_rhobar
	  CKMeta_ = VCKMIn_etabar
	else
	  CKMlambda_ = -1
	  CKMA_ = -1
	  CKMrho_ = -1
	  CKMeta_ = -1
	endif

	call FHSetSMPara(error,
     &    SMInputs_invAlfaMZ, SMInputs_AlfasMZ, SMInputs_GF,
     &    SMInputs_Ms, SMInputs_Mc, SMInputs_Mb,
     &    -1D0, SMInputs_MZ,
     &    CKMlambda_, CKMA_, CKMrho_, CKMeta_)


	if( int(ModSel_Content) .gt. 0 )
     &    Warning("SLHA record indicates model other than MSSM")

	TB_ = invalid
	M_3_ = invalid

	if( ExtPar_TB .ne. invalid ) then
	  Origin("TB from ExtPar_TB")
	  TB_ = ExtPar_TB
	else if( MinPar_TB .ne. invalid ) then
	  Origin("TB from MinPar_TB")
	  TB_ = MinPar_TB
#ifndef SLHA_STRICT
	else if( HMix_TB .ne. invalid ) then
	  Origin("TB from HMix_TB")
	  TB_ = HMix_TB
#endif
	else
	  Missing("TB")
	endif

	if( ExtPar_M3 .ne. invalid ) then
	  Origin("M_3 from ExtPar_M3")
	  M_3_ = ExtPar_M3
#ifndef SLHA_STRICT
	else if( Mass_MGl .ne. invalid ) then
	  Origin("M_3 from Mass_MGl")
	  M_3_ = Mass_MGl
	else if( MSoft_M3 .ne. invalid ) then
	  Origin("M_3 from MSoft_M3")
	  M_3_ = MSoft_M3
#endif
	else
	  Missing("M_3")
	endif

	call BeginPara(error, scalefactor_, SMInputs_Mt, TB_, M_3_)

* Higgs sector

	MA0_ = invalid
	MHp_ = invalid

	if( ExtPar_MA02 .ne. invalid ) then
	  Origin("MA0 from ExtPar_MA02")
	  MA0_ = sqrt(ExtPar_MA02)
	  Q_ = ExtPar_Q
	else if( ExtPar_MA0 .ne. invalid ) then
	  Origin("MA0 from ExtPar_MA0")
	  MA0_ = ExtPar_MA0
	  Q_ = ExtPar_Q
#ifndef SLHA_STRICT
	else if( Mass_MA0 .ne. invalid ) then
	  Origin("MA0 from Mass_MA0")
	  MA0_ = Mass_MA0
	  Q_ = 0
#endif
	endif

	if( ExtPar_MHp .ne. invalid ) then
	  Origin("MHp from ExtPar_MHp")
	  MHp_ = ExtPar_MHp
	  Q_ = ExtPar_Q
#ifndef SLHA_STRICT
	else if( Mass_MHp .ne. invalid ) then
	  Origin("MHp from Mass_MHp")
	  MHp_ = Mass_MHp
	  Q_ = 0
#endif
	endif

	if( MA0_ .ne. invalid .or. MHp_ .ne. invalid ) then
	  call HiggsMasses(error, MA0_, MHp_, Q_)
	else
	  Missing("MA0 and MHp")
	endif

* Charginos

	MUE_ = invalid
	M_2_ = invalid

	if( ExtPar_MUE .ne. invalid ) then
	  Origin("MUE from ExtPar_MUE")
	  MUE_ = ExtPar_MUE
#ifndef SLHA_STRICT
	else if( HMix_MUE .ne. invalid ) then
	  Origin("MUE from HMix_MUE")
	  MUE_ = HMix_MUE
#endif
	else
	  Missing("MUE")
	endif

	if( ExtPar_M2 .ne. invalid ) then
	  Origin("M_2 from ExtPar_M2")
	  M_2_ = ExtPar_M2
#ifndef SLHA_STRICT
	else if( MSoft_M2 .ne. invalid ) then
	  Origin("M_2 from MSoft_M2")
	  M_2_ = MSoft_M2
#endif
	else
	  Missing("M_2")
	endif

	call CharginoMasses(M_2_, MUE_)

* Neutralinos

	M_1_ = invalid

	if( ExtPar_M1 .ne. invalid ) then
	  Origin("M_1 from ExtPar_M1")
	  M_1_ = ExtPar_M1
#ifndef SLHA_STRICT
	else if( MSoft_M1 .ne. invalid ) then
	  Origin("M_1 from MSoft_M1")
	  M_1_ = MSoft_M1
#endif
	else
	  Missing("M_1")
	endif

	call NeutralinoMasses(M_1_)

* Sfermion sector

	call SLHASfermions(error, 2, slhadata)
	call SLHASfermions(error, 3, slhadata)
	call SLHASfermions(error, 4, slhadata)

	call EndPara(error)
	end


************************************************************************

	subroutine SLHASfermions(error, t, slhadata)
	implicit none
	integer error, t
	double complex slhadata(*)

#include "SLHA.h"
#include "FH.h"

	integer n1, n2
	double precision x, Q_
	double precision M1SL2_, M2SL2_, M3SL2_
	double precision M1SR2_, M2SR2_, M3SR2_
	double complex Af1_, Af2_, Af3_

	double precision SLHADiagValid
	external SLHADiagValid

#define SLHAOffdiag(M,n) M(1,2,n)+M(2,3,n)+M(1,3,n)

	Origin("Sfermion "//Digit(t))

	n1 = ior(t - 1, 1)
	n2 = t + ibits(t + 1, 2, 1)

	if( t .gt. 2 ) then
	  if( SLHADiagValid(MSS2In_MSS2(1,1,n1)) +
     &        SLHADiagValid(MSS2In_MSS2(1,1,n2)) +
     &        SLHADiagValid(TfIn_Tf(1,1,t)) .ne. 0 ) then
	    Origin("NMFV from MSS2In and TfIn")
	    call SetNMFVSLHA(t,
     &        MSS2In_MSS2(1,1,n1),
     &        MSS2In_MSS2(1,1,n2), 
     &        TfIn_Tf(1,1,t))
	    return
	  endif
#ifndef SLHA_STRICT
	  if( SLHADiagValid(MSS2_MSS2(1,1,n1)) +
     &        SLHADiagValid(MSS2_MSS2(1,1,n2)) +
     &        SLHADiagValid(Tf_Tf(1,1,t)) .ne. 0 ) then
	    Origin("NMFV from MSS2 and Tf")
	    call SetNMFVSLHA(t,
     &        MSS2_MSS2(1,1,n1),
     &        MSS2_MSS2(1,1,n2),
     &        Tf_Tf(1,1,t))
	    return
	  endif
#endif
	endif

	if( SLHADiagValid(MSS2In_MSS2(1,1,n1)) .ne. 0 ) then
	  Origin("MSL2 from MSS2In_MSS2")
	  M1SL2_ = DBLE(MSS2In_MSS2(1,1,n1))
	  M2SL2_ = DBLE(MSS2In_MSS2(2,2,n1))
	  M3SL2_ = DBLE(MSS2In_MSS2(3,3,n1))
	else if( SLHAValid(ExtPar_MSS(1,n1), 3) .ne. 0 ) then
	  Origin("MSL2 from ExtPar_MSS")
	  M1SL2_ = DBLE(ExtPar_MSS(1,n1))**2
	  M2SL2_ = DBLE(ExtPar_MSS(2,n1))**2
	  M3SL2_ = DBLE(ExtPar_MSS(3,n1))**2
	  Q_ = DBLE(ExtPar_Q)
#ifndef SLHA_STRICT
	else if( SLHADiagValid(MSS2_MSS2(1,1,n1)) .ne. 0 ) then
	  Origin("MSL2 from MSS2_MSS2")
	  M1SL2_ = DBLE(MSS2_MSS2(1,1,n1))
	  M2SL2_ = DBLE(MSS2_MSS2(2,2,n1))
	  M3SL2_ = DBLE(MSS2_MSS2(3,3,n1))
	  Q_ = DBLE(MSS2_Q(n1))
	else if( SLHAValid(MSoft_MSS(1,n1), 3) .ne. 0 ) then
	  Origin("MSL2 from MSS2_MSS2")
	  M1SL2_ = DBLE(MSoft_MSS(1,n1))**2
	  M2SL2_ = DBLE(MSoft_MSS(2,n1))**2
	  M3SL2_ = DBLE(MSoft_MSS(3,n1))**2
	  Q_ = DBLE(MSoft_Q)
#endif
	else
	  Missing("MSL2")
	endif

	if( SLHADiagValid(MSS2In_MSS2(1,1,n2)) .ne. 0 ) then
	  Origin("MSL2 from MSS2In_MSS2")
	  M1SR2_ = DBLE(MSS2In_MSS2(1,1,n2))
	  M2SR2_ = DBLE(MSS2In_MSS2(2,2,n2))
	  M3SR2_ = DBLE(MSS2In_MSS2(3,3,n2))
	else if( SLHAValid(ExtPar_MSS(1,n2), 3) .ne. 0 ) then
	  Origin("MSL2 from ExtPar_MSS")
	  M1SR2_ = DBLE(ExtPar_MSS(1,n2))**2
	  M2SR2_ = DBLE(ExtPar_MSS(2,n2))**2
	  M3SR2_ = DBLE(ExtPar_MSS(3,n2))**2
#ifndef SLHA_STRICT
	else if( SLHADiagValid(MSS2_MSS2(1,1,n2)) .ne. 0 ) then
	  Origin("MSL2 from MSS2_MSS2")
	  M1SR2_ = DBLE(MSS2_MSS2(1,1,n2))
	  M2SR2_ = DBLE(MSS2_MSS2(2,2,n2))
	  M3SR2_ = DBLE(MSS2_MSS2(3,3,n2))
	  Q_ = DBLE(MSS2_Q(n2))
	else if( SLHAValid(MSoft_MSS(1,n2), 3) .ne. 0 ) then
	  Origin("MSL2 from MSS2_MSS2")
	  M1SR2_ = DBLE(MSoft_MSS(1,n2))**2
	  M2SR2_ = DBLE(MSoft_MSS(2,n2))**2
	  M3SR2_ = DBLE(MSoft_MSS(3,n2))**2
	  Q_ = DBLE(MSoft_Q)
#endif
	else
	  Missing("MSR2")
	endif

	if( SLHADiagValid(TfIn_Tf(1,1,t)) .ne. 0 ) then
	  Origin("Af from Tf_TfIn")
	  x = vev/sqrt2*CSB(t)
	  Af1_ = x/Mf(t,1)*TfIn_Tf(1,1,t)
	  Af2_ = x/Mf(t,2)*TfIn_Tf(2,2,t)
	  Af3_ = x/Mf(t,3)*TfIn_Tf(3,3,t)
	else if( SLHAValid(ExtPar_Af(1), 3) .ne. 0 ) then
	  Origin("Af from ExtPar_Af")
	  Af1_ = ExtPar_Af(t)
	  Af2_ = ExtPar_Af(t)
	  Af3_ = ExtPar_Af(t)
	  Q_ = ExtPar_Q
#ifndef SLHA_STRICT
	else if( SLHADiagValid(Tf_Tf(1,1,t)) .ne. 0 ) then
	  Origin("Af from Tf_Tf")
	  x = vev/sqrt2*CSB(t)
	  Af1_ = x/Mf(t,1)*Tf_Tf(1,1,t)
	  Af2_ = x/Mf(t,2)*Tf_Tf(2,2,t)
	  Af3_ = x/Mf(t,3)*Tf_Tf(3,3,t)
	  Q_ = Tf_Q(t)
	else if( Af_Af(3,3,t) .ne. invalid ) then
	  Origin("Af from Af_Af")
	  Af1_ = Af_Af(1,1,t)
	  Af2_ = Af_Af(2,2,t)
	  Af3_ = Af_Af(3,3,t)
	  Q_ = Af_Q(t)
#endif
	else
	  Missing("Af")
	endif

	call SfermionMasses(t,
     &    M1SL2_, M2SL2_, M3SL2_,
     &    M1SR2_, M2SR2_, M3SR2_,
     &    Af1_, Af2_, Af3_, Q_)
	end


************************************************************************

	double precision function SLHADiagValid(mat)
	implicit none
	double complex mat(3,3)

	SLHADiagValid = abs(
     &    (mat(1,1) - invalid)*
     &    (mat(2,2) - invalid)*
     &    (mat(3,3) - invalid) )
	end

