* SetSMPara.F
* set the SM inputs for FeynHiggs
* this file is part of FeynHiggs
* last modified 10 Mar 10 th

#include "externals.h"
#include "debug.h"


	subroutine FHSetSMPara(error,
     &    invAlfaMZ_, AlfasMZ_, GF_,
     &    MS_, MC_, MB_, MW_, MZ_,
     &    CKMlambda_, CKMA_, CKMrho_, CKMeta_)
	implicit none
	integer error
	double precision invAlfaMZ_, AlfasMZ_, GF_
	double precision MS_, MC_, MB_, MW_, MZ_
	double precision CKMlambda_, CKMA_, CKMrho_, CKMeta_

#include "FH.h"
#include "FHRecord.h"
#define __SUBROUTINE__ "FHSetSMPara"

	external recordnamesini

	double precision sin12, sin23, sin13
	double precision cos12, cos23, cos13
	double complex sin13c

	double precision GFdefault, invAlfaMZdefault, AlfasMZdefault
	double precision MZdefault, MWdefault
	double precision MEdefault, MMdefault, MLdefault
	double precision MUdefault, MCdefault
	double precision MDdefault, MSdefault, MBdefault
	double precision sin12default, sin23default
	double precision sin13default, delta13default

	parameter (GFdefault = 1.16639D-5)
	parameter (invAlfaMZdefault = 128.946D0)
	parameter (AlfasMZdefault = .118D0)
	parameter (MZdefault = 91.1875D0)
	parameter (MWdefault = 80.392D0)

	parameter (MEdefault = .510998902D-3)
	parameter (MMdefault = 105.658357D-3)
	parameter (MLdefault = 1777.03D-3)
	parameter (MUdefault = 3D-3)
	parameter (MCdefault = 1.286D0)
	parameter (MDdefault = 6D-3)
	parameter (MSdefault = 95D-3)
	parameter (MBdefault = 4.2D0)

	parameter (sin12default = .2243D0)
	parameter (sin23default = .0413D0)
	parameter (sin13default = .0037D0)
	parameter (delta13default = 60*degree)

	sm_valid = 0
	para_valid = 0

	error = 0

	if( debuglevel .ge. 2 ) then
	  DPARA "============================"	ENDL
	  DPARA "SMPara input parameters:"	ENDL
	  DPARA "----------------------------"	ENDL
	  DTAGm(iinvAlfaMZ, invAlfaMZ_)
	  DTAGm(iAlfasMZ, AlfasMZ_)
	  DTAGm(iGF, GF_)
	  DTAGm(iMS, MS_)
	  DTAGm(iMC, MC_)
	  DTAGm(iMB, MB_)
	  DTAGm(iMW, MW_)
	  DTAGm(iMZ, MZ_)
	  DTAGm(iCKMlambda, CKMlambda_)
	  DTAGm(iCKMA, CKMA_)
	  DTAGm(iCKMrho, CKMrho_)
	  DTAGm(iCKMeta, CKMeta_)
	endif

	GF = GF_
	if( GF .lt. 0 ) GF = GFdefault

	MZ = MZ_
	if( MZ .lt. 0 ) MZ = MZdefault
	MZ2 = MZ**2

c FIXME: should really compute MW from MZ & Alfa here
	MW = MW_
	if( MW .lt. 0 ) MW = MWdefault
	MW2 = MW**2

	CW = MW/MZ
	CW2 = CW**2
	SW2 = 1 - CW2
	SW = sqrt(SW2)

	ME = MEdefault
	ME2 = ME**2

	MM = MMdefault
	MM2 = MM**2

	ML = MLdefault
	ML2 = ML**2

	MU = MUdefault
	MU2 = MU**2

	MC = MC_
	if( MC .lt. 0 ) MC = MCdefault
	MC2 = MC**2

	MD = MDdefault
	MD2 = MD**2

	MS = MS_
	if( MS .lt. 0 ) MS = MSdefault
	MS2 = MS**2

	MB = MB_
	if( MB .lt. 0 ) MB = MBdefault
	MB2 = MB**2

	EL0 = sqrt(4*pi*Alfa0)

	AlfaGF = sqrt2/pi*GF*MW2*SW2
	ELGF = sqrt(4*pi*AlfaGF)

	if( invAlfaMZ_ .le. 0 ) then
	  AlfaMZ = -1
	else
	  AlfaMZ = 1/invAlfaMZ_
	endif

	AlfasMZ = AlfasMZ_
	if( AlfasMZ .lt. 0 ) AlfasMZ = AlfasMZdefault

	CKMlambda = CKMlambda_
	if( CKMlambda .lt. 0 ) CKMlambda = sin12default

	sin12 = CKMlambda
	cos12 = sqrt((1 - sin12)*(1 + sin12))

	CKMA = CKMA_
	if( CKMA .lt. 0 ) then
	  CKMA = 0
	  if( sin12 .ne. 0 ) CKMA = sin23default/sin12**2
	endif

	sin23 = sin12**2*CKMA
	cos23 = sqrt((1 - sin23)*(1 + sin23))

	CKMrhoeta = DCMPLX(CKMrho_, CKMeta_)
	if( DBLE(CKMrhoeta) .lt. 0 .or. DIMAG(CKMrhoeta) .lt. 0 ) then
	  CKMrhoeta = 0
	  if( sin23 .ne. 0 )
     &      CKMrhoeta = sin13default*exp(cI*delta13default)/
     &        (sin23*sin12)
	endif

	sin13c = sin12*sin23*CKMrhoeta
	sin13 = abs(sin13c)
	cos13 = sqrt((1 - sin13)*(1 + sin13))

	CKMin(1,1) = cos12*cos13
	CKMin(2,1) = -sin12*cos23 - cos12*sin23*sin13c
	CKMin(3,1) = sin12*sin23 - cos12*cos23*sin13c
	CKMin(1,2) = sin12*cos13
	CKMin(2,2) = cos12*cos23 - sin12*sin23*sin13c
	CKMin(3,2) = -cos12*sin23 - sin12*cos23*sin13c
	CKMin(1,3) = DCONJG(sin13c)
	CKMin(2,3) = sin23*cos13
	CKMin(3,3) = cos23*cos13

	sm_valid = valid

	if( debuglevel .lt. 1 ) return

	DPARA "============================="	ENDL
	DPARA "parameters set by FHSetSMPara"	ENDL
	DPARA "-----------------------------"	ENDL
	DPARA "ME =", ME			ENDL
	DPARA "MM =", MM			ENDL
	DPARA "ML =", ML			ENDL
	DPARA "MU =", MU			ENDL
	DPARA "MC =", MC			ENDL
	DPARA "MD =", MD			ENDL
	DPARA "MS =", MS			ENDL
	DPARA "MB =", MB			ENDL
	DPARA "MW =", MW			ENDL
	DPARA "MZ =", MZ			ENDL
	DPARA "CKM(1,1) =", CKMin(1,1)		ENDL
	DPARA "CKM(1,2) =", CKMin(1,2)		ENDL
	DPARA "CKM(1,3) =", CKMin(1,3)		ENDL
	DPARA "CKM(2,1) =", CKMin(2,1)		ENDL
	DPARA "CKM(2,2) =", CKMin(2,2)		ENDL
	DPARA "CKM(2,3) =", CKMin(2,3)		ENDL
	DPARA "CKM(3,1) =", CKMin(3,1)		ENDL
	DPARA "CKM(3,2) =", CKMin(3,2)		ENDL
	DPARA "CKM(3,3) =", CKMin(3,3)		ENDL
	end

