* Sfermions.F
* initialization routines for the sfermions
* this file is part of FeynHiggs
* last modified 27 Aug 10 th

* Note:
* Sf(*,4) are the down-type values based on MBy = MB(MT)/|1 + Delta b|
*   used everywhere except in the following
* Sf(*,5) are the down-type values used in the 1L corrections
*   (and must `fit' to the 2L part)
* Sf(*,6) are the up-type values used in the decays
* Sf(*,7) are the down-type values used in the decays
* Sf(*,8) are the down-type values used in hgg and hgaga

#include "externals.h"
#include "debug.h"

#define MX2_LL(i,j) MX2(i,j)
#define MX2_LR(i,j) MX2(i,j+3)
#define MX2_RR(i,j) MX2(i+3,j+3)


	subroutine Sfermions(error)
	implicit none
	integer error

#include "FH.h"
#define __SUBROUTINE__ "Sfermions"

	integer i, fail, pass, nf, g, g2
	double precision as, Q
	double precision Mdin(3), Mdbar(3)
	double complex db, Mcorr(3,3), se(3,3)

	double precision AlphaS
	external AlphaS

	error = 0

	CheckPara()

	if( error .ne. 0 ) return

	do i = 1, 3*3
	  CKM_flat(i) = CKMin_flat(i)
	enddo

* Sneutrino masses:
	do g = 1, 3
	  MSf2(1,1,g) = DBLE(MSS2(1,2,g,g)) + DSf(1,1)
	  if( MSf2(1,1,g) .lt. 0 ) then
	    Error("negative sneutrino mass squares")
	    MSf2(1,1,g) = 0
	  endif
	  MSf(1,1,g) = sqrt(MSf2(1,1,g))
	  MSf2(2,1,g) = 1D123
	  MSf(2,1,g) = 1D123
	  USf(1,1,1,g) = 1
	  USf(2,2,1,g) = 1
	  USf(2,1,1,g) = 0
	  USf(1,2,1,g) = 0
	enddo

	if( debuglevel .ge. 1 ) then
	  DPARA "----------------------------"	ENDL
	  DPARA "MSf(1,1) =", MSf(1,1,1)	ENDL
	  DPARA "MSf(1,2) =", MSf(1,1,2)	ENDL
	  DPARA "MSf(1,3) =", MSf(1,1,3)	ENDL
	endif

	fail = 0
	call SfUpdate(fail, 2, 2)
	if( fail .ne. 0 ) Error("negative stau mass squares")

	fail = 0
	call SfUpdate(fail, 3, 3)
	if( fail .ne. 0 ) Error("negative stop mass squares")

	if( botResum .eq. 0 ) goto 999

	fail = 0
	call GauginoMasses_largeTB

	Mdin(1) = Mfy(4,1)
	Mdin(2) = Mfy(4,2)
* MBrun = MB(MT)
	Mfy(4,3) = MBrun
	Mdin(3) = Mfy(4,3)

	call SfUpdate_largeTB(fail)
	Q = MSdL(1,3)*MSdL(2,3)
	as = AlphaS(sqrt(Q), nf)

	do g = 1, 3
	  call MDcorr(se, g, g, 0D0, as, Q)
	  Deltaf(4,g) = se(1,1)/Mfy(4,g)
	  Mfy(4,g) = Mdin(g)/abs(1 + Deltaf(4,g))*
     &      (1 - .5D0*(se(2,1) + se(3,1)))
	enddo

	do pass = 1, 20
	  db = Deltaf(4,3)
	  if( debuglevel .ge. 2 ) then
	    DPARA "============================"	ENDL
	    DPARA "pass =", pass			ENDL
	    DPARA "Deltaf(4,3) =", Deltaf(4,3)		ENDL
	    DPARA "Mfy(4,3) =", Mfy(4,3)		ENDL
	  endif

	  call SfUpdate_largeTB(fail)

	  do g2 = 1, 3
	    do g = 1, 3
	      call MDcorr(se(1,g), g, g2, 0D0, as, Q)
	      Mcorr(g,g2) = se(1,g) +
     &          Mfy(4,g)*se(2,g) + Mfy(4,g2)*se(3,g)
	    enddo
	    Deltaf(4,g2) = se(1,g2)/Mfy(4,g2)
	    Mfy(4,g2) = Mdin(g2)/abs(1 + Deltaf(4,g2))*
     &        (1 - .5D0*(se(2,g2) + se(3,g2)))
	    Mcorr(g2,g2) = Mcorr(g2,g2) + Mfy(4,g2)
	  enddo

	  call SVD(3, 3, Mcorr,3, Mdbar,
     &      DR(4,1,1),3, DL(4,1,1),3, 0)

	  if( debuglevel .ge. 3 ) then
	    DPARA "MDc1 =", Mdbar(1), Mdin(1) ENDL
	    DPARA "MDc2 =", Mdbar(2), Mdin(2) ENDL
	    DPARA "MDc3 =", Mdbar(3), Mdin(3) ENDL
	  endif

	  do g2 = 1, 3
	    do g = 1, 3
	      CKM(g,g2) = CKMin(g,1)*DL(4,1,g2) +
     &                    CKMin(g,2)*DL(4,2,g2) + 
     &                    CKMin(g,3)*DL(4,3,g2)
	    enddo
	    DL(4,g2,g2) = DL(4,g2,g2) - 1
	    DR(4,g2,g2) = DR(4,g2,g2) - 1
	  enddo

	  if( abs(Deltaf(4,3) - db) .lt. 1D-6 ) goto 99
	enddo

	Warning("resummation of MB did not converge")

99	continue
	if( fail .ne. 0 )
     &    Error("negative sbottom mass squares in Delta b")

999	fail = 0
	call SfUpdate(fail, 4, 4)

	do i = 1, NSf
	  Sf(i,5) = Sf(i,4)
	  Sf(i,8) = Sf(i,4)
	enddo
c	if( looplevel .lt. 2 ) call SfUpdate(fail, 5, 4)

	Mfy(8,1) = Mf(4,1)/abs(1 + Deltaf(4,1))
	Mfy(8,2) = Mf(4,2)/abs(1 + Deltaf(4,2))
	Mfy(8,3) = Mf(4,3)/abs(1 + Deltaf(4,3))
	call SfUpdate(fail, 8, 4)

	if( fail .ne. 0 ) Error("negative sbottom mass squares")

	if( error .eq. 0 ) sf_valid = valid
	end

************************************************************************

	subroutine GauginoMasses_largeTB
	implicit none

#include "FH.h"

	double complex X(2,2), Y(4,4)

* chargino masses in large-TB (CB -> 0) limit
        X(1,1) = M_2
        X(1,2) = sqrt2*MW*SB
        X(2,1) = 0
        X(2,2) = MUE
        call SVD(2, 2, X,2, MChaL, UChaL,2, VChaL,2, 0)

* neutralino masses in large-TB (CB -> 0) limit
	Y(1,1) = M_1
	Y(2,2) = M_2
	Y(3,3) = 0
	Y(4,4) = 0
	Y(1,2) = 0
	Y(1,3) = 0
	Y(1,4) = MZ*SW*SB
	Y(2,3) = 0
	Y(2,4) = -MZ*CW*SB
	Y(3,4) = -MUE
	call TakagiFactor(4, Y,4, MNeuL, ZNeuL,4, 0)
	end

************************************************************************

	subroutine SfUpdate_largeTB(fail)
	implicit none
	integer fail

#include "FH.h"

	call EigenSf(fail, 1,
     &    MSdL(1,1), MSdL2(1,1), USdL(1,1,1),
     &    DSS2(1,4,1), Mfy(4,1), MUETB(4), 0)
	call EigenSf(fail, 1,
     &    MSdL(1,2), MSdL2(1,2), USdL(1,1,2),
     &    DSS2(1,4,2), Mfy(4,2), MUETB(4), 0)
	call EigenSf(fail, 1,
     &    MSdL(1,3), MSdL2(1,3), USdL(1,1,3),
     &    DSS2(1,4,3), Mfy(4,3), MUETB(4), 0)
	end

************************************************************************

	subroutine SfUpdate(fail, ty, t)
	implicit none
	integer fail, ty, t

#include "FH.h"

	integer g

	fail = 0

	Mfy2(ty,1) = Mfy(ty,1)**2
	Mfy2(ty,2) = Mfy(ty,2)**2
	Mfy2(ty,3) = Mfy(ty,3)**2

	Kf(ty,1,1) = Mfy(ty,1)*Af(t,1)
	Kf(ty,2,2) = Mfy(ty,2)*Af(t,2)
	Kf(ty,3,3) = Mfy(ty,3)*Af(t,3)

	if( btest(nmfv, t) ) then
	  call SfUpdateNMFV(fail, ty, t)
	else
	  call SfUpdateMFV(fail, ty, t)
	endif

	do g = 1, 3
	  USf2(1,1,ty,g) = Sq(USf(1,1,ty,g))
	  USf2(1,2,ty,g) = Sq(USf(1,2,ty,g))
	  USf2(2,1,ty,g) = Sq(USf(2,1,ty,g))
	  USf2(2,2,ty,g) = Sq(USf(2,2,ty,g))
	enddo
	end

************************************************************************

	subroutine SfUpdateMFV(fail, ty, t)
	implicit none
	integer fail, ty, t

#include "FH.h"

	call EigenSf(fail, 3,
     &    MSf(1,ty,1), MSf2(1,ty,1), USf(1,1,ty,1),
     &    DSS2(1,ty,1), Mfy(ty,1), Xf(t,1), 1)
	call EigenSf(fail, 3,
     &    MSf(1,ty,2), MSf2(1,ty,2), USf(1,1,ty,2),
     &    DSS2(1,ty,2), Mfy(ty,2), Xf(t,2), 1)
	call EigenSf(fail, 3,
     &    MSf(1,ty,3), MSf2(1,ty,3), USf(1,1,ty,3),
     &    DSS2(1,ty,3), Mfy(ty,3), Xf(t,3), 1)

	if( debuglevel .lt. 1 ) return

	DPARA "----------------------------" ENDL
	DPARA "MSf("//Digit(ty)//",1) =", MSf(1,ty,1), MSf(2,ty,1) ENDL
	DPARA "MSf("//Digit(ty)//",2) =", MSf(1,ty,2), MSf(2,ty,2) ENDL
	DPARA "MSf("//Digit(ty)//",3) =", MSf(1,ty,3), MSf(2,ty,3) ENDL

	if( debuglevel .lt. 2 ) return

	DPARA "USf(1,1,"//Digit(ty)//",3) =", USf(1,1,ty,3) ENDL
	DPARA "USf(1,2,"//Digit(ty)//",3) =", USf(1,2,ty,3) ENDL
	DPARA "USf(2,1,"//Digit(ty)//",3) =", USf(2,1,ty,3) ENDL
	DPARA "USf(2,2,"//Digit(ty)//",3) =", USf(2,2,ty,3) ENDL
	end

************************************************************************

	subroutine SfUpdateNMFV(fail, ty, t)
	implicit none
	integer fail, ty, t

#include "FH.h"

	integer g, g2, as
	double complex MX2(6,6), m1, m2, m3

	MX2_LL(1,1) = MSS2(1,ty,1,1) + DSf(1,t) + Mfy2(ty,1)
	MX2_LL(2,2) = MSS2(1,ty,2,2) + DSf(1,t) + Mfy2(ty,2)
	MX2_LL(3,3) = MSS2(1,ty,3,3) + DSf(1,t) + Mfy2(ty,3)
	MX2_LL(1,2) = MSS2(1,ty,1,2)
	MX2_LL(2,3) = MSS2(1,ty,2,3)
	MX2_LL(1,3) = MSS2(1,ty,1,3)

	MX2_RR(1,1) = MSS2(2,ty,1,1) + DSf(2,t) + Mfy2(ty,1)
	MX2_RR(2,2) = MSS2(2,ty,2,2) + DSf(2,t) + Mfy2(ty,2)
	MX2_RR(3,3) = MSS2(2,ty,3,3) + DSf(2,t) + Mfy2(ty,3)
	MX2_RR(1,2) = MSS2(2,ty,1,2)
	MX2_RR(2,3) = MSS2(2,ty,2,3)
	MX2_RR(1,3) = MSS2(2,ty,1,3)

	MX2_LR(1,1) = Kf(ty,1,1) + Mfy(ty,1)*MUETB(t)
	MX2_LR(2,2) = Kf(ty,2,2) + Mfy(ty,2)*MUETB(t)
	MX2_LR(3,3) = Kf(ty,3,3) + Mfy(ty,3)*MUETB(t)
	MX2_LR(1,2) = Kf(ty,1,2)
	MX2_LR(2,3) = Kf(ty,2,3)
	MX2_LR(1,3) = Kf(ty,1,3)
	MX2_LR(2,1) = Kf(ty,2,1)
	MX2_LR(3,2) = Kf(ty,3,2)
	MX2_LR(3,1) = Kf(ty,3,1)

	if( t .eq. 3 ) then
	  do g2 = 1, 3
	    m1 = MSS2(1,ty,1,1)*CKMinC(g2,1) +
     &           MSS2(1,ty,1,2)*CKMinC(g2,2) +
     &           MSS2(1,ty,1,3)*CKMinC(g2,3)
	    m2 = DCONJG(MSS2(1,ty,1,2))*CKMinC(g2,1) +
     &           MSS2(1,ty,2,2)*CKMinC(g2,2) +
     &           MSS2(1,ty,2,3)*CKMinC(g2,3)
	    m3 = DCONJG(MSS2(1,ty,1,3))*CKMinC(g2,1) +
     &           DCONJG(MSS2(1,ty,2,3))*CKMinC(g2,2) +
     &           MSS2(1,ty,3,3)*CKMinC(g2,3)
	    do g = 1, g2
	      MX2_LL(g,g2) = MX2_LL(g,g2) +
     &          (CKMin(g,1)*m1 + CKMin(g,2)*m2 + CKMin(g,3)*m3 -
     &            MSS2(1,ty,g,g2))
	    enddo
	  enddo
	endif

	if( debuglevel .ge. 3 ) then
	  DPARA "============================"	ENDL
	  DPARA "MX2_LL(1,1) =", MX2_LL(1,1)	ENDL
	  DPARA "      (2,2) =", MX2_LL(2,2)	ENDL
	  DPARA "______(3,3) =", MX2_LL(3,3)	ENDL
	  DPARA "      (1,2) =", MX2_LL(1,2)	ENDL
	  DPARA "      (1,3) =", MX2_LL(1,3)	ENDL
	  DPARA "      (2,3) =", MX2_LL(2,3)	ENDL
	  DPARA "----------------------------"	ENDL
	  DPARA "MX2_LR(1,1) =", MX2_LR(1,1)	ENDL
	  DPARA "      (2,2) =", MX2_LR(2,2)	ENDL
	  DPARA "______(3,3) =", MX2_LR(3,3)	ENDL
	  DPARA "      (1,2) =", MX2_LR(1,2)	ENDL
	  DPARA "      (1,3) =", MX2_LR(1,3)	ENDL
	  DPARA "      (2,3) =", MX2_LR(2,3)	ENDL
	  DPARA "----------------------------"	ENDL
	  DPARA "MX2_RL(1,2) =", DCONJG(MX2_LR(2,1)) ENDL
	  DPARA "      (1,3) =", DCONJG(MX2_LR(3,1)) ENDL
	  DPARA "      (2,3) =", DCONJG(MX2_LR(3,2)) ENDL
	  DPARA "----------------------------"	ENDL
	  DPARA "MX2_RR(1,1) =", MX2_RR(1,1)	ENDL
	  DPARA "      (2,2) =", MX2_RR(2,2)	ENDL
	  DPARA "______(3,3) =", MX2_RR(3,3)	ENDL
	  DPARA "      (1,2) =", MX2_RR(1,2)	ENDL
	  DPARA "      (1,3) =", MX2_RR(1,3)	ENDL
	  DPARA "      (2,3) =", MX2_RR(2,3)	ENDL
        endif

	call HEigensystem(6, MX2,6, MASf2(1,ty), UASf(1,1,ty),6, 1)

	if( MASf2(1,ty) .lt. 0 ) then
	  fail = 1
	  return
	endif

	do as = 1, 6
	  MASf(as,ty) = sqrt(MASf2(as,ty))
	enddo

	if( debuglevel .lt. 1 ) return

	DPARA "----------------------------"	ENDL
	DPARA "MASf("//Digit(ty)//") =", MASf(1,ty), MASf(2,ty) ENDL
	DPARA "         ", MASf(3,ty), MASf(4,ty) ENDL
	DPARA "         ", MASf(5,ty), MASf(6,ty) ENDL

	if( debuglevel .lt. 2 ) return

	DPARA "UASf(1,1,"//Digit(ty)//") =", UASf(1,1,ty) ENDL
	DPARA "    (1,2)   =", UASf(1,2,ty)	ENDL
	DPARA "    (1,3)   =", UASf(1,3,ty)	ENDL
	DPARA "    (1,4)   =", UASf(1,4,ty)	ENDL
	DPARA "    (1,5)   =", UASf(1,5,ty)	ENDL
	DPARA "    (1,6)   =", UASf(1,6,ty)	ENDL
	DPARA "    (2,1)   =", UASf(2,1,ty)	ENDL
	DPARA "    (2,2)   =", UASf(2,2,ty)	ENDL
	DPARA "    (2,3)   =", UASf(2,3,ty)	ENDL
	DPARA "    (2,4)   =", UASf(2,4,ty)	ENDL
	DPARA "    (2,5)   =", UASf(2,5,ty)	ENDL
	DPARA "    (2,6)   =", UASf(2,6,ty)	ENDL
	DPARA "    (3,1)   =", UASf(3,1,ty)	ENDL
	DPARA "    (3,2)   =", UASf(3,2,ty)	ENDL
	DPARA "    (3,3)   =", UASf(3,3,ty)	ENDL
	DPARA "    (3,4)   =", UASf(3,4,ty)	ENDL
	DPARA "    (3,5)   =", UASf(3,5,ty)	ENDL
	DPARA "    (3,6)   =", UASf(3,6,ty)	ENDL
	DPARA "    (4,1)   =", UASf(4,1,ty)	ENDL
	DPARA "    (4,2)   =", UASf(4,2,ty)	ENDL
	DPARA "    (4,3)   =", UASf(4,3,ty)	ENDL
	DPARA "    (4,4)   =", UASf(4,4,ty)	ENDL
	DPARA "    (4,5)   =", UASf(4,5,ty)	ENDL
	DPARA "    (4,6)   =", UASf(4,6,ty)	ENDL
	DPARA "    (5,1)   =", UASf(5,1,ty)	ENDL
	DPARA "    (5,2)   =", UASf(5,2,ty)	ENDL
	DPARA "    (5,3)   =", UASf(5,3,ty)	ENDL
	DPARA "    (5,4)   =", UASf(5,4,ty)	ENDL
	DPARA "    (5,5)   =", UASf(5,5,ty)	ENDL
	DPARA "    (5,6)   =", UASf(5,6,ty)	ENDL
	DPARA "    (6,1)   =", UASf(6,1,ty)	ENDL
	DPARA "    (6,2)   =", UASf(6,2,ty)	ENDL
	DPARA "    (6,3)   =", UASf(6,3,ty)	ENDL
	DPARA "    (6,4)   =", UASf(6,4,ty)	ENDL
	DPARA "    (6,5)   =", UASf(6,5,ty)	ENDL
	DPARA "    (6,6)   =", UASf(6,6,ty)	ENDL
	end

************************************************************************

	double precision function Tdmb(x, y, z)
	implicit none
	double precision x, y, z

	double precision n, d

	n = 0
	d = 1

	if( x .ne. y ) then
	  if( x*y .ne. 0 ) n = n + x*y*log(x/y)
	  d = d*(x - y)
	endif
	if( x .ne. z ) then
	  if( x*z .ne. 0 ) n = n + x*z*log(z/x)
	  d = d*(x - z)
	endif
	if( y .ne. z ) then
	  if( y*z .ne. 0 ) n = n + y*z*log(y/z)
	  d = d*(y - z)
	endif

	Tdmb = n/d
	end

