! This file is part of HiggsBounds
!  -KW
!******************************************************************
module S95tables_type2
!******************************************************************	

 implicit none
 
 !table type 2------------------------------
 type table2
  logical :: needs_Mh2_gt_2Mh1
  integer :: id,nx1,nx2,OBid
  character(LEN=20) :: label
  character(LEN=3) :: expt	
  double precision :: Mhmax1,Mhmin1,Mhmax2,Mhmin2,sep1,sep2,deltaMh
  double precision, pointer :: dat(:,:,:) !in dat(a,b,1:2)...obs=1,	pred=2
 end type
						
 !------------------------------------------ 

 contains

 !*********************************************************** 
 subroutine initializetables2(S95_t2)
 !***********************************************************  
 ! fills S95_t2
 !***********************************************************  
  use store_pathname
  implicit none

  !--------------------------------------input
  type(table2) :: S95_t2(:)
  !-----------------------------------internal
  integer :: i,tno,j,x,xbeg,xend  
  character(LEN=2) :: tableno		
  character(len=100),allocatable :: filename(:) 
  double precision :: dummy
  !------------------------------------------- 	
  
  xbeg=lbound(S95_t2,dim=1)
  xend=ubound(S95_t2,dim=1)  
 
  allocate(filename(xbeg:xend))
  x=xbeg-1
   
  tno=14
  do i=1,9
   x=x+1
   tno=tno+1
   if(x.eq.3)tno=tno+1	 
   write(tableno,'(I2)')tno	
   
   S95_t2(x)%id=tno*10
   S95_t2(x)%expt='LEP'	
   S95_t2(x)%label='LEP table '//tableno
   S95_t2(x)%sep1=1.0D0
   S95_t2(x)%sep2=1.0D0
   S95_t2(x)%OBid=x+2
	 
   select case(x)
   case(1,2,7)
    S95_t2(x)%Mhmin1=1.0D0
    S95_t2(x)%Mhmax1=60.0D0
    S95_t2(x)%Mhmin2=2.0D0
    S95_t2(x)%Mhmax2=120.0D0
    S95_t2(x)%needs_Mh2_gt_2Mh1=.True.
   case(3,4,8,9)   
    S95_t2(x)%Mhmin1=1.0D0
    S95_t2(x)%Mhmax1=180.0D0
    S95_t2(x)%Mhmin2=1.0D0
    S95_t2(x)%Mhmax2=180.0D0 
    S95_t2(x)%needs_Mh2_gt_2Mh1=.False.
   case(5,6) 
    S95_t2(x)%Mhmin1=1.0D0
    S95_t2(x)%Mhmax1=90.0D0
    S95_t2(x)%Mhmin2=2.0D0
    S95_t2(x)%Mhmax2=180.0D0 
    S95_t2(x)%needs_Mh2_gt_2Mh1=.True.	   	            
   case default
   write(*,*)'error in initializetables2' 
    stop
   end select
   
   filename(x)='table'//tableno//'full'    
  enddo
  
  ! checks we've filled the whole array  
  if(x.ne.xend)then
   write(*,*)'error in initializetables2 (a)'
   stop
  endif  
  
  ! read in the tables
  do x=xbeg,xend
   
   S95_t2(x)%nx2=int((S95_t2(x)%Mhmax2-S95_t2(x)%Mhmin2))/S95_t2(x)%sep2+1
   S95_t2(x)%nx1=int((S95_t2(x)%Mhmax1-S95_t2(x)%Mhmin1))/S95_t2(x)%sep1+1	 	

   open(10,file=trim(adjustl(pathname))//('Expt_tables/' &
                //S95_t2(x)%expt//'tables/' &
                //S95_t2(x)%expt//'tables2/' &
                //trim(adjustl(filename(x)))//'_pred.txt'))		
   open(11,file=trim(adjustl(pathname))//('Expt_tables/' &
                //S95_t2(x)%expt//'tables/' &
                //S95_t2(x)%expt//'tables2/' &
                //trim(adjustl(filename(x)))//'_obs.txt'))	
	 
   allocate(S95_t2(x)%dat(S95_t2(x)%nx2,S95_t2(x)%nx1,2))	 	 	 	 

   ! fill S95 from file
   ! row 0 and column 0 in LEP file contain higgs masses 
   ! and (0,0) ie top left set to -100
   ! so avoid them	   
   read(10,*)
   read(11,*)
   
   do i=1,S95_t2(x)%nx2  
    read(10,*) dummy, ( S95_t2(x)%dat(i,j,2), j=1,S95_t2(x)%nx1 )
    read(11,*) dummy, ( S95_t2(x)%dat(i,j,1), j=1,S95_t2(x)%nx1 )	  	   	 
   end do
   
   ! set entries .ge. 100 to (-4): they will not be relevent
   where(S95_t2(x)%dat.ge.1.0D2)S95_t2(x)%dat= - 4.0D0	 
   
   close(10)
   close(11)	 
  enddo
  
  deallocate(filename)	
  			
 end subroutine initializetables2
 !*********************************************************** 

 !*********************************************************** 
end module S95tables_type2
!************************************************************
