! Contains the functions which allow the user to access the Standard Model Higgs
! Branching ratios, total decay width and cross sections.
!
! This file is part of HiggsBounds
!  -KW
!*********************************
function SMBR_HWW(Mh)
  use theory_BRfunctions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMBR_HWW

  call testBRSM
  
  SMBR_HWW=BRSM_HWW(Mh)

end function SMBR_HWW
!*********************************
function SMBR_HZZ(Mh)
  use theory_BRfunctions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMBR_HZZ

  call testBRSM
  
  SMBR_HZZ=BRSM_HZZ(Mh)

end function SMBR_HZZ
!*********************************
function SMBR_Hbb(Mh)
  use theory_BRfunctions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMBR_Hbb
 
  call testBRSM
  
  SMBR_Hbb=BRSM_Hbb(Mh)

end function SMBR_Hbb
!*********************************
function SMBR_Htautau(Mh)
  use theory_BRfunctions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMBR_Htautau

  call testBRSM
  
  SMBR_Htautau=BRSM_Htautau(Mh)
 
end function SMBR_Htautau
!*********************************
function SMBR_Hgamgam(Mh)
  use theory_BRfunctions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMBR_Hgamgam

  call testBRSM
  
  SMBR_Hgamgam=BRSM_Hgaga(Mh)

end function SMBR_Hgamgam
!*********************************
function SMBR_Hgg(Mh)
  use theory_BRfunctions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMBR_Hgg

  call testBRSM
  
  SMBR_Hgg=BRSM_Hgg(Mh)
end function SMBR_Hgg
!*********************************
function SMBR_Htoptop(Mh)
  use theory_BRfunctions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMBR_Htoptop

  call testBRSM
  
  SMBR_Htoptop=BRSM_Htoptop(Mh) 
end function SMBR_Htoptop
!*********************************
function SMBR_Hcc(Mh)
  use theory_BRfunctions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMBR_Hcc

  call testBRSM
  
  SMBR_Hcc=BRSM_Hcc(Mh)
 end function SMBR_Hcc
!*********************************
function SMBR_Hss(Mh)
  use theory_BRfunctions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMBR_Hss

  call testBRSM
  
  SMBR_Hss=BRSM_Hss(Mh)
end function SMBR_Hss
!*********************************
function SMBR_Hmumu(Mh)
  use theory_BRfunctions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMBR_Hmumu

  call testBRSM
  
  SMBR_Hmumu=BRSM_Hmumu(Mh)
end function SMBR_Hmumu
!*********************************
function SMBR_HZgam(Mh)
  use theory_BRfunctions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMBR_HZgam
  
  call testBRSM
  
  SMBR_HZgam=BRSM_HZga(Mh)
end function SMBR_HZgam
!*********************************
function SMGamma_h(Mh)
  use theory_BRfunctions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMGamma_h

  call testBRSM
  
  SMGamma_h=BRSM_GammaTot(Mh)

end function SMGamma_h
!*********************************
function SMCS_tev_pp_qq_HW(Mh)
!note: in pb
 use theory_XS_SM_functions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMCS_tev_pp_qq_HW
  
  call testBRSM
  
  SMCS_tev_pp_qq_HW=1.0D-3*XS_qq_HW_SM(Mh)
end function SMCS_tev_pp_qq_HW
!*********************************
function SMCS_tev_pp_qq_HZ(Mh)
!note: in pb
 use theory_XS_SM_functions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMCS_tev_pp_qq_HZ
  
  call testBRSM
  
  SMCS_tev_pp_qq_HZ=1.0D-3*XS_qq_HZ_SM(Mh)
end function SMCS_tev_pp_qq_HZ
!*********************************
function SMCS_tev_pp_gg_H(Mh)
!note: in pb
 use theory_XS_SM_functions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMCS_tev_pp_gg_H
  
  call testBRSM
  
  SMCS_tev_pp_gg_H=1.0D-3*XS_gg_H_SM(Mh)
end function SMCS_tev_pp_gg_H
!*********************************
function SMCS_tev_pp_bb_H(Mh)
!note: in pb
 use theory_XS_SM_functions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMCS_tev_pp_bb_H
  
  call testBRSM
  
  SMCS_tev_pp_bb_H=1.0D-3*XS_bb_H_SM(Mh)
end function SMCS_tev_pp_bb_H
!*********************************
function SMCS_tev_pp_vbf_H(Mh)
!note: in pb
 use theory_XS_SM_functions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMCS_tev_pp_vbf_H
  
  call testBRSM
  
  SMCS_tev_pp_vbf_H=1.0D-3*XS_vbf_SM(Mh)
end function SMCS_tev_pp_vbf_H
!*********************************
function SMCS_tev_pp_bg_Hb(Mh) 
!note: in pb
 use theory_XS_SM_functions
  implicit none
  double precision,intent(in) :: Mh
  double precision :: SMCS_tev_pp_bg_Hb
  
  call testBRSM
  
  SMCS_tev_pp_bg_Hb=1.0D-3*XS_bg_Hb_SM(Mh)
end function SMCS_tev_pp_bg_Hb
!*********************************
subroutine testBRSM
  use theory_BRfunctions
  implicit none

  if(.not.allocated(BRSM))then
   write(*,*)'You can only use this function between'
   write(*,*)'calling the subroutines'
   write(*,*)'initialize_HiggsBounds and'
   write(*,*)'finish_HiggsBounds'
   stop'error (see standard output for more info)'
  endif

end subroutine testBRSM
