! This file is part of HiggsBounds
!  -KW
!******************************************************************
module usefulbits
!******************************************************************	
 implicit none 
 integer :: nH
 logical :: debug
 character(LEN=5) :: whichexpt
 character(LEN=4) :: whichinput 
 character(LEN=7) :: inputmethod  
 character(LEN=3),parameter :: vers='1-0'
 double precision :: TEVrange_min,TEVrange_max,LEPrange_min,LEPrange_max
 integer :: n_additional 		
 character(len=100) :: infile1
 double precision :: TEVt1Mhmax,TEVt1Mhmin,LEPt12Mhmax,LEPt12Mhmin
 !for subroutine version--------------------
 integer :: stat

 !associated with 'channels'----------------
 integer :: ntot		
	
 type listprocesses	
  integer :: tlist,ttype
  integer :: findi,findj	 
 end type
	
 type(listprocesses), allocatable :: pr(:)
 !-------------------------------------------
 
 !associated with 'input'--------------------
 integer ndat

 type lepdataset
  double precision, pointer :: XS_hjZ_ratio(:)
  double precision, pointer :: XS_hjhi_ratio(:,:)  
 end type 

 type tevdataset
  double precision, pointer :: XS_hj_ratio(:) 
  double precision, pointer :: XS_hjZ_ratio(:)
  double precision, pointer :: XS_hjW_ratio(:) 
  double precision, pointer :: XS_hjb_ratio(:)  
  double precision, pointer :: XS_vbf_ratio(:)  
  double precision, pointer :: XS_HZ_SM(:)
  double precision, pointer :: XS_HW_SM(:)
  double precision, pointer :: XS_H_SM(:)
  double precision, pointer :: XS_vbf_SM(:)
  ! Higgs produced in association with b, where b is tagged, comes uncut and with 2 different cuts
  ! see subroutines in theory_XS_SM_functions.f90 for details
  double precision, pointer :: XS_Hb_SM(:),XS_Hb_c1_SM(:),XS_Hb_c2_SM(:)
 end type

 type dataset      
  logical :: gooddataset
  logical, pointer :: inLEPrange(:),inTEVrange(:)
  double precision, pointer :: additional(:)
  double precision, pointer :: Mh(:)   
  double precision, pointer :: BR_hjbb(:),BR_hjtautau(:)   
  double precision, pointer :: BR_hjhihi(:,:)  
  type(lepdataset) :: lep 
  !-------------------------------------------
  !extra input needed if also checking Tevatron limits
  double precision, pointer :: BR_hjWW(:),BR_hjgaga(:) 
  double precision, pointer :: BR_hjZZ(:),BR_hjgg(:)  
  double precision, pointer :: GammaTot(:)  
  type(tevdataset) :: tev 
  
  double precision, pointer :: BR_Hbb_SM(:),BR_Htautau_SM(:)  
  double precision, pointer :: BR_HWW_SM(:),BR_Hgaga_SM(:),BR_Hgg_SM(:)   
  double precision, pointer :: GammaTot_SM(:)  
  !------------------------------------------- 
 end type	
	
 type(dataset), allocatable :: theo(:) 

 type sqcouplratio 
  double precision, pointer :: hjbb(:),hjtautau(:) 
  double precision, pointer :: hjWW(:),hjZZ(:)
  double precision, pointer :: hjgaga(:),hjgg(:)
  double precision, pointer :: hjhiZ_in(:)  
 end type
 
 type(sqcouplratio), allocatable :: g2(:)  
 
 type tevextras  
  !nq_hjWp,nq_hjWm,nq_hj,nq_hjZ are set in allocate_tevextras_parts below 
  double precision, pointer :: qq_hjWp(:,:)
  integer :: nq_hjWp!=2 i.e. (u dbar), (c sbar)  e.g. allocate(tR%qq_hjWp(tR%nq_hjWp,nH)) 
  double precision, pointer :: qq_hjWm(:,:)
  integer :: nq_hjWm!=2 i.e. (ubar d), (cbar s) 
   
  double precision, pointer :: gg_hj(:)
  double precision, pointer :: qq_hj(:,:)
  integer :: nq_hj!=5 i.e.(d dbar), (u ubar), (s sbar), (c cbar), (b bbar) 
     
  double precision, pointer :: gg_hjZ(:)
  double precision, pointer :: qq_hjZ(:,:)
  integer :: nq_hjZ!=5 i.e.(d dbar), (u ubar), (s sbar), (c cbar), (b bbar)
     
  double precision, pointer :: bg_hjb(:)
 end type
 
 type(tevextras), allocatable :: tevR(:)
 !-------------------------------------------  
  
 !associated with 'output'--------------------
 integer rep
 
 type results	      
  integer, pointer :: chan(:)	
  double precision, pointer :: obsratio(:)
  integer, pointer :: allowed95(:)
  integer, pointer :: ncombined(:)  				
 end type	
		
 type(results), allocatable :: res(:)
 !-------------------------------------------
 	
 contains

 subroutine HiggsBounds_info
  implicit none

  write(*,*)
  write(*,*)"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
  write(*,*)"~                                                        ~"
  write(*,*)"~          HiggsBounds "//vers//" (fortran90 version)           ~"
  write(*,*)"~                                                        ~"
  write(*,*)"~     Philip Bechtle, Oliver Brein, Sven Heinemeyer,     ~"
  write(*,*)"~            Georg Weiglein, Karina E. Williams          ~"
  write(*,*)"~                                                        ~"
  write(*,*)"~ arXiv:0811.4169, http://www.ippp.dur.ac.uk/HiggsBounds ~"
  write(*,*)"~                                                        ~"
  write(*,*)"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
  write(*,*)
  write(*,*)"HiggsBounds collects together results from "
  write(*,*)
  write(*,*)"    * the LEP collaborations and LEP Higgs Working Group"
  write(*,*)"    * the CDF and D0 Collaborations"
  write(*,*)"    * the program HDECAY (arXiv:hep-ph/9704448)"
  write(*,*)"    * TeV4LHC Higgs Working Group report"
  write(*,*)"      (see arXiv:hep-ph/0612172 and ref. therein)"
  write(*,*)

 end subroutine HiggsBounds_info

 !**********************************************************	
 function div(a,b,divlimit,div0res)
 !**********************************************************
 ! be careful about using this - not a mathematical limit
  double precision :: div  
  !--------------------------------------input
  double precision :: a,b,divlimit,div0res
  !-----------------------------------internal
  double precision :: small1,small2
  !-------------------------------------------  
  small1 = 1.0D-14 
  small2 = 1.0D-10  
     
  if(abs(b).gt.small1)then
   div=a/b
  elseif(abs(a).lt.small2)then  
   div=divlimit  
   if(div.lt.0)stop 'error type divA (see function div in module usefulbits)'   
  else  
   div=div0res
   if(div.lt.0)stop 'error type divB (see function div in module usefulbits)'    
  endif
    
 end function

 !**********************************************************	
 subroutine allocate_dataset_parts(d,n_addit,nHiggs)
 !**********************************************************
  implicit none
  !--------------------------------------input  
  type(dataset) :: d(:)
  integer :: n_addit,nHiggs
  !-----------------------------------internal
  integer :: x
  !-------------------------------------------      
   
  do x=lbound(d,dim=1),ubound(d,dim=1)
  
   allocate(d(x)%additional(n_addit))   
   
   allocate(d(x)%Mh(nHiggs))  
   allocate(d(x)%lep%XS_hjZ_ratio(nHiggs))
   allocate(d(x)%lep%XS_hjhi_ratio(nHiggs,nHiggs))
   allocate(d(x)%BR_hjbb(nHiggs))	 
   allocate(d(x)%BR_hjtautau(nHiggs))	 
   allocate(d(x)%BR_hjhihi(nHiggs,nHiggs))
   allocate(d(x)%BR_hjWW(nHiggs))
   allocate(d(x)%BR_hjZZ(nHiggs))
   allocate(d(x)%BR_hjgaga(nHiggs))
   allocate(d(x)%BR_hjgg(nHiggs))
   allocate(d(x)%Gammatot(nHiggs))
   allocate(d(x)%tev%XS_hjb_ratio(nHiggs))
   allocate(d(x)%tev%XS_vbf_ratio(nHiggs))
    
   allocate(d(x)%tev%XS_hjZ_ratio( nHiggs))   
   allocate(d(x)%tev%XS_hjW_ratio( nHiggs))  
   allocate(d(x)%tev%XS_hj_ratio(  nHiggs))
    
   allocate(d(x)%inLEPrange(nHiggs))
   allocate(d(x)%inTEVrange(nHiggs))

   d(x)%Mh                =0.0D0
   d(x)%lep%XS_hjZ_ratio  =0.0D0
   d(x)%lep%XS_hjhi_ratio =0.0D0  
   d(x)%BR_hjbb           =0.0D0
   d(x)%BR_hjtautau       =0.0D0
   d(x)%BR_hjhihi         =0.0D0
   d(x)%BR_hjWW           =0.0D0
   d(x)%BR_hjZZ           =0.0D0
   d(x)%BR_hjgaga         =0.0D0
   d(x)%BR_hjgg           =0.0D0
   d(x)%GammaTot          =0.0D0   
   d(x)%tev%XS_hjb_ratio  =0.0D0
   d(x)%tev%XS_vbf_ratio  =0.0D0
   d(x)%additional        =0.0D0

   d(x)%tev%XS_hj_ratio   =0.0D0
   d(x)%tev%XS_hjW_ratio  =0.0D0
   d(x)%tev%XS_hjZ_ratio  =0.0D0

   d(x)%InTevRange=.True.
   d(x)%InLepRange=.True.
  enddo

  select case(whichexpt)
  case('onlyT','LandT','singH')
   do x=lbound(d,dim=1),ubound(d,dim=1)
    allocate(d(x)%tev%XS_HZ_SM(  nHiggs))
    allocate(d(x)%tev%XS_HW_SM(  nHiggs))
    allocate(d(x)%tev%XS_H_SM(   nHiggs))  
    allocate(d(x)%tev%XS_vbf_SM( nHiggs))
     
    allocate(d(x)%tev%XS_Hb_SM(    nHiggs)) 
    allocate(d(x)%tev%XS_Hb_c1_SM( nHiggs)) 
    allocate(d(x)%tev%XS_Hb_c2_SM( nHiggs)) 
   
    allocate(d(x)%BR_Hbb_SM(     nHiggs))   
    allocate(d(x)%BR_Htautau_SM( nHiggs))  
    allocate(d(x)%BR_HWW_SM(     nHiggs))  
    allocate(d(x)%BR_Hgaga_SM(   nHiggs))  
    allocate(d(x)%BR_Hgg_SM(     nHiggs)) 
    allocate(d(x)%GammaTot_SM(   nHiggs))
   enddo
  case('onlyL')
  case default
   stop'error in allocate_dataset_parts'
  end select

 end subroutine allocate_dataset_parts

 !**********************************************************	
 subroutine allocate_sqcouplratio_parts(gsq,nHiggs)
 ! to use this, gsq must be an array
 !********************************************************** 
  implicit none
  !--------------------------------------input  
  type(sqcouplratio) :: gsq(:)
  integer :: nHiggs
  !-----------------------------------internal
  integer :: x
  !-------------------------------------------       
  
  do x=lbound(gsq,dim=1),ubound(gsq,dim=1) 
   allocate(gsq(x)%hjbb(nH)    ,gsq(x)%hjtautau(nH))
   allocate(gsq(x)%hjWW(nH)    ,gsq(x)%hjZZ(nH)    )
   allocate(gsq(x)%hjgaga(nH)  ,gsq(x)%hjgg(nH)    )
   allocate(gsq(x)%hjhiZ_in(nH*(nH-1)/2+nH)                    )  

   gsq(x)%hjbb           =0.0D0
   gsq(x)%hjtautau       =0.0D0
   gsq(x)%hjWW           =0.0D0
   gsq(x)%hjZZ           =0.0D0
   gsq(x)%hjgaga         =0.0D0
   gsq(x)%hjgg           =0.0D0
   gsq(x)%hjhiZ_in       =0.0D0
  enddo 
 
 end subroutine allocate_sqcouplratio_parts

!  !**********************************************************	
!  subroutine deallocate_sqcouplratio_parts(gsq)
!  !********************************************************** 
!   implicit none
!   !--------------------------------------input  
!   type(sqcouplratio) :: gsq(:)
!   !-----------------------------------internal
!   integer :: x
!   !-------------------------------------------       
!   
!   do x=lbound(gsq,dim=1),ubound(gsq,dim=1) 
!    deallocate(gsq(x)%hjbb     )
!    deallocate(gsq(x)%hjtautau )
!    deallocate(gsq(x)%hjWW     )
!    deallocate(gsq(x)%hjZZ     )
!    deallocate(gsq(x)%hjgaga   )
!    deallocate(gsq(x)%hjgg     )
!    deallocate(gsq(x)%hjhiZ_in )  
!   enddo 
!  
!  end subroutine deallocate_sqcouplratio_parts
! 
 !**********************************************************	
 subroutine allocate_tevextras_parts(tR,nHiggs)
 !********************************************************** 
  implicit none
  !--------------------------------------input  
  type(tevextras) :: tR(:)
  integer :: nHiggs
  !-----------------------------------internal
  integer :: x
  !-------------------------------------------       
    
  tR%nq_hjWp=2 ! (u dbar), (c sbar)  e.g
  tR%nq_hjWm=2 ! (ubar d), (cbar s) 
  tR%nq_hj=5   !(d dbar), (u ubar), (s sbar), (c cbar), (b bbar) 
  tR%nq_hjZ=5  !(d dbar), (u ubar), (s sbar), (c cbar), (b bbar)
  
  do x=lbound(tR,dim=1),ubound(tR,dim=1) 
   allocate(tR(x)%qq_hjWp(tR(x)%nq_hjWp,nHiggs))
   allocate(tR(x)%qq_hjWm(tR(x)%nq_hjWm,nHiggs))
   allocate(tR(x)%gg_hj(nHiggs))
   allocate(tR(x)%qq_hj(tR(x)%nq_hj,nHiggs)) 
   allocate(tR(x)%gg_hjZ(nHiggs))
   allocate(tR(x)%qq_hjZ(tR(x)%nq_hjZ,nHiggs)) 
   allocate(tR(x)%bg_hjb(nHiggs)) 

   tR(x)%qq_hjWp   =0.0D0
   tR(x)%qq_hjWm   =0.0D0
   tR(x)%gg_hj     =0.0D0
   tR(x)%qq_hj     =0.0D0
   tR(x)%gg_hjZ    =0.0D0
   tR(x)%qq_hjZ    =0.0D0 
   tR(x)%bg_hjb    =0.0D0 
  enddo 
 
 end subroutine allocate_tevextras_parts
 
 !**********************************************************	
 subroutine deallocate_tevextras_parts(tR)
 !********************************************************** 
  implicit none
  !--------------------------------------input  
  type(tevextras) :: tR(:)
  !-----------------------------------internal
  integer :: x
  !-------------------------------------------       
  
  do x=lbound(tR,dim=1),ubound(tR,dim=1) 
   deallocate(tR(x)%qq_hjWp)
   deallocate(tR(x)%qq_hjWm)
   deallocate(tR(x)%gg_hj)
   deallocate(tR(x)%qq_hj) 
   deallocate(tR(x)%gg_hjZ)
   deallocate(tR(x)%qq_hjZ)   
   deallocate(tR(x)%bg_hjb)  
  enddo 
 
 end subroutine deallocate_tevextras_parts

 !**********************************************************	
 subroutine deallocate_usefulbits
 !**********************************************************
 ! deallocates theo,res (and everything inside)
 ! deallocates c,predratio,fact 
 !************************************************************
  implicit none
  !-----------------------------------internal
  integer x,y
  !-------------------------------------------

  !these are allocated in subroutine do_input
  do x=lbound(theo,dim=1),ubound(theo,dim=1)
   deallocate(theo(x)%additional)      
   deallocate(theo(x)%Mh)
   
   deallocate(theo(x)%lep%XS_hjZ_ratio) 
   deallocate(theo(x)%lep%XS_hjhi_ratio) 
   
   deallocate(theo(x)%BR_hjbb)
   deallocate(theo(x)%BR_hjtautau)
   deallocate(theo(x)%BR_hjhihi)
   deallocate(theo(x)%BR_hjWW)   
   deallocate(theo(x)%BR_hjZZ)   
   deallocate(theo(x)%BR_hjgaga)    
   deallocate(theo(x)%BR_hjgg)  
   deallocate(theo(x)%GammaTot)   
 
   deallocate(theo(x)%tev%XS_hjb_ratio)    
   deallocate(theo(x)%tev%XS_vbf_ratio)       
                 
   deallocate(theo(x)%tev%XS_hjZ_ratio)
   deallocate(theo(x)%tev%XS_hjW_ratio)
   deallocate(theo(x)%tev%XS_hj_ratio)      

   deallocate(theo(x)%inLEPrange) 
   deallocate(theo(x)%inTEVrange)                       
  enddo 
  
  select case(whichexpt)
  case('onlyT','LandT','singH')	
   do x=lbound(theo,dim=1),ubound(theo,dim=1)        
    deallocate(theo(x)%BR_Hbb_SM)       
    deallocate(theo(x)%BR_Htautau_SM)     
    deallocate(theo(x)%BR_HWW_SM)    
    deallocate(theo(x)%BR_Hgaga_SM)    
    deallocate(theo(x)%BR_Hgg_SM) 
    deallocate(theo(x)%GammaTot_SM)
                    
    deallocate(theo(x)%tev%XS_HZ_SM)
    deallocate(theo(x)%tev%XS_HW_SM)
    deallocate(theo(x)%tev%XS_H_SM) 
    deallocate(theo(x)%tev%XS_vbf_SM) 
    
    deallocate(theo(x)%tev%XS_Hb_SM) 
    deallocate(theo(x)%tev%XS_Hb_c1_SM) 
    deallocate(theo(x)%tev%XS_Hb_c2_SM)                    
   enddo 
  case('onlyL')
  case default 
   stop'error in deallocate_usefulbits'
  end select 
  
  deallocate(theo) !allocated in subroutine do_input  

  !allocated in subroutine setup_output       
  do x=lbound(res,dim=1),ubound(res,dim=1)  
   deallocate(res(x)%chan)   
   deallocate(res(x)%obsratio) 
   deallocate(res(x)%allowed95) 
   deallocate(res(x)%ncombined) 
  enddo   
  
  deallocate(res) !allocated in subroutine setup_output  	   
  
!  call deallocate_sqcouplratio_parts(g2)
  do x=lbound(g2,dim=1),ubound(g2,dim=1) 
   deallocate(g2(x)%hjbb)
   deallocate(g2(x)%hjtautau)
   deallocate(g2(x)%hjWW)  
   deallocate(g2(x)%hjZZ)
   deallocate(g2(x)%hjgaga)
   deallocate(g2(x)%hjgg)
   deallocate(g2(x)%hjhiZ_in) 
  enddo 
  deallocate(g2) 
 
  !these are allocated in subroutine do_input 
  call deallocate_tevextras_parts(tevR)          
  deallocate(tevR) !allocated in subroutine do_input      
  
  deallocate(pr)  !allocated in subroutine fill_pr or fill_pr_select
  
 end subroutine deallocate_usefulbits
 !**********************************************************	

end module usefulbits		
!******************************************************************
