      double precision function als(q)

      implicit none

      double precision q, pi, b0, b1, b2, lq, la5, nf

      pi = 3.14159265358979d0
      nf = 5d0
      la5 = 0.22d0

      b0 = (33d0 - 2d0 * nf)/3d0
      b1 = 102d0 - 38d0/3d0 * nf
      b2 = 2857d0/2d0 - 5033d0/18d0 * nf + 325d0/54d0 * nf**2

      lq = dlog(q**2/la5**2)

      als = 4d0 * pi/(b0 * lq) *
     $     (1d0 - b1/b0**2 * dlog(lq)/lq 
     $          + b1**2/b0**4 * dlog(lq)**2/lq**2
     $          - b1**2/b0**4 * dlog(lq)/lq**2)
c     $          + (b2 * b0 - b1**2)/b0**4/lq**2)

      end

c----------------------------------------------------------------
      
      double precision function cfkt(q)

      implicit none

      double precision q, pi, b0, b1, b2, g0, g1, g2, nf, zeta3, als

      pi = 3.14159265358979d0
      zeta3 = 1.2020596d0
      nf = 5d0

      b0 = (33d0 - 2d0 * nf)/3d0
      b1 = 102d0 - 38d0/3d0 * nf
      b2 = 2857d0/2d0 - 5033d0/18d0 * nf + 325d0/54d0 * nf**2

      g0 = -8d0
      g1 = -404d0/3d0 + 40d0/9d0 * nf
      g2 = 2d0/3d0 * (140d0/27d0 * nf**2 
     $                + (160d0 * zeta3 + 2216d0/9d0) * nf -3747d0)

      cfkt = (b0 * als(q)/(2d0 * pi))**(-g0/(2d0 * b0)) *
     $       (1d0 + (b1 * g0 - b0 * g1)/b0**2 * als(q)/(8d0 * pi) )
c     $            + ((b1 * g0 - b0 * g1)**2/(2d0 * b0**4)
c     $               + (g0 * (b2 * b0 - b1**2))/b0**3 
c     $               + g1 * b1/b0**2 - g2/b0) 
c     $        * (als(q)/(8d0 * pi))**2)

      end


c----------------------------------------------------------------
c define mass dependence for H -> W*W, Z*Z
c (from Gunion et al., Higgs Hunters Guide, pg. 28)
c
      
      double precision function fh(x)

      double precision x

      fh = -( 1.0d0 - x**2 ) * ( 23.5d0 * x**2 - 6.5d0 + 1.0d0 / x**2 ) 
     &     - 3.0d0 * ( 1.0d0 - 6.0d0 * x**2 + 4.0d0 * x**4 ) * dlog( x )
     &     + 3.0d0 * ( 1.0d0 - 8.0d0 * x**2 + 20.0d0 * x**4 )
     &             / dsqrt( 4.0d0 * x**2 - 1.0d0 )
     &        * dacos( ( 3.0d0 * x**2 - 1.0d0 ) / ( 2.0d0 * x**3 ) )

      end

c----------------------------------------------------------------
c     interpolation for decay rate h -> WW (SM)

      double precision function hwwapprox(mh)

      double precision mh, y0
      integer x0, x1
      double precision gshwwapp(50:250)
      common /decayrateww/ gshwwapp
      
      x0 = int(mh)
      x1 = x0 + 1
      y0 = mh - x0

      if (mh.lt.50d0) then
         hwwapprox = 0d0
      elseif((mh.ge.50d0).and.(mh.le.250d0)) then
         hwwapprox = gshwwapp(x0) + (gshwwapp(x1)-gshwwapp(x0)) * y0
      else
c         write(*,*) 'h -> WW out of range, taking mh = 250'
         hwwapprox = gshwwapp(250)
      endif

c      write(*,*) 'h -> WW'
c      write(*,*) real(mh), x0, x1, real(y0)
c      write(*,*) real(gshwwapp(x0)), real(gshwwapp(x1)), real(hwwapprox)

      end

      
