c     subroutine for the fast (and approximate) calculation of the 
c     neutral CP-even Higgs-boson masses in the MSSM
c     Two approaches are used:
c     1.) express mlh^2 through a closed formula. Here problems might 
c         arise for small MA
c     2.) the Higgs mass matrix ist determined numerically up to 2-loop,
c         the diagonalisation yields the two Higgs masses

      subroutine mhiggslle(msusytl, msusytr, mtlr, mblr, mt, mtrun, mb, 
     $           ma, tb, gf, as, cf, el, mz, mw, xmue, xdelmlhsq,
     $           mlhlle1, mlhlle2, 
     $           mlhllediag1, mlhllediag2, mhhllediag1, mhhllediag2,
     $           lleselec1, selec)

c     meaning of variables:
c     msusyt[l,r]: SSB terms in the stop mass matrix
c     ms[1,2]: The Susy mass scale: ms1^2 = Msusy^2 + MT^2 (for 1-loop contr.)
c                                   ms2^2 = Msusy^2 + MTrun^2 (2-loop contr.)
c     ms[1,2]2: == ms[1,2]^2
c     mtlr: the off-diagonal entry in the stop mass matrix
c     mt, mb: fermion masses
c     mtrun: running top mass, can be used for 2-loop contribution
c     ma, tb: the usual Higgs sector varialbes M_A and tan(beta)
c     gf: Fermi's constant
c     as: strong coupling constant at the scale mt
c     cf = 4/3
c     el: electromagnetic coupling, derived from gf
c     mz, mw: gauge boson masses
c     mlhlle[1,2]: the mass of the light Higgs boson in approach 1
c     m[l,h]hllediag[1,2]: the Higgs masses in approach 2


      double precision msusy, msusytl, msusytr, mtlr, mblr, mt, mtrun, 
     $                 mtold, mb, ma, theta, delmlhsq, xdelmlhsq,
     $                 gf, as, pi, res, cf, el, mw, mz, mue, pref, al,
     $                 sw, sw2, sw4, cw2, et, eb, xmue,
     $                 ma2, mz2, mw2, mt2, mt4, msfkt,
     $                 msusy2, ms1, ms2, ms12, ms22,
     $                 tb, b, sb, cb, sb2, cb2, c2b
      double precision nc, ng, pt, pb, pf, pg, pgp, p1h, p2h, p2hp
      double precision mlhlle1, mlhlle2, mlhlle1sq, mlhlle2sq, 
     $       mlhllediag1, mlhllediag2, mhhllediag1, mhhllediag2,
     $       mlhllediag1sq, mlhllediag2sq, mhhllediag1sq, mhhllediag2sq
      double precision m11, m12, m221loop, m222loop
      double precision m11botmix, m12botmix, m22botmix, 
     $                 m11top, m12top, m22top, m22log, 
     $                 m11rest, m12rest, m22rest,
     $                 m11botrest, m12botrest, m22botrest
      double precision allle1, allle2
      common /lle2/ allle1, allle2
      integer lleselec1, selec
      include 'LLExpansionP2MTrunvar.f'
      integer count
      count = 0

c      write(*,*) 'LLE parameters:'
c      write(*,*) mt, mtrun
c      write(*,*) mb

      pi = 3.14159265358979d0
      al = el**2/(4d0 * pi)
      cw2 = mw**2/mz**2
      sw2 = 1d0 - mw**2/mz**2
      sw = dsqrt(sw2)
      sw4 = sw2**2
      et = 2d0/3d0
      eb = -1d0/3d0
      b = datan(tb)
      sb = dsin(b)
      sb2 = sb**2
      cb = dcos(b)
      cb2 = cb**2
      c2b = dcos(2d0 * b)
      ma2 = ma**2
      mz2 = mz**2
      mw2 = mw**2
      mt2 = mt**2
      mt4 = mt**4
      delmlhsq = xdelmlhsq
      mue = -xmue
c --> reason for the above change: mue appears only in the HHH formula
c     for the sbottom contributions. There mue has the different sign
      call countprint(count)

      if (msusytl.eq.msusytr) then
         ms1 = dsqrt(msusytl**2 + mt**2)
         ms2 = dsqrt(msusytl**2 + mtrun**2)
      else
         ms1 = msfkt(msusytl, msusytr, mt)
         ms2 = msfkt(msusytl, msusytr, mtrun)
      endif
      ms12 = ms1**2
      ms22 = ms2**2
      msusy2 = ms12 - mt**2
      msusy = dsqrt(msusy2)

      call countprint(count)
      nc = 3d0
      ng = 3d0
      pt = nc * (1d0 - 4d0 * et * sw2 + 8d0 * et**2 * sw4)
      pb = nc * (1d0 + 4d0 * eb * sw2 + 8d0 * eb**2 * sw4)
      pf = nc * (ng - 1d0) * (2d0 - 4d0 * sw2 
     $           + 8d0 * (et**2 + eb**2) * sw4) 
     $           + ng * (2d0 - 4d0 * sw2 + 8d0 * sw4)
      pg = -44d0 + 106d0 * sw2 - 62d0 * sw4
      pgp = 10d0 + 34d0 * sw2 - 26d0 * sw4
      p1h = -9d0 * c2b**4 + (1d0 - 2d0 * sw2 + 2d0 * sw4) * c2b**2
      p2h = -10d0 + 2d0 * sw2 - 2d0 * sw4
      p2hp = 8d0 - 22d0 * sw2 + 10d0 * sw4

      theta = 0d0
      if (ma2.ge.mz2) theta = 1d0


      call countprint(count)
c --> 1. approach: the 'direct' formula for mh2
c     these formulas are taken over directly from LLEShortFormula.m

c      write(*,*) 'lle 1loop'
c      write(*,*) real(ms1), real(ms12), real(ms2), real(ms22),
c     $           real(msusy), real(msusy2)
      mlhlle1sq = ( 
c --> top/stop 1-loop contribution with (incomplete) terms prop MZ^2/Ms^2
c$$$     $    (MA**2 + MZ**2 - Sqrt(MA**4 + MZ**4 - 
c$$$     -        2*MA**2*MZ**2*(1 - 8*SB**2 + 8*SB**4)))/2. + 
c$$$     -   (al*MT**4*((-3*MT**4*MtLR**8)/(55.99999999999999*Ms1**12) + 
c$$$     -        (MtLR**2*(1.5 - (3*MT**2)/(4.*Ms1**2) + 
c$$$     $     (3*MZ**2)/(8.*Ms1**2) - 
c$$$     -             MZ**2/(2.*MT**2)))/Ms1**2 + 
c$$$     -     (MtLR**6*((-3*MT**2)/(40.*Ms1**2) + (3*MT**4)/(10.*Ms1**4) - 
c$$$     -             (3*MT**2*MZ**2)/(35.*Ms1**4)))/Ms1**6 + 
c$$$     -    (MtLR**4*(-0.125 + MT**2/(2.*Ms1**2) - (3*MT**4)/(8.*Ms1**4) - 
c$$$     -             (3*MZ**2)/(20.*Ms1**2) + 
c$$$     $     (3*MT**2*MZ**2)/(16.*Ms1**4)))/Ms1**4
c$$$     -        )*(1 + ((-MA**2 + MZ**2)*(1 - 2*SB**2))/
c$$$     -         Sqrt(MA**4 + MZ**4 - 2*MA**2*MZ**2*
c$$$     $     (1 - 8*SB**2 + 8*SB**4))))/
c$$$     -    (2.*MW**2*Pi*SB**2*sw2) - 
     $    (MA**2 + MZ**2 - Sqrt(MA**4 + MZ**4 - 
     -        2*MA**2*MZ**2*(1 - 8*SB**2 + 8*SB**4)))/2. + 
     -   (al*MT**4*((-3*MT**4*MtLR**8)/(55.99999999999999*Ms1**12) + 
     n    (1d0/4d0 * MZ**2/MT**2 - 11d0/80d0 * MZ**4/MT**4)
     -       + (MtLR**2*(1.5 - (3*MT**2)/(4.*Ms1**2) + 
     $                       0d0 - 
     -             MZ**2/(2.*MT**2)))/Ms1**2 + 
     -     (MtLR**6*((-3*MT**2)/(40.*Ms1**2) + (3*MT**4)/(10.*Ms1**4) - 
     -                                    0d0  ))/Ms1**6 + 
     -    (MtLR**4*(-0.125 + MT**2/(2.*Ms1**2) - (3*MT**4)/(8.*Ms1**4) - 
     -                                0d0 + 
     $                             0d0 ))/Ms1**4
     -        )*(1 + ((-MA**2 + MZ**2)*(1 - 2*SB**2))/
     -         Sqrt(MA**4 + MZ**4 - 2*MA**2*MZ**2*
     $     (1 - 8*SB**2 + 8*SB**4))))/
     -    (2.*MW**2*Pi*SB**2*sw2) - 
     -   (al*MT**4*(1.5 - (3*MZ**2*SB**2)/(4.*MT**2) + 
     -        ((32*MW**4 - 40*MW**2*MZ**2 + 17*MZ**4)*
     $     SB**2)/(72.*MT**4) + 
     -        (4*(MA**2 + MZ**2)*SB**2*(1 - SB**2)*
     -           ((3*MZ**2)/(8.*MT**2) - 
     -             ((32*MW**4 - 40*MW**2*MZ**2 + 17*MZ**4)*SB**2)/
     $     (72.*MT**4))
     -           )/Sqrt(MA**4 + MZ**4 - 2*MA**2*MZ**2*
     $     (1 - 8*SB**2 + 8*SB**4))
     -          + ((-MA**2 + MZ**2)*(1 - 2*SB**2)*
     -           (1.5 - (3*MZ**2*SB**2)/(4.*MT**2) - 
     -             ((32*MW**4 - 40*MW**2*MZ**2 + 17*MZ**4)*SB**2*
     -                (1 - 2*SB**2))/(72.*MT**4)))/
     -     Sqrt(MA**4 + MZ**4 - 2*MA**2*MZ**2*
     $     (1 - 8*SB**2 + 8*SB**4)))*
     -      Log(MT**2/Ms1**2))/(2.*MW**2*Pi*SB**2*sw2) + 
     -   (al*MZ**4*(Log(Msusy**2/MZ**2)*
     -         ((36*MB**4)/MZ**4 - (18*MB**2*Cos(2*b))/MZ**2 + 
     -           (3*(1 - (4*sw2)/3. + (8*sw4)/9.) + 
     -              6*(2 - 4*sw2 + (40*sw4)/9.) + 
     -              3*(2 - 4*sw2 + 8*sw4))*Cos(2*b)**2 - 
     -           2*(18 + 12*sw2 - 16*sw4)*Cos(b)**2*Sin(b)**2 + 
     -           (-54 + 108*sw2 - 64*sw4)*
     $     (Cos(b)**4 + Sin(b)**4)) - 
     -        theta * Log(MA**2/MZ**2)*(-((1 - 2*sw2 + 2*sw4)*
     $     Cos(2*b)**2) + 
     -           9*Cos(2*b)**4 - 
     -           2*(8 - 22*sw2 + 10*sw4)*Cos(b)**2*Sin(b)**2 + 
     -           (-10 + 2*sw2 - 2*sw4)*
     $     (Cos(b)**4 + Sin(b)**4))))/
     -    (23.99999999999999*MW**2*Pi*sw2) + 
     -   (3*al*((2*MB**4*MbLR**2*(1 - MbLR**2/
     $     (11.99999999999999*Msusy**2)))/
     -         Msusy**2 - (MB**2*MZ**2*Cos(2*b)*
     -           (MbLR**2 + (-(Mue**2*Tan(b)**2) + 
     $     (MbLR + Mue*Tan(b))**2)/3.)
     -           )/(2.*Msusy**2)))/(4.*MW**2*Pi*sw2)
     $     )
      call countprint(count)
      if (mlhlle1sq.gt.0d0) then
         mlhlle1 = dsqrt(mlhlle1sq)
      else
         mlhlle1 = 119.9999d0
      endif


      call countprint(count)
c      write(*,*) 'lle 2loop'
      mlhlle2sq = (
     $     mlhlle1sq
     $   + 1d0/sb**2 * delmlhsq
     -   + (al*as*MTrun**4*(1 + ((-MA**2 + MZ**2)*(1 - 2*SB**2))/
     -         Sqrt(MA**4 + MZ**4 - 2*MA**2*MZ**2*
     $     (1 - 8*SB**2 + 8*SB**4)))*
     -      ((6*MtLR)/Ms2 - (3*MtLR**4)/(4.*Ms2**4) + 
     -        (-6 - (3*MtLR**2)/Ms2**2)*Log(Ms2**2/MTrun**2) - 
     -        3*Log(Ms2**2/MTrun**2)**2))/
     $     (2.*MW**2*Pi**2*SB**2*sw2) 
     $     )

      call countprint(count)
      if (mlhlle2sq.gt.0d0) then
         mlhlle2 = dsqrt(mlhlle2sq)
      else
         mlhlle2 = 119.9999d0
      endif

c      write(*,*) 'LLE parameters:'
c      write(*,*) mt, mtrun, sin(b)**2, ms2


c ----------------------------------------------------------------------

c --> 2. approach: the Higgs mass matrix

      call countprint(count)
c      write(*,*) '2. approach'
      pref = el**2 * mz2 / (96d0 * Pi**2 * cw2 * sw2)
      mwmzfak = 32d0 * mw**4 - 40d0 * mw2 * mz2 + 17d0 * mz**4

c      write(*,*) '2. approach'
c --> the top sector at 1-loop
      m11top = -el**2/(288d0 * mw2 * pi**2 * sw2) 
     $         * mwmzfak * cb2 * dlog(mt2/ms12)
      m12top = el**2 * cb/(288d0 * sb * mw2 * pi**2 * sw2) 
     $         * (- 27 * mt2 * mz2 + mwmzfak * sb2) * dlog(mt2/ms12)
c --> m22top with (incomplete) terms prop. MZ^2/Ms^2
c$$$      m22top = el**2/(288d0 * sb2 * mw2 * pi**2 * sw2) *
c$$$     $  ((-108d0 * mt**4 + 54d0 * mt2 * mz2 * sb2 - mwmzfak * sb**4) *
c$$$     $     dlog(mt2/ms12)
c$$$     $     +9d0 * (MtLR/Ms1)**2 * (MT/Ms1)**2 *
c$$$     $ (-6d0 * mt**4 + 3d0 * mt2 * mz2 + 4d0 * ms12 * (3d0 * mt2 - mz2))
c$$$     $     +9d0/10d0 * (MtLR/Ms1)**4 * (MT/Ms1)**4 *
c$$$     $     (-10d0 * ms1**4 + 4 * ms1**2 * (10d0 * mt2 - 3d0 * mz2) +
c$$$     $     15d0 * mt2 * (-2d0 * mt2 + mz2))
c$$$     $     +9d0/35d0 * (MtLR/Ms1)**6 * (MT/Ms1)**6 *
c$$$     $     (-21d0 * ms1**4 + 12d0 * ms1**2 * (7d0 * mt2 - 2d0 * mz2) +
c$$$     $     35d0 * mt2 * (-2d0 * mt2 + mz2))
c$$$     $     -288d0/469762048d0 * (MtLR/Ms1)**8 * (MT/Ms1)**8 *
c$$$     $     (6291456d0 * ms1**4 + ms1**2 * (-25165824d0 * mt2
c$$$     $     + 6992875d0 * mz2) + 11010048d0 * (2d0 * mt**4 - mt2 * mz2)))
      m22top = el**2/(288d0 * sb2 * mw2 * pi**2 * sw2) *
     $  ((-108d0 * mt**4 + 54d0 * mt2 * mz2 * sb2 - mwmzfak * sb**4) *
     $     dlog(mt2/ms12)
     n     + (2d0 * MZ**2/MT**2 - 11d0/10d0 * MZ**4/MT**4) * 9d0 * MT**4
     $     +9d0 * (MtLR/Ms1)**2 * (MT/Ms1)**2 *
     $ (-6d0 * mt**4                   + 4d0 * ms12 * (3d0 * mt2 - mz2))
     $     +9d0/10d0 * (MtLR/Ms1)**4 * (MT/Ms1)**4 *
     $     (-10d0 * ms1**4 + 4 * ms1**2 * (10d0 * mt2            ) +
     $     15d0 * mt2 * (-2d0 * mt2      ))
     $     +9d0/35d0 * (MtLR/Ms1)**6 * (MT/Ms1)**6 *
     $     (-21d0 * ms1**4 + 12d0 * ms1**2 * (7d0 * mt2            ) +
     $     35d0 * mt2 * (-2d0 * mt2      ))
     $     -288d0/469762048d0 * (MtLR/Ms1)**8 * (MT/Ms1)**8 *
     $     (6291456d0 * ms1**4 + ms1**2 * (-25165824d0 * mt2
     $                      ) + 11010048d0 * (2d0 * mt**4            )))

      call countprint(count)
c$$$c --> with mtrun everywhere
c$$$      m11top = -el**2/(288d0 * mw2 * pi**2 * sw2) 
c$$$     $         * mwmzfak * cb2 * dlog(mtrun**2/ms22)
c$$$      m12top = el**2 * cb/(288d0 * sb * mw2 * pi**2 * sw2) 
c$$$     $         * (- 27 * mtrun**2 * mz2 + mwmzfak * sb2) * 
c$$$     $     dlog(mtrun**2/ms22)
c$$$      m22top = el**2/(288d0 * sb2 * mw2 * pi**2 * sw2) *
c$$$     $  ((-108d0 * mtrun**4 + 54d0 * mtrun**2 * mz2 * sb2 - 
c$$$     $     mwmzfak * sb**4) *
c$$$     $     dlog(mtrun**2/ms22)
c$$$     n     + (2d0 * MZ**2/MTRUN**2 - 11d0/10d0 * MZ**4/MTRUN**4) 
c$$$     $     * 9d0 * MTRUN**4
c$$$     $     +9d0 * (MtLR/Ms2)**2 * (MTRUN/Ms2)**2 *
c$$$     $ (-6d0 * mtrun**4                   + 4d0 * ms22 * 
c$$$     $     (3d0 * mtrun**2 - mz2))
c$$$     $     +9d0/10d0 * (MtLR/Ms2)**4 * (MTRUN/Ms2)**4 *
c$$$     $     (-10d0 * ms2**4 + 4 * ms2**2 * (10d0 * mtrun**2  ) +
c$$$     $     15d0 * mtrun**2 * (-2d0 * mtrun**2      ))
c$$$     $     +9d0/35d0 * (MtLR/Ms2)**6 * (MTRUN/Ms2)**6 *
c$$$     $     (-21d0 * ms2**4 + 12d0 * ms2**2 * (7d0 * mtrun**2  ) +
c$$$     $     35d0 * mtrun**2 * (-2d0 * mtrun**2      ))
c$$$     $     -288d0/469762048d0 * (MtLR/Ms2)**8 * (MTRUN/Ms2)**8 *
c$$$     $     (6291456d0 * ms2**4 + ms2**2 * (-25165824d0 * mtrun**2
c$$$     $                      ) + 11010048d0 * (2d0 * mtrun**4      )))
      

c      write(*,*) '2. approach'
c --> all other sectors except for the sbottom mixing
      call countprint(count)
      m11botrest = pref * cb2 *
     $          ((  12d0 * nc * mb**4/(mz**4 * cb**4) 
     $           - 6d0 * nc * mb**2/(mz**2 * cb**2) 
     $           + pb ) * dlog(msusy2/mz2))
      call countprint(count)
      m11rest = pref * cb2 *
     $          ((+ pf + pg + p2h ) * dlog(msusy2/mz2)
     $          + theta * (p1h - p2h) * dlog(ma2/mz2) )
      call countprint(count)
      m12botrest = -pref * sb * cb *
     $          ((-3d0 * nc * mb**2/(mz2 * cb2) 
     $            + pb) * dlog(msusy2/mz2) )
      call countprint(count)
      m12rest = -pref * sb * cb *
     $          (( pf + pgp + p2hp) * dlog(msusy2/mz2) 
     $           - theta * (p1h + p2hp) * dlog(ma2/mz2) )
      call countprint(count)
      m22botrest = pref * sb2 * 
     $          ((pb) * dlog(msusy2/mz2) )
      call countprint(count)
      m22rest = pref * sb2 * 
     $          ((pf + pg + p2h) * dlog(msusy2/mz2) 
     $           + theta * (p1h - p2h) * dlog(ma2/mz2) )
       
        
c      write(*,*) '2. approach'
c --> the bottom mixing at 1-loop 
c     (directly taken over from LLEShortFormula3.m)
      call countprint(count)
      m11botmix = 
     &  (AL*MW**(-2)*NC/SW2*(-(MB**2*MSUSY**(-2)*MZ**2*(MTLR+
     &  MUE/tan(B))*(MBLR+((MTLR+MUE/tan(B)))/3D0))+4D0*CB**(-2)*
     &  MB**4*MBLR*MSUSY**(-2)*(MTLR+MUE/tan(b))*(1D0-(MBLR*MSUSY**
     &  (-2)*(MTLR+MUE/tan(b)))/12D0)))/(8D0*PI)
      call countprint(count)
      m12botmix =
     &  -((AL*MW**(-2)*NC/SW2*(4D0*CB**(-2)*MB**4*MBLR*MSUSY**
     &  (-2)*MUE*(1D0-(MBLR*MSUSY**(-2)*(MTLR+MUE/tan(b)))/6D0)-MB**
     &  2*MSUSY**(-2)*MZ**2*(MBLR*(MTLR+2D0*MUE/tan(b))+((MUE**2+
     &  (MTLR+MUE/tan(b))**2))/3D0)*TAN(B)))/(16D0*PI))
      call countprint(count)
      m22botmix = 
     &  (AL*MW**(-2)*NC/SW2*(-((1D0*CB**(-2)*MB**4*MBLR**2*
     &  MSUSY**(-4)*MUE**2)/3D0)-MB**2*MSUSY**(-2)*MUE*MZ**2*TAN(B)*
     &  (MBLR+(MUE*TAN(B))/3D0)))/(8D0*PI)


c --> the top sector at 2-loop
c     m22log is taken over directly from LLEShortFormula.m
      call countprint(count)
      if (lleselec1.eq.1) then
         m22log = (al*as*MTrun**4/dsin(b)**2 * 
     $        ((6*MtLR)/Ms2 - (3*MtLR**4)/(4.*Ms2**4) + 
     $        (-6 - (3*MtLR**2)/Ms2**2)*dLog(Ms2**2/MTrun**2) 
     $        - 3*dLog(Ms2**2/MTrun**2)**2
     $        ))/(MW**2*Pi**2*SW2)
c$$$c --> without non-log terms, mtrun everywhere
c$$$         m22log = (al*as*MTrun**4/dsin(b)**2 * 
c$$$     $        (                                         
c$$$     $        (+2 - (3*MtLR**2)/Ms2**2)*dLog(Ms2**2/MTrun**2) 
c$$$     $        - 3*dLog(Ms2**2/MTrun**2)**2
c$$$     $        ))/(MW**2*Pi**2*SW2)
      else
         write(*,*) 'using long LLE for 2-loop LLE-contribution'
         include 'LLExpansionP2MTrunsum.f'
         m22log = -cf*el**2*as/
     $            (3d0 * 2d0**8 * sw**2 * pi**3 * sb**2)*
     $            mtrun**2/mw**2 * dreal(llexpansionp2mtrun)
      endif

c      write(*,*) 'Parameters in LLESub:'
c      write(*,*) 'Msusy, MT, MtLR,'
c      write(*,*) 'M22top1loop, M22rest, M22botmix, M22top2loop'
c      write(*,*) real(msusy), real(mt), real(mtlr)
c      write(*,*) real(m22top), real(m22rest), real(m22botmix), 
c     $           real(m22log)
c      write(*,*) cf, el, al, sw, pi, sb, mtrun, mw, llexpansionp2mtrun

      call countprint(count)
      if (selec.eq.1) then
         m11 = m11top
         m12 = m12top
         m221loop = m22top
         m222loop = m221loop + m22log + 1d0/sb**2 * delmlhsq
      elseif(selec.eq.2) then
         m11 = m11top + m11botrest + m11botmix
         m12 = m12top + m12botrest + m12botmix
         m221loop = m22top + m22botrest + m22botmix
         m222loop = m221loop + m22log + 1d0/sb**2 * delmlhsq
      else
         m11 = m11top + m11rest + m11botrest + m11botmix
         m12 = m12top + m12rest + m12botrest + m12botmix
         m221loop = m22top + m22rest + m22botrest + m22botmix
         m222loop = m221loop + m22log + 1d0/sb**2 * delmlhsq
      endif

      call countprint(count)
      mlhllediag1sq = 
     $     (ma2 + mz2 + m221loop + m11 -
     $      dsqrt(-8d0 * m12 * (ma2 + mz2) * sb * cb +
     $            (ma2 + mz2 + m221loop + m11)**2
     $            - 4d0 * (-m12**2 + mz2 * m221loop * cb2 
     $                     + m221loop * m11 + mz2 * sb2 * m11 
     $                     + ma2 * (mz2 - 4d0 * mz2 * sb2 
     $                     + m221loop * sb2 + 4d0 * mz2 * sb**4
     $                     + m11 * cb2))))/2d0
      call countprint(count)
      mlhllediag2sq = 
     $     (ma2 + mz2 + m222loop + m11 -
     $      dsqrt(-8d0 * m12 * (ma2 + mz2) * sb * cb +
     $            (ma2 + mz2 + m222loop + m11)**2
     $            - 4d0 * (-m12**2 + mz2 * m222loop * cb2 
     $                     + m222loop * m11 + mz2 * sb2 * m11 
     $                     + ma2 * (mz2 - 4d0 * mz2 * sb2 
     $                     + m222loop * sb2 + 4d0 * mz2 * sb**4
     $                     + m11 * cb2))))/2d0
      call countprint(count)
      mhhllediag1sq = 
     $     (ma2 + mz2 + m221loop + m11 +
     $      dsqrt(-8d0 * m12 * (ma2 + mz2) * sb * cb +
     $            (ma2 + mz2 + m221loop + m11)**2
     $            - 4d0 * (-m12**2 + mz2 * m221loop * cb2 
     $                     + m221loop * m11 + mz2 * sb2 * m11 
     $                     + ma2 * (mz2 - 4d0 * mz2 * sb2 
     $                     + m221loop * sb2 + 4d0 * mz2 * sb**4
     $                     + m11 * cb2))))/2d0
      call countprint(count)
      mhhllediag2sq = 
     $     (ma2 + mz2 + m222loop + m11 +
     $      dsqrt(-8d0 * m12 * (ma2 + mz2) * sb * cb +
     $            (ma2 + mz2 + m222loop + m11)**2
     $            - 4d0 * (-m12**2 + mz2 * m222loop * cb2 
     $                     + m222loop * m11 + mz2 * sb2 * m11 
     $                     + ma2 * (mz2 - 4d0 * mz2 * sb2 
     $                     + m222loop * sb2 + 4d0 * mz2 * sb**4
     $                     + m11 * cb2))))/2d0


      allle1 = 0d0
      allle2 = 0d0
      call countprint(count)
      if (mlhllediag1sq.gt.0d0) then
         mlhllediag1 = dsqrt(mlhllediag1sq)
         allle1 = datan((-(ma2 + mz2) * sb * cb + m12)/
     $                  (mz2 * cb2 + ma2 * sb2 + m11 - mlhllediag1sq))
      else
         mlhllediag1 = 119.9999d0
      endif

      call countprint(count)
      if (mlhllediag2sq.gt.0d0) then
         mlhllediag2 = dsqrt(mlhllediag2sq)
         allle2 = datan((-(ma2 + mz2) * sb * cb + m12)/
     $                  (mz2 * cb2 + ma2 * sb2 + m11 - mlhllediag2sq))
      else
         mlhllediag2 = 119.9999d0
      endif

      call countprint(count)
      if (mhhllediag1sq.gt.0d0) then
         mhhllediag1 = dsqrt(mhhllediag1sq)
      else
         mhhllediag1 = 119.9999d0
      endif
      if (mhhllediag2sq.gt.0d0) then
         mhhllediag2 = dsqrt(mhhllediag2sq)
      else
         mhhllediag2 = 119.9999d0
      endif
      call countprint(count)



      end


c-------------------------------------------------------------------

      double precision function msfkt(msusytl, msusytr, mtrun)

      double precision msusytl, msusytr, mtrun

c$$$       msfkt = dsqrt(dsqrt(msusytl**2 * msusytr**2 
c$$$     $                    + mt**2 * (msusytl**2 + msusytr**2) + mt**4
c$$$     $                    + mt**2 * mtlr**2) - mt**2)
      msfkt = dsqrt( dsqrt(  msusytl**2 * msusytr**2 
     $         + mtrun**2 * (msusytl**2 + msusytr**2) + mtrun**4))

      end


c-------------------------------------------------------------------

      subroutine countprint(count)
      
      integer count

      count = count + 1
c      write(*,*) 'LLE: ', count
      end

c----------------------------------------------------------------
      


      
