*      
*     SOME AUXILIARY FUNCTIONS CALLED BY THE DSZ & BDSZ ROUTINES
*
*     Last update 13/12/2001
*
***********************************************************************
*

      real*8 function myAA(m,q)      
      real*8 m,q

      if(m.ne.0d0) then
         myAA = m*(1d0-Log(m/q))
      else
         myAA = 0d0
      endif

      return
      end

*
***********************************************************************
*


      real*8 function myB0(q,m1,m2,mu2) 

c     from Degrassi and Sirlin, Phys. Rev. D46 (1992) 3104.
      
      real*8 q,m1,m2,fhOmega,mu2
      
      if(m1.eq.0d0.and.m2.ne.0d0) then
         
         if(m2.ne.q) then
            myB0 = -(Log(m2/mu2)-2-(m2/q-1d0)*Log(dabs(1d0 - q/m2))) 
         else 
            myB0 = -(Log(m2/mu2) - 2)
         endif

      elseif(m2.eq.0d0.and.m1.ne.0d0) then

         if(m1.ne.q) then
            myB0 = -(Log(m1/mu2)-2-(m1/q-1d0)*Log(dabs(1d0 - q/m1))) 
         else
            myB0 = -(Log(m1/mu2) - 2)
         endif
         
      elseif(m2.eq.0d0.and.m1.eq.0d0) then

         myB0 = -(Log(q/mu2) - 2)  ! cut the imaginary part (I Pi)
         
      else
         
         myB0 = -( dlog(q/mu2)-2.d0 + 
     1        1.d0/2.d0*( 1.d0 + (m1/q-m2/q))*dlog(m1/q) +
     2        1.d0/2.d0*( 1.d0 - (m1/q-m2/q))*dlog(m2/q) +
     3        2.d0*fhOmega(m1/q,m2/q))
         
      endif

      return
      end
      
c     function fhOmega(a,b) contained in myB0
      real*8 function fhOmega(a,b)
      real*8 a,b,cbig
      Cbig = (a+b)/2.d0 - (a-b)**2.d0/4.d0 -1.d0/4.d0
      if(Cbig.gt.0.d0) then
         fhOmega = dsqrt(Cbig)*
     1        (datan((1.d0 + a - b)/(2.d0*dsqrt(Cbig))) +
     2        datan((1.d0 - a + b)/(2.d0*dsqrt(Cbig))) )
      elseif(Cbig.lt.0d0) then
         Cbig = - Cbig
         fhOmega = 1.d0/2.d0*dsqrt(Cbig)*
     1        dlog((a/2.d0 +b/2.d0 -1.d0/2.d0 -dsqrt(Cbig))/
     2        (a/2.d0 + b/2.d0 -1.d0/2.d0 + dsqrt(Cbig)))
      else
         fhOmega = 0         
      endif

      return
      end
      
*
**********************************************************************
*

      function phi(x,y,z)

c     from Davydychev and Tausk, Nucl. Phys. B397 (1993) 23

      implicit none
      real*8 x,y,z,phi,pphi,myphi
      
      if(x.le.z.and.y.le.z) then
         pphi = myphi(x,y,z)
      elseif(z.le.x.and.y.le.x) then
         pphi = z/x*myphi(z,y,x)
      elseif(z.le.y.and.x.le.y) then
         pphi = z/y*myphi(z,x,y)
      endif

      phi = pphi
      
      end
      
      function myphi(x,y,z)
      
      implicit none

      real*8 x,y,z,myphi
      real*8 u,v
      real*8 Pi/3.14159265358979/
      complex*16 clam,cxp,cxm,bdszCLi2,ccphi

c     auxiliary variables

      u = x/z
      v = y/z
      
      if((1d0-u-v)**2.ge.4d0*u*v) then         
         clam = dcmplx(dsqrt((1d0-u-v)**2 - 4d0*u*v),0d0)
      else
         clam = dcmplx(0d0,dsqrt(4d0*u*v - (1d0-u-v)**2))
      endif

      cxp = (1d0+(u-v)-clam)/2d0
      cxm = (1d0-(u-v)-clam)/2d0
      
c     phi function from eq. (A4)

      ccphi = (2d0*log(cxp)*log(cxm) - log(u)*log(v) - 
     &     2d0*(bdszCLi2(cxp) + bdszCLi2(cxm)) + Pi**2/3d0)/clam
      myphi = dreal(ccphi)

      return
      end


*
***********************************************************************
*

      function delt(x,y,z)

      implicit none

      real*8 x,y,z,delt

      delt = x**2 + y**2 + z**2 - 2d0*(x*y + x*z + y*z)

      return
      end

*
***********************************************************************
*

      function bdszLi2(x)

      implicit none

      complex*16 bdszCLi2,z
      real*8 x,bdszLi2

      z = DCMPLX(x,0d0)
      bdszLi2 = dreal(bdszCLi2(z))

      return
      end

*
***********************************************************************
*

      COMPLEX*16 FUNCTION bdszCLi2(Z)

c     routine for the complex dilogarithm function
      
      COMPLEX*16 Z,Z1,CLI20,MOD2
      REAL*8 MOD,PI3,PI6
      DATA PI6/1.64493406684823/,PI3/3.28986813369645/
      IF(Z.EQ.(1d0,0d0)) THEN
        bdszCLi2=DCMPLX(PI6,0d0)
        RETURN
      ENDIF
      LAB=0
      Z1=Z
      CLI20=(0d0,0d0)
      MOD=SQRT(DREAL(Z)**2+DIMAG(Z)**2)
      IF(MOD.GT.1d0)  Z1=1d0/Z   
      IF(DREAL(Z1).GE.0.5d0)  THEN
          LAB=1
          Z1=1d0-Z1
      ENDIF
      DO 20 ITE=1,100
      MOD2=Z1**ITE/(ITE**2)
      CLI20=MOD2+CLI20
      IF(SQRT(DREAL(MOD2)**2+DIMAG(MOD2)**2).LE.10E-10) GO TO 30
20    CONTINUE
30    IF(LAB.EQ.1)  CLI20=-CLI20+PI6-LOG(Z1)*LOG(1d0-Z1)
      IF(MOD.LE.1d0) THEN
        bdszCLi2=CLI20
        RETURN
      ELSE
        bdszCLi2=-PI6-CLI20-CDLOG(-Z)**2/2d0
      ENDIF
      RETURN
      END

