***********************************************************************
*** Function to perform the integral required for producing a      
*** Poissonian likelihood marginalised over a systematic
*** uncertainty in the sensitivity of the experiment.  See
*** hep-ex/0202013 or arXiv:0909.3300 for details.
***
*** Now obsolete - superseeded by dslnpoisint, dslnpin and dslnpiln.
***
*** input:  counts_tot total predicted number of counts
***         counts_S   predicted number of signal counts
***         obscounts  observed number of counts
***         errorsq    squared fractional systematic error on 
***                     experimental sensitivity
***         logNorm    use log-normal distribution for sys error if T
***
*** output: lnsf       ln(scaling factor)
***         dspoisint  integral / scaling factor
***
*** Author: Pat Scott (patscott@physics.mcgill.ca)
*** Date: April 2011
***********************************************************************

      real*8 function dspoisint(counts_tot,counts_S,obscounts,errorsq,
     & lnsf,logNorm)

      implicit none
      include 'dsge.h'
      include 'dspois.h'
      include 'dsidtag.h'

      real*8 obscounts, counts_tot, counts_S, errorsq, dsf_int, eps
      real*8 upperlim, integral, extent, peakval, lnsf, dspintegrand
      parameter (eps = 1.d-6, extent = 1.d2, peakval=1.d0)
      logical logNorm, logNormal
      common /logNormalDist/ logNormal
      save /logNormalDist/
      external dspintegrand

      logNormal = logNorm
      CBG_shared = counts_tot - counts_S
      CS_shared = counts_S
      invSsq_shared = 1.d0/errorsq
      obsC_shared = obscounts
      upperlim = extent*sqrt(errorsq)
      if (abs(counts_tot) .lt. epsilon(counts_tot)) then
        scalingFactor = peakval
      else
        scalingFactor = peakval - obscounts*dlog(counts_tot) +counts_tot
      endif
      lnsf = -scalingFactor
      integral = dsf_int(dspintegrand,0.d0,upperlim,eps)

      if (integral .le. 0.d0) then
        write(*,*) 'Error in dspoisint: integral <= 0,'
        write(*,*) 'probably due to very large counts.'
        write(*,*) 'With counts this large, you really should'
        write(*,*) 'switch to a chi^2 instead.'
        stop
      endif

      dspoisint = integral / sqrt(2.d0*pi*errorsq)

      end function dspoisint


      real*8 function dspintegrand(x)

      implicit none
      include 'dspois.h'

      real*8 x, logx, expont, maxExp
      parameter(maxExp = 700.d0)
      logical logNorm, logNormal
      common /logNormalDist/ logNormal
      save /logNormalDist/

      if (logNormal) then
        !Treat percentage error as log-normal distributed
        if (x .ne. 0.d0) then
          logx = dlog(x)
          expont = logx + x*CS_shared + CBG_shared +
     &     0.5d0*logx*logx*invSsq_shared
          if (obsC_shared .ne. 0.d0) expont = expont - 
     &     obsC_shared*dlog(x*CS_shared + CBG_shared)
        else
          expont = 1.d5
        endif
      else
        !Treat percentage error as Gaussianly-distributed
        expont = x*CS_shared + CBG_shared + 
     &   0.5d0*(1.d0-x)*(1.d0-x)*invSsq_shared
        if (obsC_shared .ne. 0.d0) expont = expont - 
     &   obsC_shared*dlog(x*CS_shared + CBG_shared)
      endif

      if (-expont+scalingFactor .gt. maxExp) then
        write(*,*) 'Error in dspoisint: integrand about to go'
        write(*,*) 'infinite.  Please either switch to a Gaussian '
        write(*,*) 'pdf for epsilon or write a new rescaling if '
        write(*,*) 'you want to persist with a Poissonian likelihood.'
        write(*,*) 'With counts this large though, you may wish to '
        write(*,*) 'switch to a chi^2 instead.'
        stop
      endif
  
      dspintegrand = dexp(-expont+scalingFactor)

      end function dspintegrand

