**********************************************************************
*** function dspbtpb gives the antiproton kinetic energy at the helio-
*** sphere as a function of the kinetic energy at the earth.
***   input:
***     tp - antiproton kinetic energy in gev
*** date: 98-02-10
**********************************************************************

      real*8 function dspbtpb(tp)
      implicit none

      include 'dshmcom.h'
      include 'dsidtag.h'

c----------------------------------------------------------------------
      real*8 pp,tp,mp,e,kkc,epb,ec,ppb,smod

      parameter (mp=0.93827231d0)
c      parameter (kkc=1.015d0)  ! critical momentum, gev
c      parameter (smod=.495d0)  ! solar modulation, gev
c      parameter (smod=.365d0)  ! solar modulation, gev

c...parameters for pbar paper with lars, piero and joakim:
      parameter (kkc=0.0d0)    ! critical momentum, gev
      parameter (smod=.500d0)  ! solar modulation, gev

c----------------------------------------------------------------------

c...solar modulation: calculate kinetic energy at heliosphere boundary
      e=mp+tp
      pp=sqrt(2*mp*tp+tp**2)
      if (pp.ge.kkc) then
         epb=e+smod
c         tjac=1.0d0
      else
         ec=sqrt(mp**2+kkc**2)
         epb=kkc*log((pp+e)/(kkc+ec))+ec+smod
c         tjac=kkc/(k+e)
      endif
      ppb=sqrt(max(epb**2-mp**2,0.0d0))
      dspbtpb=epb-mp   ! pbar kinetic energy at boundary

      return
      end





