       subroutine SUSPECT2(iknowl,input,ichoice,errmess)
c  VERSION 2.002  
c: last changes : September 10 2001
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c  J.-L. Kneur, A. Djouadi, G. Moultaka
c see home page: 
c http://www.lpm.univ-montp2.fr:7082/~kneur/
c for manual, updated info and maintenance
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
c  Calculates the MSSM mass spectrum (charginos, neutralinos, 
c  squarks, sleptons and  Higgs bosons (h,H,A,H+));   
c  including RG evolution of parameters, with different
c  options on models, approximations used, etc (see below).
c   (with some routines taken from 
c   already existing codes, in particular Higgs masses from 
c   M. Carena, C. Quiros, C. Wagner available routine;
c  some Higgs-related routines also common with "HDECAY" code.)
c|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
c EXAMPLE OF CALL: SEE THE ACCOMPANYING FILE suspect2_call.f
c where new (2) version options and detailed calling examples are
c given  
c------------------------------------------------------
c * Notations, definitions, analytic expressions based (mostly) on
c    a mixture of: ** Castano,Ramond,Piard Phys. Rev. D49(1994)4882. 
c                  ** Barger,Berger,Ohmann Phys.Rev. D49(1994)4908.
c EXCEPT for some SIGN CONVENTIONS changed (see conventions below!)
c|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
c------DEFINITION OF PARAMETERS AND CONVENTIONS USED ----    |
c|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
c  INPUT parameters: there are 3 CLASS of "input" parameters 
c  1) *** OPTION FLAG PARAMETERS (driven from input file) ***
c  =ICHOICE(1--10) (see suspect2_call.f for more explanations)
c
c 2)``Standard Model" parameters (not to be changed normally)
C ALFINV:   1/ALPHA(MZ): QED Coupling (at MZ scale, MSbar scheme)
c  ( reference latest value is ALFINV = 127.938 )
C SW2:       sin^2(theta)_W(MZ) in the MSbar scheme
c  (reference value: SW2= .23117 for MTOP =175 GeV) 
C ALPHAS:  VALUE FOR ALPHA_S(M_Z) (at the MZ scale)
c  (reference value: ALPHAS= .119 )
C MT:   TOP POLE MASS (reference value is MT= 174.3 GeV)
c MB:   BOttom pole mass (ref. value 4.62 GeV)
c MC:   Charm pole mass  (ref. value 1.4 GeV)
c
c  3) MSSM models physical parameters:
c  m0, m1/2, A0, tan(beta), sign(MU) in minimal SUGRA;
c  or arbitrary soft-breaking terms in non-universal models
c (see input file suspect2.in for more details and examples)
c|||||||||||||||||||||||||||||||||||||||||||||||||||||||||
c  **** IMPORTANT: MU SIGN (AND OTHER) CONVENTIONS ***** :
c|||||||||||||||||||||||||||||||||||||||||||||||||||||||||
c  1) WE DEFINE THE SUPERPOTENTIAL with the sign of MU conventions
c  as: W = MU (H_u . H_d) +.. =(def)= MU *eps(i,j) H^i_u H^j_d +..;
c   eps(1,2) = 1 = -eps(2,1)
c  where H_u, H_d are chiral (Higgs, Higgsinos)_u,d SUPERfields; 
c  2) WE DEFINE  the susy-breaking MU-term in the scalar potential
c  with the convention: V = B* MU eps(i,j) h^i_u * h^j_d (h_u,d are now
c  ordinary scalar fields, h_u=(h^1_u, h^2_u)=(h^{+}_u, h^0_u), 
c  h_d=(h^1_d, h^2_d)=(h^0_d, h^{-}_d)
c 
c   MU signs in relevant terms then follow as:
c  
c  -Sfermions: mixing terms =  m_LR = A_i - MU *(TBETA or 1/TBETA)
c  where A_i = A_b, A_tau or A_top
c  - Chargino:  +MU in mass matrix mixing terms
c  - Neutralinos:  -MU in mass matrix mixing terms;
c  - ALSO, Higgs potential minimization condition 
c  (radiative SU(2)xU(1) breaking) takes then the following form:
c  MZ^2/2 =(m1^2 - m2^2 * tbeta^2)/(tbeta^2 -1)
c   B* MU = (m1^2+m2^2)/2 * sin( 2 beta)
c  (where m1^2 =(m_phi_d)^2 +MU^2 +one-loop corrections
c         m2^2 =(m_phi_u)^2 +MU^2 +one-loop corrections )

c   OTHER RELEVANT CONVENTIONS :
c  3) sign of M1,M2,M3:  -M_i * Gbar G in Lagrangien (i.e.
c   ``normal" fermion mass signs), where G,Gbar are gaugino fields. 
c  4)  TGBETA = vu/vd (Q=MZ); (INPUT);
c  5)  v = sqrt(vu^2+vd^2) = 1/sqrt(2*sqrt(2)*GF) =174.** GeV
c    i.e. MW^2 = g2^2 * v^2/2 and there are NO 1/sqrt(2) 
c    factor in phi_u,d: <phi_u> = vu ; <phi_d> = vd 
c 
c -------------------------------------------------------
c  Main RG relevant variables: 
c  y(n) = vector containing all (RG evolving) parameters, 
c  at various possible scales depending on evolution stages.
c  n = number of RG-evolved parameters (may be different from
c  the initial  free parameters)
c  Those RG-evolving, scale-dependent parameters are:
c  y(1) = g1^2   U(1) gauge coupling
c  y(2) = g2^2   SU(2)_L gauge coupling
c  y(3) = g3^2 = 4*pi*alphas  SU(3) gauge coupling
c
c  y(4) = Y_tau  tau Yukawa coupling 
c  y(5) = Y_b    bottom  Yukawa coupling
c  y(6) = Y_top  top Yukawa coupling
c
c  y(7) = Ln(vu)  Logarithm of vu 
c  y(8) = Ln(vd)  Logarithm of vd 
c
c  y(9) = A_tau 
c  y(10)= A_b
c  y(11)=A_top
c
c  y(12) = (m_phi_u)^2  scalar phi_u "mass" term (in potential)
c  y(13) = (m_phi_d)^2  scalar phi_d "mass" term (in potential)
c  
c  y(14) = MTAUR^2 right-handed Stau Lagrangian mass^2 term
c  y(15) = MSL^2 left-handed Stau lagrangian mass^2 term
c  y(16) = MBR^2 right-handed Sbottom lagrangian mass^2 term
c  y(17) = MTR^2 right-handed Stop Lagrangian mass^2 term 
c  y(18) = MSQ^2 left-handed Stop Lagrangian mass^2 term
c
c  y(19) = B  The (dimensionful) B parameter in scalar mixing 
c
c  y(20) = Ln(M1) Log of Bino mass term
c  y(21) = Ln(M2) Log of Wino mass term
c  y(22) = Ln(M3) Log of gluino mass term
c
c  y(23) = Ln(MU) Log of the MU parameter, as defined above.
c
c  y(24) = MER^2 right-handed Selectron(Smuon) Lagrangian mass^2
c  y(25) = MEL^2 left-handed Selctron(Smuon) Lagrangian mass^2
c  y(26) = MDR^2 right-handed Sdown(Sstrange) Lagrangian mass^2
c  y(27) = MUR^2 right-handed Sup(Scharm) Lagrangian mass^2
c  y(28) = MUQ^2 left-handed Sup(Scharm) Lagrangian mass^2
c
c******************************************************
c  PROGRAM  COMMAND LINES START HERE 
c******************************************************
      implicit real*8(a-h,m,o-z)
      real*8 nf
      logical isNaN
      PARAMETER(NI=87,NOUT=88)
c  (just input/output file driving parameters)
       parameter(n=28)
       dimension ichoice(10),errmess(10)
       dimension y(n),ysave(n),ygut(8)
      dimension gcen(2,2),gctb(2,2),glee(2,2),gltt(2,2),
     . glbb(2,2),ghee(2,2),ghtt(2,2),ghbb(2,2)
      dimension GMN(4),XMN(4),GMC(2),GMST(2),GMSB(2),GMSL(2),
     .          GMSU(2),GMSD(2),GMSE(2),GMSN(4)
c#########################################################
c                    INPUT
c#########################################################
c++++++++++++++++++++++++++++++++++++++++++     
c INPUT PARAMETERS FOR INTERFACE with other codes:
c++++++++++++++++++++++++++++++++++++++++++
c  1) ICHOICE(1)--(10) : algorithm control parameters + choice of models
c etc (see for details suspect2_call.f)
c
c  2) "standard" model INPUT parameters:
      COMMON/SMpar/dalfinv,dsw2,dalphas,dmt,dmb,dmc,dmtau
c            
c you may change those input 
c values if you need, but they are of course not totally arbitrary
c and are fixed below from (more or less) most recent data. 
c Moreover one should be aware that e.g. GUT and SUSY threshold
C are eventually very sensitive to those values):
c  RG evolution scale parameters:
c
      COMMON/RGscal/dqewsb,dehigh,delow
c 
c QEWSB: EWSB scale: by default set to QEWSB= (m_t_L*m_t_R)^(1/2) 
c (ichoice(8) = 1 for default EWSB scale=(m_t_L*m_t_R)^(1/2), =0 if not)
c then IF ichoice(8)=0 EWSB scale is set by user from input file/calling
c routine by the value of Qewsb
c EHIGH: HIGH ENERGY END of RG EVOLUTION 
c (! only if specially required, normally (default) superseeded
c i.e in that case a more adequate value will be taken automatically)
c ELOW:  LOW-ENERGY END of RG EVOLUTION
c (! only if specially required, normally (default) superseeded
c by more adequate value. 
c
c
c 4) MSSM physical parameters:
c
       COMMON/MSSMhpar/dmhu2,dmhd2,dMA,dMU
c
C TGBETA:    TAN(BETA) (AT MZ scale!!) = vu/vd 
c MHU2:  SQUARED MASS of the H_u scalar doublet 
c MHD2:  SQUARED MASS of the H_d scalar doublet
c MA: PHYSICAL pseudo-scalar MA mass
c MU: the mu-parameter (sign convention: see below)
c IMPORTANT !: only 2 parameters are independent among
c mhu2,mhd2,mu,MA: depending on ichoice(6)= 0 (MA,MU input)
c or = 1 (MHD2,MHU2 input) the two others input are irrelevant
c because they will be recalcuted consistently from Electroweak
c symmetry breaking constraints
c (Note that also the scalar sector B parameter plays a role
c in the EWSB constraints, but has not to be given as input)
c 
       COMMON/MSSMgpar/dM1,dM2,dM3 
c  
c the U(1), SU(2), SU(3) SUSY-breaking gaugino masses
c
       COMMON/MSSMslep/dMSL,dMTAUR,dMEL,dMER
c
C MSL:      SUSY BREAKING MASS OF LEFT HANDED STAU
C MTAUR:    SUSY BREAKING MASS OF RIGHT HANDED STAU
C MEL:      SUSY BREAKING MASS OF LEFT HANDED Selectron, Smuon 
C MER:      SUSY BREAKING MASS OF RIGHT HANDED Selectron, Smuon 
c
       COMMON/MSSMsqua/dMSQ,dMTR,dMBR,dMUQ,dMUR,dMDR
c
C MSQ:      SUSY BREAKING MASS OF LEFT HANDED STOP
C MTR:      SUSY BREAKING MASS OF RIGHT HANDED STOP
C MBR:      SUSY BREAKING MASS OF RIGHT HANDED SBOTTOM
C MUQ:      SUSY BREAKING MASS OF LEFT-HANDED Sup, Scharm
C MUR:      SUSY BREAKING MASS OF RIGHT HANDED Sup, Scharm
C MDR:      SUSY BREAKING MASS OF RIGHT HANDED Sdown, Sstrange 
c
       COMMON/MSSMAtri/dAL,dAU,dAD
c       
C AL:       STAU TRILINEAR SOFT BREAKING TERMS 
C AU:       STOP TRILINEAR SOFT BREAKING TERMS.
C AD:       SBOTTOM TRILINEAR SOFT BREAKING TERMS.
c
c 5) mSUGRA case parameters:
c
       COMMON/MSUGRA/m0,mhalf,A0,sgnmu0,dtgbeta
c
c M0: UNIVERSAL scalar mass at GUT SCALE
c MHALF: UNIVERSAL Majorana fermion mass at GUT SCALE
c A0 :   UNIVERSAL trilinear coupling at GUT SCALE
c SGNMU0: SIGN  of MU0(GUT) (give +/-1. (!real parameter) )
c REMARK: when MSUGRA option is chosen (ichoice(1)=10) 
c other input "MSSM" parameter values above are ignored
c
c#########################################################
c                    OUTPUT
c !! However some of the input parameters above are also OUTPUT
c at the end of the run
c#########################################################
c
c   COMMON for OUTPUT masses:
      COMMON/OUTHIGGS/dml,dmh,dmch,alfa
c
c  light, heavy, charged Higgs masses, Higgs mix angle alpha 
c
      COMMON/OUTGINOS/dmc1,dmc2,dmn1,dmn2,dmn3,dmn4,mgluino
c
c   M_charginos 1,2; M_neutralinos 1--4; M_gluino 
c
      COMMON/OUTSQU/dmst1,dmst2,dmsu1,dmsu2
c
c  stop 1,2 and sup 1,2 = scharm 1,2 masses
c
      COMMON/OUTSQD/dmsb1,dmsb2,dmsd1,dmsd2
c
c  sbottom 1,2 and sdown 1,2 = sstrange 1,2 masses
c
      COMMON/OUTSLEP/dmsl1,dmsl2,dmse1,dmse2,dmsn1,dmsntau
c
c  stau 1,2 ; selectron (=smuon) 1,2; sneut_e,mu, sneut_tau masses
c
      common/mixang/thet,theb,thel
c  stop, sbottom, stau mixing angles
      common/rcmass/mbcr,mtcr,mtaucr
c   rad. corrected (final) bottom, top tau masses 
c
c#################################################################
c  INTERNAL COMMON AND PARAMETERS DEFINITIONS START HERE:     
      common/strc/irge,isfrc,inorc
      common/fmasses/mtau,mb,mt
      common/s_masses/MTAUrun,MBrun,MTrun
      COMMON/PARAM/GF,ALPH,MZ,MW
      COMMON/HMASS/MA,ML,MH,MCH,MAR
      COMMON/QMASSES/MS,MC
      COMMON/STRANGE/MSB
      COMMON/ALS/XLAMBDA,MC0,MB0,MT0,N0
      COMMON/FLAG/NNLO,IPOLE
      COMMON/BREAK/MSL,MTAUR,MSQ,MTR,MBR,AL,AU,AD,
     .            MU,M1,M2,M3
      COMMON/BREAK2/MEL,MER,MUQ,MUR,MDR
      COMMON/SMASS/GMN,XMN,GMC,GMST,GMSB,GMSL,GMSU,GMSD,GMSE,GMSN
      COMMON/s_COUP/GAT,GAB,GLT,GLB,GHT,GHB,GZAH,GZAL,
     .            GHHH,GLLL,GHLL,GLHH,GHAA,GLAA,GLVV,GHVV,
     .            GLPM,GHPM,BETA,A
      COMMON/CHIMASS/MCHI 
      COMMON/ERR_SF/sterr,sberr,stauerr
      common/cplhsf1/gcen,gctb,glee,gltt,glbb,ghee,ghtt,ghbb
      common/cplhsf2/gatt,gabb,gaee
       common/cte/nf,cpi,zm,wm,tbeta
       common/cteloop/vu,vd,Atop,Ab,Atau,rmllt,rmllb,rmlltau,
     . rmrrt,rmrrb,rmrrtau
       common/soft/rmtaur,rmL,rmbr,rmtr,rmQ
       common/cpl/g22,sw2
       common/s_thresh/rmtop,susym,egut
       common/stepwi/wistep,h1,kpole
       common/stegut/ifirst,jfirst,ygut
       common/gunif/kunif
       common/sgnM123/sgnM1,sgnM2,sgnM3
       common/yuka/ytau,yb,ytop
      common/yukaewsb/ytauewsb,ybewsb,ytewsb,alsewsb
      common/treesfer/msbtr1,msbtr2,msttr1,msttr2
       common/renscale/scale
      common/tachyrc/tachsqrc
      common/good/iflop
       external deriv1,deriv1t,deriv2,deriv2t,rkqc
       sgn(x)=dsign(x,x)/dabs(x)
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c  INPUT/OUTPUT parameters: 
c  reinitialize various control parameters + other parameters:
           do ierr=1,10
           errmess(ierr)=0.d0
           enddo
          irge=0
          iflop=0
          irpb=0
          tachsqrc=0.d0
          icount=0
          do irg=1,28
          y(irg)=0.d0
          enddo
         if(input.ne.11) then
      OPEN(NOUT,FILE='suspect.out',status='unknown')
         endif
c  Physical input parameters:
      if(input.eq.0) then
c read all relevant physical parameters from the input file:)
      OPEN(NI,FILE='suspect2.in',status='unknown')
      read(ni,*)
      read(ni,*)
      read(ni,*)
      read(ni,*)
      read(ni,*) ichoice(1)
      read(ni,*)
      read(ni,*) ichoice(2)
      read(ni,*)
      read(ni,*) ichoice(3)
      read(ni,*)
      read(ni,*)
      read(ni,*) ichoice(4)
      read(ni,*)
      read(ni,*) ichoice(5)
      read(ni,*)
      read(ni,*) ichoice(6)
      read(ni,*)
      read(ni,*)
      read(ni,*)
      read(ni,*)
      read(ni,*)
      read(ni,*) ichoice(7)
      read(ni,*)
      read(ni,*) ichoice(8)
      read(ni,*)
      read(ni,*) ichoice(9)
      read(ni,*)
      read(ni,*) ichoice(10)
      read(ni,*)
      read(ni,*)
      read(ni,*)
      read(ni,*) alfinv, sw2, alphas, mt, mb, mtau
      read(ni,*)
      read(ni,*)
      read(ni,*) EHIGH,  ELOW, QEWSB
      read(ni,*)
      read(ni,*)
      read(ni,*)
c   minimal supergravity boundary conditions 
      read(NI,*) rm0,  rmhalf,  A0,  TGBETA,  SGNMU0
      read(ni,*)
      read(ni,*)
      read(ni,*)
      READ(NI,*) MHU2,  MHD2
      read(ni,*)
      READ(NI,*) M1,  M2,  M3
      read(ni,*)
      READ(NI,*) MSL,  MTAUR
      READ(NI,*) 
      READ(NI,*) MSQ,  MTR,  MBR
      READ(NI,*) 
      READ(NI,*) MEL, MER
      READ(NI,*)
      READ(NI,*) MUQ,  MUR, MDR
      READ(NI,*) 
      READ(NI,*) AL, AU, AD
      read(ni,*)
      read(ni,*)
      read(ni,*)
      READ(NI,*) MA, MU
c
c
      else
c      (SO that  input is not 0) 
c  i.e. in this case the user defines all physical parameters
c  they are passed here via commons above
c      	
      ALFINV   =dalfinv
      SW2      =dsw2
      ALPHAS   =dalphas
c
      MT       =dmt
      MB       = dmb
      MTAU       = dmtau
c
      QEWSB    = dqewsb
      EHIGH    = dehigh
      ELOW     = delow 
c
      TGBETA   = dtgbeta
      rm0      = m0
      rmhalf   = mhalf
c (NB parameters A0 and sgnmu) already defined via common)
c
      MHU2     = dmhu2
      MHD2     = dmhd2
c
      M1       = dm1
      M2       = dm2
      M3       = dm3
c
      MSL      = dmsl
      MTAUR    = dmtaur
      MSQ      = dmsq
      MTR      = dmtr
      MBR      = dmbr
      MEL      = dmel
      MER      = dmer
      MUQ      = dmuq
      MUR      = dmur
      MDR      = dmdr
c
      AL       = dal
      AU       = dau
      AD       = dad
      MA       = dma
      MU       = dmu      
      endif
c
      tgbet0 = tgbeta
c  blind use: assign protection default values to control parameters:
        if(ichoice(2).eq.0) ichoice(2)=11
c  (i.e. 1-loop RGE at first run by default, if 2-loop not chosen) 
        if(ichoice(5).ne.1) ichoice(5)=1
        if(ichoice(1).eq.10) ichoice(6)=1
c (protection agains wrong input use: 
c in msugra case M_Hu(GUT)=M_Hd(GUt) = m0 necesseraly as input
        if(ichoice(8).ne.0) ichoice(8) = 1
c (i.e to be sure that not choosing default  EWSB scale 
c  =(m_t_L*m_t_R)^(1/2) is on purpose) 
c choose frozen scale in RGE parameters:
        kpole = ichoice(8)
c choose susy R.C. options:
        if(ichoice(7).eq.1) then
c only mt,mb,mtau susy R.C.
        isfrc = 0
        inorc=0
        else if(ichoice(7).eq.2) then
c  mt,mb,mtau susy R.C + (all) squarks susy RC.
        isfrc = 1
        inorc=0
        else if(ichoice(7).eq.3) then
c  mt,mb,mtau  + (all) squarks + (all) gaugino susy RC:
        isfrc = 1
        inorc=1
        endif
c
c   optimize number of long (RG+ Full spectrum) iterations
      irgmax = ichoice(9)
      if(irgmax.lt.3) irgmax = 3
c      if(irgmax.eq.3.and.tgbeta.ge.30) irgmax = 1+int(tgbeta/10) 
c
      if ( ichoice(1) .eq. 10 ) then
      if(rmhalf.ne.0.d0)  sgnM1  = rmhalf/dabs(rmhalf)
         sgnM2  = sgnM1
         sgnM3  = sgnM1
      else
      sgnM1  = M1/dabs(M1)
      sgnM2  = M2/dabs(M2)
      sgnM3  = M3/dabs(M3)
      endif
c
c  save input parameters again for some special purpose
c Higs mass terms
      mhu20 = mhu2
      mhd20 = mhd2
c "heavy" sfermions
      MSL0=msl
      MTAUR0=mtaur
      MSQ0=msQ
      MTR0=mtr
      MBR0=mbr
c "light" sfermions
      MEL0=meL
      MER0=mer
      MUQ0=muQ
      MUR0=mur
      MDR0=mdr

      AL0=AL
      AU0=Au
      AD0=Ad
      MU0=mu

      M10=M1
      M20=M2
      M30=M3
      BETA = datan(tgbeta)
c
c some other basic parameter definitions
      pi = 4*datan(1.d0)
       cpi = 1.d0/(16*pi*pi)
       GF = 1.16639d-5
       fermi=GF
       MZ = 91.187d0
       zm = MZ
c  guess starting point for SUSYM , ELOW, EHIGH SCALES:
       if(ichoice(3).ne.0) ehigh = 1.d17
c       elow= MZ
       if(ichoice(1).eq.10) then
       SUSYM = .5*(rm0+rmhalf) +MZ
       else
       rm0= (msl+mtaur+msq+mtr+mbr+mel+mer+muq+mur+mdr)/10
       fermav= (m1+m2+m3)/3
       SUSYM = .5*(rm0+fermav) +MZ
       endif 
       GUT=EHIGH
       kunif=ichoice(3)
       wistep= 100.d0
       nf = 6.d0
c  MW, sw^2 MSbar at Z scale (values may be changed):
       cw2= 1.d0-sw2
       sw=dsqrt(sw2)
       cw =dsqrt(cw2)
       MW = 80.3d0
       wm = MW
c MW is approximate exp. value)
      ALPH = 1.d0/137.0359895D0
c   alph is alpha at q2=0
c  "light" (=|= top) fermion (POLE) masses:
      MS=.190d0
      MC =1.42d0
c       mtau = 1.7771d0
       rmtau=mtau
       rmtau2=rmtau**2
c  Now defining running quark masses at Z scale:
      MSB=MS
      MC0=MC
      MB0=MB
      MT0=MT
      mtaurun=mtau
      mbrun=mb0
      mtrun=mt0
c (initial values only)
      ACC=1.d-8
      NLOOP=2
      XLAMBDA=XITLA(NLOOP,alphas,ACC)
      N0 = 5
      CALL ALSINI(ACC)
      MC=RUNM(MZ,4)
      MB=RUNM(MZ,5)
c      MT=RUNM(MZ,6)
       rmb=mb 
       rmb2=rmb**2
       rmtop=MT
       rmt2=rmtop**2
       sw20=sw2
       alphas0=alphas
c
c  long iteration (on RGE + spectrum once defined) starts here:
c
 44    irge=irge+1
       g32=4*pi*alphas0/(1.d0-(1.d0/2)*alphas0/(2*pi) )
       alphas=g32/4/pi
c
c       alpha=1.d0/alfinv
c passing from alpha(MZ) MSbar to alpha(MZ) DRbar:      
       alpha =1.d0/(alfinv -1.d0/pi/4)
c
       e2=4*pi*alpha
c
       cw20=1.d0-sw20
       g12= e2/cw20
       g22=e2/sw20
c
       tbeta=TGBETA
       cbeta= 1.d0/dsqrt(1.d0+tbeta**2)
       sbeta=tbeta*cbeta
c 
c - higgs vev at Z scale: tbeta = vu/vd
c (NB in our normalization MZ = (g12+g22)/2*(vu2+vd2), and there
c are no factors of sqrt(2) in the Higgs doublet components
c (cf Ramond et al PRD49(1994) 4882)
       vd2 = 2*zm*zm/(g12+g22)/(1.d0+tbeta**2)
       vu2 = vd2*tbeta**2
       vd= dsqrt(vd2)
       vu= dsqrt(vu2)
       v= dsqrt(vu2+vd2)
c
       if(ichoice(1).eq.0.or.ichoice(1).eq.2) goto 880
c (case where only physical mass spectrum is required, without RGE)
c
c  defining Yukawa couplings at Z scale:
       if(irge.eq.1) then
       y(4) = mtau/vd
       delmt =mt*(alphas/3/pi*(+5.d0)+1.11*alphas**2)
c       
       delmttot=delmt
       y(6) = mt/vu/(1.d0+delmttot/mt)
       delmb=0.d0
       y(5) =mb/vd/(1.d0+delmb/mb)
       endif
c - higgs vev at Z scale: y(7)=Ln vu, y(8)=Ln vd
       y(7) = .5*log(vu2)
       y(8) = .5*log(vd2)
c      1st stage: evolution of gauge + yukawa cpl from Mz to GUT:
c ! for irge=1 (iter. 1) yukawa's determined from QCD corrections only
       y(1) = 5.d0*g12/3.d0
c (i.e usual SU(5) normalisation of g1)
       y(2) = g22
       y(3) = g32
c set RGE accuracy choices (3 different)
       if(ichoice(4).eq.0) then
       h1=.2d0
       eps=1.d-3
       else if(ichoice(4).eq.1) then
       h1=.06d0
       eps=1.d-3
       else if(ichoice(4).eq.2) then
       h1=.02d0
       eps=2.d-5
       endif
       x1 = dlog(zm)
       x2= dlog(ehigh)
c
        ifirst=0
        jfirst=0
c        if(irge.ge.2) qewsb = 2*dsqrt(msttr1*msttr2)
        scale = qewsb
c (latter value for RGE only relevant IF ichoice(8) = 0) 
       if(ichoice(2).eq.11.and.irge.eq.1) then
       call odeint(y,n,x1,x2,eps,h1,1.d-5,nok,nbad,deriv1,rkqc)
       else if(ichoice(2).eq.21.and.irge.eq.1) then
       call odeint(y,n,x1,x2,eps,h1,1.d-5,nok,nbad,deriv2,rkqc)
       endif
       if(ichoice(2).eq.11.and.irge.ge.2) then
c means a full running has been done and susy RC redefine yukawas:
c 1-loop RGE:
       call odeint(y,n,x1,x2,eps,h1,1.d-5,nok,nbad,deriv1T,rkqc)
       endif
       if(ichoice(2).eq.21.and.irge.ge.2) then
c means a full running has been done and susy RC redefine yukawas:
c 2-loop RGE:
       call odeint(y,n,x1,x2,eps,h1,1.d-5,nok,nbad,deriv2T,rkqc)
       endif
c protection against RGE num. pbs (Landau poles, non-perturbativity):
       if(iflop.eq.1) then
       errmess(10)=-1.d0
       goto 801
       endif
c        write(*,*)'RG UP: ',irge
c (approximate) gauge unif. if required:
       if(egut.ne.0.d0) then
       ehigh=egut
         do irg=1,8
         y(irg)=ygut(irg)
         enddo
       y(2)=y(1)
       y(3)=y(1) 
       endif
c
       do i=1,8
       ysave(i)=y(i)
       end do
       vu = dexp(y(7))
       vd = dexp(y(8)) 
       mtaugut=vd*y(4)
       mbgut = vd*y(5)
       mtgut=vu*y(6)
c******************************************************************
c      2d stage: evolution from HIGH (GUT) scale down to low energy
c******************************************************************
c Now taking input rmu0,B0 values (!only guess initialization values)
       if(ichoice(6).eq.0) then
c  MU(0) is really an input then:
       rmu0=MU
       sgnmu0=rmu0/dabs(rmu0)
c and M^2_Hu, M^2_Hd will be calculated consistently at low-energy
c from EWSB. At high scale just give  initialization values:
c a not too bad is the average of non-universal other scalar masses
c (i.e. this is a smooth departure from msugra case)
       scalav= (msl+mtaur+msq+mtr+mbr+mel+mer+muq+mur+mdr)/10
       mhu2 = scalav**2
       mhd2 = scalav**2
       else
c guess MU(Gut) value at first time run (later superseeded by EWSB MU) 
c this apply in particular in mSUGRA or non-univ cases:
       if(irge.eq.1) rmu0 = 1.1*rm0
       endif
c
   7   rmu0=sgnmu0*dabs(rmu0)
c also guess value for B(GUT):
       B0 = 2*rm0
c set up boundary conditions at GUT scale:
c   yukawa coupling (eventual) unification at GUT scale:
       if(ichoice(3).ge.2) then
       y(5)=y(4)
       ysave(5)=y(5)
       endif
c  (tau- b unification)
       if(ichoice(3).eq.3) then
       y(6)=y(5)
       ysave(6)=y(6)
       endif
c  (tau-b-top unification): ! caution then tanbeta is constrained!
c (!! NOT YET operative  !!)
c
c - Higgs initial vev at GUT scale: fixed from evolution
c   from Z scale (see above)
  2    icount=icount+1
c icount is a counter for things to be done only at first iteration
       errhu=1.d3
       errhd=1.d3
       ifix =0
c errhu,hd,ifix are convergence control parameters for ichoice(6)=0
  77     if(ichoice(1).eq.1) then
c  Unconstrained MSSM: general case
C MSL:      SUSY BREAKING MASS OF LEFT HANDED STAU
C MTAUR:   SUSY BREAKING MASS OF RIGHT HANDED STAU
C MSQ:      SUSY BREAKING MASS OF LEFT HANDED STOP
C MTR:      SUSY BREAKING MASS OF RIGHT HANDED STOP
C MBR:      SUSY BREAKING MASS OF RIGHT HANDED SBOTTOM
C MEL:      SUSY BREAKING MASS OF LEFT HANDED Selectron, Smuon 
C MER:      SUSY BREAKING MASS OF RIGHT HANDED Selectron, Smuon 
C MUQ:      SUSY BREAKING MASS OF LEFT-HANDED Sup, Scharm
C MUR:      SUSY BREAKING MASS OF RIGHT HANDED Sup, Scharm
C MDR:      SUSY BREAKING MASS OF RIGHT HANDED Sdown, Sstrange 
       y(9)=AL0
       y(10)=AD0
       y(11)=AU0
       y(12)=MHU2
       y(13)=MHD2
       y(14)=MTAUR0**2
       y(15)=MSL0**2
       y(16)=MBR0**2
       y(17)=MTR0**2
       y(18)=MSQ0**2
       y(19)=B0
       y(23)=dlog(dabs(rmu0))
       y(24)=MER0**2
       y(25)=MEL0**2
       y(26)=MDR0**2
       y(27)=MUR0**2
       y(28)=MUQ0**2
       y(20)=dlog(dabs(M10))
       y(21)=dlog(dabs(M20))
       y(22)=dlog(dabs(M30))
c      
       else if(ichoice(1).eq.10) then
c Minimal SUGRA case: only m0,m1/2,A0,sgn(mu0) arbitrary)
c then forces radiative EW breaking (if was not chosen before:)
       ichoice(5)=1
c - soft susy-breaking terms: universality at GUT scale
       do j= 9,11
       y(j) = A0
       end do
       do kk=12,18
       y(kk) = rm0*rm0
       end do
       do kk=24,28
       y(kk) =rm0*rm0
       end do
c       if(icount.eq.1) y(19) = B0
       y(19)=B0
       do l=20,22
       y(l) = dlog(rmhalf)
       end do
c       if(icount.eq.1) y(23) = dlog(dabs(rmu0))
       y(23) = dlog(dabs(rmu0)) 
       endif
       x1= dlog(ehigh)
c
c   Generic end of running scale: actually, it is
c  determined "consistently", that is by default:
c  -at MZ scale for gauge+yukawas couplings, that are defined at MZ,
c  - at ewsb scale (!by default = sqrt(mst_L*mst_R) )
c   for all others RG parameters 
       x2=dlog(mz)
       h1=-h1
 3     issb=0
       istab=0
        ifirst=0
        jfirst=0
       if(ichoice(2).eq.11.and.irge.eq.1) then
       call odeint(y,n,x1,x2,eps,h1,1.d-5,nok,nbad,deriv1,rkqc)
       else if(ichoice(2).eq.21.and.irge.eq.1) then
       call odeint(y,n,x1,x2,eps,h1,1.d-5,nok,nbad,deriv2,rkqc)
       endif
       if(ichoice(2).eq.11.and.irge.ge.2) then
c  1-loop RGE
       call odeint(y,n,x1,x2,eps,h1,1.d-5,nok,nbad,deriv1T,rkqc)
       endif
       if(ichoice(2).eq.21.and.irge.ge.2) then
c  2-loop RGE
       call odeint(y,n,x1,x2,eps,h1,1.d-5,nok,nbad,deriv2T,rkqc)
       endif
c    
c protection against big troubles in RGE (e.g. Landau poles)
       if(iflop.eq.1) then
       errmess(10)=-1.d0
       goto 801
       endif
c    
c check for problems (non-perturbativity or/and Landau poles) in RGE:
       do i=1,28
       if(isnan(y(i))) then
       errmess(10)=-1.d0
       endif
       end do
       if(errmess(10).eq.-1.d0) then
       goto 801
       endif
c
       vu=dexp(y(7))
       vd=dexp(y(8))
       rmtau=y(4)*vd
       rmb = y(5)*vd
       rmtop =y(6)*vu
       mtaurun = rmtau
       mbrun = rmb
       mtrun = rmtop
       ytau=y(4) 
       yb=y(5)
       ytop=y(6)
c
       Atau =y(9)
       Ab=y(10)
       Atop=y(11)
       rmhu2 = y(12)
       rmhd2 = y(13)
       do kk=14,18
       if(y(kk).lt.0.d0) then
       errmess(1)=-1.d0
            if(iknowl.eq.2) then
       write(*,'(a)') 'Bad input: one m^2(3rd gen. sf) <0 from RGE '
       write(*,'(a)') 'maybe temporary due to iteration. wait and see'
            endif
       y(kk)=1.d-2
c
       endif
       enddo
       do kk=24,28
       if(y(kk).lt.0.d0) then
       errmess(2)=-1.d0
             if(iknowl.eq.2) then
       write(*,'(a)') 'Bad input: one m^2(1,2 gen. sf) <0 from RGE '
       write(*,'(a)') 'maybe temporary due to iteration. wait and see'
             endif
       y(kk)=1.d-2
c
       endif
       enddo
c 
       if(errmess(1).eq.-1.d0.or.errmess(2).eq.-1.d0) then
       goto 801
       endif
c                                                   
       rmtaur = dsqrt(y(14))
       rmL = dsqrt(y(15))
       rmbr=dsqrt(y(16))
       rmtr=dsqrt(y(17))
       rmQ =dsqrt(y(18))
       rmer =dsqrt(y(24))
       rmel=dsqrt(y(25))
       rmdr=dsqrt(y(26))
       rmur=dsqrt(y(27))
       rmuQ=dsqrt(y(28))
c
       B = y(19)
       rmu = sgn(rmu0)*dexp(y(23))
       if(ichoice(5).eq.1) then
       Bold  =B
       rmuold =rmu
c
       else
c  means no radiative EW required
       continue
       endif
       rmino1=sgnM1*dexp(y(20))
       rmino2=sgnM2*dexp(y(21))
       rmino3=sgnM3*dexp(y(22))
c  interface with Higgs mass spectrum calculations:
      IPOLE=ichoice(10)
c "heavy" sfermions
      MSL=rmL
      MTAUR=rmtaur
      MSQ=rmQ
      MTR=rmtr
      MBR=rmbr
c "light" sfermions
      MEL=rmeL
      MER=rmer
      MUQ=rmuQ
      MUR=rmur
      MDR=rmdr

      AL=Atau
      AU=Atop
      AD=Ab
      MU=rmu
       tgbeta=vu/vd
c tgbeta is tbeta at Z scale
      M1=rmino1
      M2=rmino2
      M3=rmino3
c  tan beta at the relevant (ewsb) scale: tbeta
c (extracted from common/cte/nf,cpi,zm,wm,tbeta )
      BETA = datan(tbeta)
c
c set EWSB scale (used in RGE, eff. potential, and susy R.C):
      if(ichoice(8).eq.0) then
      scale= qewsb 
      ewsb2= qewsb**2
      else if(ichoice(8).eq.1) then
c DEFAULT EWSB scale:
         if(msttr1.ne.0.d0) then
      scale= dsqrt(msttr1*msttr2)
         else
      scale = dsqrt(msq*mtr)
         endif
      ewsb2= scale**2
      endif
c    Gaugino masses
       CALL GAUGINO(mu,M1,M2,M3,BETA,A,GMC,GMN,XMN)
c******
c- set up the condition for radiative sym.break and stability:
c******
       cbeta=1.d0/dsqrt(1.d0+tbeta*tbeta)
       sbeta = tbeta*cbeta
       c2beta = cbeta*cbeta-sbeta*sbeta
       wm2=wm*wm
       zm2=zm*zm
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                 if(ichoice(6).eq.0) then
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c  input is MA, MU(high). Consistent M^2_Hu, M^2_d from EWSB
c  with iteration. 
       ifix=ifix+1
       MCHI =gmc(2)
c the user's MA input value is used in Higgs spectrum calc.:
       CALL SUSYCP(tgbeta)      
c
c  calculate sfermion masses and mixing angle:

       CALL SFERMION(MSQ,MTR,MBR,MSL,MTAUR,AL,AU,AD,MU,
     .               GMST,GMSB,GMSL,GMSU,GMSD,GMSE,GMSN)
         if(ifix.gt.5) then
         errmess(4)=-1.d0
           if(sterr.eq.-1.d0) gmst(1)=0.d0
           if(sberr.eq.-1.d0) gmsb(1)=0.d0
           if(stauerr.eq.-1.d0) gmsl(1)=0.d0
         errmess(4)=-1.d0
             if(iknowl.eq.2) then
         write(*,'(a)')' CAUTION: m^2_sf < 0! . was changed to 0  '
             endif
         endif
c
c  call one-loop effective potential corrections to Mh^2_1,2:
c 
c  dVdvd2, dVdvu2 are d(V_eff)/d(vd^2) and d(V_eff)/d(vu^2) which
c  add corrections to m^2_PHId (rmhd2) and m^2_PHIu (rmhu2) resp. 
c  
c
       rmst12= msttr1**2
       rmst22= msttr2**2
       rmsb12= msbtr1**2
       rmsb22= msbtr2**2
       rmstau12=gmsl(1)**2
       rmstau22=gmsl(2)**2
       dmsu1=gmsu(1)
       dmsu2=gmsu(2)
       dmsd1=gmsd(1)
       dmsd2=gmsd(2)
       dmse1=gmse(1)
       dmse2=gmse(2)
       dmsn1=gmsn(1)
       dmsntau=gmsn(3)
       alfa=a
       dmc1=gmc(1)
       dmc2=gmc(2)
       dmn1=gmn(1)
       dmn2=gmn(2)
       dmn3=gmn(3)
       dmn4=gmn(4)
c   
       call vloop2(ewsb2,rmtop**2,rmst12,rmst22,rmb2,rmsb12,rmsb22,
     . rmtau2,rmstau12,rmstau22,-rmu,dVdvd2,dVdvu2)
c   check of EWSB in this parametrization:
c   (note it includes the 1-loop effective potential contributions)   
c  However at first iteration neglect V_eff correction in the EWSB
c  consistency relation: that to avoid a totally wrong temporary
c sfermion spectrum and (hope to) accelerate convergence..
          if(ifix.eq.1) then
          dVdvd2=0.d0
          dVdvu2=0.d0
          endif
      rMHU2 = (MU**2+MZ**2/2)*(1.d0-tgbeta**2)/(1.d0+tgbeta**2) +
     . (MA**2-2*mu**2)/(1.d0+tgbeta**2)   -dVdvu2
      rMHD2 = MA**2-rMHU2-2*MU**2    -dVdvd2
      B=(rMHD2+dVdvd2 +rMHU2+dVdvu2+2*MU**2)*dsin(2*beta)/(2*MU)
c - to be compared to previous M^2_Hu,Hd values:
       errhuold=errhu
       errhdold=errhd
       errhu = (rmhu2-rmhu2old)/rmhu2
       errhd = (rmhd2-rmhd2old)/rmhd2
       if(dabs(errhu).lt.1.d-3.and.dabs(errhd).lt.1.d-3) then
       goto 8
       endif
       rmhu2old=rmhu2
       rmhd2old=rmhd2
c
c   running RGE UP to find corresponding M^2_Hu,M^2_Hd(high):
       y(12) = rmhu2 
       y(13) = rmhd2
c (all other values have not changed)
       x1=x2
       x2=dlog(Ehigh)
       h1=-h1
       if(ichoice(2).eq.11) then
       call odeint(y,n,x1,x2,2.d-4,h1,1.d-5,nok,nbad,deriv1,rkqc)
       else if(ichoice(2).eq.21) then
       call odeint(y,n,x1,x2,2.d-4,h1,1.d-5,nok,nbad,deriv2,rkqc)
       endif
       do i=1,8
       y(i)=ysave(i)
       end do
       mhu2=y(12)
       mhd2=y(13)
       B0= y(19)
c                if(ifix.gt.20) goto 88
       goto 77      
c
 8     continue
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    else
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c      input parameters M^2_Hu, M^2_Hd
c      consistent MU, B from EWSB conditions
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c --- algorithm to find a consistent MU with V_eff corrections:
       errB=1.d5
       errmu=1.d5
       if(ichoice(5).eq.1) then
c i.e. we want EWSB to determine MU and B
       dVdvd2=0.d0
       dVdvu2=0.d0
       ifix=0
  80   ifix=ifix+1
       MU=rmu
       CALL GAUGINO(mu,M1,M2,M3,BETA,A,GMC,GMN,XMN)
       MCHI =gmc(2)
c equation for MA:
        MA2 =(rmhu2+dVdvu2 -rmhd2-dVdvd2)/dcos(2*beta)-zm**2
     . -sbeta**2*dVdvd2 -cbeta**2*dVdvu2   
c     
        if(ma2.ge.0.d0) then
        MA=dsqrt(ma2)
        masave=ma
        errmess(3)=0.d0
        else
c allows for temporary MA^2 < 0 (before EWSB converges)
c and attempt to retrieve a correct MA via a correct MU etc.
c gives wrong MA and likely MU values just so that calculation 
c (EWSB iteration) can proceed for a while:
        MA=mz
        masave=ma
          if(irge.eq.irgmax.and.ifix.ge.10) then
c (consider the MA^2 <0 problem irremediable after 10 such erratic iter):
        errmess(3)=-1.d0
        goto 801
          endif
        rmu= sgn(rmu0)*dsqrt((MA**2-rmhu2 -rmhd2)/2)
       MU=rmu
       CALL GAUGINO(mu,M1,M2,M3,BETA,A,GMC,GMN,XMN)
       MCHI =gmc(2)
        endif
c
c  calculate sfermion masses and mixing angle:
c
       CALL SFERMION(MSQ,MTR,MBR,MSL,MTAUR,AL,AU,AD,MU,
     .               GMST,GMSB,GMSL,GMSU,GMSD,GMSE,GMSN)
c  next is to protect against initial MU input that could be
c  (temporarily!) inconsistent with m_sfermion^2 >0 :
          if(ifix.eq.1) then
          if(sterr.eq.-1.d0.or.sberr.eq.-1.d0.or.stauerr.eq.-1.d0) then 
c calculate an apriori better MU value:
       rmu2 =(rmhd2-rmhu2*tgbeta**2)/(tgbeta**2-1.d0)
     .        -zm**2/2.d0
       sterr=0.d0
       sberr=0.d0
       stauerr=0.d0
       if(rmu2.le.0.d0) then
       rmu= rmu/2.d0
       else
       rmu =sgn(rmu0)*dsqrt(rmu2)
       endif 
c then recalculate a corresponding, a priori better, Higgs+SF spectrum:
       CALL SFERMION(MSQ,MTR,MBR,MSL,MTAUR,AL,AU,AD,MU,
     .               GMST,GMSB,GMSL,GMSU,GMSD,GMSE,GMSN)
         endif 
         endif
         if(sterr.eq.-1.d0.or.sberr.eq.-1.d0.or.stauerr.eq.-1.d0) then
c means there is really the tachyonic sfermion mass problem:
c can't even calculate Higgs spectrum etc, so has to stop really.
         errmess(4)=-1.d0
             if(iknowl.eq.2) then
         write(*,'(a)')' CAUTION: m^2_sf < 0! . Has been changed to 0 '
             endif
         goto 801
         endif
         if(tachsqrc.eq.-1.d0) then
          errmess(4)=-1.d0
         goto 801
        endif
c Otherwise (= no tachyonic sfermions) calculate Higgs mass spectrum:
       CALL SUSYCP(tbeta)
c protection against NAN Higgs that could occurs despite previous protec.
       if(isNAN(ml).or.isNAN(mH).or.isNAN(MCH)) then
       errmess(9)=-1.d0
       goto 801
       endif
       if(ml.eq.0.d0.or.ml.gt.1.d10.or.mH.gt.1.d10) then
       errmess(9)=-1.d0
       goto 801
       endif
c
       rmst12= msttr1**2
       rmst22= msttr2**2
       rmsb12= msbtr1**2
       rmsb22= msbtr2**2
       rmstau12=gmsl(1)**2
       rmstau22=gmsl(2)**2
       dmsu1=gmsu(1)
       dmsu2=gmsu(2)
       dmsd1=gmsd(1)
       dmsd2=gmsd(2)
       dmse1=gmse(1)
       dmse2=gmse(2)
       dmsn1=gmsn(1)
       dmsntau=gmsn(3)
       alfa= a
       dmc1=gmc(1)
       dmc2=gmc(2)
       dmn1=gmn(1)
       dmn2=gmn(2)
       dmn3=gmn(3)
       dmn4=gmn(4)
c  call one-loop effective potential corrections to Mh^2_1,2:
c 
c  dVdvd2, dVdvu2 are d(V_eff)/d(vd^2) and d(V_eff)/d(vu^2) which
c  add corrections to m^2_PHId (rmhd2) and m^2_PHIu (rmhu2) resp. 
c   
       call vloop2(ewsb2,rmtop**2,rmst12,rmst22,rmb2,rmsb12,rmsb22,
     . rmtau2,rmstau12,rmstau22,-rmu,dVdvd2,dVdvu2)
c NB: Note the -rmu =-MU entry in the subroutine above: this is
c  because vloop was calculated with the other convention:
c   m_LR =  A_i + mu/tbeta
c
              if(isNAN(dvdvd2).or.isNaN(dvdvu2)) then
              errmess(3)=-1.d0
              goto 801
              endif  
c - Now the radiative breaking conditions DEFINE true mu(mz):
c 
c tree-level EWSB conditions as (first time!) MU guess:
       if(ifix.eq.1) then
       rmu2 =(rmhd2-(rmhu2)*tbeta**2)/(tbeta**2-1.d0)
     .        -zm**2/2.d0 
       else
       rmu2 =(rmhd2+dVdvd2-(rmhu2+dVdvu2)*tbeta**2)/(tbeta**2-1.d0)
     .        -zm**2/2.d0
       endif
       if(rmu2.le.0.d0) then
        errmess(5)=-1.d0
            if(iknowl.eq.2) then
       write(*,'(a)')'Warning: MU^2(EWSB) <0 (may be temporary) '
             endif    
       if(irge.eq.irgmax.and.ifix.ge.10) then
c consider the MU^2 < 0 problem irremediable:
       errmess(6)=-1.d0
       goto 801
       else
c take approximate MU "RG" =f(MA,M_Hu,Mhd) to attempt to retrieve
c EWSB convergence:
         if(MA**2-rmhu2 -rmhd2.gt.0.d0) then
       rmu= sgn(rmu0)*dsqrt((MA**2-rmhu2 -rmhd2)/2)
         else
c take arbitrary small MU to attempt to retrieve EWSB convergence:
        rmu=sgn(rmu0)*10.d0
         endif
       endif
c       rmu= rmu/2.d0
      else
       rmu =sgn(rmu0)*dsqrt(rmu2)
       endif
c      
c - ..and true B(EWSB):
c  tree-level EWSB conditions as first time MU guess:
       if(ifix.eq.1) then
       B = (rmhd2 +rmhu2 +2*rmu**2)*sbeta*cbeta/rmu 
       else
       B = (rmhu2+dVdvu2 +rmhd2+dVdvd2 +2*rmu**2)*sbeta*cbeta/rmu  
       endif
c
c - to be compared to evolved mu values:
       errmuold=errmu
       errmu= (rmu-rmuold)/rmuold
       if(dabs(errmu).lt.5.d-5.and.ma2.gt.0.d0.and.rmu2.gt.0.d0
     & .or.ifix.eq.20) then
c i.e. considers as unconvergent MU from EWSB either:
c   -inaccurate (> 1e-4) convergence;
c   - more than 20 iterations;
c   if MA^2 was in fact <0, so that convergence is around fake MA,MU
c since MA was articifially forced = MZ temporarily in that case
c  
       goto 81
       else
       rmuold=rmu
       goto 80
       endif
c  ( end of the iterative loop on consistent MU,B )
c
  81    continue 
                     endif
c (previous endif = end of the choice M_Hu,MHd or MA,MU input)
       if(ifix.eq.20.and.irge.eq.irgmax) then
       errmess(6)=-1.d0
          if(iknowl.eq.2) then
       write(*,'(a)')' consistent EWSB unconvergent below 1d-4' 
          endif
       endif
c
 88    if(ichoice(1).eq.1.and.ifix.eq.20) then
       errmess(6)=-1.d0
          if(iknowl.eq.2) then
       write(*,'(a)')' consistent EWSB unconvergent below 1d-4' 
          endif
       endif
c
       endif
c    control SSB V stability (naive RG improved checks of UFB/CCB):
      r1= rmhd2 +dvdvd2 +rmu2
      r2= rmhu2 +dvdvu2 +rmu2
      r3= B*rmu
      test1= r1*r2-r3*r3
c      test2= r1+r2+2*r3
c      test3=r1+r2-2*r3
      test2 = ma2 +2*r3
      test3 = ma2 -2*r3
      if(ichoice(5).eq.1) then
      if(test1.ge.0.d0.and.irge.eq.irgmax) then
      errmess(7)=-1.d0
            if(iknowl.eq.2) then
      write(*,'(a)')' Warning!: EW Sym. Break may be not realized '
            endif
      endif
      if(test2.lt.0.d0.or.test3.lt.0.d0.and.irge.eq.irgmax) then
      errmess(8)=-1.d0
           if(iknowl.eq.2) then
      write(*,'(a)')' Warning: Potential maybe unbounded from below  '
            endif
      endif
c CCB (simplest!) constraints:
          MCCB= MSQ**2 +MTR**2 +rmhu2 +rMU**2
          ccbcheck= AU**2-3*MCCB
          if(ccbcheck.gt.0.d0) then
c ! these are points which do not pass those naive CCB constraints
          errmess(8)=-1.d0
          endif
       else
c  means no radiative EW required
c Now B = y(19) and MU =Exp(y(23)) are determined from EW breaking
c (however not RADIATIVE breaking in this case)
       rmu2 =(rmhd2+dVdvd2-(rmhu2+dVdvu2)*tgbeta**2)/(tgbeta**2-1.d0)
     .        -zm**2/2.d0
       if(rmu2.le.0.d0) then
       errmess(5)=-1.d0
            if(iknowl.eq.2) then
       write(*,'(a)')' Warning: initial rmu0(HIGH) inconsistent. '
       write(*,'(a)')' has been changed '
            endif
       rmu0  = rmu0/2
       do i=1,8
       y(i)=ysave(i)
       end do
       x2=x1
       h1=-h1
       goto 7
       else
       continue 
       endif
c      
       rmu =sgn(rmu0)*dsqrt(rmu2)
c - .. B(mz):
       B = (rmhd2+dVdvd2+rmhu2+dVdvu2+2*rmu2)*sbeta*cbeta/rmu 
c    control of SSB and V stability scales:
      r1= rmhd2 +dVdvd2 +rmu2
      r2= rmhu2 +dVdvu2 +rmu2
      r3= B*rmu
      test1= R1*R2-R3*r3
      test2= r1+r2+2*r3
      test3=r1+r2-2*r3
       if(test1.gt.0.d0) then
       errmess(7)=-1.d0
             if(iknowl.eq.2) then
       write(*,'(a)')'Warning: m^2(Hu),m^2(Hd) inconsistent with EWSB' 
       write(*,13) rmhu2,rmhd2
       write(*,'(a)')' have been changed '
             endif       
       MHU2= 1.5*MHU2
       MHD2= MHD2
       do i=1,8
       y(i)=ysave(i)
       end do
       x2=x1
       h1=-h1
       goto 7 
       endif
       if(test2.lt.0.d0.or.test3.lt.0.d0) then
       errmess(8)=-1.d0
            if(iknowl.eq.2) then
       write(*,'(a)')' Warning: Potential unbounded from below! ' 
       write(*,'(a)')' m^2(Hu),m^2(Hd) values been changed ' 
             endif
       MHU2 = 1.5*MHU2
       MHD2  =MHD2
       do i=1,8
       y(i)=ysave(i)
       end do
       x2=x1
       h1=-h1
       goto 7 
       endif
       endif
       if(ichoice(5).ne.1) then
       MCHI =gmc(2)
       MA = dsqrt(rmhu2 +rmhd2 +2*rmu**2 )
c  calculate Higgs mass spectrum
       CALL SUSYCP(tgbeta)      
c
c  calculate sfermion masses and mixing angle:

       CALL SFERMION(MSQ,MTR,MBR,MSL,MTAUR,AL,AU,AD,MU,
     .               GMST,GMSB,GMSL,GMSU,GMSD,GMSE,GMSN)
        endif
c
c----   special case of unconstrained MSSM with low-en input: ----
c
 880   if(ichoice(1).eq.0.or.ichoice(1).eq.2) then
c  case of the pMSSM (unconstrained MSSM, low-en input) 
c
                                  if(ichoice(6).eq.0) then
c   input parameter of the pMSSM  is M_A , MU
c    Gaugino masses
 881    beta= datan(tgbeta)
       CALL GAUGINO(mu,M1,M2,M3,BETA,A,GMC,GMN,XMN)
c   Higgs masses
       MCHI =gmc(2)
       CALL SUSYCP(tgbeta)      
c
c  sfermion masses

       CALL SFERMION(MSQ,MTR,MBR,MSL,MTAUR,AL,AU,AD,MU,
     .               GMST,GMSB,GMSL,GMSU,GMSD,GMSE,GMSN)

c   check of EWSB in this parametrization:
c Note we include in the EWSB consistency relations the 1-loop
c V_eff contributions: indeed, it is consistent with the fact
c that all Higgs masses are calculated with one-loop contributions:
c
c  call one-loop effective potential corrections to MH^2_u,d:
c 
c  dVdvd2, dVdvu2 are d(V_eff)/d(vd^2) and d(V_eff)/d(vu^2) which
c  add corrections to m^2_PHId (rmhd2) and m^2_PHIu (rmhu2) resp. 
c  
       rmtaur = mtaur
       rmL = msl
       rmbr= mbr
       rmtr= mtr
       rmQ = msq
       Atau= AL
       Ab= Ad
       Atop = Au
c
       rmst12= msttr1**2
       rmst22= msttr2**2
       rmsb12= msbtr1**2
       rmsb22= msbtr2**2
       rmstau12=gmsl(1)**2
       rmstau22=gmsl(2)**2
       rmt2=mtrun**2
       rmtop=mtrun
       rmb2=mbrun**2
       rmtau2= mtaurun**2
       dmsu1=gmsu(1)
       dmsu2=gmsu(2)
       dmsd1=gmsd(1)
       dmsd2=gmsd(2)
       dmse1=gmse(1)
       dmse2=gmse(2)
       dmsn1=gmsn(1)
       dmsntau=gmsn(3)
       alfa = a
       dmc1=gmc(1)
       dmc2=gmc(2)
       dmn1=gmn(1)
       dmn2=gmn(2)
       dmn3=gmn(3)
       dmn4=gmn(4)
c        
      rmu=MU
       ewsb2=msttr1*msttr2
c       ewsb2=mt**2
       call vloop2(ewsb2,rmt2,rmst12,rmst22,rmb2,rmsb12,rmsb22,
     . rmtau2,rmstau12,rmstau22,-rmu,dVdvd2,dVdvu2)
c   
      rMHU2 = (rMU**2+MZ**2/2)*(1.d0-tgbeta**2)/(1.d0+tgbeta**2) +
     . (MA**2-2*rmu**2)/(1.d0+tgbeta**2)    -dVdvu2
      rMHD2 = MA**2-rMHU2 -dVdvu2 -2*rMU**2    -dVdvd2
      B=(rMHD2+rMHU2+dVdvd2+dVdvu2+2*rMU**2)*dsin(2*beta)/(2*rMU)
c (note: this is to take into account that we would like to get
c the "tree-level" values of M^2_Hu, M^2_Hd, thus without V_eff
c corrections)
c    control of SSB and V stability scales:
      r1= rmhd2 +dVdvd2  +rmu**2
      r2= rmhu2  +dVdvu2 +rmu**2
      r3= B*rmu
      test1= r1*r2-r3*r3
      test2= r1+r2+2*r3
      test3=r1+r2-2*r3
      if(input.ne.11) then
      write(nout,'(a)')' loop-level EW breaking + stability tests:'
      write(nout,'(a)')' m1^2*m2^2-B^2*MU^2, m1^2+m2^2 +/-2*B*MU:'
      write(nout,777) test1,test2,test3 
      endif
c
      MHU2=rmhu2
      MHD2=rmhd2
         if(ichoice(1).eq.0) then
        rmino1=m1
        rmino2=m2
        rmino3=m3
         else
       rmino1=sgnM1*dexp(y(20))
       rmino2=sgnM2*dexp(y(21))
       rmino3=sgnM3*dexp(y(22))
         endif
        if(test1.ge.0.d0) then
      errmess(7)=-1.d0
        endif
        if(test2.lt.0.d0.or.test3.lt.0.d0) then
       errmess(8)=-1.d0
        endif
c
                       else
c
c   input parameter of the pMSSM (no RGE) is MHD2,MHU2
c    
c  in that case the  one-loop effective potential contributions are
c implicit, since MHD2,MHU2 now are "physical" low-en input, so include
c  these one-loop contributions! (otherwise it would be inconsistent
c  with the fact that e.g. all Higgs masses are calculated with one-loop
c  corrections. Thus the V_eff corrections are not explicitly needed). 
      ichoice(5) = 1
       rmhd2=MHD2
       rmhu2=MHU2
c --- algorithm to find a consistent MU with V_eff corrections:
c
c  
c - the radiative breaking conditions DEFINE true mu(mz):
c 
       rmu2 =(rmhd2-rmhu2*tgbeta**2)/(tgbeta**2-1.d0)
     .        -zm**2/2.d0
           if(rmu2.le.0.d0) then
               if(iknowl.eq.2) then
       write(*,'(a)')' CAUTION: initial M^2_Hu,Hd inconsistent'
       write(*,'(a)')' their values were changed so that mu^2 >=0! ' 
c
               endif
c  find the minimal values of M^2_Hu,Hd to guarantee mu^2 >0,MA>0:
      rMHU2 = (1.d-7+MZ**2/2)*(1.d0-tgbeta**2)/(1.d0+tgbeta**2) +
     . (MA**2-2*1.d-7)/(1.d0+tgbeta**2)    
      rMHD2 = -rMHU2    
            else
c rmu^2 >0 from the input
       rmu =sgn(rmu)*dsqrt(rmu2)
            endif
c      
c - ..and true B(mz):
       B = (rmhd2+rmhu2+2*rmu2)*sbeta*cbeta/rmu 
       Mu=rmu
       CALL GAUGINO(MU,M1,M2,M3,BETA,A,GMC,GMN,XMN)
       MCHI =gmc(2)
       MA = dsqrt(rmhu2 +rmhd2 +2*rmu**2 )
       CALL SUSYCP(tgbeta)      
c
c  calculate sfermion masses and mixing angle:

       CALL SFERMION(MSQ,MTR,MBR,MSL,MTAUR,AL,AU,AD,MU,
     .               GMST,GMSB,GMSL,GMSU,GMSD,GMSE,GMSN)
c
c    control of SSB and V stability scales:
      r1= rmhd2  +rmu2
      r2= rmhu2  +rmu2
      r3= B*rmu
      test1= r1*r2-r3*r3
      test2= r1+r2+2*r3
      test3=r1+r2-2*r3
      if(input.ne.11) then
      write(nout,'(a)')' loop-level EW breaking + stability tests:'
      write(nout,'(a)')' m1^2*m2^2-B^2*MU^2, m1^2+m2^2 +/-2*B*MU:'
      write(nout,777) test1,test2,test3 
      endif
c
  
            if(test1.ge.0.d0) then
           errmess(7)=-1.d0
            endif
            if(test2.lt.0.d0.or.test3.lt.0.d0) then
       errmess(8)=-1.d0
              if(iknowl.eq.2) then
      write(*,'(a)')' Warning: Potential unbounded from below! '
      write(*,'(a)')' give new MHD2, MHU2 input: '
      write(*,'(a)')' (0, 0 if proceed still)   '
      read(*,*) rMHD2,rMHU2
               endif
c      if(rmhd2.eq.0.d0) goto 888
            endif
       rmino1=sgnM1*dexp(y(20))
       rmino2=sgnM2*dexp(y(21))
       rmino3=sgnM3*dexp(y(22))
c
        endif
      endif
c
            if(ichoice(1).eq.2) then
c
c unconstrained MSSM runned up to High scale 
       y(1) = 5.d0*g12/3.d0
c (i.e usual SU(5) normalisation of g1)
       y(2) = g22
       y(3) = g32
       y(4) = rmtau/vd
       y(5) = rmb/vd
       y(6) = rmtop/vu/(1.d0+4*alphas/3.d0/pi)
c - higgs vev at Z scale: y(7)=Ln vu, y(8)=Ln vd
       y(7) = .5*log(vu2)
       y(8) = .5*log(vd2)
c
       if(ichoice(4).eq.0) then
       h1=.2d0
       eps=1.d-3
       else if(ichoice(4).eq.1) then
       h1=.06d0
       eps=1.d-3
       else if(ichoice(4).eq.2) then
       h1=.02d0
       eps=2.d-5
       endif
c  first little run from z scale (where g1,g2,g3 are defined!)
c to scale elow(where all soft etc parameters are assumed to be given
c if ichoice(1) = 0 or 2 ):
       x1=dlog(mz)
       x2=dlog(elow)
c
       if(ichoice(2).eq.11) then
       call odeint(y,n,x1,x2,eps,h1,1.d-5,nok,nbad,deriv1,rkqc)
       else if(ichoice(2).eq.21) then
       call odeint(y,n,x1,x2,eps,h1,1.d-5,nok,nbad,deriv2,rkqc)
       endif
c  Now the big run from elow to ehigh, with non-zero soft terms:
       x1 = dlog(elow)
       x2= dlog(ehigh)
c
       y(9)=AL
       y(10)=AD
       y(11)=AU
       y(12)=MHU2
       y(13)=MHD2
       y(14)=MTAUR**2
       y(15)=MSL**2
       y(16)=MBR**2
       y(17)=MTR**2
       y(18)=MSQ**2
       y(19)=B
       y(23)=dlog(dabs(mu))
       y(24)=MER**2
       y(25)=MEL**2
       y(26)=MDR**2
       y(27)=MUR**2
       y(28)=MUQ**2
       y(20)=dlog(dabs(M1))
       y(21)=dlog(dabs(M2))
       y(22)=dlog(dabs(M3))
c
       if(ichoice(2).eq.11) then
       call odeint(y,n,x1,x2,eps,h1,1.d-5,nok,nbad,deriv1,rkqc)
       else if(ichoice(2).eq.21) then
       call odeint(y,n,x1,x2,eps,h1,1.d-5,nok,nbad,deriv2,rkqc)
       endif
c redefining output parameters in terms of y(i):
       vu=dexp(y(7))
       vd=dexp(y(8))
       rmtau=y(4)*vd
       rmb = y(5)*vd
       rmtop =y(6)
c (note these are high scale mass, so only running: the "pole"
c top mass at gut scale would not make much sense)
       Atau =y(9)
       Ab=y(10)
       Atop=y(11)
       rmhu2 = y(12)
       rmhd2 = y(13)
       do kk=14,18
       if(y(kk).lt.0.d0) then
       errmess(1)=-1.d0
            if(iknowl.eq.2) then
       write(*,'(a)') 'Bad input: one m^2(3rd gen. sf) <0 from RGE '
       write(*,'(a)') 'maybe temporary due to iteration. wait and see'
            endif
c
       endif
       enddo
       do kk=24,28
       if(y(kk).lt.0.d0) then
       errmess(2)=-1.d0
             if(iknowl.eq.2) then
       write(*,'(a)') 'Bad input: one m^2(1,2 gen. sf) <0 from RGE '
       write(*,'(a)') 'maybe temporary due to iteration. wait and see'
             endif
c
       endif
       enddo
c                                                                      
       rmtaur = dsqrt(y(14))
       rmL = dsqrt(y(15))
       rmbr=dsqrt(y(16))
       rmtr=dsqrt(y(17))
       rmQ =dsqrt(y(18))
       rmer =dsqrt(y(24))
       rmel=dsqrt(y(25))
       rmdr=dsqrt(y(26))
       rmur=dsqrt(y(27))
       rmuQ=dsqrt(y(28))
c
       B = y(19)
       rmu = sgn(mu)*dexp(y(23))
       rmino1=dexp(y(20))
       rmino2=dexp(y(21))
       rmino3=dexp(y(22))
c 
c 3rd gen. sfermions
      MSL=rmL
      MTAUR=rmtaur
      MSQ=rmQ
      MTR=rmtr
      MBR=rmbr
c 1st, 2d gen. sfermions
      MEL=rmeL
      MER=rmer
      MUQ=rmuQ
      MUR=rmur
      MDR=rmdr

      AL=Atau
      AU=Atop
      AD=Ab
      MU=rmu
       tbeta=vu/vd
      M1=rmino1
      M2=rmino2
      M3=rmino3
      BETA = datan(tgbeta)
c 
                 endif
c
c################################
c   SUSY tau,b,t RAD. CORR. section
c################################
       if(ichoice(7).eq.3) then
c Incorporating leading susy RC to gluino mass:  
c***
       call ginocr(alsewsb,rmino3,gmsu(1),gmsu(2),
     . gmsd(1),gmsd(2),gmsb(1),gmsb(2),gmst(1),gmst(2), delgino)
c       Mgluino = M3 +delgino
       Mgluino = M3/(1.d0 -delgino/M3)
       else
       Mgluino= M3
       endif
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         if(ichoice(7).ge.1) then
c   incorporating mb,mt,mtau susy radiative corrections:
      if(irge.eq.1) then
       mtausave = rmtau
       mbsave = rmb
       mtsave= rmtop
       endif
       call bmsusycr(alphas,ytop,AU,tbeta,mu,m2,mgluino,gmsb(1),
     . gmsb(2),gmst(1),gmst(2),  delmb)
c Now susy RC to tau and top  masses:
       call taumscr(tgbeta,mu,m2,gmsn(3),  delmtau)
c
       call topmscr(alphas,mgluino,gmst(1),gmst(2),  delmtop)
c   
c  Note!! normally in suspect mb etc are at this stage RUNNING masses:
c having taken into account susy R.C. in fact redefines yukawas
c (we assume the top, b, tau pole masses do not change, within exp.acc.)
       if(irge.lt.irgmax) then
c  redefining running mtau,mb,mtop masses and Yuk. cplgs at Z scale:
       rmtau = mtausave-delmtau
       rmb= mbsave/(1.d0-delmb/mbsave)
c (i.e. geometric "resummation" may be relevant for large tanbeta)
       rmtop = mtsave-delmtop
       y(4) = rmtau/vd
       y(5) = rmb/vd
       y(6) = rmtop/vu
c 
c  now this will redefine Yukawas at high scale as well: 
       else
c means this is final step, no longer need to redefine yukawas
c define final mtau, mb, mtop masses including R.C.:
       mtaucr= rmtau+delmtau
       mbcr= rmb*(1.d0-delmb/mbsave)
       mtcr = (rmtop+delmtop)*(1.d0+alphas/3/pi*(6*dlog(mt/mt)+5.d0)+
     . 1.11*alphas**2)         
       endif
       mtaurun = rmtau
       mbrun = rmb
       mtrun = rmtop
              if(irge.lt.irgmax) then
c saving some parameters for threshold effects in RGE :
      dmhu2=rmhu2
      dmhd2=rmhd2
       dm1=rmino1
       dm2=rmino2
       dm3=rmino3
      dtgbeta=tgbeta
      dma=ma
      dml=ml
      dmh=mh
      dmch=mch
      dmc1=gmc(1)
      dmc2=gmc(2)
      dmn1=gmn(1)
      dmn2=gmn(2)
      dmn3=gmn(3)
      dmn4=gmn(4)
      dmst1=gmst(1)
      dmst2=gmst(2)
      dmsu1=gmsu(1)
      dmsu2=gmsu(2)
      dmsb1=gmsb(1)
      dmsb2=gmsb(2)
      dmsd1=gmsd(1)
      dmsd2=gmsd(2)
      dmsl1=gmsl(1)
      dmsl2=gmsl(2)
      dmse1=gmse(1)
      dmse2=gmse(2)
      dmsn1=gmsn(1)
      dmsntau=gmsn(3)
c 
      dMSL      = msl
      dMTAUR    = mtaur
      dMSQ      = msq
      dMTR      = mtr
      dMBR      = mbr
      dMEL      = mel
      dMER      = mer
      dMUQ      = muq
      dMUR      = mur
      dMDR      = mdr
      dAL       = al
      dAU       = au
      dAD       = ad
      dMA       = ma
      dMU       = mu      

              goto 44 
              endif
       else
c mean no RC required
        mtcr=mt
        mbcr=mb
        mtaucr=mtau
         endif
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
 801              if(input.ne.11) then
C ************  SUSPECT OUTPUT WRITING (in SUSPECT.out)
        if(ichoice(1).eq.10) then  
        write(nout,'(a)')'             SUSPECT2.0 OUTPUT: MSUGRA CASE'
        write(nout,'(a)')'             ------------------------------'
        write(nout,'(a)')
        else
        write(nout,'(a)')'        SUSPECT2.0 OUTPUT: NON-UNIVERSAL CASE'
        write(nout,'(a)')'        -------------------------------------'
        write(nout,'(a)') 
       endif
        write(nout,'(a)')'INPUT VALUES:'
        write(nout,'(a)')'-------------'
        if(ichoice(1).eq.10) then
        write(nout,580)'m0','m1/2','A0','TANBETA','sign(MU)'
            write(nout,102) rm0,rmhalf,A0,tgbet0,sgnmu0
       write(nout,'(a)')
       endif
        write(nout,581)'MTOP','MBOTTOM','MTAU','1/ALPHA','SW2(MZ)',
     . 'ALPHAS' 
             write(nout,104) mt,mb0,mtau,alfinv,sw2,alphas0 
       write(nout,'(a)')
        write(nout,582)'M(GUT)','M(EWSB)','ELOW'
       if(ichoice(3).eq.0) then
            write(nout,105) ehigh,dsqrt(ewsb2),elow
       write(nout,'(a)')
       else if(ichoice(3).eq.1) then
            write(nout,105) egut,dsqrt(ewsb2),elow
       write(nout,'(a)')
       endif
        if(ichoice(1).eq.1) then        
       write(nout,'(a)')'INPUT SOFT TERMS AT M(GUT)'
       write(nout,'(a)')'---------------------------------------'
       write(nout,584)'MHU**2','MHD**2','MU','MA','TANBETA'
            write(nout,102) mhu20,mhd20,mu0,MA,tgbet0
c            write(nout,'(a)')
       write(nout,585)'M1','M2','M3'
            write(nout,105) m10,m20,m30
c            write(nout,'(a)')
            write(nout,586) 'm_eR','m_eL','m_dR','m_uR','m_qL' 
            write(nout,102) mer0,mel0,mdr0,mur0,muq0
c            write(nout,'(a)')
       write(nout,587)'m_tauR','m_tauL','m_bR','m_tR','m_QL'
            write(nout,102) mtaur0,msl0,mbr0,mtr0,msq0
c       write(nout,'(a)')
       write(nout,588)'Atau','Abottom','Atop'
            write(nout,105) al0,ad0,au0
c            write(nout,'(a)')
            endif
       if(ichoice(1).eq.1.or.ichoice(1).eq.10) then
        write(nout,'(a)')'COUPLINGS AND MASSES AT M(GUT):'
        write(nout,'(a)')'------------------------------'
        write(nout,583)'MTOP','MBOTTOM','MTAU','g1**2','g2**2','g3**2'
        write(nout,104) mtgut,mbgut,mtaugut,ysave(1),ysave(2),ysave(3) 
       endif
            write(nout,'(a)')
        write(nout,'(a)')'RUNNING top, b, tau MASSES (incl. SUSY RC):'
        write(nout,'(a)')'------------------------------'
        write(nout,105)  mtrun, mbrun, mtaurun  
       write(nout,'(a)')
         if(ichoice(1).eq.10) then
       write(nout,'(a)')'SOFT TERMS AT M(EWSB) and MIXING ANGLES'
       write(nout,'(a)')'---------------------------------------'
       write(nout,584)'MHU**2','MHD**2','MU','B','TANBETA','ALPHA'
            write(nout,104) rmhu2,rmhd2,mu,B,tgbeta,alfa
            write(nout,'(a)')
       write(nout,585)'M1','M2','M3'
            write(nout,105) rmino1,rmino2,rmino3
            write(nout,'(a)')
            write(nout,586) 'm_eR','m_eL','m_dR','m_uR','m_qL' 
            write(nout,102) rmer,rmel,rmdr,rmur,rmuq
            write(nout,'(a)')
       write(nout,587)'m_tauR','m_tauL','m_bR','m_tR','m_QL'
            write(nout,102) rmtaur,rml,rmbr,rmtr,rmq
       write(nout,'(a)')
       write(nout,588)'Atau','Abottom','Atop','thetaTau','thetaB',
     . 'thetaT'
            write(nout,104) atau,ab,atop,thel,theb,thet
            write(nout,'(a)')
         else
       write(nout,'(a)')'SOFT TERMS AT M(EWSB) and MIXING ANGLES'
       write(nout,'(a)')'---------------------------------------'
       write(nout,584)'MHU**2','MHD**2','MU','B','TANBETA','ALPHA'
            write(nout,104) rmhu2,rmhd2,mu,B,tgbeta,alfa
c            write(nout,'(a)')
       write(nout,585)'M1','M2','M3'
            write(nout,105) rmino1,rmino2,rmino3
c            write(nout,'(a)')
            write(nout,586) 'm_eR','m_eL','m_dR','m_uR','m_qL' 
            write(nout,102) rmer,rmel,rmdr,rmur,rmuq
c            write(nout,'(a)')
       write(nout,587)'m_tauR','m_tauL','m_bR','m_tR','m_QL'
            write(nout,102) rmtaur,rml,rmbr,rmtr,rmq
c       write(nout,'(a)')
       write(nout,588)'Atau','Abottom','Atop','thetaTau','thetaB',
     . 'thetaT'
            write(nout,104) atau,ab,atop,thel,theb,thet
            write(nout,'(a)')
        endif
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        write(nout,'(a)')'FINAL HIGGS AND SPARTICLE MASSES: '
        write(nout,'(a)')'--------------------------------- '
        if(ma2.gt.0.d0) then
        write(nout,589)'h','A','H','H+'
        write(nout,101) ml, ma, mh, mch
        else
        write(nout,'(a)')
        write(nout,'(a)')'MA**2 <0! NO further Higgs masses calculated'
        endif
        write(nout,'(a)')
        write(nout,590)'chi+_1','chi+_2','chi0_1','chi0_2','chi0_3',
     . 'chi0_4' 
            write(nout,104) gmc(1),gmc(2),xmn(1),xmn(2),xmn(3),xmn(4)
        write(nout,'(a)')
        write(nout,582)'gluino'
            write(nout,106) mgluino
        write(nout,'(a)')
        write(nout,591)'stop_1','stop_2','sup_1','sup_2'
            write(nout,101) gmst(1),gmst(2),gmsu(1),gmsu(2)
        write(nout,'(a)')
        write(nout,592)'sbot_1','sbot_2','sdown_1','sdown_2'
            write(nout,101) gmsb(1),gmsb(2),gmsd(1),gmsd(2)                  
                 write(nout,'(a)')
        write(nout,593)'stau_1','stau_2','snutau','selec_1','selec_2',
     .'snuelec'
        write(nout,104) gmsl(1),gmsl(2),gmsn(3),gmse(1),gmse(2),gmsn(1)
        write(nout,'(a)')
        write(nout,'(a)')
      if(iknowl.ne.0) then
        write(nout,'(a)')'WARNING/ERRORS FLAGS: errmess(1)-(10):'
        write(nout,'(a)')'--------------------------------------'
        write(nout,595) (errmess(ierr),ierr=1,10)
        write(nout,'(a)')'--------------------------------------'
        write(nout,'(a)')'errmess(i)= 0: Everything is fine.'
        write(nout,'(a)')'errmess(1)=-1: tachyon 3rd gen. sfermion from 
     .RGE'
        write(nout,'(a)')'errmess(2)=-1: tachyon 1,2 gen. sfermion from 
     .RGE'
        write(nout,'(a)')'errmess(3)=-1: tachyon A (maybe temporary: 
     .see final mass) '
        write(nout,'(a)')'errmess(4)=-1: tachyon 3rd gen. sf from mix.'
        write(nout,'(a)')'errmess(5)=-1: MU(GUT) guess inconsistent '
        write(nout,'(a)')'errmess(6)=-1: unconvergent MU from EWSB '
        write(nout,'(a)')'errmess(7)=-1: EWSB maybe inconsistent 
     .(!but RG-improved only check)' 
        write(nout,'(a)')'errmess(8)=-1: V_Higgs maybe UFB or CCB 
     .(!but RG-improved only check)' 
        write(nout,'(a)')'errmess(9)=-1: Higgs masses are NaN '
        write(nout,'(a)')'errmess(10)=-1: RGE problems (non-pert and/or
     .Landau poles)'
      if(errmess(1).eq.-1.d0) then
      write(nout,'(a)') 'Bad input: one m^2(3rd gen. sf) <0 from RGE '
      write(nout,'(a)') 'maybe artefact of algorithm, see final result'
      endif
      if(errmess(2).eq.-1.d0) then
      write(nout,'(a)') 'Bad input: one m^2(1,2 gen. sf) <0 from RGE '
      write(nout,'(a)') 'maybe artefact of algorithm, see final result'
      endif
       if(errmess(1).eq.-1.d0.or.errmess(2).eq.-1.d0) then
       write(nout,'(a)')' Tachyonic RGE: NO further OUTPUT Generated '
       goto 900
        endif
      if(errmess(3).eq.-1.d0) then
      write(nout,'(a)') 'Warning:  MA^2 <0: May be temporary '
      write(nout,'(a)') 'check final results '
      endif
      if(errmess(4).eq.-1.d0) then
      write(nout,'(a)') 'STOP: one tachyonic m^2(3rd gen. sf) <0 '
      write(nout,'(a)') 'NO further OUTPUT Generated '
      goto 900
      endif
      if(errmess(5).eq.-1.d0) then
      write(nout,'(a)')' Warning: initial rmu0(HIGH) inconsistent. '
      write(nout,'(a)')' has been recalculated consistently '
      endif
      if(errmess(6).eq.-1.d0) then
      write(nout,'(a)')'WARNING: EWSB unconvergent after 20 iter.' 
      endif
      if(errmess(7).eq.-1.d0) then
      write(nout,'(a)')'EW Sym. Break. may be not realized '
      write(nout,'(a)')'(however from naive tree-level analysis) '
      endif
      if(errmess(8).eq.-1.d0) then
      write(nout,'(a)')'Potential may be unbounded from below '
      write(nout,'(a)')'(however from naive tree-level analysis) '
      endif
      if(errmess(9).eq.-1.d0) then
      write(nout,'(a)')' PROBLEM: some Higgs masses are NaN! '
      endif
      if(errmess(10).eq.-1.d0) then
      write(nout,'(a)')' STOP: serious RGE pb! (non-pert. or Landau)'
      goto 900
      endif
      endif
c
c saving some parameters for output:
       mhu2=rmhu2
       mhd2=rmhd2
       m1=rmino1
       m2=rmino2
       m3=rmino3
      dtgbeta=tgbeta
      dma=ma
      dml=ml
      dmh=mh
      dmch=mch
      alfa = a
      dmc1=gmc(1)
      dmc2=gmc(2)
      dmn1=gmn(1)
      dmn2=gmn(2)
      dmn3=gmn(3)
      dmn4=gmn(4)
      dmst1=gmst(1)
      dmst2=gmst(2)
      dmsu1=gmsu(1)
      dmsu2=gmsu(2)
      dmsl1=gmsl(1)
      dmsl2=gmsl(2)
      dmse1=gmse(1)
      dmse2=gmse(2)
      dmsn1=gmsn(1)
c 
c      endif

 580    FORMAT(2x,A2,9X,A4,7X,A2,9X,A7,4X,A8,3X)
 581    FORMAT(2x,A4,7X,A7,4X,A4,7X,A7,4X,A7,3x,A6,5x)
 582    FORMAT(2x,A6,5X,A7,4X,A4,9X)
 583    FORMAT(2x,A4,7X,A7,4X,A4,6X,A5,6X,A5,6x,A5,6x)
 584    FORMAT(2x,A6,5X,A6,5X,A2,9X,A2,9X,A7,4x,A5,6x)
 585    FORMAT(2x,A2,9X,A2,9X,A2,9X)
 586    FORMAT(2x,A4,7X,A4,7X,A4,7X,A4,7X,a4,7x)
 587    FORMAT(2x,A6,5X,A6,5X,A4,7X,A4,7X,a4,7x)
 588    FORMAT(2x,A4,7X,A7,4X,A4,6X,A9,3X,a6,5x,a6,5x)
 589    FORMAT(2x,A1,10X,A1,10X,A1,10X,A2,3X)
 590    FORMAT(2x,A6,5X,A6,5X,A6,5X,A6,5X,A6,5X,a6,5X)
 591    FORMAT(2x,A6,5X,A6,5X,A5,5X,A6,6X)
 592    FORMAT(2x,A6,5X,A6,5X,A7,4X,A7,4X)
 593    FORMAT(2x,A6,5X,A6,5X,A6,5X,A7,4X,A7,4x,A7)
 595    format(10(f3.0))
 105   format(3(g11.4))
 101   format(4(g11.4))
 102   format(5(g11.4))
 103   format(2(g11.4))
 104   format(6(g11.4))
 106   format(1(g11.4))
c
 13    format(1x,'m(phi_u)^2, m(phi_d)^2 ',g14.6,1x,g14.6)
 777  format(3(g14.6))
      write(*,'(a)')' RUN TERMINATED : OUTPUT in suspect.out'
      CLOSE(NOUT)      
                 endif
c saving some parameters for loops etc
      mhu2=rmhu2
      mhd2=rmhd2
       m1=rmino1
       m2=rmino2
       m3=rmino3
      dtgbeta=tgbeta
      dma=ma
      dml=ml
      dmh=mh
      dmch=mch
      dmc1=gmc(1)
      dmc2=gmc(2)
      dmn1=gmn(1)
      dmn2=gmn(2)
      dmn3=gmn(3)
      dmn4=gmn(4)
      dmst1=gmst(1)
      dmst2=gmst(2)
      dmsu1=gmsu(1)
      dmsu2=gmsu(2)
      dmsb1=gmsb(1)
      dmsb2=gmsb(2)
      dmsd1=gmsd(1)
      dmsd2=gmsd(2)
      dmsl1=gmsl(1)
      dmsl2=gmsl(2)
      dmse1=gmse(1)
      dmse2=gmse(2)
      dmsn1=gmsn(1)
      dmsntau=gmsn(3)
c  final interface for all soft terms:
      dMHU2     = mhu2
      dMHD2     = mhd2
      dM1       = m1
      dM2       = m2
      dM3       = m3
      dMSL      = msl
      dMTAUR    = mtaur
      dMSQ      = msq
      dMTR      = mtr
      dMBR      = mbr
      dMEL      = mel
      dMER      = mer
      dMUQ      = muq
      dMUR      = mur
      dMDR      = mdr
      dAL       = al
      dAU       = au
      dAD       = ad
      dMA       = ma
      dMU       = mu
 900   continue      
       end
c-------------------------------------------------
       subroutine ginocr(alphas,m3,msu1,msu2,msd1,msd2,
     . msb1,msb2,mst1,mst2,  delgino) 
c susy leading corrections to gluino mass 
c OUTPUT: delgino is the (additive) correction to running mass M3 
      implicit real*8(a-h,m,o-z)
      complex*16 SB1,SB0
       common/renscale/scale
      COMMON/PARAM/GF,ALPH,MZ,MW
      common/s_masses/MTAU,MB,MT
      common/mixang/thet,theb,thel
      common/treesfer/msbtr1,msbtr2,msttr1,msttr2
       common/stepwi/wistep,h1,kpole
      if(kpole.eq.1) scale = dsqrt(msttr1*msttr2)
       pi=4*datan(1.d0)
       mu=.005d0
       md=.015d0
       ms=.2d0
       mc=1.7d0
       msc1=msu1
       msc2=msu2
       mss1=msd1
       mss2=msd2
c       
       sumB1= 
     .        dble(SB1(M3**2,mu,msu1))+dble(SB1(M3**2,mu,msu2) )
     .       +dble(SB1(M3**2,md,msd1) )+dble(SB1(M3**2,md,msd2) )
     .       +dble(SB1(M3**2,ms,mss1) )+dble(SB1(M3**2,ms,mss2) )
     .       +dble(SB1(M3**2,mc,msc1) )+dble(SB1(M3**2,mc,msc2) )
     .       +dble(SB1(M3**2,mb,msb1) )+dble(SB1(M3**2,mb,msb2) )
     .       +dble(SB1(M3**2,mt,mst1) )+dble(SB1(M3**2,mt,mst2) )
       sumB0= mb*dsin(2*theb)*
     .       (dble(SB0(M3**2,mb,msb1))-dble(SB0(M3**2,mb,msb2)) )
     .       +mt*dsin(2*thet)*        
     .       (dble(SB0(M3**2,mt,mst1))-dble(SB0(M3**2,mt,mst2)) )  
c
        delgino =3*alphas/pi/4*M3*(5.d0-6*dlog(M3/scale))
     .          -alphas/pi/4*M3*sumB1
     .          +alphas/pi/4*sumB0
       end
c---------------------------------------------------------------
      subroutine radcino(ml1,mq1,mq3,mu3,md3,ma,yt,yb,m1,m2,mu,tb,
     .            rcm1,rcm2,rcmu)
c Radiative corrections to the gaugino parameters m1,m2 and mu
c Based on expressions in Pierce, Bagger, Matchev, Zhang 
c hep-ph/9606211
c NB valid only in the limit given in the paper by Pierce et al.  
      implicit real*8(a-h,m,o-z)
      common/param/gf,alph,mz,mw 
      common/renscale/scale
      common/treesfer/msbtr1,msbtr2,msttr1,msttr2
       common/stepwi/wistep,h1,kpole
      complex*16 SB1,SB0
c  fix re. scale (used in SB0, SB1):
      if(kpole.eq.1) scale = dsqrt(msttr1*msttr2)
      pi=4*datan(1.d0)
      cw=mw/mz
      sw=dsqrt(1.d0-cw**2)
      b=datan(tb)
      ep=1.d-5
      amu =dabs(mu)
c
c      scale= m1
      rm1=11.d0*dble(SB1(M1**2,ep,mq1))+9.d0*dble(SB1(M1**2,ep,ml1))
     .+mu/m1*dsin(2*b)*(dble(SB0(M1**2,amu,ma))-dble(SB0(M1**2,amu,mz)))
     . +dble(SB1(M1**2,amu,ma))+dble(SB1(M1**2,amu,mz))
      rcm1=-alph/(4.d0*pi*cw**2)*rm1   *M1
c
c      scale = m2
      rm2=9.d0*dble(SB1(M2**2,ep,mq1))+3.d0*dble(SB1(M2**2,ep,ml1))
     .+mu/m2*dsin(2*b)*(dble(SB0(M2**2,amu,ma))-dble(SB0(M2**2,amu,mz)))
     . +dble(SB1(M2**2,amu,ma))+dble(SB1(M2**2,amu,mz))
     . -4.d0*(2.d0*dble(SB0(M2**2,m2,mw))-dble(SB1(M2**2,m2,mw)))
      rcm2=-alph/(4.d0*pi*sw**2)*rm2    *M2
c
c      scale = dabs(mu)
      rmu1=(yb**2+yt**2)*dble(SB1(aMU**2,ep,mq3))
     .    + yt**2*dble(SB1(aMU**2,ep,mu3))    
     .    + yb**2*dble(SB1(aMU**2,ep,md3))    
      rmu2=dble(SB1(aMU**2,m2,ma))+dble(SB1(aMU**2,m2,mz))
     .    +2.d0*dble(SB1(aMU**2,amu,mz))-4.d0*dble(SB0(aMU**2,amu,mz))
      rcmu=(-3.d0/(16*pi**2)*rmu1-3*alph/(16*pi*sw**2)*rmu2 ) *MU
      end
c-------------------------------------------------
       subroutine bmsusycr(alphas,yt,AT,tgbeta,mu,m2,mgluino,msb1,msb2,
     . mst1,mst2,  delmb)
c susy leading corrections to b mass
c  based on Bagger et al hep-ph/9606211
c OUTPUT: dmb is (susy only) (additive) correction to mb
      implicit real*8(a-h,m,o-z)
      complex*16 SB0
      COMMON/PARAM/GF,ALPH,MZ,MW
      common/fmasses/MTAU,MB,MT
      common/treesfer/msb1tr,msb2tr,mst1tr,mst2tr
      common/treeang/thettr,thebtr
      common/mixang/thet,theb,thel
      common/renscale/scale
      B11(x)= .5d0*(.5d0+1.d0/(1.d0-x)+dlog(x)/(1.d0-x)**2)
      B12(x)= .5d0*(.5d0+1.d0/(1.d0-x)+dlog(x)/(1.d0-x)**2-dlog(x))
c   fix ren. scale (used in SB0, SB1):
c !! caution: since threshold are included in beta-RG functions,
c one should remove from here all RG log terms !:
c 
      scalsave=scale
      scale= mz      
c !!! ren. scale should = MZ always for b mass R.C. 
c    NOTE : Normally in SUSPECT MB is redefined to be running b mass!
c    this susy R. C. contributes to mb pole
c NB remember scale is ren. scale, SB0,SB1 implicitly depend on it
       pi=4*datan(1.d0)
       sw2= 1.d0-(mw/mz)**2
       B = datan(tgbeta)
       ct2=dcos(thet)**2
       st2=dsin(thet)**2
c
c use only tree-level masses and mix. angle in 1-loop order:
       msb1save=msb1
       msb2save=msb2
       mst1save=mst1
       mst2save=mst2
       thebsave=theb
c       
       msb1 =msb1tr
       msb2 =msb2tr
       mst1 =mst1tr
       mst2 =mst2tr
       theb=thebtr
c
       x1= (msb1/mgluino)**2
       x2= (msb2/mgluino)**2
       if(x1.ge.1.d0) then
       r1= b11(x1)
       else
       r1=b12(x1)
       endif
        if(x2.ge.1.d0) then
       r2= b11(x2)
       else
       r2=b12(x2)
       endif
       ginosq = -alphas/pi/3*(r1+r2 -mgluino/mb*dsin(2*theb)*
     . (dble(SB0(mb**2,mgluino,msb1))-dble(SB0(mb**2,mgluino,msb2)) ))
        cinost = -yt**2/pi**2/8*mu*(AT*tgbeta-mu)/(mst1**2-mst2**2)*
     . (dble(SB0(mb**2,mu,mst1))-dble(SB0(mb**2,mu,mst2)) )   
     . -alph/pi/4/sw2*mu*m2*tgbeta/(mu**2-m2**2)*
     . (ct2*dble(SB0(mb**2,m2,mst1))+st2*dble(SB0(mb**2,m2,mst2)) 
     . -ct2*dble(SB0(mb**2,mu,mst1)) -st2*dble(SB0(mb**2,mu,mst2)) ) 
       delmb= -mb*(ginosq +cinost)
c
       msb1=msb1save
       msb2=msb2save
       mst1=mst1save
       mst2=mst2save
       theb=thebsave
c
       scale=scalsave
       end
c-------------------------------------------------
       subroutine topmscr(alphas,mgluino,mst1,mst2,  delmtop)
c susy leading corrections to top mass
c  
c  based on Bagger et al hep-ph/9606211
c OUTPUT: delmtop is (susy only) (additive) correction to mtop
      implicit real*8(a-h,m,o-z)
      complex*16 SB0
c      complex*16 SB1
      COMMON/PARAM/GF,ALPH,MZ,MW
      common/fmasses/MTAU,MB,MT
      common/mixang/thet,theb,thel
      common/renscale/scale
      B11(x)= .5d0*(.5d0+1.d0/(1.d0-x)+dlog(x)/(1.d0-x)**2)
      B12(x)= .5d0*(.5d0+1.d0/(1.d0-x)+dlog(x)/(1.d0-x)**2-dlog(x))
c   fix ren. scale (used in SB0, SB1):
      scalsave=scale
      scale= mt    
c!!! ren. scale should be = MT always for top mass R.C. 
c
      mt0=mt
c
c
       pi=4*datan(1.d0)
       sw2= 1.d0-(mw/mz)**2
       ct2=dcos(thet)**2
       st2=dsin(thet)**2
c
c
       x1= (mst1/mgluino)**2
       x2= (mst2/mgluino)**2
       if(x1.ge.1.d0) then
       r1= b11(x1)
       else
       r1=b12(x1)
       endif
        if(x2.ge.1.d0) then
       r2= b11(x2)
       else
       r2=b12(x2)
       endif
       ginost = -alphas/pi/3*(r1+r2 -mgluino/mt*dsin(2*thet)*
     . (dble(SB0(mt0**2,mgluino,mst1))-dble(SB0(mt0**2,mgluino,mst2))))
c        cinost = -alphas/pi/3*(dble(SB1(mt0**2,mgluino,mst1))
c     .  +dble(SB1(mt0**2,mgluino,mst2)) )
       cinost=0.d0
       delmtop= mt*(ginost +cinost)
       scale=scalsave
       end
c-------------------------------------------------
       subroutine taumscr(tgbeta,mu,m2,mnstau,  delmtau)
c OUTPUT: delmtau is (susy only) (additive) correction to mtau
c  base on Bagger et al hep-ph/9606211
      implicit real*8(a-h,m,o-z)
       complex*16 SB0
      COMMON/PARAM/GF,ALPH,MZ,MW
      common/fmasses/MTAU,MB,MT
      common/renscale/scale
c   fix ren. scale (used in SB0, SB1):
      scalsave=scale
      scale = mz   
c!!! ren. scale should be = MZ always for tau mass R.C. 

c
c
       pi=4*datan(1.d0)
       sw2= 1.d0-(mw/mz)**2
c
       cinostau=  
     . alph/pi/4/sw2*mu*m2*tgbeta/(mu**2-m2**2)*
     . (dble(SB0(mtau**2,m2,mnstau))-dble(SB0(mtau**2,mu,mnstau)) ) 
       delmtau= mtau*cinostau
       scale=scalsave
       end
c----------------------------------------------------
       subroutine sqcr(alphas,mgluino,msquark,   dmsquark)
c OUTPUT: dmsquark is LIGHT (I.e. not stop) squark leading susy R.C.
c  based on Bagger et al hep-ph/9606211
      implicit real*8(a-h,m,o-z)
      common/renscale/scale
      common/tachyrc/tachsqrc
      COMMON/PARAM/GF,ALPH,MZ,MW
      common/treesfer/msbtr1,msbtr2,msttr1,msttr2
       common/stepwi/wistep,h1,kpole
c   fix ren. scale (used in SB0, SB1):
       if(kpole.eq.1) scale = dsqrt(msttr1*msttr2)    
c
       pi=4*datan(1.d0)
       x=(mgluino/msquark)**2
       corr2=2*alphas/pi/3*(1.d0+3*x+(x-1.d0)**2*dlog(dabs(x-1.d0))
     . -x**2*dlog(x)+4*x*dlog(scale/msquark) )
       if(corr2.gt.-1.d0) then
       corr = dsqrt(1.d0+corr2)-1.d0
       dmsquark = msquark*corr
       else
       dmsquark =0.d0
       tachsqrc= -1.d0
       endif
       end
c---------------------------------------------------------
      subroutine stopcr(alphas,mgluino,mu,MA,thet,bet,mst1,mst2,msb,
     . yt,yb,AT,AB, crLL,crLR,crRR)
c OUTPUT: crLL, crLR,crRR leading susy R.C. to stop mass mat. elts.
c  base on Bagger et al hep-ph/9606211
c NB difference of convention: their Yukawa's = sqrt(2)* us 
      implicit real*8(a-h,m,o-z)
      complex*16 SB0,SB1
      common/renscale/scale     
      COMMON/PARAM/GF,ALPH,MZ,MW
      common/fmasses/MTAU,MB,MT
       common/stepwi/wistep,h1,kpole
c   fix ren. scale (used in SB0, SB1):
      if(kpole.eq.1) scale = dsqrt(mst1*mst2)     
c
       pi=4*datan(1.d0)
       cpi=1.d0/16/pi**2
       if(mst1.eq.0.d0.or.msb.eq.0.d0) goto 100
c (NB protection: means mst1 or msb undefined yet)        
       zero=1.d-2
       zero2=zero**2
       ct=dcos(thet)
       st=dsin(thet)
       ct2=ct*ct
       st2=st*st
       cb=dcos(bet)
       sb=dsin(bet)
       cb2=cb*cb
       sb2=sb*sb
       crLL =2*alphas/pi/3*(2*mst1**2*(st2*dble(SB1(mst1**2,mst2,zero))+
     . ct2*dble(SB1(mst1**2,mst1,zero)) )
     .+A(mgluino)+A(mt)-(mst1**2-mgluino**2-mt**2)*
     .  dble(SB0(zero2,mgluino,mt)))
     . -cpi*(2*yt**2*ct2*A(mst2)+2*yb**2*A(msb)
     . -4*(yt**2+yb**2)*A(mu)+(2*yt**2*cb2+2*yb**2*sb2)*A(ma)  )
     . -cpi*yt**2*(  crlam(mt,mu,at,thet,bet)*dble(SB0(zero2,mst2,ma))
     . +crlam(mt,mu,at,thet-pi/2,bet)*dble(SB0(zero2,zero,ma))
     . +crlam(mt,mu,at,thet,bet-pi/2)*dble(SB0(zero2,mst2,zero))
     . +crlam(mt,mu,at,thet-pi/2,bet-pi/2)*dble(SB0(mst1**2,mst1,mz)) )
     .-cpi*((2*yt**2*mt**2*cb2+2*yb**2*(mu*cb+ab*sb)**2)*
     . dble(SB0(zero2,msb,ma))
     .+(2*yt**2*mt**2*sb2+2*yb**2*(mu*sb-ab*cb)**2)*
     .  dble(SB0(zero2,msb,zero)) )
c
      crRR=2*alphas/pi/3*(2*mst1**2*(ct2*dble(SB1(mst1**2,mst2,zero))+
     . st2*dble(SB1(mst1**2,mst1,zero)) )
     . +A(mgluino)+A(mt)-(mst1**2-mgluino**2-mt**2)*
     . dble(SB0(zero2,mgluino,mt)) )
     . -cpi*2*yt**2*(  st2*A(mst2)+A(msb)-4*A(mu)+2*cb2*A(ma)  )
     . -cpi*yt**2*(  crlam(mt,mu,at,pi/2-thet,bet)*
     .  dble(SB0(zero2,mst2,ma) )
     . +crlam(mt,mu,at,-thet,bet)*dble(SB0(zero2,zero,ma))
     . +crlam(mt,mu,at,pi/2-thet,bet-pi/2)*dble(SB0(zero2,mst2,zero))
     . +crlam(mt,mu,at,-thet,bet-pi/2)*dble(SB0(mst1**2,mst1,mz)) )
     . -cpi*((2*yb**2*mt**2*sb2+2*yt**2*(mu*sb+at*cb)**2)*
     . dble(SB0(zero2,msb,ma))
     .  + 2*yt**2*(mu*cb-at*sb)**2*dble(SB0(zero2,msb,zero))  )
c
      crLR =2*alphas/pi/3*ct*st*((mst1**2+mst2**2)*
     . dble(SB0(mst1**2,mst2,zero))
     .+2*mst1**2*dble(SB0(mst1**2,mst1,zero)) ) 
     . -4*alphas/pi/3*mt*mgluino*dble(SB0(zero2,mt,mgluino))
     . +6*cpi*yt**2*ct*st*A(mst2)
     . -cpi*yt**2*(  cromeg(mt,mu,at,thet,bet)*
     .  dble(SB0(zero2,mst2,ma))
     . +cromeg(mt,mu,at,-thet,bet)*dble(SB0(zero2,zero,ma))
     . +cromeg(mt,mu,at,thet,bet+pi/2)*dble(SB0(zero2,mst2,zero))
     . +cromeg(mt,mu,at,-thet,bet+pi/2)*dble(SB0(mst1**2,mst1,mz))  )
     .-cpi*((2*yt**2*mt*cb*(mu*sb+at*cb)+2*yb**2*mt*sb*(mu*cb+ab*sb))
     .  *dble(SB0(zero2,msb,ma))
     .  -2*yt**2*mt*sb*(mu*cb-at*sb)*dble(SB0(zero2,msb,zero))    )
c
 100  continue
      end
c-------------------------------------------------
       real*8 function crlam(mt,mu,At,t,b)
      implicit real*8(a-h,m,o-z)
      crlam =(2*mt*dcos(b)*dsin(t)-(mu*dsin(b)+At*dcos(b))*dcos(t))**2
     . +(mu*dsin(b)+At*dcos(b))**2*dcos(t)**2
      end
c-------------------------------------------------
       real*8 function cromeg(mt,mu,At,t,b)
      implicit real*8(a-h,m,o-z)
      cromeg = -2*mt**2*dcos(b)**2*dsin(2*t)
     . +2*mt*dcos(b)*(mu*dsin(b)+At*dcos(b))
      end
c-----------------------------------------------------------
      subroutine vloop2(q2,rmt2,rmst12,rmst22,rmb2,rmsb12,rmsb22,
     . rmtau2,rmstau12,rmstau22,rmu,dVdvd2,dVdvu2)
c complete one-loop effective potential (tadpole) expressions needed
c to evaluate MU (among other things)
c  !! IMPORTANT note: as written below, the corrections have been
c     defined such that rmst1,rmsb1,rmstau1 is the LIGHTEST sfermion. 
       implicit real*8(a-h,m,o-z)
       real*8 nf
       dimension u(2,2),v(2,2),z(4,4),dxmn(4)
       common/cte/nf,cpi,zm,wm,tbeta
       common/cteloop/vu,vd,Atop,Ab,Atau,rmllt,rmllb,rmlltau,
     . rmrrt,rmrrb,rmrrtau
       common/soft/rmtaur,rmL,rmbr,rmtr,rmQ
      common/mixang/thet,theb,thel
      COMMON/OUTSQU/dmst1,dmst2,msu1,msu2
      COMMON/OUTSQD/dmsb1,dmsb2,msd1,msd2
      COMMON/OUTSLEP/dmsl1,dmsl2,mse1,mse2,msn1,msntau
      COMMON/HMASS/MA,ML,MH,MCH,MAR
      COMMON/OUTHIGGS/dml,dmh,dmch,alfa 
      COMMON/OUTGINOS/mc1,mc2,mn1,mn2,mn3,mn4,mgluino
      COMMON/matino/U,V,Z,dxmn
       common/cpl/g22,sw2
      common/yukaewsb/ytau,yb,yt,alsewsb
      common/renscale/scale     
c
       scale= dsqrt(q2)      
c basic parameters and definitions used:
       g=dsqrt(g22)
       mz=zm
       mw=wm
       mst1= dsqrt(rmst12)
       mst2= dsqrt(rmst22)
       msb1=dsqrt(rmsb12)
       msb2=dsqrt(rmsb22)
       msta1=dsqrt(rmstau12)
       msta2=dsqrt(rmstau22)
       ct=dcos(thet)
       st=dsin(thet)
       cb=dcos(theb)
       sb=dsin(theb)
       cta=dcos(thel)
       sta=dsin(thel)
c
       cw2= 1.d0-sw2
       cwm2 =1.d0/cw2
       cw=dsqrt(cw2)
       sw=dsqrt(sw2)
       cbeta2=1.d0/(1.d0+tbeta**2)
       cbet= dsqrt(cbeta2)
       sbet=dsqrt(1.d0-cbeta2)
       c2b =2*cbeta2-1.d0
c 
       sal=dsin(alfa)
       cal=dcos(alfa)
       s2a = 2*sal*cal 
       tm=dsqrt(rmt2)
       bm=dsqrt(rmb2)
       taum=dsqrt(rmtau2)
c       yt = tm/vu
c       yb=bm/vd
c       ytau = taum/vd
c ! yt, yb, ytau at ewsb scale are taken from common/yukaewsb/..
c relevant Sfermion couplings contributions:
c
       s1bLbL = g*mz/cw*(-.5d0 +sw2/3)*cbet +2*yb*bm
       s1bRbR = g*mz/cw*(+sw2/3)*cbet +2*yb*bm
       s1bLbR = yb*Ab
       s2bLbL = -g*mz/cw*(-.5d0 +sw2/3)*sbet
       s2bRbR = -g*mz/cw*(+sw2/3)*sbet
       s2bLbR = yb*rmu
c
       gs1d1d1 = g*mz/cw*(-.5d0 +sw2/3)*cbet 
       gs1d2d2 = g*mz/cw*(+sw2/3)*cbet 
       gs2d1d1 = -g*mz/cw*(-.5d0 +sw2/3)*sbet
       gs2d2d2 = -g*mz/cw*(+sw2/3)*sbet
c  
       s1tauLL = g*mz/cw*(-.5d0 +sw2)*cbet +2*ytau*taum
       s1tauRR = g*mz/cw*(+sw2)*cbet +2*ytau*taum
       s1tauLR = ytau*Atau
       s2tauLL = -g*mz/cw*(-.5d0 +sw2)*sbet
       s2tauRR = -g*mz/cw*(+sw2)*sbet
       s2tauLR = ytau*rmu
c
       gs1e1e1 = g*mz/cw*(-.5d0 +sw2)*cbet 
       gs1e2e2 = g*mz/cw*(+sw2)*cbet 
       gs2e1e1 = -g*mz/cw*(-.5d0 +sw2)*sbet
       gs2e2e2 = -g*mz/cw*(+sw2)*sbet
c
       s2tLtL = -g*mz/cw*(.5d0 -2*sw2/3)*sbet +2*yt*tm
       s2tRtR = -g*mz/cw*(-2*sw2/3)*sbet +2*yt*tm
       s2tLtR = yt*Atop
       s1tLtL = g*mz/cw*(.5d0 -2*sw2/3)*cbet
       s1tRtR = g*mz/cw*(-2*sw2/3)*cbet
       s1tLtR = yt*rmu
c
       gs2u1u1 = -g*mz/cw*(.5d0 -2*sw2/3)*sbet 
       gs2u2u2 = -g*mz/cw*(-2*sw2/3)*sbet
       gs1u1u1 = g*mz/cw*(.5d0 -2*sw2/3)*cbet
       gs1u2u2 = g*mz/cw*(-2*sw2/3)*cbet
c
       gs2n1n1 = -g*mz/cw*(.5d0 )*sbet 
       gs1n1n1 = g*mz/cw*(.5d0 )*cbet
c
       gs1b1b1 = cb**2*s1bLbL +2*cb*sb*s1bLbR +sb**2*s1bRbR 
       gs1b2b2 = sb**2*s1bLbL -2*cb*sb*s1bLbR +cb**2*s1bRbR 
       gs2b1b1 = cb**2*s2bLbL +2*cb*sb*s2bLbR +sb**2*s2bRbR 
       gs2b2b2 = sb**2*s2bLbL -2*cb*sb*s2bLbR +cb**2*s2bRbR 
c
       gs1t1t1 = ct**2*s1tLtL +2*ct*st*s1tLtR +st**2*s1tRtR 
       gs1t2t2 = st**2*s1tLtL -2*ct*st*s1tLtR +ct**2*s1tRtR 
       gs2t1t1 = ct**2*s2tLtL +2*ct*st*s2tLtR +st**2*s2tRtR 
       gs2t2t2 = st**2*s2tLtL -2*ct*st*s2tLtR +ct**2*s2tRtR 
c
       gs1tau11 = cta**2*s1tauLL +2*cta*sta*s1tauLR +sta**2*s1tauRR 
       gs1tau22 = sta**2*s1tauLL -2*cta*sta*s1tauLR +cta**2*s1tauRR 
       gs2tau11 = cta**2*s2tauLL +2*cta*sta*s2tauLR +sta**2*s2tauRR 
       gs2tau22 = sta**2*s2tauLL -2*cta*sta*s2tauLR +cta**2*s2tauRR 
c
cc  down fermion contributions:
       confd= -2*2*ytau**2*A(ytau*vd) -2*2*3*yb**2*A(yb*vd)
c  up fermion contributions:
       confu = -2*2*3*yt**2*A(yt*vu)
c    vd sfermion contributions:
       dvdsf= g/2/mw/cbet*(3*gs1t1t1*A(mst1)+3*gs1t2t2*A(mst2) +
     . 3*gs1b1b1*A(msb1)+3*gs1b2b2*A(msb2) +   
     . gs1tau11*A(msta1)+ gs1tau22*A(msta2) +
     .  2*( 3*gs1u1u1*A(msu1)+3*gs1u2u2*A(msu2) +
     . 3*gs1d1d1*A(msd1)+3*gs1d2d2*A(msd2) +   
     . gs1e1e1*A(mse1)+ gs1e2e2*A(mse2) ) + 
     .  2*gs1n1n1*A(msn1) +gs1n1n1*A(msntau) )
c    vu sfermion contributions:
       dvusf= g/2/mw/sbet*(3*gs2t1t1*A(mst1)+3*gs2t2t2*A(mst2) +
     . 3*gs2b1b1*A(msb1)+3*gs2b2b2*A(msb2) +   
     . gs2tau11*A(msta1)+ gs2tau22*A(msta2) +
     .  2*( 3*gs2u1u1*A(msu1)+3*gs2u2u2*A(msu2) +
     . 3*gs2d1d1*A(msd1)+3*gs2d2d2*A(msd2) +   
     . gs2e1e1*A(mse1)+ gs2e2e2*A(mse2) ) + 
     .  2*gs2n1n1*A(msn1) +gs2n1n1*A(msntau) )
c   vd Higgs contributions:
       dvdH = -g**2*c2b/cw**2/8*(A(ma)+2*A(mch))+g**2/2*A(mch)
     . + g**2/cw**2/8*(3*sal**2-cal**2+s2a*tbeta)*A(ml)
     . + g**2/cw**2/8*(3*cal**2-sal**2-s2a*tbeta)*A(mh)
c   vd gauge contributions:  
       dvdWZ = 3*g**2/4*(2*A(mw)+A(mz)/cw**2) 
     . +g**2*c2b/cw**2/8*(2*A(mw) +A(mz) )
c   vd gaugino contributions:
       dvdino = -g**2/mw/cbet*
     .  (dxmn(1)*Z(1,3)*(Z(1,2)-Z(1,1)*sw/cw)*A(mn1)
     . + dxmn(2)*Z(2,3)*(Z(2,2)-Z(2,1)*sw/cw)*A(mn2)
     . + dxmn(3)*Z(3,3)*(Z(3,2)-Z(3,1)*sw/cw)*A(mn3)
     . + dxmn(4)*Z(4,3)*(Z(4,2)-Z(4,1)*sw/cw)*A(mn4)  )
     . -dsqrt(2.d0)*g**2/mw/cbet*(mc1*V(1,1)*U(1,2)*A(mc1) 
     .                           +mc2*V(2,1)*U(2,2)*A(mc2)  )
c
c   vu Higgs contributions:
       dvuH = g**2*c2b/cw**2/8*(A(ma)+2*A(mch))+g**2/2*A(mch)
     . + g**2/cw**2/8*(3*cal**2-sal**2+s2a/tbeta)*A(ml)
     . + g**2/cw**2/8*(3*sal**2-cal**2-s2a/tbeta)*A(mh)
c   vu gauge contributions:  
       dvuWZ = 3*g**2/4*(2*A(mw)+A(mz)/cw**2) 
     . -g**2*c2b/cw**2/8*(2*A(mw) +A(mz) )
c   vu gaugino contributions:
       dvuino = g**2/mw/sbet*
     . (dxmn(1)*Z(1,4)*(Z(1,2)-Z(1,1)*sw/cw)*A(mn1)
     . + dxmn(2)*Z(2,4)*(Z(2,2)-Z(2,1)*sw/cw)*A(mn2)
     . + dxmn(3)*Z(3,4)*(Z(3,2)-Z(3,1)*sw/cw)*A(mn3)
     . + dxmn(4)*Z(4,4)*(Z(4,2)-Z(4,1)*sw/cw)*A(mn4)  )
     . -dsqrt(2.d0)*g**2/mw/sbet*(mc1*V(1,2)*U(1,1)*A(mc1) 
     .                           +mc2*V(2,2)*U(2,1)*A(mc2)  )
c
c final contributions:
        dVdvd2 = -cpi*(confd + dvdsf +dvdH +dvdWZ +dvdino)   
        dVdvu2 = -cpi*(confu + dvusf +dvuH +dvuWZ +dvuino)   
        end      
c========================================
C ************* SUBROUTINE FOR THE HIGGS MASSES AND COUPLINGS ***********
C *****************************************************************
      SUBROUTINE SUSYCP(TGBET)
      IMPLICIT DOUBLE PRECISION (A-H,M,O-Z)
      DOUBLE PRECISION LA1,LA2,LA3,LA4,LA5,LA6,LA7,LA3T
      COMPLEX*16 SF0
      DIMENSION MST(2),GLTT(2,2),GHTT(2,2),
     .          MSB(2),GLBB(2,2),GHBB(2,2)
      COMMON/FLAG/NNLO,IPOLE
      common/fmasses/AMTAU,AMB,AMT
      COMMON/HMASS/AMA,AML,AMH,AMCH,AMAR
      COMMON/CHIMASS/AMCHI
      COMMON/HSELF/LA1,LA2,LA3,LA4,LA5,LA6,LA7
      COMMON/BREAK/AMEL,AMER,AMSQ,AMUR,AMDR,AL,AU,AD,AMU,AM1,AM2,AM3
      COMMON/PARAM/GF,ALPH,AMZ,AMW
      COMMON/s_COUP/GAT,GAB,GLT,GLB,GHT,GHB,GZAH,GZAL,
     .            GHHH,GLLL,GHLL,GLHH,GHAA,GLAA,GLVV,GHVV,
     .            GLPM,GHPM,B,A
      PI=4*DATAN(1D0)
      V=1.D0/DSQRT(DSQRT(2.D0)*GF)
      BET=DATAN(TGBET)
      SB = DSIN(BET)
      CB = DCOS(BET)
      AMAR = AMA
C  ============ HEAVIEST CHARGINO MASS NEEDED FOR SUBH ========== 
c      AMCHI2=AM2**2+AMU**2+2.D0*AMW**2+DSQRT((AM2**2-AMU**2)**2
c     .      +4.D0*AMW**4*DCOS(2.D0*BET)**2+4.D0*AMW**2*
c     .      (AM2**2+AMU**2+2.D0*AMU*AM2*DSIN(2.D0*BET) ) ) 
c      AMCHI=DSQRT(0.5D0*AMCHI2)
C ===============================================================
C ========== RUNNING MASSES
      CALL SUBH(AMA,TGBET,AMSQ,AMUR,AMDR,AMT,AU,AD,AMU,AMCHI,
     .          AMLR,AMHR,AMCH,SA,CA,TANBA)
      LA3T=LA3+LA4+LA5
      AMA2=AMAR**2
      AML2=AMLR**2
      AMH2=AMHR**2
      AMP2=AMCH**2
C ========== HIGGS COUPLINGS 
      SBMA = SB*CA-CB*SA
      CBMA = CB*CA+SB*SA
      SBPA = SB*CA+CB*SA
      CBPA = CB*CA-SB*SA
      S2A = 2*SA*CA
      C2A = CA**2-SA**2
      S2B = 2*SB*CB
      C2B = CB**2-SB**2
      GLZZ = 1.d0/V/2*AML2*SBMA
      GHZZ = 1.d0/V/2*AMH2*CBMA
      GLWW = 2*GLZZ
      GHWW = 2*GHZZ
      GLAZ = 1.d0/V*(AML2-AMA2)*CBMA
      GHAZ = -1.d0/V*(AMH2-AMA2)*SBMA
      GLPW = -1.d0/V*(AMP2-AML2)*CBMA
      GLMW = GLPW
      GHPW = 1.d0/V*(AMP2-AMH2)*SBMA
      GHMW = GHPW
      GAPW = 1.d0/V*(AMP2-AMA2)
      GAMW = -GAPW
      GHHH = V/2*(LA1*CA**3*CB + LA2*SA**3*SB + LA3T*SA*CA*SBPA
     .     + LA6*CA**2*(3*SA*CB+CA*SB) + LA7*SA**2*(3*CA*SB+SA*CB))
      GLLL = -V/2*(LA1*SA**3*CB - LA2*CA**3*SB + LA3T*SA*CA*CBPA
     .     - LA6*SA**2*(3*CA*CB-SA*SB) + LA7*CA**2*(3*SA*SB-CA*CB))
      GLHH = -3*V/2*(LA1*CA**2*CB*SA - LA2*SA**2*SB*CA
     .     + LA3T*(SA**3*CB-CA**3*SB+2*SBMA/3)
     .     - LA6*CA*(CB*C2A-SA*SBPA) - LA7*SA*(C2A*SB+CA*SBPA))
      GHLL = 3*V/2*(LA1*SA**2*CB*CA + LA2*CA**2*SB*SA
     .     + LA3T*(SA**3*SB+CA**3*CB-2*CBMA/3)
     .     - LA6*SA*(CB*C2A+CA*CBPA) + LA7*CA*(C2A*SB+SA*CBPA))
      GLAA = -V/2*(LA1*SB**2*CB*SA - LA2*CB**2*SB*CA
     .     - LA3T*(SB**3*CA-CB**3*SA) + 2*LA5*SBMA
     .     - LA6*SB*(CB*SBPA+SA*C2B) - LA7*CB*(C2B*CA-SB*SBPA))
      GHAA = V/2*(LA1*SB**2*CB*CA + LA2*CB**2*SB*SA
     .     + LA3T*(SB**3*SA+CB**3*CA) - 2*LA5*CBMA
     .     - LA6*SB*(CB*CBPA+CA*C2B) + LA7*CB*(SB*CBPA+SA*C2B))
      GLPM = 2*GLAA + V*(LA5 - LA4)*SBMA
      GHPM = 2*GHAA + V*(LA5 - LA4)*CBMA
      GLZZ = 2*GLZZ
      GHZZ = 2*GHZZ
      GLLL = 6*GLLL
      GHHH = 6*GHHH
      GLHH = 2*GLHH
      GHLL = 2*GHLL
      GLAA = 2*GLAA
      GHAA = 2*GHAA
      XNORM = AMZ**2/V
      GLLL = GLLL/XNORM
      GHLL = GHLL/XNORM
      GLHH = GLHH/XNORM
      GHHH = GHHH/XNORM
      GHAA = GHAA/XNORM
      GLAA = GLAA/XNORM
      GLPM = GLPM/XNORM
      GHPM = GHPM/XNORM
      GAT=1.D0/TGBET
      GAB=TGBET
      GLT=CA/SB
      GLB=-SA/CB
      GHT=SA/SB
      GHB=CA/CB
      GZAL=-CBMA
      GZAH=SBMA
      GLVV=SBMA
      GHVV=CBMA
      B=BET
       if(ca.ne.0.d0) then
      A=DATAN(SA/CA)
       else
       if(sa.gt.0.d0) a=pi/2
       if(sa.lt.0.d0) a=-pi/2
       endif
      IF(CA.LT.0D0)THEN
       IF(SA.LT.0D0)THEN
        A = A-PI
       ELSE
        A = A+PI
       ENDIF
      ENDIF
C ===============================================================
C ========== POLE MASSES 
      IF(IPOLE.EQ.1) THEN 
      MT=RUNM(AMT,6) 
      MB=RUNM(AMT,5) 
      SW2=1.D0-AMW**2/AMZ**2
C===== STOP MASSES
      MSTL2=AMSQ**2+(0.5D0-2.D0/3.D0*SW2)*AMZ**2*DCOS(2.D0*B)
      MSTR2=AMUR**2+2.D0/3.D0*SW2*AMZ**2*DCOS(2.D0*B)
      MLRT=AU-AMU/TGBET
      DELT=(MSTL2-MSTR2)**2+4*MT**2*MLRT**2
      MST12=MT**2+0.5D0*(MSTL2+MSTR2-DSQRT(DELT))
      MST22=MT**2+0.5D0*(MSTL2+MSTR2+DSQRT(DELT))
        IF(MST12.LT.0.D0) goto 111
      MST(1)=DSQRT(MST12)
      MST(2)=DSQRT(MST22)
      THET=0.5D0*DATAN(2.D0*MT*MLRT / (MSTL2-MSTR2) )
      IF(MSTL2.GT.MSTR2) THET = THET + PI/2
      CST= DCOS(THET)
      SST= DSIN(THET)
C===== SBOTTOM MASSES
      MSBL2=AMSQ**2+(-0.5D0+1.D0/3.D0*SW2)*AMZ**2*DCOS(2.D0*B)
      MSBR2=AMDR**2-1.D0/3.D0*SW2*AMZ**2*DCOS(2.D0*B)
      MLRB=AD-AMU*TGBET
      DELB=(MSBL2-MSBR2)**2+4*MB**2*MLRB**2
      MSB12=MB**2+0.5D0*(MSBL2+MSBR2-DSQRT(DELB))
      MSB22=MB**2+0.5D0*(MSBL2+MSBR2+DSQRT(DELB))
        IF(MSB12.LT.0.D0)GOTO 111
      MSB(1)=DSQRT(MSB12)
      MSB(2)=DSQRT(MSB22)
      THEB=0.5D0*DATAN(2.D0*MB*MLRB / (MSBL2-MSBR2) )
      IF(MSBL2.GT.MSBR2) THEB = THEB + PI/2
      CSB= DCOS(THEB)
      SSB= DSIN(THEB)
C===== LIGHT HIGGS COUPLINGS 
      GLTT(1,1)=-SBPA*(0.5D0*CST**2-2.D0/3.D0*SW2*DCOS(2*THET) )
     .    +MT**2/AMZ**2*GLT + MT*SST*CST/AMZ**2*(AU*GLT+AMU*GHT)
      GLTT(2,2)=-SBPA*(0.5D0*SST**2+2.D0/3.D0*SW2*DCOS(2*THET) )
     .    +MT**2/AMZ**2*GLT - MT*SST*CST/AMZ**2*(AU*GLT+AMU*GHT)
      GLTT(1,2)=-2*SBPA*SST*CST*(2.D0/3.D0*SW2-0.25D0)
     .    + MT*DCOS(2*THET)/2.D0/AMZ**2*(AU*GLT+AMU*GHT)
      GLTT(2,1)=-2*SBPA*SST*CST*(2.D0/3.D0*SW2-0.25D0)
     .    + MT*DCOS(2*THET)/2.D0/AMZ**2*(AU*GLT+AMU*GHT)
      GLBB(1,1)=-SBPA*(-0.5D0*CSB**2+1.D0/3.D0*SW2*DCOS(2*THEB))
     .    +MB**2/AMZ**2*GLB + MB*SSB*CSB/AMZ**2*(AD*GLB-AMU*GHB)
      GLBB(2,2)=-SBPA*(-0.5D0*SSB**2-1.D0/3.D0*SW2*DCOS(2*THEB))
     .    +MB**2/AMZ**2*GLB - MB*SSB*CSB/AMZ**2*(AD*GLB-AMU*GHB)
      GLBB(1,2)=-2*SBPA*SSB*CSB*(-1.D0/3.D0*SW2+0.25D0)
     .    + MB*DCOS(2*THEB)/2.D0/AMZ**2*(AD*GLB-AMU*GHB)
      GLBB(2,1)=-2*SBPA*SSB*CSB*(-1.D0/3.D0*SW2+0.25D0)
     .    + MB*DCOS(2*THEB)/2.D0/AMZ**2*(AD*GLB-AMU*GHB)
C===== HEAVY HIGGS COUPLINGS 
      GHTT(1,1)=CBPA*(0.5D0*CST**2-2.D0/3.D0*SW2*DCOS(2*THET))
     .    +MT**2/AMZ**2*GHT + MT*SST*CST/AMZ**2*(AU*GHT-AMU*GLT)
      GHTT(2,2)=CBPA*(0.5D0*SST**2+2.D0/3.D0*SW2*DCOS(2*THET))
     .    +MT**2/AMZ**2*GHT - MT*SST*CST/AMZ**2*(AU*GHT-AMU*GLT)
      GHTT(1,2)=2*CBPA*SST*CST*(2.D0/3.D0*SW2-0.25D0)
     .    +MT*DCOS(2*THET)/2.D0/AMZ**2*(AU*GHT-AMU*GLT)
      GHTT(2,1)=2*CBPA*SST*CST*(2.D0/3.D0*SW2-0.25D0)
     .    + MT*DCOS(2*THET)/2.D0/AMZ**2*(AU*GHT-AMU*GLT)
      GHBB(1,1)=CBPA*(-0.5D0*CSB**2+1.D0/3.D0*SW2*DCOS(2*THEB))
     .    +MB**2/AMZ**2*GHB + MB*SSB*CSB/AMZ**2*(AD*GHB+AMU*GLB)
      GHBB(2,2)=CBPA*(-0.5D0*SSB**2-1.D0/3.D0*SW2*DCOS(2*THEB))
     .    + MB**2/AMZ**2*GHB - MB*SSB*CSB/AMZ**2*(AD*GHB+AMU*GLB)
      GHBB(1,2)=2*CBPA*SSB*CSB*(-1.D0/3.D0*SW2+0.25D0)
     .    + MB*DCOS(2*THEB)/2.D0/AMZ**2*(AD*GHB+AMU*GLB)
      GHBB(2,1)=2*CBPA*SSB*CSB*(-1.D0/3.D0*SW2+0.25D0)
     .    + MB*DCOS(2*THEB)/2.D0/AMZ**2*(AD*GHB+AMU*GLB)
C===== PSEUDOSCALAR HIGGS COUPLINGS 
      GATT=-MT/2.D0/AMZ**2*(AMU+AU*GAT) 
      GABB=-MB/2.D0/AMZ**2*(AMU+AD*GAB) 
C======= LOOP CORRECTIONS  
      XDLT=GF/(2.D0*DSQRT(2.D0)*PI**2)*GLT**2*(-2.D0*MT**2+0.5D0*AML2)
     .    *DBLE(SF0(MT,MT,AML2))
     .    *3*MT**2
      XDLB=GF/(2.D0*DSQRT(2.D0)*PI**2)*GLB**2*(-2.D0*MB**2+0.5D0*AML2)
     .    *DBLE(SF0(MB,MB,AML2))
     .    *3*MB**2
      XDHT=GF/(2.D0*DSQRT(2.D0)*PI**2)*GHT**2*(-2.D0*MT**2+0.5D0*AMH2)
     .    *DBLE(SF0(MT,MT,AMH2))
     .    *3*MT**2
      XDHB=GF/(2.D0*DSQRT(2.D0)*PI**2)*GHB**2*(-2.D0*MB**2+0.5D0*AMH2)
     .    *DBLE(SF0(MB,MB,AMH2))
     .    *3*MB**2
      XDAT=GF/(2.D0*DSQRT(2.D0)*PI**2)*GAT**2*(-0.5D0*AMA2)
     .    *DBLE(SF0(MT,MT,AMA2))
     .    *3*MT**2
      XDAB=GF/(2.D0*DSQRT(2.D0)*PI**2)*GAB**2*(-0.5D0*AMA2)
     .    *DBLE(SF0(MB,MB,AMA2))
     .    *3*MB**2
      XDLST=0.D0
      XDLSB=0.D0
      XDHST=0.D0
      XDHSB=0.D0
         DO 311 I=1,2
         DO 311 J=1,2
      XDLST=XDLST+GF/(2.D0*DSQRT(2.D0)*PI**2)*GLTT(I,J)**2*
     .      DBLE(SF0(MST(I),MST(J),AML2))
     .    *3*AMZ**4
      XDLSB=XDLSB+GF/(2.D0*DSQRT(2.D0)*PI**2)*GLBB(I,J)**2*
     .      DBLE(SF0(MSB(I),MSB(J),AML2))
     .    *3*AMZ**4
      XDHST=XDHST+GF/(2.D0*DSQRT(2.D0)*PI**2)*GHTT(I,J)**2*
     .      DBLE(SF0(MST(I),MST(J),AMH2))
     .    *3*AMZ**4
      XDHSB=XDHSB+GF/(2.D0*DSQRT(2.D0)*PI**2)*GHBB(I,J)**2*
     .      DBLE(SF0(MSB(I),MSB(J),AMH2))
     .    *3*AMZ**4
311    CONTINUE
      XDAST=GF/(1.D0*DSQRT(2.D0)*PI**2)*GATT**2*
     .      DBLE(SF0(MST(1),MST(2),AMA2))
     .    *3*AMZ**4
      XDASB=GF/(1.D0*DSQRT(2.D0)*PI**2)*GABB**2*
     .      DBLE(SF0(MSB(1),MSB(2),AMA2))
     .    *3*AMZ**4
      
       AML=DSQRT(AML2+XDLT+XDLB+XDLST+XDLSB)
       AMH=DSQRT(AMH2+XDHT+XDHB+XDHST+XDHSB)  
       AMA=DSQRT(AMA2+XDAT+XDAB+XDAST+XDASB)  
      ELSE
       AML=AMLR
       AMH=AMHR     
       AMA=AMAR     
      ENDIF 
 111  continue  
c111   STOP
      END

C ===================== THE FUNCTION SF0 ===============
      COMPLEX*16 FUNCTION SF0(M1,M2,QSQ)
      IMPLICIT REAL*8 (A-H,M,O-Z)
      COMPLEX*16 CD,CR,CQ2,IEPS,CBET,CXX
      M1SQ = M1*M1
      M2SQ = M2*M2
      AQSQ = DABS(QSQ)
      IEPS = DCMPLX(1.D0,1.D-12)
      IF(QSQ.EQ.0.D0) THEN
       SF0 = 0.D0
      ELSE
      CQ2 = QSQ*IEPS
      CD = (M1SQ-M2SQ)/CQ2
      CR = CDSQRT((1.d0+CD)**2 - 4*M1SQ/CQ2)
       IF(M1.EQ.M2) THEN
        SF0 = -2.D0 + CR*CDLOG(-(1.d0+CR)/(1.d0-CR))
       ELSE
        CBET = CDSQRT(1.d0-4*M1*M2/(CQ2 - (M1-M2)**2))
        CXX = (CBET-1.d0)/(CBET+1.d0)
        SF0 = -1.d0 + ((QSQ+M2SQ-M1SQ)/2/QSQ - M2SQ/(M2SQ-M1SQ))
     .                                           *DLOG(M2SQ/M1SQ)
     .     - (QSQ-(M1-M2)**2)/QSQ*CBET*CDLOG(CXX)
       ENDIF
      ENDIF
      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     THIS PROGRAM COMPUTES THE RENORMALIZATION GROUP IMPROVED
C     VALUES OF HIGGS MASSES AND COUPLINGS IN THE MSSM.
C
C     INPUT: MA,TANB = TAN(BETA),MQ,MUR,MDR,MTOP,AU,AD,MU,MCHI
C
C     ALL MASSES IN GEV UNITS. MA IS THE CP-ODD HIGGS MASS,
C     MTOP IS THE PHYSICAL TOP MASS, MQ AND MUR/MDR ARE THE SOFT
C     SUPERSYMMETRY BREAKING MASS PARAMETERS OF LEFT HANDED
C     AND RIGHT HANDED STOPS RESPECTIVELY, AU AND AD ARE THE
C     STOP AND SBOTTOM TRILINEAR SOFT BREAKING TERMS,
C     RESPECTIVELY,  AND MU IS THE SUPERSYMMETRIC
C     HIGGS MASS PARAMETER. WE USE THE  CONVENTIONS FROM
C     THE PHYSICS REPORT OF HABER AND KANE: LEFT RIGHT
C     STOP MIXING TERM PROPORTIONAL TO (AU - MU/TANB).
C     MCHI IS THE HEAVIEST CHARGINO MASS. 
C     WE USE AS INPUT TANB DEFINED AT THE SCALE MTOP.

C     OUTPUT: MH,HM,MCH, SA = SIN(ALPHA), CA= COS(ALPHA), TANBA
C     WHERE MHP AND HPM ARE THE LIGHTEST AND HEAVIEST CP-EVEN
C     HIGGS MASSES, MHCH IS THE CHARGED HIGGS MASS AND
C     ALPHA IS THE HIGGS MIXING ANGLE.
C     TANBA IS THE ANGLE TANB AT THE CP-ODD HIGGS MASS SCALE.

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c       Program based on the work by M. Carena, M. Quiros
c       and C.E.M. Wagner, "Effective potential methods and
c       the Higgs mass spectrum in the MSSM", Phys. Lett.
c       B335 (1995) 209. 
c       (and also Nucl.Phys. B461 (1996) 407)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      SUBROUTINE SUBH(MA,TANB,MQ,MUR,MD,MTOP,AU,AD,MU,MCHI0,
     *                MHP,HMP,MCH,SA,CA,TANBA)

      IMPLICIT REAL*8(A-H,L,M,O-Z)
      DIMENSION VH(2,2),M2(2,2),M2P(2,2)
      COMMON/PARAM/GF,ALPH,AMZ,AMW
      common/fmasses/MTAU,AMB,MT
      COMMON/HSELF/LAMBDA1,LAMBDA2,LAMBDA3,LAMBDA4,LAMBDA5,
     .             LAMBDA6,LAMBDA7

      MCHI = MCHI0
      TANBA = TANB
      TANBT = TANB
      
      PI = 4*DATAN(1D0)
      MZ = AMZ
      MW = AMW
      V  = 1/DSQRT(2*DSQRT(2D0)*GF)
      CW = AMW**2/AMZ**2
      SW = 1.d0-CW
      ALPHA2  = (2*AMW/V/DSQRT(2D0))**2/4/PI
      ALPHA1  = ALPHA2*SW/CW
      ALPHA3Z = ALPHAS(AMZ,2)
      ALPHA3  = ALPHAS(MTOP,2)
      MB      = RUNM(MTOP,5)
      RMTOP   = RUNM(MTOP,6)
      TQ = LOG((MQ**2+MTOP**2)/MTOP**2)
      TU = LOG((MUR**2 + MTOP**2)/MTOP**2)
      TD = LOG((MD**2 + MTOP**2)/MTOP**2)
      SINB = TANB/DSQRT(1.D0 + TANB**2)
      COSB = SINB/TANB

      IF(MA.GT.MTOP)
     *       TANBA = TANB*(1.D0-3.D0/32.D0/PI**2*
     *       (RMTOP**2/V**2/SINB**2-MB**2/V**2/COSB**2)*
     *       DLOG(MA**2/MTOP**2))
      IF(MA.LT.MTOP.OR.MA.EQ.MTOP) TANBT = TANBA

      SINB = TANBT/DSQRT(1.D0 + TANBT**2)
      COSB = 1.D0/DSQRT(1.D0 + TANBT**2)
      COS2B = (TANBT**2 - 1.D0)/(TANBT**2 + 1.D0)
      G1 = DSQRT(ALPHA1*4.D0*PI)
      G2 = DSQRT(ALPHA2*4.D0*PI)
      G3 = DSQRT(ALPHA3*4.D0*PI)
      HU = RMTOP/V/SINB
      HD =  MB/V/COSB
C

      IF(MQ.GT.MUR) TP = TQ - TU
      IF(MQ.LT.MUR.OR.MQ.EQ.MUR) TP = TU - TQ
      IF(MQ.GT.MUR) TDP = TU
      IF(MQ.LT.MUR.OR.MQ.EQ.MUR) TDP = TQ
      IF(MQ.GT.MD) TPD = TQ - TD
      IF(MQ.LT.MD.OR.MQ.EQ.MD) TPD = TD - TQ
      IF(MQ.GT.MD) TDPD = TD
      IF(MQ.LT.MD.OR.MQ.EQ.MD) TDPD = TQ

      IF(MQ.GT.MD) DLAMBDA1 = 6./96./PI**2*G1**2*HD**2*TPD
      IF(MQ.LT.MD.OR.MQ.EQ.MD) DLAMBDA1 = 3./32./PI**2*
     * HD**2*(G1**2/3.+G2**2)*TPD

      IF(MQ.GT.MUR) DLAMBDA2 =12./96./PI**2*G1**2*HU**2*TP
      IF(MQ.LT.MUR.OR.MQ.EQ.MUR) DLAMBDA2 = 3./32./PI**2*
     * HU**2*(-G1**2/3.+G2**2)*TP

      DLAMBDA3 = 0.
      DLAMBDA4 = 0.

      IF(MQ.GT.MD) DLAMBDA3 = -1./32./PI**2*G1**2*HD**2*TPD
      IF(MQ.LT.MD.OR.MQ.EQ.MD) DLAMBDA3 = 3./64./PI**2*HD**2*
     *(G2**2-G1**2/3.)*TPD
      
      IF(MQ.GT.MUR) DLAMBDA3 = DLAMBDA3 - 
     *1./16./PI**2*G1**2*HU**2*TP
      IF(MQ.LT.MUR.OR.MQ.EQ.MUR) DLAMBDA3 = DLAMBDA3 + 
     * 3./64./PI**2*HU**2*(G2**2+G1**2/3.)*TP

      IF(MQ.LT.MUR) DLAMBDA4 = -3./32./PI**2*G2**2*HU**2*TP
      IF(MQ.LT.MD) DLAMBDA4 = DLAMBDA4 - 3./32./PI**2*G2**2*
     *                        HD**2*TPD
C
      LAMBDA1 = ((G1**2 + G2**2)/4.)*
     *(1.-3.*HD**2*(TPD + TDPD)/8./PI**2)
     *+(3.*HD**4./16./PI**2) *TPD*(1.   
     *+ (3.*HD**2/2. + HU**2/2.       
     *- 8.*G3**2) * (TPD + 2.*TDPD)/16./PI**2) 
     *+(3.*HD**4./8./PI**2) *TDPD*(1.  + (3.*HD**2/2. + HU**2/2.       
     *- 8.*G3**2) * TDPD/16./PI**2) + DLAMBDA1 
C
      LAMBDA2 = ((G1**2 + G2**2)/4.)*(1.-3.*HU**2*
     *(TP + TDP)/8./PI**2)
     *+(3.*HU**4./16./PI**2) *TP*(1.   
     *+ (3.*HU**2/2. + HD**2/2.       
     *- 8.*G3**2) * (TP + 2.*TDP)/16./PI**2) 
     *+(3.*HU**4./8./PI**2) *TDP*(1. + (3.*HU**2/2. + HD**2/2.       
     *- 8.*G3**2) * TDP/16./PI**2) + DLAMBDA2 
C
      LAMBDA3 = ((G2**2 - G1**2)/4.)*(1.-3.*
     *(HU**2)*(TP + TDP)/16./PI**2 -3.*
     *(HD**2)*(TPD + TDPD)/16./PI**2) +DLAMBDA3 
C
      LAMBDA4 = (- G2**2/2.)*(1.
     *-3.*(HU**2)*(TP + TDP)/16./PI**2
     *-3.*(HD**2)*(TPD + TDPD)/16./PI**2) +DLAMBDA4
C     
	LAMBDA5 = 0.
	LAMBDA6 = 0.
	LAMBDA7 = 0.

      M2(1,1) = 2.*V**2*(LAMBDA1*COSB**2+2.*LAMBDA6*
     *COSB*SINB + LAMBDA5*SINB**2) + MA**2*SINB**2
      M2(2,2) = 2.*V**2*(LAMBDA5*COSB**2+2.*LAMBDA7*
     *COSB*SINB + LAMBDA2*SINB**2) + MA**2*COSB**2
      M2(1,2) = 2.*V**2*(LAMBDA6*COSB**2+(LAMBDA3+LAMBDA4)*
     *COSB*SINB + LAMBDA7*SINB**2) - MA**2*SINB*COSB
      M2(2,1) = M2(1,2)

C
C     THIS IS THE CONTRIBUTION FROM LIGHT CHARGINOS/NEUTRALINOS
C     CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
  	 MSSUSY=DSQRT(0.5D0*(MQ**2+MUR**2)+MTOP**2)
	IF(MCHI.GT.MSSUSY)GOTO 3790
	IF(MCHI.LT.MTOP) MCHI=MTOP
	TCHAR=LOG(MSSUSY**2/MCHI**2)
	DELTAL12=(9./64./PI**2*G2**4+5./192./PI**2*G1**4)*TCHAR
	DELTAL3P4=(3./64./PI**2*G2**4+7./192./PI**2*G1**4
     *       +4./32/PI**2*G1**2*G2**2)*TCHAR
	DELTAM112=2.*DELTAL12*V**2*COSB**2
	DELTAM222=2.*DELTAL12*V**2*SINB**2
	DELTAM122=2.*DELTAL3P4*V**2*SINB*COSB
	M2(1,1)=M2(1,1)+DELTAM112
	M2(2,2)=M2(2,2)+DELTAM222
	M2(1,2)=M2(1,2)+DELTAM122
	M2(2,1)=M2(2,1)+DELTAM122

 3790	CONTINUE
CCCCCCCCCCCCCCC    END OF CHARGINOS AND NEUTRALINOS  CCCCCCCCCCCC 


      CALL GFUN(MA,TANBA,MQ,MUR,MD,MTOP,AU,AD,MU,VH)
      
      DO 9800 I = 1,2
      DO 9801 J = 1,2
      M2P(I,J) = M2(I,J) + VH(I,J)
 9801 CONTINUE
 9800 CONTINUE

      TRM2P  = M2P(1,1) + M2P(2,2)
      DETM2P = M2P(1,1)*M2P(2,2) - M2P(1,2)*M2P(2,1)

      MH2P = (TRM2P - DSQRT(TRM2P**2 - 4.D0* DETM2P))/2.D0
      HM2P = (TRM2P + DSQRT(TRM2P**2 - 4.D0* DETM2P))/2.D0
C !!!!!!!!!!!!!!!!!!!
      MCH2=MA**2+(LAMBDA5-LAMBDA4)*V**2
C !!!!!!!!!!!!!!!!!!!
      if(mch2.ge.0.d0) then
      MCH=DSQRT(MCH2)
      else
      goto 5555
      endif
      HMP = DSQRT(HM2P) 
      IF(MH2P.ge.0.) then
      MHP = DSQRT(MH2P) 
      else
      goto 5555
      endif
C
      SIN2ALPHA = 2.*M2P(1,2)/DSQRT(TRM2P**2-4.D0*DETM2P)
      COS2ALPHA = (M2P(1,1)-M2P(2,2))/DSQRT(TRM2P**2-4.D0*DETM2P)
      IF(COS2ALPHA.GT.0.) ALPHA = DASIN(SIN2ALPHA)/2.D0
      IF(COS2ALPHA.LT.0.) ALPHA = -PI/2.D0-DASIN(SIN2ALPHA)/2.D0
      SA = DSIN(ALPHA)
      CA = DCOS(ALPHA)  
      SQBMA = (SINB*CA - COSB*SA)**2

5555  RETURN
      END
C

CCCCCCCCCCCCCCCCCCCCCCCC NON DEGENERATE STOP/SBOTTOM EFFECTS CCCCCCCCC
C
        SUBROUTINE GFUN(MA,TANB,MQ,MUR,MD,MTOP,AT,AB,MU,VH)
        IMPLICIT REAL*8 (A-H,L,M,O-Z)
        DIMENSION VH(2,2),VH1(2,2),VH2(2,2),
     *            VH3T(2,2),VH3B(2,2),AL(2,2)
        COMMON/PARAM/GF,ALPH,AMZ,AMW
        G(X,Y) = 2.D0 - (X+Y)/(X-Y)*DLOG(X/Y)

        IF(DABS(MU).LT.0.000001) MU = 0.000001
        MQ2   = MQ**2
        MUR2  = MUR**2
        MD2   = MD**2
        TANBA = TANB
        SINBA = TANBA/DSQRT(TANBA**2+1.D0)
        COSBA = SINBA/TANBA        
        SINB = TANB/DSQRT(TANB**2+1.D0)
        COSB = SINB/TANB

      MB = RUNM(MTOP,5)
      PI = 4*DATAN(1D0)
      MZ = AMZ
      MW = AMW
      V  = 1/DSQRT(2*DSQRT(2D0)*GF)
      CW = AMW**2/AMZ**2
      SW = 1.d0-CW
      ALPHA2  = (2*AMW/V/DSQRT(2D0))**2/4/PI
      ALPHA1  = ALPHA2*SW/CW
      ALPHA3Z = ALPHAS(AMZ,2)
      ALPHA3  = ALPHAS(MTOP,2)

      G1 = DSQRT(ALPHA1*4.*PI)
      G2 = DSQRT(ALPHA2*4.*PI)
      G3 = DSQRT(ALPHA3*4.*PI)
      
        IF(MQ.GT.MUR) MST = MQ
        IF(MUR.GT.MQ.OR.MUR.EQ.MQ) MST = MUR
        MSUSYT = DSQRT(MST**2  + MTOP**2)

	IF(MQ.GT.MD) MSB = MQ
	IF(MD.GT.MQ.OR.MD.EQ.MQ) MSB = MD
	MSUSYB = DSQRT(MSB**2 + MB**2)

	TT = LOG(MSUSYT**2/MTOP**2)
	TB = LOG(MSUSYB**2/MTOP**2)

        RMTOP   = RUNM(MTOP,6) 
                
        HT = RMTOP/V/SINB
        HTST = RMTOP/V
        HB =  MB/V/COSB
        G32 = ALPHA3*4.*PI

        BT2 = -(8.*G32 - 9.*HT**2/2. - HB**2/2.)/(4.*PI)**2
	BB2 = -(8.*G32 - 9.*HB**2/2. - HT**2/2.)/(4.*PI)**2
        AL2 = 3./8./PI**2*HT**2
        BT2ST = -(8.*G32 - 9.*HTST**2/2.)/(4.*PI)**2
        ALST = 3./8./PI**2*HTST**2
        AL1 = 3./8./PI**2*HB**2

        AL(1,1) = AL1
        AL(1,2) = (AL2+AL1)/2.
        AL(2,1) = (AL2+AL1)/2.
        AL(2,2) = AL2

        MTOP4 = RMTOP**4.*(1.+2.*BT2*TT- AL2*TT)
        MTOP2 = DSQRT(MTOP4)
	MBOT4 = MB**4.*(1.+2.*BB2*TB - AL1*TB)
	MBOT2 = DSQRT(MBOT4)

	IF(MA.GT.MTOP) THEN
        VI = V*(1. + 3./32./PI**2*HTST**2*LOG(MTOP**2/MA**2))
        H1I = VI*COSBA
        H2I = VI*SINBA
        H1T = H1I*(1.+3./8./PI**2*HB**2*LOG(MA**2/MSUSYT**2))**.25
        H2T = H2I*(1.+3./8./PI**2*HT**2*LOG(MA**2/MSUSYT**2))**.25
        H1B = H1I*(1.+3./8./PI**2*HB**2*LOG(MA**2/MSUSYB**2))**.25
        H2B = H2I*(1.+3./8./PI**2*HT**2*LOG(MA**2/MSUSYB**2))**.25
	ELSE
	VI =  V
	H1I = VI*COSB
	H2I = VI*SINB
        H1T = H1I*(1.+3./8./PI**2*HB**2*LOG(MTOP**2/MSUSYT**2))**.25
        H2T = H2I*(1.+3./8./PI**2*HT**2*LOG(MTOP**2/MSUSYT**2))**.25
        H1B = H1I*(1.+3./8./PI**2*HB**2*LOG(MTOP**2/MSUSYB**2))**.25
        H2B = H2I*(1.+3./8./PI**2*HT**2*LOG(MTOP**2/MSUSYB**2))**.25
	END IF

        TANBST = H2T/H1T
        SINBT = TANBST/(1.+TANBST**2)**.5
        COSBT = SINBT/TANBST

        TANBSB = H2B/H1B
        SINBB = TANBSB/(1.+TANBSB**2)**.5
        COSBB = SINBB/TANBSB

        STOP12 = (MQ2 + MUR2)*.5 + MTOP2 
     *   +1./8.*(G2**2+G1**2)*(H1T**2-H2T**2)
     *   +(((G2**2-5.*G1**2/3.)/4.*(H1T**2-H2T**2) +
     *   MQ2 - MUR2)**2*0.25 + MTOP2*(AT-MU/TANBST)**2)**.5

        STOP22 = (MQ2 + MUR2)*.5 + MTOP2 
     *  +1./8.*(G2**2+G1**2)*(H1T**2-H2T**2) 
     *   - (((G2**2-5.*G1**2/3.)/4.*(H1T**2-H2T**2) +
     *  MQ2 - MUR2)**2*0.25 
     *  + MTOP2*(AT-MU/TANBST)**2)**.5

        IF(STOP22.LT.0.) GOTO 4237

        SBOT12 = (MQ2 + MD2)*.5  
     *   - 1./8.*(G2**2+G1**2)*(H1B**2-H2B**2)
     *  + (((G1**2/3.-G2**2)/4.*(H1B**2-H2B**2) +
     *  MQ2 - MD2)**2*0.25 + MBOT2*(AB-MU*TANBSB)**2)**.5

        SBOT22 = (MQ2 + MD2)*.5  
     *   - 1./8.*(G2**2+G1**2)*(H1B**2-H2B**2)
     *   - (((G1**2/3.-G2**2)/4.*(H1B**2-H2B**2) +
     *   MQ2 - MD2)**2*0.25 + MBOT2*(AB-MU*TANBSB)**2)**.5

        IF(SBOT22.LT.0.) GOTO 4237

        STOP1 = STOP12**.5
        STOP2 = STOP22**.5
        SBOT1 = SBOT12**.5
        SBOT2 = SBOT22**.5

        VH1(1,1) = 1./TANBST
        VH1(2,1) = -1.
        VH1(1,2) = -1.
        VH1(2,2) = TANBST
        VH2(1,1) = TANBST
        VH2(1,2) = -1.
        VH2(2,1) = -1.
        VH2(2,2) = 1./TANBST

C CCCCCCCCCCCCCCCCCCCCCCCCCCC  D-terms CCCCCCCCCCCCCCCCCCCCCCCCCCCCC
	STW=SW

	F1T=(MQ2-MUR2)/(STOP12-STOP22)*(.5-4./3.*STW)*
     *         LOG(STOP1/STOP2)
     *        +(.5-2./3.*STW)*LOG(STOP1*STOP2/(MQ2+MTOP2))
     *        + 2./3.*STW*LOG(STOP1*STOP2/(MUR2+MTOP2))

	F1B=(MQ2-MD2)/(SBOT12-SBOT22)*(-.5+2./3.*STW)*
     *        LOG(SBOT1/SBOT2)
     *        +(-.5+1./3.*STW)*LOG(SBOT1*SBOT2/(MQ2+MBOT2))
     *        - 1./3.*STW*LOG(SBOT1*SBOT2/(MD2+MBOT2))

	F2T=MTOP2**.5*(AT-MU/TANBST)/(STOP12-STOP22)*
     *         (-.5*LOG(STOP12/STOP22)
     *        +(4./3.*STW-.5)*(MQ2-MUR2)/(STOP12-STOP22)*
     *         G(STOP12,STOP22))

	F2B=MBOT2**.5*(AB-MU*TANBSB)/(SBOT12-SBOT22)*
     *         (.5*LOG(SBOT12/SBOT22)
     *        +(-2./3.*STW+.5)*(MQ2-MD2)/(SBOT12-SBOT22)*
     *        G(SBOT12,SBOT22))

        VH3B(1,1) = MBOT4/(COSBB**2)*(LOG(SBOT1**2*SBOT2**2/
     *  (MQ2+MBOT2)/(MD2+MBOT2)) 
     *  + 2.*(aB*(aB-MU*TANBSB)/(SBOT1**2-SBOT2**2))*
     *  LOG(SBOT1**2/SBOT2**2)) + 
     *  mBOT4/(COSBB**2)*(aB*(aB-MU*TANBSB)/
     *  (SBOT1**2-SBOT2**2))**2*G(SBOT12,SBOT22) 

	VH3T(1,1) =
     *  MTOP4/(SINBT**2)*(MU*(-aT+MU/TANBST)/(STOP1**2
     * -STOP2**2))**2*G(STOP12,STOP22)

	VH3B(1,1)=VH3B(1,1)+
     *    MZ**2*(2*MBOT2*F1B-MBOT2**.5*AB*F2B)

	VH3T(1,1) = VH3T(1,1) + 
     *  MZ**2*(MTOP2**.5*MU/TANBST*F2T)  

        VH3T(2,2) = MTOP4/(SINBT**2)*(LOG(STOP1**2*STOP2**2/
     *  (MQ2+MTOP2)/(MUR2+MTOP2)) 
     *  + 2.*(aT*(aT-MU/TANBST)/(STOP1**2-STOP2**2))*
     *  LOG(STOP1**2/STOP2**2)) + 
     *  MTOP4/(SINBT**2)*(aT*(aT-MU/TANBST)/
     *  (STOP1**2-STOP2**2))**2*G(STOP12,STOP22) 

	VH3B(2,2) =
     *  mBOT4/(COSBB**2)*(MU*(-aB+MU*TANBSB)/(SBOT1**2
     * -SBOT2**2))**2*G(SBOT12,SBOT22)

	VH3T(2,2)=VH3T(2,2)+
     *    MZ**2*(-2*MTOP2*F1T+MTOP2**.5*AT*F2T)

	VH3B(2,2) = VH3B(2,2) -MZ**2*MBOT2**.5*MU*TANBSB*F2B  
 
        VH3T(1,2) = -
     *   MTOP4/(SINBT**2)*MU*(aT-MU/TANBST)/
     * (STOP1**2-STOP2**2)*(LOG(STOP1**2/STOP2**2) + aT*
     * (AT - MU/TANBST)/(STOP1**2-STOP2**2)*G(STOP12,STOP22))

	VH3B(1,2) =
     * - MBOT4/(COSBB**2)*MU*(aT-MU*TANBSB)/
     * (SBOT1**2-SBOT2**2)*(LOG(SBOT1**2/SBOT2**2) + AB*
     * (AB - MU*TANBSB)/(SBOT1**2-SBOT2**2)*G(SBOT12,SBOT22))

	VH3T(1,2)=VH3T(1,2) +
     *      MZ**2*(MTOP2/TANBST*F1T-MTOP2**.5*(AT/TANBST+MU)/2.*F2T)

	VH3B(1,2)=VH3B(1,2)
     *  +MZ**2*(-MBOT2*TANBSB*F1B+MBOT2**.5*(AB*TANBSB+MU)/2.*F2B)

        VH3T(2,1) = VH3T(1,2)
        VH3B(2,1) = VH3B(1,2)

       TQ = LOG((MQ2 + MTOP2)/MTOP2)
       TU = LOG((MUR2+MTOP2)/MTOP2)
       TQD = LOG((MQ2 + MB**2)/MB**2)
       TD = LOG((MD2+MB**2)/MB**2)

        DO 8910 I = 1,2
        DO 8911 J = 1,2
        VH(I,J) = 
     *  6./(8.*PI**2*(H1T**2+H2T**2))
     *  *VH3T(I,J)*0.5*(1.-AL(I,J)*TT/2.) +
     *  6./(8.*PI**2*(H1B**2+H2B**2))
     *  *VH3B(I,J)*0.5*(1.-AL(I,J)*TB/2.)
 8911   CONTINUE
 8910   CONTINUE

        GOTO 4236
 4237   DO 6868 I =1,2
        DO 6867 J = 1,2
        VH(I,J) = -1.D+15
 6867   CONTINUE
 6868   CONTINUE

4236    RETURN
        END
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c       End of program from M. Carena, M. Quiros and C.E.M. Wagner.
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C     ****************************************************************
C	  CHARGINO AND NEUTRALINO MASS MATRICES AND COUPLINGS
C     ****************************************************************
      SUBROUTINE GAUGINO(MU,M1,M2,M3,B,A,MC,MN,XMN)
      IMPLICIT REAL*8(A-H,K-Z)
      COMPLEX*16 CXA,CXB,CXC,CXD,CX1,CX2,CX3
      DIMENSION MC(2),MN(4),XMN(4),Z(4,4),ZX(4,4),U(2,2),V(2,2),
     .          QQ(4,4),SS(4,4),S(2,2),Q(2,2),AC1(2,2),AC2(2,2),
     .          AC3(2,2),AN1(4,4),AN2(4,4),AN3(4,4),ACNL(2,4),
     .          ACNR(2,4),IORD(4),IREM(2)
      DIMENSION X(2,2)
      dimension dxmn(4)
      DIMENSION YMN(4),YZ(4,4),XMC(2),BU(2),BV(2)
      COMMON/PARAM/GF,ALPH,MZ,MW
       common/cpl/g22,sw2
      COMMON/BREAK/MSL,MTAUR,MQ3,Mu3,Md3,AL,AU,AD,
     .            MUdum,M1dum,M2dum,M3dum
      COMMON/BREAK2/ML1,MERdum,MQ1,MURdum,MDRdum
      common/mssmhpar/dum1,dum2,ma,dumu
      common/yuka/ytau,yb,ytop
      common/strc/irge,isfrc,inorc
        COMMON/matino/U,V,Z,dxmn
c  #&! included for omega #&!
      COMMON /NEUT/ MCHI,N01,N02,N03,N04    
c  #&! included for omega #&!
c      CW=MW/MZ
c      SW=DSQRT(1-CW**2)
      sw=dsqrt(sw2)
      cw=dsqrt(1.d0-sw2)
      PI=4.D0*DATAN(1.D0)
      SB=DSIN(B)
      CB=DCOS(B)
      TW=SW/CW
C      M1=5.D0/3.D0*TW**2*M2
c  
      if(inorc.eq.1.and.irge.ge.2) then
c  adding R.C to M1, M2, MU:
      m1save=m1
      m2save=m2
      musave=mu
      call radcino(ml1,mq1,mq3,mu3,md3,ma,ytop,yb,m1,m2,-mu,tan(b),
     .            rcm1,rcm2,rcmu)
      m1 = m1+rcm1
      m2 = m2 +rcm2
      mu = mu +rcmu
      endif
C     ************  NEUTRALINO MASSES AND MATRIX ELEMENTS ***********
      EPS=-1.D-10
      XC2=(M1*M2-MZ**2-MU**2)-3.D0/8.D0*(M1+M2)**2
      XC3=-1.D0/8.D0*(M1+M2)**3+1.D0/2.D0*(M1+M2)*(M1*M2-MZ**2
     .    -MU**2)+(M1+M2)*MU**2+(M1*CW**2+M2*SW**2)*MZ**2
     .    -MU*MZ**2*DSIN(2.D0*B)
      XC4=+(M1*CW**2+M2*SW**2)*MU*MZ**2*DSIN(2.D0*B)-M1*M2*MU**2
     .    +1.D0/4.D0*(M1+M2)*( (M1+M2)*MU**2+(M1*CW**2+M2*SW**2)
     .    *MZ**2-MU*MZ**2*DSIN(2.D0*B) )+1.D0/16.D0*(M1+M2)**2*
     .    (M1*M2-MZ**2-MU**2)-3.D0/256.D0*(M1+M2)**4
      XS=-XC3**2-2.D0/27.D0*XC2**3+8.D0/3.D0*XC2*XC4
      XU=-1.D0/3.D0*XC2**2-4.D0*XC4
      CXD=(-4*XU**3-27*XS**2)*DCMPLX(1.D0,EPS)
      CXC=1.D0/2.D0*(-XS+DCMPLX(0.D0,1.D0)*CDSQRT(CXD/27.D0))
      CXA=DBLE(CXC**(1.D0/3.D0))*DCMPLX(1.D0,-EPS)
      CXB=8.D0*CXA-8.D0/3.D0*XC2*DCMPLX(1.D0,-EPS)
C     *********** MASSES AND COUPLINGS:
      X0=(M1+M2)/4.D0
      CX1= CXA/2.D0-XC2/6.D0*DCMPLX(1.D0,-EPS)
      CX2=-CXA/2.D0-XC2/3.D0*DCMPLX(1.D0,-EPS)
      CX3=XC3*DCMPLX(1.D0,-EPS)/CDSQRT(CXB)
      XMN(1)=X0-CDABS(CDSQRT(CX1))+CDABS(CDSQRT(CX2+CX3))
      XMN(2)=X0+CDABS(CDSQRT(CX1))-CDABS(CDSQRT(CX2-CX3))
      XMN(3)=X0-CDABS(CDSQRT(CX1))-CDABS(CDSQRT(CX2+CX3))
      XMN(4)=X0+CDABS(CDSQRT(CX1))+CDABS(CDSQRT(CX2-CX3))
      DO 10 I=1,4
      MN(I)=DABS(XMN(I))
      YMN(I)=XMN(I)
      ZX(I,2)=-CW/SW*(M1-XMN(I))/(M2-XMN(I))
      ZX(I,3)=(MU*(M2-XMN(I))*(M1-XMN(I))-MZ**2*SB*CB*((M1-M2)*CW**2
     .       +M2-XMN(I)))/MZ/(M2-XMN(I))/SW/(MU*CB+XMN(I)*SB)
      ZX(I,4)=(-XMN(I)*(M2-XMN(I))*(M1-XMN(I))-MZ**2*CB*CB*((M1-M2)
     .       *CW**2+M2-XMN(I)))/MZ/(M2-XMN(I))/SW/(MU*CB+XMN(I)*SB)
      ZX(I,1)=1.D0/DSQRT(1.D0+ZX(I,2)**2+ZX(I,3)**2+ZX(I,4)**2) 
      YZ(I,1)=ZX(I,1)
      YZ(I,2)=ZX(I,2)*ZX(I,1)
      YZ(I,3)=ZX(I,3)*ZX(I,1)
      YZ(I,4)=ZX(I,4)*ZX(I,1)
 10   CONTINUE
C     *************  ORDERING THE DISORDER ******************
      XX0 = DMIN1(MN(1),MN(2),MN(3),MN(4))
      XX1 = DMAX1(MN(1),MN(2),MN(3),MN(4))
      IDUMMY = 1
      DO I = 1,4
       IF(MN(I).EQ.XX0)THEN
        IORD(1) = I
       ELSEIF(MN(I).EQ.XX1)THEN
        IORD(4) = I
       ELSE
        IREM(IDUMMY) = I
        IDUMMY = IDUMMY+1
       ENDIF
      ENDDO
      IF(MN(IREM(1)).LE.MN(IREM(2)))THEN
       IORD(2) = IREM(1)
       IORD(3) = IREM(2)
      ELSE
       IORD(2) = IREM(2)
       IORD(3) = IREM(1)
      ENDIF
C 
      DO 98 J=1,4
      I=IORD(J)
      XMN(J)=YMN(I)
c  #&! included for omega #&!
      dxmn(j) = xmn(j)
      MN(J) =DABS(YMN(I))
        DO I1=1,4
        Z(J,I1)=YZ(I,I1)
        ENDDO
 98   CONTINUE
C     ************  NEUTRALINO COUPLINGS TO HIGGS BOSONS ***********
	DO 11 I=1,4
	DO 11 J=1,4
	QQ(I,J)=1.D0/2.D0*(Z(I,3)*(Z(J,2)-TW*Z(J,1))+Z(J,3)*
     .		(Z(I,2)-TW*Z(I,1)))
	SS(I,J)=1.D0/2.D0*(Z(I,4)*(Z(J,2)-TW*Z(J,1))+Z(J,4)*
     .		(Z(I,2)-TW*Z(I,1)))
 11	CONTINUE
	DO 21 I=1,4
	DO 21 J=1,4
	AN1(I,J)= QQ(I,J)*DCOS(A)-SS(I,J)*DSIN(A)
	AN2(I,J)=-QQ(I,J)*DSIN(A)-SS(I,J)*DCOS(A)
	AN3(I,J)= QQ(I,J)*DSIN(B)-SS(I,J)*DCOS(B)
 21	CONTINUE

C       ************* CHARGINO MASSES AND MATRIX ELEMENTS ***********
	DELTA=DABS(B-.25*PI)
	DDD=MU*DCOS(B)+M2*DSIN(B)
	CCC=MU*DSIN(B)+M2*DCOS(B)
	IF(DELTA.LT.0.01D0) THEN
	PHIM=PI/4.D0-.5D0*DATAN((M2-MU)/(2.D0*MW))
	PHIP=PHIM
	ELSE IF	(DABS(CCC).LT.1.D-5) THEN
	PHIM=0.D0
	PHIP=DATAN(DSQRT(2.D0)*MW*DSIN(B)/(M2+1.D-5))
	ELSE IF	(DABS(DDD).LT.1.D-5) THEN
	PHIP=0.D0
	PHIM=DATAN(DSQRT(2.D0)*MW*DCOS(B)/(M2+1.D-5))
	ELSE
	RAD=DSQRT((M2**2-MU**2)**2+4.D0*MW**4*DCOS(2.D0*B)**2
     +	+4.D0*MW**2*(M2**2+MU**2+2.D0*M2*MU*DSIN(2.D0*B)))
	PHIP=DATAN((RAD-(M2**2-MU**2+2.D0*MW**2*DCOS(2.D0*B)))
     +	/(2.D0*DSQRT(2.D0)*MW*(MU*DCOS(B)+M2*DSIN(B))))
	PHIM=DATAN((RAD-(M2**2-MU**2-2.D0*MW**2*DCOS(2.D0*B)))
     +	/(2.D0*DSQRT(2.D0)*MW*(MU*DSIN(B)+M2*DCOS(B))))
	ENDIF
	CP=DCOS(PHIP)
	SP=DSIN(PHIP)
	CM=DCOS(PHIM)
	SM=DSIN(PHIM)
C  MY CONVENTION
	U(2,2)=CM
	U(2,1)=-SM
	U(1,2)=SM
	U(1,1)=CM
	V(1,1)=CP
	V(1,2)=SP
	V(2,1)=-SP
	V(2,2)=CP
        X(1,1)=M2
        X(1,2)=DSQRT(2.D0)*MW*DSIN(B)
        X(2,1)=DSQRT(2.D0)*MW*DCOS(B)
        X(2,2)=MU
 555    CONTINUE
       XMC(1)=(U(1,1)*X(1,1)+U(1,2)*X(2,1))*V(1,1)
     .       +(U(1,1)*X(1,2)+U(1,2)*X(2,2))*V(1,2)
       XMC(2)=(U(2,1)*X(1,1)+U(2,2)*X(2,1))*V(2,1)
     .       +(U(2,1)*X(1,2)+U(2,2)*X(2,2))*V(2,2)
        IF(XMC(1).LT.0.D0) THEN
	V(1,1)=-CP
	V(1,2)=-SP
	V(2,1)=-SP
	V(2,2)=CP
        GOTO 555
        ENDIF
        IF(XMC(2).LT.0.D0) THEN
	V(1,1)=CP
	V(1,2)=SP
	V(2,1)=SP
	V(2,2)=-CP
        GOTO 555
        ENDIF
        IF(XMC(1).GT.XMC(2)) THEN
        MTEMP=XMC(1)
        XMC(1)=XMC(2)
        XMC(2)=MTEMP
        DO J=1,2
        BU(J)=U(1,J)
        U(1,J)=U(2,J)
        U(2,J)=BU(J)
        BV(J)=V(1,J)
        V(1,J)=V(2,J)
        V(2,J)=BV(J)
        ENDDO
        ENDIF        
        MC(1)=DABS(XMC(1))
        MC(2)=DABS(XMC(2))

C     ************  CHARGINO COUPLINGS TO HIGGS BOSONS ***********
	DO 12 I=1,2
	DO 12 J=1,2
	Q(I,J)=DSQRT(1.D0/2.D0)*U(J,2)*V(I,1)
	S(I,J)=DSQRT(1.D0/2.D0)*U(J,1)*V(I,2)
 12	CONTINUE
	DO 22 I=1,2
	DO 22 J=1,2	
	AC1(I,J)= Q(I,J)*DCOS(A)+S(I,J)*DSIN(A)
	AC2(I,J)=-Q(I,J)*DSIN(A)+S(I,J)*DCOS(A)
	AC3(I,J)= Q(I,J)*DSIN(B)+S(I,J)*DCOS(B)
 22	CONTINUE
C     **** CHARGINO-NEUTRALINO COUPLINGS TO CHARGED HIGGS BOSONS 
	DO 13 I=1,2
	DO 13 J=1,4
        ACNL(I,J)=DCOS(B)*(Z(J,4)*V(I,1)+(Z(J,2)+Z(J,1)*TW)
     .       *V(I,2)/DSQRT(2.D0)) 
        ACNR(I,J)=DSIN(B)*(Z(J,3)*U(I,1)-(Z(J,2)+Z(J,1)*TW)
     .       *U(I,2)/DSQRT(2.D0)) 
 13     CONTINUE
       mchi = xmn(1)
       n01 = z(1,1)
       n02 = z(2,1)
       n03 = z(3,1)
       n04 = z(4,1)        
c
      if(inorc.eq.1.and.irge.ge.2) then
      m1 = m1save
      m2 = m2save
      mu = musave
      endif
       RETURN
       END

C   ****************************************************************
C     SUBROUTINE FOR SFERMION MASSES, MIXING AND COUPLINGS 
C   ****************************************************************

       SUBROUTINE SFERMION(MQL,MUR,MDR,MEL,MER,AL,AT,AB,MU,
     .                    MST,MSB,MSL,MSU,MSD,MSE,MSN)
      IMPLICIT REAL*8(A-H,K-Z)
      DIMENSION MST(2),MSB(2),MSL(2),MSU(2),MSD(2),MSE(2),MSN(4),
     .          GCEN(2,2),GCTB(2,2),GLEE(2,2),GLTT(2,2),GLBB(2,2),
     .          GHEE(2,2),GHTT(2,2),GHBB(2,2)
      common/s_masses/AMTAU,AMB,AMT
      COMMON/PARAM/GF,ALPH,MZ,MW
       common/cpl/g22,sw2
      COMMON/s_COUP/GAT,GAB,GLT,GLB,GHT,GHB,GZAH,GZAL,GHHH,GLLL,GHLL,
     .            GLHH,GHAA,GLAA,GLVV,GHVV,GLPM,GHPM,B,A
      COMMON/BREAK2/MELdum,MERdum,MUQdum,MURdum,MDRdum
      common/mixang/thet,theb,thel
      common/treeang/thettree,thebtree
      common/cplhsf1/gcen,gctb,glee,gltt,glbb,ghee,ghtt,ghbb
      common/cplhsf2/gatt,gabb,gaee
      COMMON/ERR_SF/sterr,sberr,stauerr
      common/mssmhpar/dum1,dum2,ma,dumu
      COMMON/OUTGINOS/dmc1,dmc2,dmn1,dmn2,dmn3,dmn4,mgluino
      common/yukaewsb/ytauewsb,ybewsb,ytewsb,alsewsb
      common/yuka/ytau,yb,ytop
      common/treesfer/msb1,msb2,mst1,mst2
      common/strc/irge,isfrc,inorc
C
      sferr = 0.d0
      MQL1=MUQdum
      MUR1=MURdum
      MDR1=MDRdum
      MEL1=MELdum
      MER1=MERdum
      PI = 4*DATAN(1.D0)
c      SW2=1.D0-MW**2/MZ**2
      TB=DTAN(B)
c  bug corrected next line: 17-11-99
c      MB = RUNM(AMT,5) 
c      MB = AMB
c      MT = RUNM(AMT,6) 
c      ML = AMTAU
c  !! CHANGE 02/04/01 due to inclusion of susy R.C.: 
c  MB, MT, ML used in sfermion mat. elts should be running masses 
c including susy corrections
       mb=amb
       mt=amt
       ml=amtau 
C FIRST TWO GENERATIONS:  NO MIXING INCLUDED 
C UP SQUARKS: 
      MSTL2=MQL1**2+(0.5D0-2.D0/3.D0*SW2)*MZ**2*DCOS(2.D0*B)
      MSTR2=MUR1**2+2.D0/3.D0*SW2*MZ**2*DCOS(2.D0*B) 
      MSU(1)=DSQRT(MSTL2)
      MSU(2)=DSQRT(MSTR2)
      if(msu(1).gt.msu(2)) then
      msutemp= msu(1)
      msu(1)=msu(2)
      msu(2)=msutemp
      endif
C DOWN SQUARKS
      MSBL2=MQL1**2+(-0.5D0+1.D0/3.D0*SW2)*MZ**2*DCOS(2.D0*B)
      MSBR2=MDR1**2-1.D0/3.D0*SW2*MZ**2*DCOS(2.D0*B) 
      MSD(1)=DSQRT(MSBL2)
      MSD(2)=DSQRT(MSBR2)
      if(msd(1).gt.msd(2)) then
      msdtemp= msd(1)
      msd(1)=msd(2)
      msd(2)=msdtemp
      endif
C SLEPTONS
      MSEL2=MEL1**2+(-0.5D0+SW2)*MZ**2*DCOS(2.D0*B)
      MSER2=MER1**2- SW2*MZ**2*DCOS(2.D0*B) 
      MSNL2=MEL1**2+0.5D0*MZ**2*DCOS(2.D0*B)
      MSE(1)=DSQRT(MSEL2)
      MSE(2)=DSQRT(MSER2)
      if(mse(1).gt.mse(2)) then
      msetemp= mse(1)
      mse(1)=mse(2)
      mse(2)=msetemp
      endif
      MSN(1)=DSQRT(MSNL2)
      MSN(2)=1.D+15
c  add radiative corrections to first gen. squarks
      if(isfrc.eq.1.and.irge.ge.2) then
      call sqcr(alsewsb,mgluino,msu(1),   dmsu1)
      call sqcr(alsewsb,mgluino,msu(2),   dmsu2)
      call sqcr(alsewsb,mgluino,msd(1),   dmsd1)
      call sqcr(alsewsb,mgluino,msd(2),   dmsd2)
      msu(1)=msu(1)+dmsu1
      msu(2)=msu(2)+dmsu2
      msd(1)=msd(1)+dmsd1
      msd(2)=msd(2)+dmsd2
      endif
C NOW THE THIRD GENERATION
C
C STOP MASSES/MIXING
C     first some reinitializations:
       ifirst=0
       crll=0.d0
       crRR=0.d0
       crlr=0.d0
 1     MSTL2=MQL**2+(0.5D0-2.D0/3.D0*SW2)*MZ**2*DCOS(2.D0*B)  +crLL
      MSTR2=MUR**2+2.D0/3.D0*SW2*MZ**2*DCOS(2.D0*B)        +crRR
      MLRT=AT-MU/TB                                     +crLR/mt
      DELT=(MSTL2-MSTR2)**2+4*MT**2*MLRT**2
      MST12=MT**2+0.5D0*(MSTL2+MSTR2-DSQRT(DELT))
      MST22=MT**2+0.5D0*(MSTL2+MSTR2+DSQRT(DELT))
        IF(MST12.LT.0.D0)THEN 
c  tachyonic sfermion 1 mass
       sterr=-1.d0
      GOTO 111 
      ELSE 
      MST(1)=DSQRT(MST12)
      MST(2)=DSQRT(MST22)
      THET=0.5D0*DATAN(2.D0*MT*MLRT / (MSTL2-MSTR2) )
      IF(MSTL2.GT.MSTR2) THET = THET + PI/2
        ENDIF 
      if(ifirst.eq.0) then
c save tree-level values for other uses:
      mst1=mst(1)
      mst2=mst(2)
      thettree=thet
      endif
c  adding rad. corr.
      if(isfrc.eq.1.and.irge.ge.2.and.ifirst.eq.0) then
      ifirst=1
c 
      call stopcr(alsewsb,mgluino,mu,MA,thet,b,mst(1),mst(2),msb(1),
     . ytewsb,ybewsb,AT,AB,  crLL,crLR,crRR)
       goto 1
       endif
      CT= DCOS(THET)
      ST= DSIN(THET) 
C
C SBOTTOM MASSES/MIXING
C
      MSBL2=MQL**2+(-0.5D0+1.D0/3.D0*SW2)*MZ**2*DCOS(2.D0*B)
      MSBR2=MDR**2-1.D0/3.D0*SW2*MZ**2*DCOS(2.D0*B) 
      MLRB=AB-MU*TB
      DELB=(MSBL2-MSBR2)**2+4*MB**2*MLRB**2
      MSB12=MB**2+0.5D0*(MSBL2+MSBR2-DSQRT(DELB))
      MSB22=MB**2+0.5D0*(MSBL2+MSBR2+DSQRT(DELB))
        IF(MSB12.LT.0.D0)THEN
c   Tachyonic sfermion mass
      sberr=-1.d0
      GOTO 111
        ELSE
      MSB(1)=DSQRT(MSB12)
      MSB(2)=DSQRT(MSB22)
c      msb1=msb(1)
c      msb2=msb(2)
      THEB=0.5D0*DATAN(2.D0*MB*MLRB / (MSBL2-MSBR2) )
      IF(MSBL2.GT.MSBR2) THEB = THEB + PI/2
        ENDIF  
       if(ifirst.eq.0) then
c save tree-level values for other uses:
      msb1=msb(1)
      msb2=msb(2)
      thebtree=theb
      endif
      CB= DCOS(THEB)
      SB= DSIN(THEB) 
c  add radiative corrections to sbottom  quarks
      if(isfrc.eq.1.and.irge.ge.2) then
      call sqcr(alsewsb,mgluino,msb(1),   dmsb1)
      call sqcr(alsewsb,mgluino,msb(2),   dmsb2)
      msb(1)=msb(1)+dmsb1
      msb(2)=msb(2)+dmsb2
      endif
      
C
C  STAU MASSES/MIXING
C
      MSEL2=MEL**2+(-0.5D0+SW2)*MZ**2*DCOS(2.D0*B)
      MSER2=MER**2- SW2*MZ**2*DCOS(2.D0*B) 
      MSNtau2=MEL**2+0.5D0*MZ**2*DCOS(2.D0*B)
      MLRE=AL-MU*TB
      DELE=(MSEL2-MSER2)**2+4*ML**2*MLRE**2
      MSE12=ML**2+0.5D0*(MSEL2+MSER2-DSQRT(DELE))
      MSE22=ML**2+0.5D0*(MSEL2+MSER2+DSQRT(DELE))
        IF(MSE12.LT.0.D0)THEN
c   Tachyonic sfermion mass
      stauerr=-1.d0
      GOTO 111
        ELSE
      MSL(1)=DSQRT(MSE12)
      MSL(2)=DSQRT(MSE22)
      THEL=0.5D0*DATAN(2.D0*ML*MLRE / (MSEL2-MSER2) )
      IF(MSEL2.GT.MSER2) THEL = THEL + PI/2
        ENDIF  
      CL= DCOS(THEL)
      SL= DSIN(THEL) 
c   tau sneutrino:
      msn(3)=dsqrt(msntau2) 
      MSN(4)=1.D+15
c NB: for convenience msn(1--4) contains: 
c msn_{e,mu}(1),msn_{e,mu}(2), msn_{tau}(1),msn_{tau}(2) 
C
C LIGHT CP--EVEN HIGGS COUPLINGS TO STOPS
C 
      GLTT(1,1)=-DSIN(B+A)*(0.5D0*CT**2-2.D0/3.D0*SW2*DCOS(2*THET)) 
     .    + MT**2/MZ**2*GLT + MT*ST*CT/MZ**2*(AT*GLT+MU*GHT)
      GLTT(2,2)=-DSIN(B+A)*(0.5D0*ST**2+2.D0/3.D0*SW2*DCOS(2*THET))
     .    + MT**2/MZ**2*GLT - MT*ST*CT/MZ**2*(AT*GLT+MU*GHT)
      GLTT(1,2)=-2*DSIN(B+A)*ST*CT*(2.D0/3.D0*SW2-0.25D0)
     .    + MT*DCOS(2*THET)/2.D0/MZ**2*(AT*GLT+MU*GHT) 
      GLTT(2,1)=-2*DSIN(B+A)*ST*CT*(2.D0/3.D0*SW2-0.25D0)
     .    + MT*DCOS(2*THET)/2.D0/MZ**2*(AT*GLT+MU*GHT) 
C
C LIGHT CP--EVEN HIGGS COUPLINGS TO SBOTTOMS
C
      GLBB(1,1)=-DSIN(B+A)*(-0.5D0*CB**2+1.D0/3.D0*SW2*DCOS(2*THEB)) 
     .    + MB**2/MZ**2*GLB + MB*SB*CB/MZ**2*(AB*GLB-MU*GHB)
      GLBB(2,2)=-DSIN(B+A)*(-0.5D0*SB**2-1.D0/3.D0*SW2*DCOS(2*THEB)) 
     .    + MB**2/MZ**2*GLB - MB*SB*CB/MZ**2*(AB*GLB-MU*GHB)
      GLBB(1,2)=-2*DSIN(B+A)*SB*CB*(-1.D0/3.D0*SW2+0.25D0)
     .    + MB*DCOS(2*THEB)/2.D0/MZ**2*(AB*GLB-MU*GHB) 
      GLBB(2,1)=-2*DSIN(B+A)*SB*CB*(-1.D0/3.D0*SW2+0.25D0)
     .    + MB*DCOS(2*THEB)/2.D0/MZ**2*(AB*GLB-MU*GHB) 

C
C LIGHT CP--EVEN HIGGS COUPLINGS TO STAU'S 
C
      GLEE(1,1)=-DSIN(B+A)*(-0.5D0*CL**2+SW2*DCOS(2*THEL)) 
     .    + ML**2/MZ**2*GLB + ML*SL*CL/MZ**2*(AL*GLB-MU*GHB)
      GLEE(2,2)=-DSIN(B+A)*(-0.5D0*SL**2-SW2*DCOS(2*THEL)) 
     .    + ML**2/MZ**2*GLB - ML*SL*CL/MZ**2*(AL*GLB-MU*GHB)
      GLEE(1,2)=-2*DSIN(B+A)*SL*CL*(-SW2+0.25D0)
     .    + ML*DCOS(2*THEL)/2.D0/MZ**2*(AL*GLB-MU*GHB) 
      GLEE(2,1)=-2*DSIN(B+A)*SL*CL*(-SW2+0.25D0)
     .    + ML*DCOS(2*THEL)/2.D0/MZ**2*(AL*GLB-MU*GHB) 
C
C HEAVY CP--EVEN HIGGS COUPLINGS TO STOPS
C
      GHTT(1,1)=DCOS(B+A)*(0.5D0*CT**2-2.D0/3.D0*SW2*DCOS(2*THET)) 
     .    + MT**2/MZ**2*GHT + MT*ST*CT/MZ**2*(AT*GHT-MU*GLT)
      GHTT(2,2)= DCOS(B+A)*(0.5D0*ST**2+2.D0/3.D0*SW2*DCOS(2*THET))
     .    + MT**2/MZ**2*GHT - MT*ST*CT/MZ**2*(AT*GHT-MU*GLT)
      GHTT(1,2)=2*DCOS(B+A)*ST*CT*(2.D0/3.D0*SW2-0.25D0)
     .    + MT*DCOS(2*THET)/2.D0/MZ**2*(AT*GHT-MU*GLT) 
      GHTT(2,1)=2*DCOS(B+A)*ST*CT*(2.D0/3.D0*SW2-0.25D0)
     .    + MT*DCOS(2*THET)/2.D0/MZ**2*(AT*GHT-MU*GLT) 
C
C HEAVY CP--EVEN HIGGS COUPLINGS TO SBOTTOMS
C
      GHBB(1,1)= DCOS(B+A)*(-0.5D0*CB**2+1.D0/3.D0*SW2*DCOS(2*THEB)) 
     .    + MB**2/MZ**2*GHB + MB*SB*CB/MZ**2*(AB*GHB+MU*GLB)
      GHBB(2,2)= DCOS(B+A)*(-0.5D0*SB**2-1.D0/3.D0*SW2*DCOS(2*THEB)) 
     .    + MB**2/MZ**2*GHB - MB*SB*CB/MZ**2*(AB*GHB+MU*GLB)
      GHBB(1,2)=2*DCOS(B+A)*SB*CB*(-1.D0/3.D0*SW2+0.25D0)
     .    + MB*DCOS(2*THEB)/2.D0/MZ**2*(AB*GHB+MU*GLB) 
      GHBB(2,1)=2*DCOS(B+A)*SB*CB*(-1.D0/3.D0*SW2+0.25D0)
     .    + MB*DCOS(2*THEB)/2.D0/MZ**2*(AB*GHB+MU*GLB) 
C
C HEAVY CP--EVEN HIGGS COUPLINGS TO STAU'S 
C
      GHEE(1,1)= DCOS(B+A)*(-0.5D0*CL**2+SW2*DCOS(2*THEL)) 
     .    + ML**2/MZ**2*GHB + ML*SL*CL/MZ**2*(AL*GHB+MU*GLB)
      GHEE(2,2)= DCOS(B+A)*(-0.5D0*SL**2-SW2*DCOS(2*THEL)) 
     .    + ML**2/MZ**2*GHB - ML*SL*CL/MZ**2*(AL*GHB+MU*GLB)
      GHEE(1,2)=2*DCOS(B+A)*SL*CL*(-SW2+0.25D0)
     .    + ML*DCOS(2*THEL)/2.D0/MZ**2*(AL*GHB+MU*GLB) 
      GHEE(2,1)=2*DCOS(B+A)*SL*CL*(-SW2+0.25D0)
     .    + ML*DCOS(2*THEL)/2.D0/MZ**2*(AL*GHB+MU*GLB) 

C
C PSEUDOSCALAR COUPLINGS 
C
      GATT=-MT/2.D0/MZ**2*(MU+AT*GAT) 
      GABB=-MB/2.D0/MZ**2*(MU+AB*GAB) 
      GAEE=-ML/2.D0/MZ**2*(MU+AL*GAB) 
C
C CHARGED HIGGS COUPLINGS STOPS/SBOTTOMS 
C
      CLL=(MW**2*DSIN(2*B)-MT**2*GAT-MB**2*GAB)/DSQRT(2.D0)/MW**2
      CRR=-MT*MB*(GAT+GAB)/DSQRT(2.D0)/MW**2
      CLR=-MB*(MU+AB*GAB)/DSQRT(2.D0)/MW**2
      CRL=-MT*(MU+AT*GAT)/DSQRT(2.D0)/MW**2
      GCTB(1,1)=+CT*CB*CLL+ST*SB*CRR+CT*SB*CLR+ST*CB*CRL
      GCTB(1,2)=-CT*SB*CLL+ST*CB*CRR+CT*CB*CLR-ST*SB*CRL
      GCTB(2,1)=-ST*CB*CLL+CT*SB*CRR-ST*SB*CLR+CT*CB*CRL
      GCTB(2,2)=+ST*SB*CLL+CT*CB*CRR-ST*CB*CLR-CT*SB*CRL

C
C CHARGED HIGGS COUPLINGS TAU'S AND NEUTRINOS 
C
      CLL=(MW**2*DSIN(2*B)-ML**2*GAB)/DSQRT(2.D0)/MW**2
      CLR=-ML*(MU+AL*GAB)/DSQRT(2.D0)/MW**2
      GCEN(1,1)=CL*CLL+SL*CLR
      GCEN(1,2)=-SL*CLL+CL*CLR
      GCEN(2,1)=0.D0
      GCEN(2,2)=0.D0 
c
 111  RETURN 
      END 
C ******************************************************************
C        SUBROUTINES FOR RUNNING MASSES AND ALPHAS
C *******************************************************************
      DOUBLE PRECISION FUNCTION RUNM(Q,NF)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NN=6)
      PARAMETER (ZETA3 = 1.202056903159594D0)
      DIMENSION AM(NN),YMSB(NN)
      COMMON/ALS/XLAMBDA,AMCA,AMBA,AMTA,N0A
      common/fmasses/AMTAU,AMB,AMT
      COMMON/QMASSES/AMS,AMC
      COMMON/STRANGE/AMSB
      COMMON/RUN/XMSB,XMHAT,XKFAC
      COMMON/FLAG/NNLO,IPOLE
      SAVE ISTRANGE
      B0(NF)=(33.D0-2.D0*NF)/12D0
      B1(NF) = (102D0-38D0/3D0*NF)/16D0
      B2(NF) = (2857D0/2D0-5033D0/18D0*NF+325D0/54D0*NF**2)/64D0
      G0(NF) = 1D0
      G1(NF) = (202D0/3D0-20D0/9D0*NF)/16D0
      G2(NF) = (1249D0-(2216D0/27D0+160D0/3D0*ZETA3)*NF
     .       - 140D0/81D0*NF**2)/64D0
      C1(NF) = G1(NF)/B0(NF) - B1(NF)*G0(NF)/B0(NF)**2
      C2(NF) = ((G1(NF)/B0(NF) - B1(NF)*G0(NF)/B0(NF)**2)**2
     .       + G2(NF)/B0(NF) + B1(NF)**2*G0(NF)/B0(NF)**3
     .       - B1(NF)*G1(NF)/B0(NF)**2 - B2(NF)*G0(NF)/B0(NF)**2)/2D0
      TRAN(X,XK)=1D0+4D0/3D0*ALPHAS(X,2)/PI+XK*(ALPHAS(X,2)/PI)**2
      CQ(X,NF)=(2D0*B0(NF)*X)**(G0(NF)/B0(NF))
     .            *(1D0+C1(NF)*X+C2(NF)*X**2)
      DATA ISTRANGE/0/
      PI=4D0*DATAN(1D0)
      ACC = 1.D-8
      AM(1) = 0
      AM(2) = 0
C--------------------------------------------
      IMSBAR = 0
      IF(IMSBAR.EQ.1)THEN
       IF(ISTRANGE.EQ.0)THEN
C--STRANGE POLE MASS FROM MSBAR-MASS AT 1 GEV
        AMSD = XLAMBDA
        AMSU = 1.D8
123     AMS  = (AMSU+AMSD)/2
        AM(3) = AMS
        XMSB = AMS/CQ(ALPHAS(AMS,2)/PI,3)
     .            *CQ(ALPHAS(1.D0,2)/PI,3)/TRAN(AMS,0D0)
        DD = (XMSB-AMSB)/AMSB
        IF(DABS(DD).GE.ACC)THEN
         IF(DD.LE.0.D0)THEN
          AMSD = AM(3)
         ELSE
          AMSU = AM(3)
         ENDIF
         GOTO 123
        ENDIF
        ISTRANGE=1
       ENDIF
       AM(3) = AMSB
      ELSE
       AMS=AMSB
       AM(3) = AMS
      ENDIF
C--------------------------------------------
      AM(3) = AMSB
      AM(4) = AMC
      AM(5) = AMB
      AM(6) = AMT
      XK = 16.11D0
      DO 1 I=1,NF-1
       XK = XK - 1.04D0*(1.D0-AM(I)/AM(NF))
1     CONTINUE
      IF(NF.GE.4)THEN
       XMSB = AM(NF)/TRAN(AM(NF),0D0)
       XMHAT = XMSB/CQ(ALPHAS(AM(NF),2)/PI,NF)
      ELSE
       XMSB = 0
       XMHAT = 0
      ENDIF
      YMSB(3) = AMSB
      IF(NF.EQ.3)THEN
       YMSB(4) = YMSB(3)*CQ(ALPHAS(AM(4),2)/PI,3)/
     .                   CQ(ALPHAS(1.D0,2)/PI,3)
       YMSB(5) = YMSB(4)*CQ(ALPHAS(AM(5),2)/PI,4)/
     .                   CQ(ALPHAS(AM(4),2)/PI,4)
       YMSB(6) = YMSB(5)*CQ(ALPHAS(AM(6),2)/PI,5)/
     .                   CQ(ALPHAS(AM(5),2)/PI,5)
      ELSEIF(NF.EQ.4)THEN
       YMSB(4) = XMSB
       YMSB(5) = YMSB(4)*CQ(ALPHAS(AM(5),2)/PI,4)/
     .                   CQ(ALPHAS(AM(4),2)/PI,4)
       YMSB(6) = YMSB(5)*CQ(ALPHAS(AM(6),2)/PI,5)/
     .                   CQ(ALPHAS(AM(5),2)/PI,5)
      ELSEIF(NF.EQ.5)THEN
       YMSB(5) = XMSB
       YMSB(4) = YMSB(5)*CQ(ALPHAS(AM(4),2)/PI,4)/
     .                   CQ(ALPHAS(AM(5),2)/PI,4)
       YMSB(6) = YMSB(5)*CQ(ALPHAS(AM(6),2)/PI,5)/
     .                   CQ(ALPHAS(AM(5),2)/PI,5)
      ELSEIF(NF.EQ.6)THEN
       YMSB(6) = XMSB
       YMSB(5) = YMSB(6)*CQ(ALPHAS(AM(5),2)/PI,5)/
     .                   CQ(ALPHAS(AM(6),2)/PI,5)
       YMSB(4) = YMSB(5)*CQ(ALPHAS(AM(4),2)/PI,4)/
     .                   CQ(ALPHAS(AM(5),2)/PI,4)
      ENDIF
      IF(Q.LT.AMC)THEN
       N0=3
       Q0 = 1.D0
      ELSEIF(Q.LE.AMB)THEN
       N0=4
       Q0 = AMC
      ELSEIF(Q.LE.AMT)THEN
       N0=5
       Q0 = AMB
      ELSE
       N0=6
       Q0 = AMT
      ENDIF
      IF(NNLO.EQ.1.AND.NF.GT.3)THEN
       XKFAC = TRAN(AM(NF),0D0)/TRAN(AM(NF),XK)
      ELSE
       XKFAC = 1D0
      ENDIF
      RUNM = YMSB(N0)*CQ(ALPHAS(Q,2)/PI,N0)/
     .               CQ(ALPHAS(Q0,2)/PI,N0)
     .       * XKFAC
      RETURN
      END

      DOUBLE PRECISION FUNCTION ALPHAS(Q,N)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION XLB(6)
      COMMON/ALSLAM/XLB1(6),XLB2(6)
      COMMON/ALS/XLAMBDA,AMC,AMB,AMT,N0
      B0(NF)=33.D0-2.D0*NF
      B1(NF)=6.D0*(153.D0-19.D0*NF)/B0(NF)**2
      ALS1(NF,X)=12.D0*PI/(B0(NF)*DLOG(X**2/XLB(NF)**2))
      ALS2(NF,X)=12.D0*PI/(B0(NF)*DLOG(X**2/XLB(NF)**2))
     .          *(1.D0-B1(NF)*DLOG(DLOG(X**2/XLB(NF)**2))
     .           /DLOG(X**2/XLB(NF)**2))
      PI=4.D0*DATAN(1.D0)
      IF(N.EQ.1)THEN
       DO 1 I=1,6
        XLB(I)=XLB1(I)
1      CONTINUE
      ELSE
       DO 2 I=1,6
        XLB(I)=XLB2(I)
2      CONTINUE
      ENDIF
      IF(Q.LT.AMC)THEN
       NF=3
      ELSEIF(Q.LE.AMB)THEN
       NF=4
      ELSEIF(Q.LE.AMT)THEN
       NF=5
      ELSE
       NF=6
      ENDIF
      IF(N.EQ.1)THEN
        ALPHAS=ALS1(NF,Q)
      ELSE
        ALPHAS=ALS2(NF,Q)
      ENDIF
      RETURN
      END

      SUBROUTINE ALSINI(ACC)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION XLB(6)
      COMMON/ALSLAM/XLB1(6),XLB2(6)
      COMMON/ALS/XLAMBDA,AMC,AMB,AMT,N0
      PI=4.D0*DATAN(1.D0)
      XLB1(1)=0D0
      XLB1(2)=0D0
      XLB2(1)=0D0
      XLB2(2)=0D0
      IF(N0.EQ.3)THEN
       XLB(3)=XLAMBDA
       XLB(4)=XLB(3)*(XLB(3)/AMC)**(2.D0/25.D0)
       XLB(5)=XLB(4)*(XLB(4)/AMB)**(2.D0/23.D0)
       XLB(6)=XLB(5)*(XLB(5)/AMT)**(2.D0/21.D0)
      ELSEIF(N0.EQ.4)THEN
       XLB(4)=XLAMBDA
       XLB(5)=XLB(4)*(XLB(4)/AMB)**(2.D0/23.D0)
       XLB(3)=XLB(4)*(XLB(4)/AMC)**(-2.D0/27.D0)
       XLB(6)=XLB(5)*(XLB(5)/AMT)**(2.D0/21.D0)
      ELSEIF(N0.EQ.5)THEN
       XLB(5)=XLAMBDA
       XLB(4)=XLB(5)*(XLB(5)/AMB)**(-2.D0/25.D0)
       XLB(3)=XLB(4)*(XLB(4)/AMC)**(-2.D0/27.D0)
       XLB(6)=XLB(5)*(XLB(5)/AMT)**(2.D0/21.D0)
      ELSEIF(N0.EQ.6)THEN
       XLB(6)=XLAMBDA
       XLB(5)=XLB(6)*(XLB(6)/AMT)**(-2.D0/23.D0)
       XLB(4)=XLB(5)*(XLB(5)/AMB)**(-2.D0/25.D0)
       XLB(3)=XLB(4)*(XLB(4)/AMC)**(-2.D0/27.D0)
      ENDIF
      DO 1 I=1,6
       XLB1(I)=XLB(I)
1     CONTINUE
      IF(N0.EQ.3)THEN
       XLB(3)=XLAMBDA
       XLB(4)=XLB(3)*(XLB(3)/AMC)**(2.D0/25.D0)
     .             *(2.D0*DLOG(AMC/XLB(3)))**(-107.D0/1875.D0)
       XLB(4)=XITER(AMC,XLB(3),3,XLB(4),4,ACC)
       XLB(5)=XLB(4)*(XLB(4)/AMB)**(2.D0/23.D0)
     .             *(2.D0*DLOG(AMB/XLB(4)))**(-963.D0/13225.D0)
       XLB(5)=XITER(AMB,XLB(4),4,XLB(5),5,ACC)
       XLB(6)=XLB(5)*(XLB(5)/AMT)**(2.D0/21.D0)
     .            *(2.D0*DLOG(AMT/XLB(5)))**(-321.D0/3381.D0)
       XLB(6)=XITER(AMT,XLB(5),5,XLB(6),6,ACC)
      ELSEIF(N0.EQ.4)THEN
       XLB(4)=XLAMBDA
       XLB(5)=XLB(4)*(XLB(4)/AMB)**(2.D0/23.D0)
     .             *(2.D0*DLOG(AMB/XLB(4)))**(-963.D0/13225.D0)
       XLB(5)=XITER(AMB,XLB(4),4,XLB(5),5,ACC)
       XLB(3)=XLB(4)*(XLB(4)/AMC)**(-2.D0/27.D0)
     .             *(2.D0*DLOG(AMC/XLB(4)))**(107.D0/2025.D0)
       XLB(3)=XITER(AMC,XLB(4),4,XLB(3),3,ACC)
       XLB(6)=XLB(5)*(XLB(5)/AMT)**(2.D0/21.D0)
     .            *(2.D0*DLOG(AMT/XLB(5)))**(-321.D0/3381.D0)
       XLB(6)=XITER(AMT,XLB(5),5,XLB(6),6,ACC)
      ELSEIF(N0.EQ.5)THEN
       XLB(5)=XLAMBDA
       XLB(4)=XLB(5)*(XLB(5)/AMB)**(-2.D0/25.D0)
     .             *(2.D0*DLOG(AMB/XLB(5)))**(963.D0/14375.D0)
       XLB(4)=XITER(AMB,XLB(5),5,XLB(4),4,ACC)
       XLB(3)=XLB(4)*(XLB(4)/AMC)**(-2.D0/27.D0)
     .             *(2.D0*DLOG(AMC/XLB(4)))**(107.D0/2025.D0)
       XLB(3)=XITER(AMC,XLB(4),4,XLB(3),3,ACC)
       XLB(6)=XLB(5)*(XLB(5)/AMT)**(2.D0/21.D0)
     .            *(2.D0*DLOG(AMT/XLB(5)))**(-321.D0/3381.D0)
       XLB(6)=XITER(AMT,XLB(5),5,XLB(6),6,ACC)
      ELSEIF(N0.EQ.6)THEN
       XLB(6)=XLAMBDA
       XLB(5)=XLB(6)*(XLB(6)/AMT)**(-2.D0/23.D0)
     .            *(2.D0*DLOG(AMT/XLB(6)))**(321.D0/3703.D0)
       XLB(5)=XITER(AMT,XLB(6),6,XLB(5),5,ACC)
       XLB(4)=XLB(5)*(XLB(5)/AMB)**(-2.D0/25.D0)
     .             *(2.D0*DLOG(AMB/XLB(5)))**(963.D0/14375.D0)
       XLB(4)=XITER(AMB,XLB(5),5,XLB(4),4,ACC)
       XLB(3)=XLB(4)*(XLB(4)/AMC)**(-2.D0/27.D0)
     .             *(2.D0*DLOG(AMC/XLB(4)))**(107.D0/2025.D0)
       XLB(3)=XITER(AMC,XLB(4),4,XLB(3),3,ACC)
      ENDIF
      DO 2 I=1,6
       XLB2(I)=XLB(I)
2     CONTINUE
      RETURN
      END

      DOUBLE PRECISION FUNCTION XITER(Q,XLB1,NF1,XLB,NF2,ACC)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      B0(NF)=33.D0-2.D0*NF
      B1(NF)=6.D0*(153.D0-19.D0*NF)/B0(NF)**2
      ALS2(NF,X,XLB)=12.D0*PI/(B0(NF)*DLOG(X**2/XLB**2))
     .              *(1.D0-B1(NF)*DLOG(DLOG(X**2/XLB**2))
     .              /DLOG(X**2/XLB**2))
      AA(NF)=12D0*PI/B0(NF)
      BB(NF)=B1(NF)/AA(NF)
      XIT(A,B,X)=A/2.D0*(1D0+DSQRT(1D0-4D0*B*DLOG(X)))
      PI=4.D0*DATAN(1.D0)
      XLB2=XLB
      II=0
1     II=II+1
      X=DLOG(Q**2/XLB2**2)
      ALP=ALS2(NF1,Q,XLB1)
      A=AA(NF2)/ALP
      B=BB(NF2)*ALP
      XX=XIT(A,B,X)
      XLB2=Q*DEXP(-XX/2.D0)
      Y1=ALS2(NF1,Q,XLB1)
      Y2=ALS2(NF2,Q,XLB2)
      DY=DABS(Y2-Y1)/Y1
      IF(DY.GE.ACC) GOTO 1
      XITER=XLB2
      RETURN
      END

      DOUBLE PRECISION FUNCTION XITLA(NO,ALP,ACC)
C--ITERATION ROUTINE TO DETERMINE IMPROVED LAMBDA'S
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/PARAM/GF,ALPH,AMZ,AMW
      B0(NF)=33.D0-2.D0*NF
      B1(NF)=6.D0*(153.D0-19.D0*NF)/B0(NF)**2
      ALS2(NF,X,XLB)=12.D0*PI/(B0(NF)*DLOG(X**2/XLB**2))
     .              *(1.D0-B1(NF)*DLOG(DLOG(X**2/XLB**2))
     .              /DLOG(X**2/XLB**2))
      AA(NF)=12D0*PI/B0(NF)
      BB(NF)=B1(NF)/AA(NF)
      XIT(A,B,X)=A/2.D0*(1D0+DSQRT(1D0-4D0*B*DLOG(X)))
      PI=4.D0*DATAN(1.D0)
      NF=5
      Q=AMZ
      XLB=Q*DEXP(-AA(NF)/ALP/2.D0)
      IF(NO.EQ.1)GOTO 111
      II=0
1     II=II+1
      X=DLOG(Q**2/XLB**2)
      A=AA(NF)/ALP
      B=BB(NF)*ALP
      XX=XIT(A,B,X)
      XLB=Q*DEXP(-XX/2.D0)
      Y1=ALP
      Y2=ALS2(NF,Q,XLB)
      DY=DABS(Y2-Y1)/Y1
      IF(DY.GE.ACC) GOTO 1
111   XITLA=XLB
      RETURN
      END

c---------------------------------------- 
       subroutine DERIV1(x,y,dydx)
       implicit real*8(a-h,m,o-z)
c--- derivatives of y(xN) i.e these are exactly the beta functions)
c NB: DERIV1 includes only 1-LOOP RGE, 
c with simple (unique scale) threshold. 
c NB: only used at first iteration (when mssm spectrum undefined yet) 
       parameter (n=28)
       dimension y(n),dydx(n),ygut(8)
       common/cte/nf,cpi,zm,wm,tbeta
       common/s_thresh/rmtop,susym,egut
       common/stepwi/wistep,h1,kpole
       common/stegut/ifirst,jfirst,ygut
       common/gunif/kunif
       common/sgnM123/sgnM1,sgnM2,sgnM3
      COMMON/MSSMsqua/MSQ,MTR,MBR,MUQ,MUR,MDR
      common/treesfer/msbtr1,msbtr2,msttr1,msttr2
      common/yukaewsb/ytau,yb,yt,alsewsb
       common/renscale/scale
       real*8 nu,nd,ne,nn,nsq,n1sq,nsu,n1su,nsd,n1sd,nsl,n1sl,nse,n1se,
     . nhino,nh,nbino,nwino,ngino,nf,nhl,nhlino,nhh,nhhino
       data nn/3.d0/,nd/3.d0/,ne/3.d0/
c NB: cpi = 1/(16 pi^2)
c  a smoothed step function :   
      step(x) = .5d0*(1.d0 +dtanh(wistep*x))
      pi=4*datan(1.d0)
      Q = dexp(x)
c g1, g2 gauge unif.:
      if(kunif.ne.0.and.h1.gt.0.d0.and.Q.gt.1.d15) then
      if(y(1).ge.y(2)) then
       if(ifirst.eq.0) then
       egut=Q
c freeze out the gauge +Yukawa+vu,vd couplings after that
       do ii=1,8 
       ygut(ii)=y(ii)
       enddo
       endif
      ifirst=1      
      endif
      endif
c simple (unique scale) threshold in beta functions:
      mtop =174.3d0
      st1= step(Q-mtop)
      st2= step(Q-susym)
      nu=2.d0 +st1
      nsq=3*st2
      n1sq=st2
      nsu=3*st2
      n1su=st2
      nsd=3*st2
      n1sd=st2
      nsl=3*st2
      n1sl=st2
      nse=3*st2
      n1se=st2      
      nhino=2.d0*st2
      nbino=st2
      nwino=st2
      ngino=st2
      nh=1.d0+st2
      nhl=st2
      nhh=st2
      nhlino=st2
      nhhino=st2
c       
c
       b10 = 2.d0/5.d0*(17*nu/12.d0+5*nd/12.d0+5*ne/4.d0+nn/4.d0)
     . +nsq/30.d0
     . +4*nsu/15.d0+nsd/15.d0+nsl/10.d0+nse/5.d0
     . +nhino/5.d0 +nh/10.d0
       b20 = -22.d0/3.d0+(nu+nd)/2.d0+(ne+nn)/6.d0
     . +nsq/2.d0 + nsl/6.d0 + nhino/3.d0
     . +nh/6.d0 +4*nwino/3.d0
       b30 = 2*(nu+nd)/3.d0 +nsq/3.d0+nsu/6.d0+nsd/6.d0+2*ngino -11.d0
c - gauge coupling beta functions (nb the variables are g^2):
c -y(1)--y(3): g1^2,g2^2,g3^2.

       dydx(1) = 2*cpi*b10*y(1)*y(1) 
       dydx(2) = 2*cpi*b20*y(2)*y(2)
       dydx(3) = 2*cpi*b30*y(3)*y(3)

c - Yukawa coupling beta function (only Ytau,Yb,Ytop included):
c -y(4)--y(6): Ytau,Yb,Ytop
       ytau2 =y(4)*y(4)
       yb2 = y(5)*y(5)
       ytop2 = y(6)*y(6)

c threshold effects:
      tbet=dexp(y(7)-y(8))
      cb2=1.d0/(1.d0+tbet*tbet)
      sb2=1.d0-cb2
      Ytaubeta = 3*(cb2*(nhl+nhlino)+sb2*(nhh+nhhino))/2.d0*y(4)*ytau2 
     .  +3*sb2*(nhl-nhh)*y(4)*ytop2 
     .  +(cb2*nhl+sb2*nhh)*(3*yb2+ytau2)*y(4) 
     . +(-3.d0/5.d0*y(1)*(15.d0/4.d0+3*nhlino/4.d0-nbino*
     . (n1sl/4.d0+n1se +nhlino/4.d0))
     . -y(2)*(9.d0/4.d0+9*nhlino/4.d0-3*nwino*(n1sl+nhlino)/4.d0))*y(4)

      Ybbeta = 3*(cb2*(nhl+nhlino)+sb2*(nhh+nhhino))/2.d0*y(5)*yb2 +
     . ((sb2*(nhl+nhlino)+cb2*(nhh+nhhino))/2.d0 -sb2*(nhl-nhh))*
     . y(5)*ytop2+(cb2*nhl+sb2*nhh)*(3*yb2+ytau2)*y(5) 
     . +(-3.d0/5.d0*y(1)*(5.d0/12.d0+3*nhlino/4.d0-nbino*
     . (n1sq/36.d0+n1sd/9.d0+nhlino/4.d0))
     . -y(2)*(9.d0/4.d0+9*nhlino/4.d0-3*nwino*(n1sq+nhlino)/4.d0)
     . -y(3)*(8.d0-4*ngino*(n1sq+n1sd)/3.d0))*y(5)

      Ytbeta =3*(sb2*(nhl+nhlino)+cb2*(nhh+nhhino))/2.d0*y(6)*ytop2 +
     . ((cb2*(nhl+nhlino)+sb2*(nhh+nhhino))/2.d0 -cb2*(nhl-nhh))*
     . y(6)*yb2+3*(sb2*nhl+cb2*nhh)*y(6)*ytop2+cb2*(nhl-nhh)*y(6)*ytau2
     .+(-3.d0/5.d0*y(1)*(17.d0/12.d0+3*nhlino/4.d0-nbino*
     . (n1sq/36.d0+4*n1su/9.d0+nhlino/4.d0))
     . -y(2)*(9.d0/4.d0+9*nhlino/4.d0-3*nwino*(n1sq+nhlino)/4.d0)
     . -y(3)*(8.d0-4*ngino*(n1sq+n1su)/3.d0))*y(6)
c  
       dydx(4) =cpi*Ytaubeta

        dydx(5) = cpi*Ybbeta

        dydx(6) = cpi*Ytbeta 

c - Higgs vev beta functions:
c - y(7), y(8) = Ln(vu), Ln(vd)

       dydx(7) = cpi*(3.d0/4.d0*(y(1)/5.d0 +y(2)) -3*ytop2)
       dydx(8) = cpi*(3.d0/4.d0*(y(1)/5.d0 +y(2)) -3*yb2-ytau2)

c - soft susy-breaking terms beta functions:
c - y(9)--y(11) : Atau, Ab, Atop

       dydx(9) =cpi*(8*ytau2*y(9) +6*yb2*y(10)
     . +6*(3*y(1)*sgnM1*exp(y(20))/5.d0 +sgnM2*y(2)*exp(y(21))))

       dydx(10) =cpi*(12*y(10)*yb2 +2*y(9)*ytau2+2*y(11)*ytop2
     . +14*y(1)/15.d0*sgnM1*exp(y(20))+6*y(2)*sgnM2*exp(y(21))
     . +32*y(3)/3.d0*sgnM3*exp(y(22)))

       dydx(11) =cpi*(12*y(11)*ytop2 +2*y(10)*yb2
     . +26*y(1)/15.d0*sgnM1*exp(y(20))+6*y(2)*sgnM2*exp(y(21))
     . +32*y(3)/3.d0*sgnM3*exp(y(22)))

c - y(12)--y(13) : m^2(phi_u), m^2(phi_d)
       trym2 = y(18)-2*y(17)+y(16)-y(15)+y(14)+y(12)-y(13)

       dydx(12) = 2*cpi*(3*ytop2*(y(12)+y(18)+y(17)+y(11)*y(11))
     . +3.d0/10.d0*y(1)*trym2 -3*y(1)/5.d0*exp(2*y(20))
     . -3*y(2)*exp(2*y(21)))

       dydx(13) = 2*cpi*(ytau2*(y(13)+y(15)+y(14)+y(9)*y(9))
     . +3*yb2*(y(13)+y(18)+y(16)+y(10)*y(10))
     . -3.d0/10.d0*y(1)*trym2 -3*y(1)/5.d0*exp(2*y(20))
     . -3*y(2)*exp(2*y(21)))

c - y(14)--y(19) : 3rd gen. sfermion mass^2 terms:
c   m^2_tau, m^2_L, m^2_b, m^2_top, m^2_Q, B

       dydx(14) = 2*cpi*(2*ytau2*(y(13)+y(14)+y(15)+y(9)*y(9))
     . +3*y(1)/5.d0*trym2 -12*y(1)/5.d0*exp(2*y(20)))

       dydx(15) = 2*cpi*(ytau2*(y(13)+y(15)+y(14)+y(9)*y(9))
     . -3*y(1)/10.d0*trym2 -3*y(1)/5.d0*exp(2*y(20))
     . -3*y(2)*exp(2*y(21)))

       dydx(16) = 2*cpi*(2*yb2*(y(13)+y(16)+y(18)+y(10)*y(10))
     . +y(1)/5.d0*trym2-4*y(1)/15.d0*exp(2*y(20))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(17) =2*cpi*(2*ytop2*(y(12)+y(17)+y(18)+y(11)*y(11))
     . -2*y(1)/5.d0*trym2 -16*y(1)/15.d0*exp(2*y(20))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(18) =2*cpi*(ytop2*(y(12)+y(17)+y(18)+y(11)*y(11))
     . +yb2*(y(13)+y(18)+y(16)+y(10)*y(10))
     . +y(1)/10.d0*trym2 -y(1)/15.d0*exp(2*y(20))-3*y(2)*exp(2*y(21))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(19) = 2*cpi*(3*y(11)*ytop2 +3*y(10)*yb2 +y(9)*ytau2
     . +3*y(1)/5.d0*sgnM1*exp(y(20)) +3*y(2)*sgnM2*exp(y(21)))

c - Gauginos masses beta functions:
c - y(20)--y(22) : Ln (M1,M2,M3)

       dydx(20) = -2*cpi*(-3.d0/5.d0-nf)*y(1)

       dydx(21) = -2*cpi*(5-nf)*y(2)

       dydx(22) = -2*cpi*(9-nf)*y(3)

c -  the mu parameter:
c - y(23) = Ln mu

       dydx(23) = cpi*(3*ytop2 +3*yb2+ytau2 -3*y(1)/5.d0-3*y(2))

c - y(24)--y(28) : 1st and 2d gen. sfermion mass^2 terms:
c   m^2_er, m^2_eL, m^2_dr, m^2_ur, m^2_uL

       dydx(24) = 2*cpi*(3*y(1)/5.d0*trym2 -12*y(1)/5.d0*exp(2*y(20)))

       dydx(25) = 2*cpi*(-3*y(1)/10.d0*trym2 -3*y(1)/5.d0*exp(2*y(20))     
     . -3*y(2)*exp(2*y(21)))

       dydx(26) = 2*cpi*(
     . y(1)/5.d0*trym2-4*y(1)/15.d0*exp(2*y(20))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(27) =2*cpi*(
     . -2*y(1)/5.d0*trym2 -16*y(1)/15.d0*exp(2*y(20))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(28) =2*cpi*(
     . y(1)/10.d0*trym2 -y(1)/15.d0*exp(2*y(20))-3*y(2)*exp(2*y(21))
     . -16*y(3)/3.d0*exp(2*y(22)))
c Now mimics pole masses: stop RG evolution at m(Q=m) for soft parameters m: 
c      if(kpole.eq.1) then
c      ewsbscale = dsqrt(msttr1*msttr2)
c      else if(kpole.eq.0) then
c      ewsbscale = scale
c      endif
c      if(ewsbscale.lt.zm) ewsbscale=zm
       ewsbscale=zm
      if(q.lt.ewsbscale) then
c  Everything (except gauges+ yukawas) is frozen at (m_tL mt_R)^1/2:
       freeze=step(q-ewsbscale)
       if(jfirst.eq.0) then
       tbeta=dexp(y(7))/dexp(y(8))
c saving Yukawas at EWSB scale:
       ytau=y(4)
       yb=y(5)
       yt=y(6)
       alsewsb=y(3)/(4*pi)
       endif
       jfirst=1
       do jpole= 9,28
       dydx(jpole)= freeze*dydx(jpole) 
       enddo
      endif
c
       end
c-----------------------------------------
       subroutine DERIV2(x,y,dydx)
c--- derivatives of y(xN) i.e these are exactly the beta functions)
c NB:  includes 2-LOOP rge for gauge, yukawa, gaugino 
c with simple (unique scale) threshold in 1-loop dependence. 
c NB: only used at first iteration (when mssm spectrum undefined yet) 
       implicit real*8(a-h,m,n,o-z)
       integer*2 n
c--- derivatives of y(xN) i.e these are exactly the beta functions)
c NB: DERIV1 includes 1-loop only rge, 
c with simple (unique scale) threshold. 
       parameter (n=28)
       dimension y(n),dydx(n),ygut(8)
       common/cte/nf,cpi,zm,wm,tbeta
       common/s_thresh/rmtop,susym,egut
       common/stepwi/wistep,h1,kpole
       common/stegut/ifirst,jfirst,ygut
       common/gunif/kunif
       common/sgnM123/sgnM1,sgnM2,sgnM3
      COMMON/MSSMsqua/MSQ,MTR,MBR,MUQ,MUR,MDR
      common/treesfer/msbtr1,msbtr2,msttr1,msttr2
      common/yukaewsb/ytau,yb,yt,alsewsb
       common/renscale/scale
       data nn/3.d0/,nd/3.d0/,ne/3.d0/
c NB: cpi = 1/(16 pi^2)
c  a smoothed step function:   
      step(x) = .5d0*(1.d0 +dtanh(wistep*x))
ccccccccccccccccccccccccccccccc
      pi=4*datan(1.d0)
      MTop=174.3d0
      Q = dexp(x)
c g1, g2 gauge unif.:
      if(kunif.ne.0.and.h1.gt.0.d0.and.Q.gt.1.d15) then
      if(y(1).ge.y(2)) then
       if(ifirst.eq.0) then
       egut=Q
c freeze out the gauge +Yukawa+vu,vd couplings at GUT scale:
       do ii=1,8 
       ygut(ii)=y(ii)
       enddo
       endif
      ifirst= 1      
      endif
      endif
      st1= step(Q-mtop)
c
      st2= step(Q-susym)
      nu=2.d0 +st1
      nsq=3*st2
      n1sq=st2
      nsu=3*st2
      n1su=st2
      nsd=3*st2
      n1sd=st2
      nsl=3*st2
      n1sl=st2
      nse=3*st2
      n1se=st2      
      nhino=2.d0*st2
      nbino=st2
      nwino=st2
      ngino=st2
      nh=1.d0+st2
      nhl=st2
      nhh=st2
      nhlino=st2
      nhhino=st2
c
       b10 = 2.d0/5.d0*(17*nu/12.d0+5*nd/12.d0+5*ne/4.d0+nn/4.d0)
     . +nsq/30.d0
     . +4*nsu/15.d0+nsd/15.d0+nsl/10.d0+nse/5.d0
     . +nhino/5.d0 +nh/10.d0
       b20 = -22.d0/3.d0+(nu+nd)/2.d0+(ne+nn)/6.d0
     . +nsq/2.d0 + nsl/6.d0 + nhino/3.d0
     . +nh/6.d0 +4*nwino/3.d0
       b30 = 2*(nu+nd)/3.d0 +nsq/3.d0+nsu/6.d0+nsd/6.d0+2*ngino -11.d0
c - gauge coupling beta functions (nb the variables are g^2):
c -y(1)--y(3): g1^2,g2^2,g3^2.
       ytau2 =y(4)*y(4)
       yb2 = y(5)*y(5)
       ytop2 = y(6)*y(6)
c
       dydx(1) = 2*cpi*b10*y(1)*y(1) 
     . +2*cpi*cpi*y(1)*y(1)*((19*nf/15.d0+9.d0/25.d0)*y(1)
     . +(3*nf/5.d0+9.d0/5.d0)
     . *y(2)+44*nf/15.d0*y(3)-26*ytop2/5.d0-14*yb2/5.d0-18*ytau2/5.d0)

       dydx(2) = 2*cpi*b20*y(2)*y(2)
     . +2*cpi*cpi*y(2)*y(2)*((nf/5.d0+3.d0/5.d0)*y(1)
     . +(7*nf-17.d0)*y(2)
     . +4*nf*y(3) -6*ytop2 -6*yb2-2*ytau2 )

       dydx(3) = 2*cpi*b30*y(3)*y(3)
     . +2*cpi*cpi*y(3)*y(3)*(11*nf/30.d0*y(1)+3*nf/2.d0*y(2)
     . +(34*nf/3.d0-54.d0)
     . *y(3) -4*ytop2 -4*yb2 )
c - Yukawa coupling beta function (only Ytau,Yb,Ytop included):
c -y(4)--y(6): Ytau,Yb,Ytop
c threshold effects:
      tbet=dexp(y(7)-y(8))
      cb2=1.d0/(1.d0+tbet*tbet)
      sb2=1.d0-cb2
      Ytaubeta = 3*(cb2*(nhl+nhlino)+sb2*(nhh+nhhino))/2.d0*y(4)*ytau2 
     .  +3*sb2*(nhl-nhh)*y(4)*ytop2 
     .  +(cb2*nhl+sb2*nhh)*(3*yb2+ytau2)*y(4) 
     . +(-3.d0/5.d0*y(1)*(15.d0/4.d0+3*nhlino/4.d0-nbino*
     . (n1sl/4.d0+n1se +nhlino/4.d0))
     . -y(2)*(9.d0/4.d0+9*nhlino/4.d0-3*nwino*(n1sl+nhlino)/4.d0))*y(4)

      Ybbeta = 3*(cb2*(nhl+nhlino)+sb2*(nhh+nhhino))/2.d0*y(5)*yb2 +
     . ((sb2*(nhl+nhlino)+cb2*(nhh+nhhino))/2.d0 -sb2*(nhl-nhh))*
     . y(5)*ytop2+(cb2*nhl+sb2*nhh)*(3*yb2+ytau2)*y(5) 
     . +(-3.d0/5.d0*y(1)*(5.d0/12.d0+3*nhlino/4.d0-nbino*
     . (n1sq/36.d0+n1sd/9.d0+nhlino/4.d0))
     . -y(2)*(9.d0/4.d0+9*nhlino/4.d0-3*nwino*(n1sq+nhlino)/4.d0)
     . -y(3)*(8.d0-4*ngino*(n1sq+n1sd)/3.d0))*y(5)

      Ytbeta =3*(sb2*(nhl+nhlino)+cb2*(nhh+nhhino))/2.d0*y(6)*ytop2 +
     . ((cb2*(nhl+nhlino)+sb2*(nhh+nhhino))/2.d0 -cb2*(nhl-nhh))*
     . y(6)*yb2+3*(sb2*nhl+cb2*nhh)*y(6)*ytop2+cb2*(nhl-nhh)*y(6)*ytau2
     .+(-3.d0/5.d0*y(1)*(17.d0/12.d0+3*nhlino/4.d0-nbino*
     . (n1sq/36.d0+4*n1su/9.d0+nhlino/4.d0))
     . -y(2)*(9.d0/4.d0+9*nhlino/4.d0-3*nwino*(n1sq+nhlino)/4.d0)
     . -y(3)*(8.d0-4*ngino*(n1sq+n1su)/3.d0))*y(6)
c   
       dydx(4) =cpi*(Ytaubeta
     . +cpi*(-10*ytau2*ytau2-9*yb2*yb2 -9*yb2*ytau2-3*yb2*ytop2
     . +(6*y(2)+6*y(1)/5.)*ytau2 +(-2*y(1)/5.+16*y(3))*yb2
     . +(9*nf/5.+27./10.)*y(1)*y(1)+(3*nf-21./2.)*y(2)*y(2)
     . +9*y(1)*y(2)/5. )  ) 

        dydx(5) = cpi*(Ybbeta
     . +cpi*(-22*yb2*yb2-5*ytop2*ytop2-5*yb2*ytop2-3*yb2*ytau2 
     . -3*ytau2*ytau2 
     . +4*y(1)/5.d0*ytop2+(2*y(1)/5.d0+6*y(2)+16*y(3))*yb2
     . +6*y(1)/5.d0*ytau2 
     . +(7*nf/15.d0+7.d0/18.d0)*y(1)*y(1)+(3*nf-21.d0/2.d0)*y(2)*y(2)
     . +(16*nf/3.d0-304.d0/9.d0)*y(3)*y(3)+y(1)*y(2)+8*y(1)*y(3)/9.d0
     . +8*y(2)*y(3) )  )

        dydx(6) = cpi*(Ytbeta 
     . +cpi*(-22*ytop2*ytop2-5*yb2*yb2-5*yb2*ytop2-yb2*ytau2
     . +(6*y(1)/5.d0+6*y(2)+16*y(3))*ytop2+2*y(1)/5.d0*yb2 
     . +(13*nf/15.d0+403.d0/450.d0)*y(1)*y(1)+(3*nf-21.d0/2.d0)
     . *y(2)*y(2)
     . +(16*nf/3.d0-304.d0/9.d0)*y(3)*y(3) +y(1)*y(2)+136.d0/45.d0
     . *y(1)*y(3)
     . +8*y(2)*y(3) ) )
c
c - Higgs vev beta functions:
c - y(7), y(8) = Ln vu, Ln vd

       dydx(7) = cpi*(3.d0/4.d0*(y(1)/5.d0 +y(2)) -3*ytop2)
     . +cpi*cpi*(9*ytop2*ytop2/4.d0+9*ytop2*yb2/4.d0
     . -(19*y(1)/10.d0+9*y(2)/2.d0+20*y(3))*ytop2
     . -(279.d0/800.d0+1803*nf/3200.d0)*y(1)*y(1)
     . -(207.d0/32.d0+357*nf/128.d0)*y(2)*y(2)
     . -(27.d0/80.d0+9*nf/160.d0)*y(1)*y(2))

       dydx(8) = cpi*(3.d0/4.d0*(y(1)/5.d0 +y(2)) -3*yb2-ytau2)
     . +cpi*cpi*(9*yb2*yb2/4.d0+9*yb2*ytop2/4.d0+3*ytau2*ytau2/4.d0
     . -(2*y(1)/5.d0+9*y(2)/2.d0+20*y(3))*yb2
     . -(9*y(1)/5.d0+3*y(2)/2.d0)*ytau2
     . -(279.d0/800.d0+1803*nf/3200.d0)*y(1)*y(1)
     . -(207.d0/32.d0+357*nf/128.d0)*y(2)*y(2)
     . -(27.d0/80.d0+9*nf/160.d0)*y(1)*y(2))

c - soft susy-breaking terms beta functions:
c - y(9)--y(11) : Atau, Ab, Atop

       dydx(9) =cpi*(8*ytau2*y(9) +6*yb2*y(10)
     . +6*(3*y(1)*sgnM1*exp(y(20))/5.d0 +sgnM2*y(2)*exp(y(21))))

       dydx(10) =cpi*(12*y(10)*yb2 +2*y(9)*ytau2+2*y(11)*ytop2
     . +14*y(1)/15.d0*sgnM1*exp(y(20))+6*y(2)*sgnM2*exp(y(21))
     . +32*y(3)/3.d0*sgnM3*exp(y(22)))

       dydx(11) =cpi*(12*y(11)*ytop2 +2*y(10)*yb2
     . +26*y(1)/15.d0*sgnM1*exp(y(20))+6*y(2)*sgnM2*exp(y(21))
     . +32*y(3)/3.d0*sgnM3*exp(y(22)))

c - y(12)--y(13) : m^2(phi_u), m^2(phi_d)
       trym2 = y(18)-2*y(17)+y(16)-y(15)+y(14)+y(12)-y(13)

       dydx(12) = 2*cpi*(3*ytop2*(y(12)+y(18)+y(17)+y(11)*y(11))
     . +3.d0/10.d0*y(1)*trym2 -3*y(1)/5.d0*exp(2*y(20))
     . -3*y(2)*exp(2*y(21)))

       dydx(13) = 2*cpi*(ytau2*(y(13)+y(15)+y(14)+y(9)*y(9))
     . +3*yb2*(y(13)+y(18)+y(16)+y(10)*y(10))
     . -3.d0/10.d0*y(1)*trym2 -3*y(1)/5.d0*exp(2*y(20))
     . -3*y(2)*exp(2*y(21)))

c - y(14)--y(19) : m^2_tau, m^2_L, m^2_b, m^2_top, m^2_Q, B

       dydx(14) = 2*cpi*(2*ytau2*(y(13)+y(14)+y(15)+y(9)*y(9))
     . +3*y(1)/5.d0*trym2 -12*y(1)/5.d0*exp(2*y(20)))

       dydx(15) = 2*cpi*(ytau2*(y(13)+y(15)+y(14)+y(9)*y(9))
     . -3*y(1)/10.d0*trym2 -3*y(1)/5.d0*exp(2*y(20))
     . -3*y(2)*exp(2*y(21)))

       dydx(16) = 2*cpi*(2*yb2*(y(13)+y(16)+y(18)+y(10)*y(10))
     . +y(1)/5.d0*trym2-4*y(1)/15.d0*exp(2*y(20))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(17) =2*cpi*(2*ytop2*(y(12)+y(17)+y(18)+y(11)*y(11))
     . -2*y(1)/5.d0*trym2 -16*y(1)/15.d0*exp(2*y(20))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(18) =2*cpi*(ytop2*(y(12)+y(17)+y(18)+y(11)*y(11))
     . +yb2*(y(13)+y(18)+y(16)+y(10)*y(10))
     . +y(1)/10.d0*trym2 -y(1)/15.d0*exp(2*y(20))-3*y(2)*exp(2*y(21))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(19) = 2*cpi*(3*y(11)*ytop2 +3*y(10)*yb2 +y(9)*ytau2
     . +3*y(1)/5.d0*sgnM1*exp(y(20)) +3*y(2)*sgnM2*exp(y(21)))

c - Gauginos masses beta functions (includes two-loop):
c - y(20)--y(22) : Ln (M1,M2,M3)
       mm1= sgnM1*dexp(y(20))
       mm2= sgnM2*dexp(y(21))
       mm3= sgnM3*dexp(y(22))
       dydx(20) = -2*cpi*(-3.d0/5.d0-nf)*y(1) 
     . +2*cpi*cpi*y(1)*((19*nf/15.d0+9.d0/25.d0)*y(1)*(1.d0+1.d0)
     . +(3*nf/5.d0+9.d0/5.d0)*y(2)*(1.d0+Mm2/Mm1) 
     . +44*nf/15.d0*y(3)*(1.d0+Mm3/Mm1)
     . -26*ytop2*(1.d0-y(11)/mm1)/5.d0
     . -14*yb2*(1.d0-y(10)/mm1)/5.d0
     . -18*ytau2*(1.d0-y(9)/mm1)/5.d0)

       dydx(21) = -2*cpi*(5.d0-nf)*y(2)
     . +2*cpi*cpi*y(2)*((nf/5.d0+3.d0/5.d0)*y(1)*(1.d0+mm1/mm2)
     . +(7*nf-17.d0)*y(2)*(1.d0+1.d0)
     . +4*nf*y(3)*(1.d0+mm3/mm2)
     . -6*ytop2*(1.d0-y(11)/mm2)
     . -6*yb2*(1.d0 -y(10)/mm2)
     . -2*ytau2*(1.d0 -y(9)/mm2) )

       dydx(22) = -2*cpi*(9.d0-nf)*y(3)
     . +2*cpi*cpi*y(3)*(11*nf/30.d0*y(1)*(1.d0+mm1/mm3)
     . +3*nf/2.d0*y(2)*(1.d0+mm2/mm3)
     . +(34*nf/3.d0-54.d0)*y(3)*(1.d0+1.d0) 
     . -4*ytop2*(1.d0 -y(11)/mm3)
     . -4*yb2*(1.d0 -y(10)/mm3)  )

c - the mu parameter:
c - y(23) = Ln mu

       dydx(23) = cpi*(3*ytop2 +3*yb2+ytau2 -3*y(1)/5.d0-3*y(2))

c - y(24)--y(28) : 1st and 2d gen. sfermion mass^2 terms:
c   m^2_er, m^2_eL, m^2_dr, m^2_ur, m^2_uL

       dydx(24) = 2*cpi*(3*y(1)/5.d0*trym2 -12*y(1)/5.d0*exp(2*y(20)))

       dydx(25) = 2*cpi*(-3*y(1)/10.d0*trym2 -3*y(1)/5.d0*exp(2*y(20))     
     . -3*y(2)*exp(2*y(21)))

       dydx(26) = 2*cpi*(
     . y(1)/5.d0*trym2-4*y(1)/15.d0*exp(2*y(20))-16*y(3)/3.d0
     . *exp(2*y(22)))

       dydx(27) =2*cpi*(
     . -2*y(1)/5.d0*trym2 -16*y(1)/15.d0*exp(2*y(20))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(28) =2*cpi*(
     . y(1)/10.d0*trym2 -y(1)/15.d0*exp(2*y(20))-3*y(2)*exp(2*y(21))
     . -16*y(3)/3.d0*exp(2*y(22)))
c Now mimics pole masses: stop RG evolution at m(Q=m) for soft parameters m: 
c      if(kpole.eq.1) then
c      ewsbscale = dsqrt(msttr1*msttr2)
c      else if(kpole.eq.0) then
c      ewsbscale = scale
c      endif
c      if(ewsbscale.lt.zm) ewsbscale=zm
       ewsbscale=zm
      if(q.lt.ewsbscale) then
c  Everything (except gauges+ yukawas) is frozen at (m_tL mt_R)^1/2:
       freeze= step(q-ewsbscale)
       if(h1.lt.0.d0.and.jfirst.eq.0) then
       tbeta=dexp(y(7))/dexp(y(8))
c saving Yukawas at EWSB scale:
       ytau=y(4)
       yb=y(5)
       yt=y(6)
       alsewsb=y(3)/(4*pi)
       endif
       jfirst=1
       do jpole= 7,19
       dydx(jpole)= freeze*dydx(jpole) 
       enddo
       do jpole= 20,28
       dydx(jpole)= freeze*dydx(jpole) 
       enddo
      endif
c always freezing ytop at mtop:
       if(q.le.MTop) dydx(6) = 0.d0
c
        end
c-----------------------------------------------------------------
       subroutine DERIV1T(x,y,dydx)
c--- derivatives of y(xN) i.e these are exactly the beta functions)
c NB:  1-loop level, realistic multi scale threshold for gauge+Yukawas 
       implicit real*8(a-h,m,n,o-z)
       integer*2 n
c--- derivatives of y(xN) i.e these are exactly the beta functions)
c NB: DERIV1 includes 1-loop only rge, 
c with simple (unique scale) threshold. 
       parameter (n=28)
       dimension y(n),dydx(n),ygut(8)
      common/good/iflop
      COMMON/MSSMgpar/M1,M2,M3  
      COMMON/MSSMslep/MSL,MTAUR,MEL,MER
      COMMON/MSSMsqua/MSQ,MTR,MBR,MUQ,MUR,MDR
      common/treesfer/msbtr1,msbtr2,msttr1,msttr2
      COMMON/MSSMAtri/AL,AU,AD
      COMMON/MSSMhpar/mhu2,mhd2,MA,MU
      COMMON/OUTHIGGS/ml,mh,mch,alpha 
       common/cte/nf,cpi,zm,wm,tbeta
       common/s_thresh/rmtop,susym,egut
       common/stepwi/wistep,h1,kpole
       common/stegut/ifirst,jfirst,ygut
      common/yukaewsb/ytau,yb,yt,alsewsb
       common/renscale/scale
       common/gunif/kunif
       common/sgnM123/sgnM1,sgnM2,sgnM3
       data nn/3.d0/,nd/3.d0/,ne/3.d0/
c NB: cpi = 1/(16 pi^2)
c a smoothed step function:    
      step(x) = .5d0*(1.d0 +dtanh(wistep*x))
ccccccccccccccccccccccccccccccccccccccccccccccc
      pi=4*datan(1.d0)
      MTOP=174.3d0
      Q = dexp(x)
c g1, g2 gauge unif.:
      if(kunif.ne.0.and.h1.gt.0.d0.and.Q.gt.1.d15) then
      if(y(1).ge.y(2)) then
       if(ifirst.eq.0) then
       egut=Q
c freeze out the gauge +Yukawa+vu,vd couplings at GUT scale:
       do ii=1,8 
       ygut(ii)=y(ii)
       enddo
       endif
      ifirst= 1      
      endif
      endif
c NB threshold effects are approximated. We assume for (some) particle a step
c at the relevant soft terms, i.e. before mixing to true physical masses, 
c except when the mass appear explicitly (see expressions below) 
      tbet=dexp(y(7)-y(8))
      cb2=1.d0/(1.d0+tbet*tbet)
      sb2=1.d0-cb2
      cb=dsqrt(cb2)
      sb=dsqrt(sb2)
      sw2=.2315d0
      cw2=1.d0-sw2
      sw=dsqrt(sw2)
      cw=dsqrt(cw2)
c
      nu=2.d0 +step(q-mtop)
c      nu=3.d0
      nsq= 2*step(q-muq) +step(q-msq)
      n1sq= step(q-msq)
      nsu= 2*step(q-mur) +step(q-mtr)
      n1su= step(q-mtr)
      nsd= 2*step(q-mdr) +step(q-mbr)
      n1sd= step(q-mbr)
      nsl= 2*step(q-mel) +step(q-msl)
      n1sl= step(q-msl)
      nse= 2*step(q-mer)+step(q-mtaur)
      n1se= step(q-mtaur)     
c
      nbino= step(q-dabs(m1))
      nwino= step(q-dabs(m2))
      ngino= step(q-dabs(m3))
      nh=  step(q-dsqrt(dabs(mhu2)))+step(q-dsqrt(dabs(mhd2)))
      nhl=step(q-ml)
      nhh=step(q-mh)
      scal1 = min(zm,dabs(mu))
      if(scal1.lt.zm) scal1=zm
      scal2 = max(zm,dabs(mu))
      nhlino= step(q-scal1)
      nhhino= step(q-scal2)
      nhino= nhlino +nhhino
c NB nhlino,nhhino,nhino are determined as a compromise: in fact they
c are the "masses" (before physical mixing) of the two Higgsino states.
c We thus arbitrarily define  a light and heavy scale, as function of
c the parameters of the neutralino mass matrix that are relevant to 
c determine those scales.
c
      b10 = 2.d0/5.d0*(17*nu/12.d0+5*nd/12.d0+5*ne/4.d0+nn/4.d0)
     . +nsq/30.d0
     . +4*nsu/15.d0+nsd/15.d0+nsl/10.d0+nse/5.d0
     . +nhino/5.d0 +nh/10.d0
       b20 = -22.d0/3.d0+(nu+nd)/2.d0+(ne+nn)/6.d0
     . +nsq/2.d0 + nsl/6.d0 + nhino/3.d0
     . +nh/6.d0 +4*nwino/3.d0
       b30 = 2*(nu+nd)/3.d0 +nsq/3.d0+nsu/6.d0+nsd/6.d0+2*ngino -11.d0
c
       ytau2 =y(4)*y(4)
       yb2 = y(5)*y(5)
       ytop2 = y(6)*y(6)
c
c - gauge coupling beta functions (nb the variables are g^2):
c -y(1)--y(3): g1^2,g2^2,g3^2.

       dydx(1) = 2*cpi*b10*y(1)*y(1) 
       dydx(2) = 2*cpi*b20*y(2)*y(2)
       dydx(3) = 2*cpi*b30*y(3)*y(3)
c threshold effects:
      tbet=dexp(y(7)-y(8))
      cb2=1.d0/(1.d0+tbet*tbet)
      sb2=1.d0-cb2
      Ytaubeta = 3*(cb2*(nhl+nhlino)+sb2*(nhh+nhhino))/2.d0*y(4)*ytau2 
     .  +3*sb2*(nhl-nhh)*y(4)*ytop2 
     .  +(cb2*nhl+sb2*nhh)*(3*yb2+ytau2)*y(4) 
     . +(-3.d0/5.d0*y(1)*(15.d0/4.d0+3*nhlino/4.d0-nbino*
     . (n1sl/4.d0+n1se +nhlino/4.d0))
     . -y(2)*(9.d0/4.d0+9*nhlino/4.d0-3*nwino*(n1sl+nhlino)/4.d0))*y(4)

      Ybbeta = 3*(cb2*(nhl+nhlino)+sb2*(nhh+nhhino))/2.d0*y(5)*yb2 +
     . ((sb2*(nhl+nhlino)+cb2*(nhh+nhhino))/2.d0 -sb2*(nhl-nhh))*
     . y(5)*ytop2+(cb2*nhl+sb2*nhh)*(3*yb2+ytau2)*y(5) 
     . +(-3.d0/5.d0*y(1)*(5.d0/12.d0+3*nhlino/4.d0-nbino*
     . (n1sq/36.d0+n1sd/9.d0+nhlino/4.d0))
     . -y(2)*(9.d0/4.d0+9*nhlino/4.d0-3*nwino*(n1sq+nhlino)/4.d0)
     . -y(3)*(8.d0-4*ngino*(n1sq+n1sd)/3.d0))*y(5)

      Ytbeta =3*(sb2*(nhl+nhlino)+cb2*(nhh+nhhino))/2.d0*y(6)*ytop2 +
     . ((cb2*(nhl+nhlino)+sb2*(nhh+nhhino))/2.d0 -cb2*(nhl-nhh))*
     . y(6)*yb2+3*(sb2*nhl+cb2*nhh)*y(6)*ytop2+cb2*(nhl-nhh)*y(6)*ytau2
     .+(-3.d0/5.d0*y(1)*(17.d0/12.d0+3*nhlino/4.d0-nbino*
     . (n1sq/36.d0+4*n1su/9.d0+nhlino/4.d0))
     . -y(2)*(9.d0/4.d0+9*nhlino/4.d0-3*nwino*(n1sq+nhlino)/4.d0)
     . -y(3)*(8.d0-4*ngino*(n1sq+n1su)/3.d0))*y(6)
c  
       dydx(4) =cpi*Ytaubeta


        dydx(5) = cpi*Ybbeta

        dydx(6) = cpi*Ytbeta 

c - (1-loop) Higgs vev beta functions:
c - y(7), y(8) = Ln vu, Ln vd

       dydx(7) = cpi*(3.d0/4.d0*(y(1)/5.d0 +y(2)) -3*ytop2)

       dydx(8) = cpi*(3.d0/4.d0*(y(1)/5.d0 +y(2)) -3*yb2-ytau2)

c - (1-loop) soft susy-breaking terms beta functions:
c - y(9)--y(11) : Atau, Ab, Atop

       dydx(9) =cpi*(8*ytau2*y(9) +6*yb2*y(10)
     . +6*(3*y(1)*sgnM1*exp(y(20))/5.d0 +sgnM2*y(2)*exp(y(21))))

       dydx(10) =cpi*(12*y(10)*yb2 +2*y(9)*ytau2+2*y(11)*ytop2
     . +14*y(1)/15.d0*sgnM1*exp(y(20))+6*y(2)*sgnM2*exp(y(21))
     . +32*y(3)/3.d0*sgnM3*exp(y(22)))

       dydx(11) =cpi*(12*y(11)*ytop2 +2*y(10)*yb2
     . +26*y(1)/15.d0*sgnM1*exp(y(20))+6*y(2)*sgnM2*exp(y(21))
     . +32*y(3)/3.d0*sgnM3*exp(y(22)))

c - (1-loop) y(12)--y(13) : m^2(phi_u), m^2(phi_d)
       trym2 = y(18)-2*y(17)+y(16)-y(15)+y(14)+y(12)-y(13)

       dydx(12) = 2*cpi*(3*ytop2*(y(12)+y(18)+y(17)+y(11)*y(11))
     . +3.d0/10.d0*y(1)*trym2 -3*y(1)/5.d0*exp(2*y(20))
     . -3*y(2)*exp(2*y(21)))

       dydx(13) = 2*cpi*(ytau2*(y(13)+y(15)+y(14)+y(9)*y(9))
     . +3*yb2*(y(13)+y(18)+y(16)+y(10)*y(10))
     . -3.d0/10.d0*y(1)*trym2 -3*y(1)/5.d0*exp(2*y(20))
     . -3*y(2)*exp(2*y(21)))

c - (1-loop) y(14)--y(19) : m^2_tau, m^2_L, m^2_b, m^2_top, m^2_Q, B

       dydx(14) = 2*cpi*(2*ytau2*(y(13)+y(14)+y(15)+y(9)*y(9))
     . +3*y(1)/5.d0*trym2 -12*y(1)/5.d0*exp(2*y(20)))

       dydx(15) = 2*cpi*(ytau2*(y(13)+y(15)+y(14)+y(9)*y(9))
     . -3*y(1)/10.d0*trym2 -3*y(1)/5.d0*exp(2*y(20))
     . -3*y(2)*exp(2*y(21)))

       dydx(16) = 2*cpi*(2*yb2*(y(13)+y(16)+y(18)+y(10)*y(10))
     . +y(1)/5.d0*trym2-4*y(1)/15.d0*exp(2*y(20))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(17) =2*cpi*(2*ytop2*(y(12)+y(17)+y(18)+y(11)*y(11))
     . -2*y(1)/5.d0*trym2 -16*y(1)/15.d0*exp(2*y(20))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(18) =2*cpi*(ytop2*(y(12)+y(17)+y(18)+y(11)*y(11))
     . +yb2*(y(13)+y(18)+y(16)+y(10)*y(10))
     . +y(1)/10.d0*trym2 -y(1)/15.d0*exp(2*y(20))-3*y(2)*exp(2*y(21))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(19) = 2*cpi*(3*y(11)*ytop2 +3*y(10)*yb2 +y(9)*ytau2
     . +3*y(1)/5.d0*sgnM1*exp(y(20)) +3*y(2)*sgnM2*exp(y(21)))

c - Gauginos masses 1-loop beta functions:
c - y(20)--y(22) : Ln (M1,M2,M3)
       mm1= sgnM1*dexp(y(20))
       mm2= sgnM2*dexp(y(21))
       mm3= sgnM3*dexp(y(22))
       dydx(20) = -2*cpi*(-3.d0/5.d0-nf)*y(1) 

       dydx(21) = -2*cpi*(5.d0-nf)*y(2)

       dydx(22) = -2*cpi*(9.d0-nf)*y(3)

c - the mu parameter:
c - y(23) = Ln mu

       dydx(23) = cpi*(3*ytop2 +3*yb2+ytau2 -3*y(1)/5.d0-3*y(2))

c - (1-loop) y(24)--y(28) : 1st and 2d gen. sfermion mass^2 terms:
c   m^2_er, m^2_eL, m^2_dr, m^2_ur, m^2_uL

       dydx(24) = 2*cpi*(3*y(1)/5.d0*trym2 -12*y(1)/5.d0*exp(2*y(20)))

       dydx(25) = 2*cpi*(-3*y(1)/10.d0*trym2 -3*y(1)/5.d0*exp(2*y(20))     
     . -3*y(2)*exp(2*y(21)))

       dydx(26) = 2*cpi*(
     . y(1)/5.d0*trym2-4*y(1)/15.d0*exp(2*y(20))-16*y(3)/3.d0
     . *exp(2*y(22)))

       dydx(27) =2*cpi*(
     . -2*y(1)/5.d0*trym2 -16*y(1)/15.d0*exp(2*y(20))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(28) =2*cpi*(
     . y(1)/10.d0*trym2 -y(1)/15.d0*exp(2*y(20))-3*y(2)*exp(2*y(21))
     . -16*y(3)/3.d0*exp(2*y(22)))
c Now mimics pole masses: stop RG evolution at m(Q=m) for soft parameters m: 
c  Everything (except gauges+ yukawas) is frozen at (m_tL mt_R)^1/2:
      if(kpole.eq.1) then
      ewsbscale = dsqrt(msttr1*msttr2)
      else if(kpole.eq.0) then
      ewsbscale = scale
      endif
      if(ewsbscale.lt.zm) ewsbscale=zm
      if(q.lt.ewsbscale) then
       freeze= step(q-ewsbscale)
c retaining tbeta at this (ewsbscale) scale:
       if(h1.lt.0.d0.and.jfirst.eq.0) then
       tbeta=dexp(y(7))/dexp(y(8))
c saving Yukawas at EWSB scale:
       ytau=y(4)
       yb=y(5)
       yt=y(6)
       alsewsb=y(3)/(4*pi)
       endif
       jfirst=1
       do jpole= 7,19
       dydx(jpole)=  freeze*dydx(jpole) 
       enddo
       do jpole= 20,28
       dydx(jpole)= freeze*dydx(jpole) 
       enddo
      endif
c also ALWAYS freezing ytop at mtop:
       if(q.le.MTop) dydx(6) = 0.d0
        end
c-----------------------------------------------------------------
       subroutine DERIV2T(x,y,dydx)
c--- derivatives of y(xN) i.e these are exactly the beta functions)
c NB:  includes 2-loop rge for gauge, yukawa, gauginos 
c ! with "realistic" (multiscale) thresholds (in 1-loop dependence). 
       implicit real*8(a-h,m,n,o-z)
       integer*2 n
c       logical isnan
c--- derivatives of y(xN) i.e these are exactly the beta functions)
c NB: DERIV1 includes 1-loop only rge, 
c with simple (unique scale) threshold. 
       parameter (n=28)
       dimension y(n),dydx(n),ygut(8)
      common/good/iflop
      COMMON/MSSMgpar/M1,M2,M3  
      COMMON/MSSMslep/MSL,MTAUR,MEL,MER
      COMMON/MSSMsqua/MSQ,MTR,MBR,MUQ,MUR,MDR
      common/treesfer/msbtr1,msbtr2,msttr1,msttr2
      COMMON/MSSMAtri/AL,AU,AD
      COMMON/MSSMhpar/mhu2,mhd2,MA,MU
      COMMON/OUTHIGGS/ml,mh,mch,alpha 
       common/cte/nf,cpi,zm,wm,tbeta
       common/s_thresh/rmtop,susym,egut
       common/stepwi/wistep,h1,kpole
       common/stegut/ifirst,jfirst,ygut
      common/yukaewsb/ytau,yb,yt,alsewsb
       common/renscale/scale
       common/gunif/kunif
       common/sgnM123/sgnM1,sgnM2,sgnM3
       data nn/3.d0/,nd/3.d0/,ne/3.d0/
c NB: cpi = 1/(16 pi^2)
c  a smoothed step function:    
      step(x) = .5d0*(1.d0 +dtanh(wistep*x))
ccccccccccccccccccccccccccccccccccccccccccccccc
      pi=4*datan(1.d0)
      MTOP=174.3d0
      Q = dexp(x)
c g1, g2 gauge unif.:
      if(kunif.ne.0.and.h1.gt.0.d0.and.Q.gt.1.d15) then
      if(y(1).ge.y(2)) then
       if(ifirst.eq.0) then
       egut=Q
c freeze out the gauge +Yukawa+vu,vd couplings at GUT scale:
       do ii=1,8 
       ygut(ii)=y(ii)
       enddo
       endif
      ifirst= 1      
      endif
      endif
c NB threshold effects are approximated. We assume for (some) particle a step
c at the relevant soft terms, i.e. before mixing to true physical masses, 
c except when the mass appear explicitly (see expressions below) 
      tbet=dexp(y(7)-y(8))
      cb2=1.d0/(1.d0+tbet*tbet)
      sb2=1.d0-cb2
      cb=dsqrt(cb2)
      sb=dsqrt(sb2)
      sw2=.2315d0
      cw2=1.d0-sw2
      sw=dsqrt(sw2)
      cw=dsqrt(cw2)
c
      nu=2.d0 +step(q-mtop)
c      nu=3.d0
      nsq= 2*step(q-muq) +step(q-msq)
      n1sq= step(q-msq)
      nsu= 2*step(q-mur) +step(q-mtr)
      n1su= step(q-mtr)
      nsd= 2*step(q-mdr) +step(q-mbr)
      n1sd= step(q-mbr)
      nsl= 2*step(q-mel) +step(q-msl)
      n1sl= step(q-msl)
      nse= 2*step(q-mer)+step(q-mtaur)
      n1se= step(q-mtaur)     
c
      nbino= step(q-dabs(m1))
      nwino= step(q-dabs(m2))
      ngino= step(q-dabs(m3))
      nh=  step(q-dsqrt(dabs(mhu2)))+step(q-dsqrt(dabs(mhd2)))
      nhl=step(q-ml)
      nhh=step(q-mh)
      scal1 = min(zm,dabs(mu))
      if(scal1.lt.zm) scal1=zm
      scal2 = max(zm,dabs(mu))
      nhlino= step(q-scal1)
      nhhino= step(q-scal2)
      nhino= nhlino +nhhino
c NB nhlino,nhhino,nhino are determined as a compromise: in fact they
c are the "masses" (before physical mixing) of the two Higgsino states.
c We thus arbitrarily define  a light and heavy scale, as function of
c the parameters of the neutralino mass matrix that are relevant to 
c determine those scales.
c
      b10 = 2.d0/5.d0*(17*nu/12.d0+5*nd/12.d0+5*ne/4.d0+nn/4.d0)
     . +nsq/30.d0
     . +4*nsu/15.d0+nsd/15.d0+nsl/10.d0+nse/5.d0
     . +nhino/5.d0 +nh/10.d0
       b20 = -22.d0/3.d0+(nu+nd)/2.d0+(ne+nn)/6.d0
     . +nsq/2.d0 + nsl/6.d0 + nhino/3.d0
     . +nh/6.d0 +4*nwino/3.d0
       b30 = 2*(nu+nd)/3.d0 +nsq/3.d0+nsu/6.d0+nsd/6.d0+2*ngino -11.d0
c - 2-loop gauge coupling beta functions (nb the variables are g^2):
c -y(1)--y(3): g1^2,g2^2,g3^2.
       ytau2 =y(4)*y(4)
       yb2 = y(5)*y(5)
       ytop2 = y(6)*y(6)
c
       dydx(1) = 2*cpi*b10*y(1)*y(1) 
     . +2*cpi*cpi*y(1)*y(1)*((19*nf/15.d0+9.d0/25.d0)*y(1)
     . +(3*nf/5.d0+9.d0/5.d0)
     . *y(2)+44*nf/15.d0*y(3)-26*ytop2/5.d0-14*yb2/5.d0-18*ytau2/5.d0)

       dydx(2) = 2*cpi*b20*y(2)*y(2)
     . +2*cpi*cpi*y(2)*y(2)*((nf/5.d0+3.d0/5.d0)*y(1)
     . +(7*nf-17.d0)*y(2)
     . +4*nf*y(3) -6*ytop2 -6*yb2-2*ytau2 )

       dydx(3) = 2*cpi*b30*y(3)*y(3)
     . +2*cpi*cpi*y(3)*y(3)*(11*nf/30.d0*y(1)+3*nf/2.d0*y(2)
     . +(34*nf/3.d0-54.d0)
     . *y(3) -4*ytop2 -4*yb2 )
c - 2-loop Yukawa coupling beta function (only Ytau,Yb,Ytop included):
c -y(4)--y(6): Ytau,Yb,Ytop
c threshold effects:
      tbet=dexp(y(7)-y(8))
      cb2=1.d0/(1.d0+tbet*tbet)
      sb2=1.d0-cb2
      Ytaubeta = 3*(cb2*(nhl+nhlino)+sb2*(nhh+nhhino))/2.d0*y(4)*ytau2 
     .  +3*sb2*(nhl-nhh)*y(4)*ytop2 
     .  +(cb2*nhl+sb2*nhh)*(3*yb2+ytau2)*y(4) 
     . +(-3.d0/5.d0*y(1)*(15.d0/4.d0+3*nhlino/4.d0-nbino*
     . (n1sl/4.d0+n1se +nhlino/4.d0))
     . -y(2)*(9.d0/4.d0+9*nhlino/4.d0-3*nwino*(n1sl+nhlino)/4.d0))*y(4)

      Ybbeta = 3*(cb2*(nhl+nhlino)+sb2*(nhh+nhhino))/2.d0*y(5)*yb2 +
     . ((sb2*(nhl+nhlino)+cb2*(nhh+nhhino))/2.d0 -sb2*(nhl-nhh))*
     . y(5)*ytop2+(cb2*nhl+sb2*nhh)*(3*yb2+ytau2)*y(5) 
     . +(-3.d0/5.d0*y(1)*(5.d0/12.d0+3*nhlino/4.d0-nbino*
     . (n1sq/36.d0+n1sd/9.d0+nhlino/4.d0))
     . -y(2)*(9.d0/4.d0+9*nhlino/4.d0-3*nwino*(n1sq+nhlino)/4.d0)
     . -y(3)*(8.d0-4*ngino*(n1sq+n1sd)/3.d0))*y(5)

      Ytbeta =3*(sb2*(nhl+nhlino)+cb2*(nhh+nhhino))/2.d0*y(6)*ytop2 +
     . ((cb2*(nhl+nhlino)+sb2*(nhh+nhhino))/2.d0 -cb2*(nhl-nhh))*
     . y(6)*yb2+3*(sb2*nhl+cb2*nhh)*y(6)*ytop2+cb2*(nhl-nhh)*y(6)*ytau2
     .+(-3.d0/5.d0*y(1)*(17.d0/12.d0+3*nhlino/4.d0-nbino*
     . (n1sq/36.d0+4*n1su/9.d0+nhlino/4.d0))
     . -y(2)*(9.d0/4.d0+9*nhlino/4.d0-3*nwino*(n1sq+nhlino)/4.d0)
     . -y(3)*(8.d0-4*ngino*(n1sq+n1su)/3.d0))*y(6)
c       dydx(4) = cpi*y(4)*(4*ytau2 +3*yb2-9*y(1)/5.-3*y(2)
       dydx(4) =cpi*(Ytaubeta
     . +cpi*(-10*ytau2*ytau2-9*yb2*yb2 -9*yb2*ytau2-3*yb2*ytop2
     . +(6*y(2)+6*y(1)/5.)*ytau2 +(-2*y(1)/5.+16*y(3))*yb2
     . +(9*nf/5.+27./10.)*y(1)*y(1)+(3*nf-21./2.)*y(2)*y(2)
     . +9*y(1)*y(2)/5. )  ) 

c       dydx(5) = cpi*y(5)*(6*yb2 +ytau2 +ytop2
c     .            -7*y(1)/15.d0 -3*y(2) -16*y(3)/3.d0) 
        dydx(5) = cpi*(Ybbeta
     . +cpi*(-22*yb2*yb2-5*ytop2*ytop2-5*yb2*ytop2-3*yb2*ytau2 
     . -3*ytau2*ytau2 
     . +4*y(1)/5.d0*ytop2+(2*y(1)/5.d0+6*y(2)+16*y(3))*yb2
     . +6*y(1)/5.d0*ytau2 
     . +(7*nf/15.d0+7.d0/18.d0)*y(1)*y(1)+(3*nf-21.d0/2.d0)*y(2)*y(2)
     . +(16*nf/3.d0-304.d0/9.d0)*y(3)*y(3)+y(1)*y(2)+8*y(1)*y(3)/9.d0
     . +8*y(2)*y(3) )  )

c       dydx(6) = cpi*y(6)*(6*ytop2 +yb2
c     .            -13*y(1)/15.d0 -3*y(2) -16*y(3)/3.d0)
        dydx(6) = cpi*(Ytbeta 
     . +cpi*(-22*ytop2*ytop2-5*yb2*yb2-5*yb2*ytop2-yb2*ytau2
     . +(6*y(1)/5.d0+6*y(2)+16*y(3))*ytop2+2*y(1)/5.d0*yb2 
     . +(13*nf/15.d0+403.d0/450.d0)*y(1)*y(1)+(3*nf-21.d0/2.d0)
     . *y(2)*y(2)
     . +(16*nf/3.d0-304.d0/9.d0)*y(3)*y(3) +y(1)*y(2)+136.d0/45.d0
     . *y(1)*y(3)
     . +8*y(2)*y(3) ) )

c - (1-loop) Higgs vev beta functions:
c - y(7), y(8) = Ln vu, Ln vd

       dydx(7) = cpi*(3.d0/4.d0*(y(1)/5.d0 +y(2)) -3*ytop2)
     . +cpi*cpi*(9*ytop2*ytop2/4.d0+9*ytop2*yb2/4.d0
     . -(19*y(1)/10.d0+9*y(2)/2.d0+20*y(3))*ytop2
     . -(279.d0/800.d0+1803*nf/3200.d0)*y(1)*y(1)
     . -(207.d0/32.d0+357*nf/128.d0)*y(2)*y(2)
     . -(27.d0/80.d0+9*nf/160.d0)*y(1)*y(2))

       dydx(8) = cpi*(3.d0/4.d0*(y(1)/5.d0 +y(2)) -3*yb2-ytau2)
     . +cpi*cpi*(9*yb2*yb2/4.d0+9*yb2*ytop2/4.d0+3*ytau2*ytau2/4.d0
     . -(2*y(1)/5.d0+9*y(2)/2.d0+20*y(3))*yb2
     . -(9*y(1)/5.d0+3*y(2)/2.d0)*ytau2
     . -(279.d0/800.d0+1803*nf/3200.d0)*y(1)*y(1)
     . -(207.d0/32.d0+357*nf/128.d0)*y(2)*y(2)
     . -(27.d0/80.d0+9*nf/160.d0)*y(1)*y(2))

c - (1-loop) soft susy-breaking terms beta functions:
c - y(9)--y(11) : Atau, Ab, Atop

       dydx(9) =cpi*(8*ytau2*y(9) +6*yb2*y(10)
     . +6*(3*y(1)*sgnM1*exp(y(20))/5.d0 +sgnM2*y(2)*exp(y(21))))

       dydx(10) =cpi*(12*y(10)*yb2 +2*y(9)*ytau2+2*y(11)*ytop2
     . +14*y(1)/15.d0*sgnM1*exp(y(20))+6*y(2)*sgnM2*exp(y(21))
     . +32*y(3)/3.d0*sgnM3*exp(y(22)))

       dydx(11) =cpi*(12*y(11)*ytop2 +2*y(10)*yb2
     . +26*y(1)/15.d0*sgnM1*exp(y(20))+6*y(2)*sgnM2*exp(y(21))
     . +32*y(3)/3.d0*sgnM3*exp(y(22)))

c - (1-loop) y(12)--y(13) : m^2(phi_u), m^2(phi_d)
       trym2 = y(18)-2*y(17)+y(16)-y(15)+y(14)+y(12)-y(13)

       dydx(12) = 2*cpi*(3*ytop2*(y(12)+y(18)+y(17)+y(11)*y(11))
     . +3.d0/10.d0*y(1)*trym2 -3*y(1)/5.d0*exp(2*y(20))
     . -3*y(2)*exp(2*y(21)))

       dydx(13) = 2*cpi*(ytau2*(y(13)+y(15)+y(14)+y(9)*y(9))
     . +3*yb2*(y(13)+y(18)+y(16)+y(10)*y(10))
     . -3.d0/10.d0*y(1)*trym2 -3*y(1)/5.d0*exp(2*y(20))
     . -3*y(2)*exp(2*y(21)))

c - (1-loop) y(14)--y(19) : m^2_tau, m^2_L, m^2_b, m^2_top, m^2_Q, B

       dydx(14) = 2*cpi*(2*ytau2*(y(13)+y(14)+y(15)+y(9)*y(9))
     . +3*y(1)/5.d0*trym2 -12*y(1)/5.d0*exp(2*y(20)))

       dydx(15) = 2*cpi*(ytau2*(y(13)+y(15)+y(14)+y(9)*y(9))
     . -3*y(1)/10.d0*trym2 -3*y(1)/5.d0*exp(2*y(20))
     . -3*y(2)*exp(2*y(21)))

       dydx(16) = 2*cpi*(2*yb2*(y(13)+y(16)+y(18)+y(10)*y(10))
     . +y(1)/5.d0*trym2-4*y(1)/15.d0*exp(2*y(20))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(17) =2*cpi*(2*ytop2*(y(12)+y(17)+y(18)+y(11)*y(11))
     . -2*y(1)/5.d0*trym2 -16*y(1)/15.d0*exp(2*y(20))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(18) =2*cpi*(ytop2*(y(12)+y(17)+y(18)+y(11)*y(11))
     . +yb2*(y(13)+y(18)+y(16)+y(10)*y(10))
     . +y(1)/10.d0*trym2 -y(1)/15.d0*exp(2*y(20))-3*y(2)*exp(2*y(21))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(19) = 2*cpi*(3*y(11)*ytop2 +3*y(10)*yb2 +y(9)*ytau2
     . +3*y(1)/5.d0*sgnM1*exp(y(20)) +3*y(2)*sgnM2*exp(y(21)))

c - Gauginos masses beta functions (includes two-loop):
c - y(20)--y(22) : Ln (M1,M2,M3)
       mm1= sgnM1*dexp(y(20))
       mm2= sgnM2*dexp(y(21))
       mm3= sgnM3*dexp(y(22))
       dydx(20) = -2*cpi*(-3.d0/5.d0-nf)*y(1) 
     . +2*cpi*cpi*y(1)*((19*nf/15.d0+9.d0/25.d0)*y(1)*(1.d0+1.d0)
     . +(3*nf/5.d0+9.d0/5.d0)*y(2)*(1.d0+Mm2/Mm1) 
     . +44*nf/15.d0*y(3)*(1.d0+Mm3/Mm1)
     . -26*ytop2*(1.d0-y(11)/mm1)/5.d0
     . -14*yb2*(1.d0-y(10)/mm1)/5.d0
     . -18*ytau2*(1.d0-y(9)/mm1)/5.d0)

       dydx(21) = -2*cpi*(5.d0-nf)*y(2)
     . +2*cpi*cpi*y(2)*((nf/5.d0+3.d0/5.d0)*y(1)*(1.d0+mm1/mm2)
     . +(7*nf-17.d0)*y(2)*(1.d0+1.d0)
     . +4*nf*y(3)*(1.d0+mm3/mm2)
     . -6*ytop2*(1.d0-y(11)/mm2)
     . -6*yb2*(1.d0 -y(10)/mm2)
     . -2*ytau2*(1.d0 -y(9)/mm2) )

       dydx(22) = -2*cpi*(9.d0-nf)*y(3)
     . +2*cpi*cpi*y(3)*(11*nf/30.d0*y(1)*(1.d0+mm1/mm3)
     . +3*nf/2.d0*y(2)*(1.d0+mm2/mm3)
     . +(34*nf/3.d0-54.d0)*y(3)*(1.d0+1.d0) 
     . -4*ytop2*(1.d0 -y(11)/mm3)
     . -4*yb2*(1.d0 -y(10)/mm3)  )

c - the mu parameter:
c - y(23) = Ln mu

       dydx(23) = cpi*(3*ytop2 +3*yb2+ytau2 -3*y(1)/5.d0-3*y(2))

c - (1-loop) y(24)--y(28) : 1st and 2d gen. sfermion mass^2 terms:
c   m^2_er, m^2_eL, m^2_dr, m^2_ur, m^2_uL

       dydx(24) = 2*cpi*(3*y(1)/5.d0*trym2 -12*y(1)/5.d0*exp(2*y(20)))

       dydx(25) = 2*cpi*(-3*y(1)/10.d0*trym2 -3*y(1)/5.d0*exp(2*y(20))     
     . -3*y(2)*exp(2*y(21)))

       dydx(26) = 2*cpi*(
     . y(1)/5.d0*trym2-4*y(1)/15.d0*exp(2*y(20))-16*y(3)/3.d0
     . *exp(2*y(22)))

       dydx(27) =2*cpi*(
     . -2*y(1)/5.d0*trym2 -16*y(1)/15.d0*exp(2*y(20))
     . -16*y(3)/3.d0*exp(2*y(22)))

       dydx(28) =2*cpi*(
     . y(1)/10.d0*trym2 -y(1)/15.d0*exp(2*y(20))-3*y(2)*exp(2*y(21))
     . -16*y(3)/3.d0*exp(2*y(22)))
c Now mimics pole masses: stop RG evolution at m(Q=m) for soft parameters m: 
c  Everything (except gauges+ yukawas) is frozen at (m_tL mt_R)^1/2:
      if(kpole.eq.1) then
      ewsbscale = dsqrt(msttr1*msttr2)
      else if(kpole.eq.0) then
      ewsbscale = scale
      endif
      if(ewsbscale.lt.zm) ewsbscale=zm
      if(q.lt.ewsbscale) then
       freeze= step(q-ewsbscale)
c retaining tbeta at this (ewsbscale) scale:
       if(h1.lt.0.d0.and.jfirst.eq.0) then
       tbeta=dexp(y(7))/dexp(y(8))
c saving Yukawas at EWSB scale:
       ytau=y(4)
       yb=y(5)
       yt=y(6)
       alsewsb=y(3)/(4*pi)
       endif
       jfirst=1
       do jpole= 7,19
       dydx(jpole)=  freeze*dydx(jpole) 
       enddo
       do jpole= 20,28
       dydx(jpole)= freeze*dydx(jpole) 
       enddo
      endif
c also ALWAYS freezing ytop at mtop:
       if(q.le.MTop) dydx(6) = 0.d0
        end
c----------------------------------------
      SUBROUTINE ODEINT(YSTART,NVAR,X1,X2,EPS,H1,HMIN,NOK,NBAD,DERIVS,
     .RKQC) 
      implicit real*8(a-h,o-z)
      PARAMETER (MAXSTP=10000,NMAX=28,TWO=2.d0,ZERO=0.d0,TINY=1.d-30)
      COMMON /PATH/ KMAX,KOUNT,DXSAV,XP(200),YP(10,200)
      DIMENSION YSTART(NVAR),YSCAL(NMAX),Y(NMAX),DYDX(NMAX)
      common/good/iflop
      external derivs, rkqc 
      X=X1
      H=dSIGN(H1,X2-X1)
      NOK=0
      NBAD=0
      KOUNT=0
      DO 11 I=1,NVAR
        Y(I)=YSTART(I)
11    CONTINUE
      XSAV=X-DXSAV*TWO
      DO 16 NSTP=1,MAXSTP
        CALL DERIVS(X,Y,DYDX)
        DO 12 I=1,NVAR
          YSCAL(I)=dABS(Y(I))+dABS(H*DYDX(I))+TINY
12      CONTINUE
        IF(KMAX.GT.0)THEN
          IF(dABS(X-XSAV).GT.dABS(DXSAV)) THEN
            IF(KOUNT.LT.KMAX-1)THEN
              KOUNT=KOUNT+1
              XP(KOUNT)=X
              DO 13 I=1,NVAR
                YP(I,KOUNT)=Y(I)
13            CONTINUE
              XSAV=X
            ENDIF
          ENDIF
        ENDIF
        IF((X+H-X2)*(X+H-X1).GT.ZERO) H=X2-X
        CALL RKQC(Y,DYDX,NVAR,X,H1,EPS,YSCAL,HDID,HNEXT,DERIVS)
        IF(HDID.EQ.H)THEN
          NOK=NOK+1
        ELSE
          NBAD=NBAD+1
        ENDIF
        IF((X-X2)*(X2-X1).GE.ZERO)THEN
          DO 14 I=1,NVAR
            YSTART(I)=Y(I)
14        CONTINUE
          IF(KMAX.NE.0)THEN
            KOUNT=KOUNT+1
            XP(KOUNT)=X
            DO 15 I=1,NVAR
              YP(I,KOUNT)=Y(I)
15          CONTINUE
          ENDIF
          RETURN
        ENDIF
c        IF(dABS(HNEXT).LT.HMIN) PAUSE 'Stepsize smaller than minimum.'
        iflop=0
        IF(dABS(HNEXT).LT.HMIN) then
c        write(*,'(a)') 'Stepsize smaller than minimum.'
        iflop=1
        endif
        H=HNEXT
16    CONTINUE
c      PAUSE 'Too many steps.'
c       write(*,'(a)') 'Too many steps.'
      iflop=1
      RETURN
      END
      SUBROUTINE RKQC(Y,DYDX,N,X,HTRY,EPS,YSCAL,HDID,HNEXT,DERIVS)
      implicit real*8(a-h,o-z)
      PARAMETER (NMAX=28,FCOR=.066666666666666667d0,
     *    ONE=1.d0,SAFETY=0.9d0,ERRCON=6.d-4)
      common/good/iflop
      EXTERNAL DERIVS
      DIMENSION Y(N),DYDX(N),YSCAL(N),YTEMP(NMAX),YSAV(NMAX),DYSAV(NMAX)
      PGROW=-0.20d0
      PSHRNK=-0.25d0
      XSAV=X
      DO 11 I=1,N
        YSAV(I)=Y(I)
        DYSAV(I)=DYDX(I)
11    CONTINUE
      H=HTRY
1     HH=0.5d0*H
      CALL RK4(YSAV,DYSAV,N,XSAV,HH,YTEMP,DERIVS)
      X=XSAV+HH
      CALL DERIVS(X,YTEMP,DYDX)
      CALL RK4(YTEMP,DYDX,N,X,HH,Y,DERIVS)
      X=XSAV+H
      IF(X.EQ.XSAV) then
c      PAUSE 'Stepsize not significant in RKQC.'
c      write(*,'(a)') 'Stepsize not significant in RKQC.'
      iflop =1
      return
      endif
      CALL RK4(YSAV,DYSAV,N,XSAV,H,YTEMP,DERIVS)
      ERRMAX=0.d0
      DO 12 I=1,N
        YTEMP(I)=Y(I)-YTEMP(I)
        ERRMAX=MAX(ERRMAX,ABS(YTEMP(I)/YSCAL(I)))
12    CONTINUE
      ERRMAX=ERRMAX/EPS
      IF(ERRMAX.GT.ONE) THEN
        H=SAFETY*H*(ERRMAX**PSHRNK)
        GOTO 1
      ELSE
        HDID=H
        IF(ERRMAX.GT.ERRCON)THEN
          HNEXT=SAFETY*H*(ERRMAX**PGROW)
        ELSE
          HNEXT=4.d0*H
        ENDIF
      ENDIF
      DO 13 I=1,N
        Y(I)=Y(I)+YTEMP(I)*FCOR
13    CONTINUE
      RETURN
      END
      SUBROUTINE RK4(Y,DYDX,N,X,H,YOUT,DERIVS)
      implicit real*8(a-h,o-z)
      PARAMETER (NMAX=28)
      DIMENSION Y(N),DYDX(N),YOUT(N),YT(NMAX),DYT(NMAX),DYM(NMAX)
      HH=H*0.5d0
      H6=H/6.d0
      XH=X+HH
      DO 11 I=1,N
        YT(I)=Y(I)+HH*DYDX(I)
11    CONTINUE
      CALL DERIVS(XH,YT,DYT)
      DO 12 I=1,N
        YT(I)=Y(I)+HH*DYT(I)
12    CONTINUE
      CALL DERIVS(XH,YT,DYM)
      DO 13 I=1,N
        YT(I)=Y(I)+H*DYM(I)
        DYM(I)=DYT(I)+DYM(I)
13    CONTINUE
      CALL DERIVS(X+H,YT,DYT)
      DO 14 I=1,N
        YOUT(I)=Y(I)+H6*(DYDX(I)+DYT(I)+2.d0*DYM(I))
14    CONTINUE
      RETURN
      END
c----------------------------------------------------
       COMPLEX*16 FUNCTION SB0(QSQ,M1,M2)
      IMPLICIT REAL*8 (A-H,M,O-Z)
      COMPLEX*16 I,one,xp,xm,fbxp,fbxm
      common/renscale/scale
      eps=1.d-10
      one=dcmplx(1.d0,0.d0)
      I=dcmplx(0.d0,1.d0)
      s = qsq +m1**2-m2**2
      xp = (s +cdsqrt(s**2-4*qsq*(m1**2-I*eps)) )/qsq/2
      xm = (s -cdsqrt(s**2-4*qsq*(m1**2-I*eps)) )/qsq/2
      fBxp = cdlog(1.d0-xp)-xp*cdlog(1.d0-1.d0/xp)-1.d0
      fBxm = cdlog(1.d0-xm)-xm*cdlog(1.d0-1.d0/xm)-1.d0
      SB0 = -dlog(qsq/scale**2) -fBxp-fBxm
      END
c-----------------------------------------------------
      COMPLEX*16 FUNCTION SB1(S,MI,MJ)
      IMPLICIT REAL*8 (A-H,M-Z)
      COMPLEX*16 SB0
       common/renscale/scale
      SB1= ( A(MJ)-A(MI)+(S+MI**2-MJ**2)*SB0(S,MI,MJ) )/S/2
      END
c------------------------------------------------------
      COMPLEX*16 FUNCTION BT22(S,MI,MJ)
      IMPLICIT REAL*8 (A-H,M-Z)
      COMPLEX*16 SB0
      common/renscale/scale
      BT22 = 1.D0/6*( A(MI)/2 +A(MJ)/2 +
     .   (MI**2+MJ**2-S/2)*SB0(S,MI,MJ) 
     .  +(MJ**2-MI**2)/S/2*( A(MJ)-A(MI)-(MJ**2-MI**2)*SB0(S,MI,MJ) )
     .  +MI**2+MJ**2-S/3 ) 
     .  -A(MI)/4 -A(MJ)/4
      END
c------------------------------------------------------
      COMPLEX*16 FUNCTION BH(S,MI,MJ)
      IMPLICIT REAL*8 (A-H,M-Z)
      COMPLEX*16 SB0
      common/renscale/scale
      BH = 4.d0/6*( A(MI)/2 +A(MJ)/2 +
     .  (MI**2+MJ**2-S/2)*SB0(S,MI,MJ) 
     .  +(MJ**2-MI**2)/S/2*( A(MJ)-A(MI)-(MJ**2-MI**2)*SB0(S,MI,MJ) )
     .  +MI**2+MJ**2-S/3 )
     .  +(S-MI**2-MJ**2)*SB0(S,MI,MJ)-A(MI)-A(MJ)
      END
c------------------------------------------------------
      real*8 FUNCTION A(M)
      IMPLICIT REAL*8 (A-H,M,O-Z)
      common/renscale/scale
      A = M*M * (1.D0-2*DLOG(M/scale)) 
      END
c---------------------------------------------------------------
      logical function IsNaN(a)
C---------------------------------------------------------------
C test if a is NaN. Comparison of NaN with any number gives FALSE
C---------------------------------------------------------------
      real*8 a
      IsNaN = .not. (a.gt.0d0.or.a.lt.0d0.or.a.eq.0d0)
      end

