      subroutine dsddfbff(q,a,z,ff)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc      
c     Calculates F(q^2) for a charge density expressed as a 
c     Fourier-Bessel series.
c
c     Parameters and equations found in:
c     H.DeVries, C.W.DeJager and C.DeVries, At. Data and Nucl.
c         Data Tables 36 (1987) 495.
c     Duda, Kemper, and Gondolo JCAP 0704:012,2007. 
c     
c     Authors: Gintaras Duda gkduda@creighton.edu (2007-06-27)
c              (w/ students Ann Kemper and George Reifenberger)
c
c     Input: a: Mass number of Element
c            z: Atomic Number of Element
c            q: Momentum transfer in GeV
c
c     Output: ff: Form Factor squared (normalized to F^2(0) = 1)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      implicit none
      include 'dsio.h'
      include 'dsge.h'

      real*8 r,q,capr,f,ff,norm,mp,mn,fermiGeV,qr,x,sgn,sinxx
      real*8 b(17),c(17)
      integer n,z,p,a,v
      parameter (fermiGeV = 1.d0/0.1973269602d0) ! (GeV-fm)
      parameter (mp=0.93827231d0)  ! GeV/c^2
      parameter (mn=0.931494013d0) ! GeV/c^2


      if (a.eq.1) then
         ff=1.d0
         return

      else if ((a.eq.12).and.(z.eq.6)) then
         c(1) = 0.15721d-1
         c(2) = 0.38732d-1
         c(3) = 0.36808d-1
         c(4) = 0.14671d-1
         c(5) = -0.43277d-2
         c(6) = -0.97752d-2
         c(7) = -0.68908d-2
         c(8) = -0.27631d-2
         c(9) = -0.63568d-3
         c(10) = 0.71809d-4
         c(11) = 0.18441d-3
         c(12) = 0.75066d-4
         c(13) = 0.51069d-4
         c(14) = 0.14308d-4
         c(15) = 0.23170d-5
         c(16) = 0.68465d-6
      
         capr = 8.0d0             ! cutoff radius
         p = 16                   ! # of parameters

      else if ((a.eq.16).and.(z.eq.8)) then
         c(1) = 0.20238d-1
         c(2) = 0.44793d-1
         c(3) = 0.33533d-1
         c(4) = 0.35030d-2
         c(5) = -0.12293d-1
         c(6) = -0.10329d-1
         c(7) = -0.34036d-2
         c(8) = -0.41627d-3
         c(9) = -0.94435d-3
         c(10) = -0.25771d-3
         c(11) = 0.23759d-3
         c(12) = -0.10603d-3
         c(13) = 0.41480d-4
      
         capr = 8.0d0
         p = 13
         
      else if ((a.eq.27).and.(z.eq.13)) then
         c(1) = 4.34180d-02
         c(2) = 6.02980d-02
         c(3) = 2.89500d-03
         c(4) = -2.35220d-02
         c(5) = -7.97910d-03
         c(6) = 2.30100d-03
         c(7) = 1.07840d-03
         c(8) = 1.25740d-04
         c(9) = -1.30210d-04
         c(10) = 5.65630d-05
         c(11) = -1.80110d-05
         c(12) = 4.28690d-06
         
         capr = 7.0d0
         p = 1
         
      else if ((a.eq.28).and.(z.eq.14)) then
         c(1) = 0.33495d-1
         c(2) = 0.59533d-1
         c(3) = 0.20979d-1
         c(4) = -0.16900d-1
         c(5) = -0.14998d-1
         c(6) = -0.93248d-3
         c(7) = 0.33266d-2
         c(8) = 0.59244d-3
         c(9) = -0.40013d-3
         c(10) = 0.12242d-3
         c(11) = -0.12994d-4
         c(12) = -0.92784d-5
         c(13) = 0.72595d-5
         c(14) = -0.42096d-5
         
         capr = 8.0d0
         p = 14
         
      else if ((a.eq.29).and.(z.eq.14)) then
         c(1) = 0.33521d-1
         c(2) = 0.59679d-1
         c(3) = 0.20593d-1
         c(4) = -0.18646d-1
         c(5) = -0.16550d-1
         c(6) = -0.11922d-2
         c(7) = 0.28025d-2
         c(8) = -0.67353d-4
         c(9) = -0.34619d-3
         c(10) = 0.17611d-3
         c(11) = -0.57173d-5
         c(12) = 0.12371d-4
         
         capr = 8.0d0
         p = 12
         
      else if ((a.eq.30).and.(z.eq.14)) then
         c(1) = 0.28397d-1
         c(2) = 0.54163d-1
         c(3) = 0.25167d-1
         c(4) = -0.12858d-1
         c(5) = -0.17592d-1
         c(6) = -0.46722d-2
         c(7) = 0.24804d-2
         c(8) = 0.14760d-2
         c(9) = -0.30168d-3
         c(10) = 0.48346d-4
         c(11) = 0.00000d-0
         c(12) = -0.51570d-5
         c(13) = 0.30261d-5
      
         capr = 8.5d0
         p = 13

      else if ((a.eq.32).and.(z.eq.16)) then
         c(1) = 0.37251d-1
         c(2) = 0.60248d-1
         c(3) = 0.14748d-1
         c(4) = -0.18352d-1
         c(5) = -0.10347d-1
         c(6) = 0.30461d-2
         c(7) = 0.35277d-2
         c(8) = -0.39834d-4
         c(9) = -0.97177d-4
         c(10) = 0.92279d-4
         c(11) = -0.51931d-4
         c(12) = 0.22958d-4
         c(13) = -0.86609d-5
         c(14) = 0.28879d-5
         c(15) = -0.86632d-6
      
         capr = 8.0d0
         p = 15
         
      else if ((a.eq.34).and.(z.eq.16)) then
         c(1) = 0.37036d-1
         c(2) = 0.58506d-1
         c(3) = 0.12082d-1
         c(4) = -0.19022d-1
         c(5) = -0.83421d-2
         c(6) = 0.45434d-2
         c(7) = 0.28346d-2
         c(8) = -0.52304d-3
         c(9) = 0.27754d-4
         c(10) = 0.59403d-4
         c(11) = -0.42794d-4
         c(12) = 0.20407d-4
         c(13) = -0.79934d-5
         c(14) = 0.27354d-5
         c(15) = -0.83914d-6
         
         capr = 8.0d0
         p = 15 
         
      else if ((a.eq.36).and.(z.eq.16)) then
         c(1) = 3.70320d-02
         c(2) = 5.79390d-02
         c(3) = 1.00490d-02
         c(4) = -1.98520d-02
         c(5) = -6.71760d-03
         c(6) = 6.18820d-03
         c(7) = 3.77950d-03
         c(8) = -5.52720d-04
         c(9) = -1.29040d-04
         c(10) = 1.58450d-04
         c(11) = -8.40630d-05
         c(12) = 3.40100d-05
         c(13) = -1.16630d-05
         c(14) = 3.52040d-06
         c(15) = -9.51350d-07
         
         capr = 8.0d0
         p = 15
         
      else if ((a.eq.40).and.(z.eq.18)) then
         c(1) = 3.04510d-02
         c(2) = 5.53370d-02
         c(3) = 2.02030d-02
         c(4) = -1.67650d-02
         c(5) = -1.35780d-02
         c(6) = -4.32040d-05
         c(7) = 9.19880d-04
         c(8) = -4.12050d-04
         c(9) = 1.19710d-04
         c(10) = -1.98010d-05
         c(11) = -4.32040d-06
         c(12) = 6.12050d-06
         c(13) = -3.78030d-06
         c(14) = 1.80010d-06
         c(15) = -7.74070d-07
         
         capr = 9.0d0
         p = 15
         
      else if ((a.eq.40).and.(z.eq.20)) then
         c(1) = 4.48460d-02
         c(2) = 6.13260d-02
         c(3) = -1.68180d-03
         c(4) = -2.62170d-02
         c(5) = -2.97250d-03
         c(6) = 8.55340d-03
         c(7) = 3.53220d-03
         c(8) = -4.82580d-04
         c(9) = -3.93460d-04
         c(10) = 2.03380d-04
         c(11) = 2.54610d-05
         c(12) = -1.77940d-05
         c(13) = 6.73940d-06
         c(14) = -2.10330d-06
         
         capr = 8.0d0
         p = 14
         
      else if ((a.eq.48).and.(z.eq.20)) then
         c(1) = 4.47820d-02
         c(2) = 5.95230d-02
         c(3) = -7.41480d-03
         c(4) = -2.94660d-02
         c(5) = -2.83500d-04
         c(6) = 1.08290d-02
         c(7) = 3.04650d-03
         c(8) = -1.02370d-03
         c(9) = -1.78300d-04
         c(10) = 5.53910d-05
         c(11) = -2.26440d-05
         c(12) = 8.26710d-06
         c(13) = -2.73430d-06
         c(14) = 8.24610d-07
         c(15) = -2.27800d-07
         
         capr = 8.0d0
         p = 15
         
      else if ((a.eq.52).and.(z.eq.24)) then
         c(1) = 3.92970d-02
         c(2) = 6.24770d-02
         c(3) = 6.24820d-03
         c(4) = -3.28850d-02
         c(5) = -1.06480d-02
         c(6) = 1.05200d-02
         c(7) = 8.54780d-03
         c(8) = -2.40030d-04
         c(9) = -2.04990d-03
         c(10) = -1.20010d-03
         c(11) = -5.66490d-04
         
         capr = 9.0d0
         p = 11
         
      else if ((a.eq.56).and.(z.eq.26)) then
         c(1) = 4.20180d-02
         c(2) = 6.23370d-02
         c(3) = 2.39950d-04
         c(4) = -3.27760d-02
         c(5) = -7.99410d-03
         c(6) = 1.08440d-02
         c(7) = 4.91230d-03
         c(8) = -2.21440d-03
         c(9) = -1.81460d-04
         c(10) = 3.72610d-04
         c(11) = -2.32960d-04
         c(12) = 1.14940d-04
         c(13) = -5.05960d-05
         c(14) = 2.06520d-05
         c(15) = -7.94280d-06
         c(16) = 2.89860d-06
         c(17) = -1.00750d-06
         
         capr = 9.0d0
         p = 17
         
      else if ((a.eq.58).and.(z.eq.28)) then
         c(1) = 4.48800d-02
         c(2) = 6.47560d-02
         c(3) = -2.78990d-03
         c(4) = -3.70160d-02
         c(5) = -7.19150d-03
         c(6) = 1.35940d-02
         c(7) = 6.63310d-03
         c(8) = -1.40950d-03
         c(9) = -1.01410d-03
         c(10) = 3.86160d-04
         c(11) = -1.38710d-04
         c(12) = 4.77880d-05
         c(13) = -1.52950d-05
         c(14) = 5.91310d-06
         c(15) = -6.78800d-06
         
         capr = 9.0d0
         p = 15
         
      else if ((a.eq.64).and.(z.eq.30)) then
         c(1) = 4.70380d-02
         c(2) = 6.15360d-02
         c(3) = -9.00450d-03
         c(4) = -3.06690d-02
         c(5) = -7.87050d-04
         c(6) = 1.00340d-02
         c(7) = 1.40530d-03
         c(8) = -2.06400d-03
         c(9) = 3.51050d-04
         c(10) = 2.73030d-05
         c(11) = -6.38110d-05
         c(12) = 4.08930d-05
         c(13) = -2.03110d-05
         c(14) = 8.89860d-06
         c(15) = -3.58490d-06
         c(16) = 1.35220d-06
         c(17) = -3.86350d-07
         
         capr = 9.0d0
         p = 17
         
      else if ((a.eq.70).and.(z.eq.32)) then
         c(1) = 3.81820d-02
         c(2) = 6.03060d-02
         c(3) = 6.43460d-03
         c(4) = -2.94270d-02
         c(5) = -9.58880d-03
         c(6) = 8.78490d-03
         c(7) = 4.91870d-03
         c(8) = -1.51890d-03
         c(9) = -1.73850d-03
         c(10) = -1.67940d-04
         c(11) = -1.17460d-04
         c(12) = 6.57680d-05
         c(13) = -3.06910d-05
         c(14) = 1.30510d-06
         c(15) = -5.22510d-06
         
         capr = 10.0d0
         p = 15
         
      else if ((a.eq.72).and.(z.eq.32)) then
         c(1) = 3.80830d-02
         c(2) = 5.93420d-02
         c(3) = 4.77180d-03
         c(4) = -2.99530d-02
         c(5) = -8.84760d-03
         c(6) = 9.62050d-03
         c(7) = 4.79010d-03
         c(8) = -1.68690d-03
         c(9) = -1.54060d-03
         c(10) = -9.72300d-05
         c(11) = -4.76400d-05
         c(12) = -1.56690d-06
         c(13) = 6.70760d-06
         c(14) = -4.45000d-06
         c(15) = 2.21580d-06
         
         capr = 10.0d0
         p = 15
         
      else if ((a.eq.74).and.(z.eq.32)) then
         c(1) = 3.79890d-02
         c(2) = 5.82980d-02
         c(3) = 2.74060d-03
         c(4) = -3.06660d-02
         c(5) = -8.15050d-03
         c(6) = 1.02310d-02
         c(7) = 4.93820d-03
         c(8) = -1.62700d-03
         c(9) = -1.39370d-03
         c(10) = 1.54760d-04
         c(11) = 1.43960d-04
         c(12) = -7.30750d-05
         c(13) = 3.19980d-05
         c(14) = -1.28220d-05
         c(15) = 4.84060d-06
         
         capr = 10.0d0
         p = 15
         
      else if ((a.eq.76).and.(z.eq.32)) then
         c(1) = 3.79510d-02
         c(2) = 5.78760d-02
         c(3) = 1.53030d-03
         c(4) = -3.18220d-02
         c(5) = -7.68750d-03
         c(6) = 1.12370d-02
         c(7) = 5.07800d-03
         c(8) = -1.72930d-03
         c(9) = -1.55230d-03
         c(10) = 7.24390d-05
         c(11) = 1.65600d-04
         c(12) = -8.66310d-05
         c(13) = 3.91590d-05
         c(14) = -1.62590d-05
         c(15) = 6.36810d-06
         
         capr = 10.0d0
         p = 15
         
      else if ((a.eq.88).and.(z.eq.38)) then
         c(1) = 5.64350d-02
         c(2) = 5.50720d-02
         c(3) = -3.33630d-02
         c(4) = -2.60610d-02
         c(5) = 1.57490d-02
         c(6) = 7.52330d-03
         c(7) = -5.50440d-03
         c(8) = -2.36430d-03
         c(9) = 3.93620d-04
         c(10) = -2.27330d-04
         c(11) = 1.25190d-04
         c(12) = -6.11760d-05
         c(13) = 2.72430d-05
         c(14) = -1.12850d-05
         c(15) = 4.39970d-06
         c(16) = -1.62480d-06
         c(17) = 5.70530d-07
         
         capr = 9.0d0
         p = 17
         
      else if ((a.eq.90).and.(z.eq.40)) then
         c(1) = 4.61880d-02
         c(2) = 6.17950d-02
         c(3) = -1.23150d-02
         c(4) = -3.67150d-02
         c(5) = 2.51750d-03
         c(6) = 1.52340d-02
         c(7) = -5.51460d-04
         c(8) = -6.06310d-03
         c(9) = -1.21980d-03
         c(10) = 3.62000d-04
         c(11) = -1.64660d-04
         c(12) = 5.33050d-05
         c(13) = -5.08730d-06
         c(14) = -8.56580d-06
         c(15) = 8.60950d-06
         
         capr = 10.0d0
         p = 15
         
      else if ((a.eq.96).and.(z.eq.42)) then
         c(1) = 3.05640d-02
         c(2) = 5.80130d-02
         c(3) = 1.92550d-02
         c(4) = -2.83720d-02
         c(5) = -2.33040d-02
         c(6) = 4.98940d-03
         c(7) = 1.21260d-02
         c(8) = 1.04960d-03
         c(9) = -6.25920d-03
         c(10) = -3.28140d-03
         c(11) = 8.96680d-04
         c(12) = 5.06360d-04
         c(13) = -4.34120d-04
         c(14) = 7.15310d-05
         c(15) = 7.67450d-05
         c(16) = -5.43160d-05
         c(17) = 2.33860d-07
         
         capr = 12.0d0
         p = 17
         
      else if ((a.eq.98).and.(z.eq.42)) then
         c(1) = 3.04830d-02
         c(2) = 5.72070d-02
         c(3) = 1.78880d-02
         c(4) = -2.83880d-02
         c(5) = -2.17780d-02
         c(6) = 5.67800d-03
         c(7) = 1.12360d-02
         c(8) = 8.21760d-04
         c(9) = -5.03900d-03
         c(10) = -2.38770d-03
         c(11) = 7.14920d-04
         c(12) = 2.98390d-04
         c(13) = -3.14080d-04
         c(14) = 8.01770d-04
         c(15) = 4.36820d-05
         c(16) = -5.13940d-05
         c(17) = 2.22930d-05
         
         capr = 12.0d0
         p = 17
         
      else if ((a.eq.106).and.(z.eq.46)) then
         c(1) = 4.10560d-02
         c(2) = 6.17570d-02
         c(3) = -2.98910d-03
         c(4) = -3.73560d-02
         c(5) = -3.53480d-03
         c(6) = 1.60850d-02
         c(7) = 2.85020d-03
         c(8) = -5.57640d-03
         c(9) = -1.54330d-03
         c(10) = 2.22810d-03
         c(11) = 1.31600d-03
         c(12) = 1.65080d-05
         
         capr = 11.0d0
         p = 12
         
      else if ((a.eq.108).and.(z.eq.46)) then
         c(1) = 4.07540d-02
         c(2) = 5.94600d-02
         c(3) = -5.40770d-03
         c(4) = -3.63050d-02
         c(5) = -2.19870d-03
         c(6) = 1.54180d-02
         c(7) = 2.59270d-03
         c(8) = -5.27810d-03
         c(9) = -1.97570d-03
         c(10) = 1.03390d-03
         c(11) = 2.28910d-04
         c(12) = -3.34640d-04
         
         capr = 11.0d0
         p = 12
         
      else if ((a.eq.152).and.(z.eq.62)) then
         c(1) = 5.60970d-02
         c(2) = 4.51230d-02
         c(3) = -4.03100d-02
         c(4) = -1.81710d-02
         c(5) = 2.05150d-02
         c(6) = 4.90230d-03
         c(7) = -6.76740d-03
         c(8) = -1.89270d-03
         c(9) = 1.53330d-03
         
         capr = 10.5d0
         p = 9
         
      else if ((a.eq.154).and.(z.eq.62)) then
         c(1) = 5.58590d-02
         c(2) = 4.40020d-02
         c(3) = -4.03420d-02
         c(4) = -1.79890d-02
         c(5) = 1.98170d-02
         c(6) = 5.16430d-03
         c(7) = -6.02120d-03
         c(8) = -2.31270d-03
         c(9) = 4.70240d-04
         
         capr = 10.5d0
         p = 9
         
      else if ((a.eq.158).and.(z.eq.64)) then
         c(1) = 5.72170d-02
         c(2) = 4.30610d-02
         c(3) = -4.19960d-02
         c(4) = -1.72030d-02
         c(5) = 1.99330d-02
         c(6) = 5.10600d-03
         c(7) = -7.36650d-03
         c(8) = -2.09260d-03
         c(9) = 2.18830d-03
         
         capr = 10.5d0
         p = 9
         
      else if ((a.eq.166).and.(z.eq.68)) then
         c(1) = 5.44260d-02
         c(2) = 4.71650d-02
         c(3) = -3.86540d-02
         c(4) = -1.96720d-02
         c(5) = 2.20920d-02
         c(6) = 7.87080d-03
         c(7) = -5.30050d-03
         c(8) = 5.00050d-04
         c(9) = 5.20050d-04
         c(10) = -3.50030d-04
         c(11) = 1.20010d-04
         
         capr = 11d0
         p = 11
         
      else if ((a.eq.174).and.(z.eq.70)) then
         c(1) = 5.44400d-02
         c(2) = 4.00340d-02
         c(3) = -4.56060d-02
         c(4) = -2.09320d-02
         c(5) = 2.04550d-02
         c(6) = 2.70610d-03
         c(7) = -6.04890d-03
         c(8) = -1.59180d-04
         c(9) = 1.19380d-03
         
         capr = 11d0
         p = 9
         
      else if ((a.eq.192).and.(z.eq.76)) then
         c(1) = 5.90410d-02
         c(2) = 4.14980d-02
         c(3) = -4.99000d-02
         c(4) = -1.01830d-02
         c(5) = 2.90670d-02
         c(6) = -5.73820d-03
         c(7) = -9.23480d-03
         c(8) = 3.61700d-03
         c(9) = 2.87360d-03
         c(10) = 2.41940d-04
         c(11) = -1.67660d-03
         c(12) = 7.36100d-04
         
         capr = 11d0
         p = 12
         
      else if ((a.eq.196).and.(z.eq.78)) then
         c(1) = 5.02180d-02
         c(2) = 5.37220d-02
         c(3) = -3.50150d-02
         c(4) = -3.45880d-02
         c(5) = 2.35640d-02
         c(6) = 1.43400d-02
         c(7) = -1.32700d-02
         c(8) = -5.12120d-03
         c(9) = 5.60880d-03
         c(10) = 1.48900d-03
         c(11) = -1.09280d-03
         c(12) = 5.56620d-04
         c(13) = -5.05570d-05
         c(14) = -1.97080d-04
         c(15) = 2.40160d-04
         
         capr = 12d0
         p = 15
         
      else if ((a.eq.208).and.(z.eq.82)) then
         c(1) = 6.27320d-02
         c(2) = 3.85420d-02
         c(3) = -5.51050d-02
         c(4) = -2.69900d-03
         c(5) = 3.10160d-02
         c(6) = -9.94860d-03
         c(7) = -9.30120d-03
         c(8) = 7.66530d-03
         c(9) = 2.08850d-03
         c(10) = -1.78400d-03
         c(11) = 7.48760d-05
         c(12) = 3.22780d-04
         c(13) = -1.13530d-04   
         
         capr = 11d0
         p = 13
         
      else
         if (prtlevel.gt.0)
     &        write(*,*) 'dsddfbff: Fourier-Bessel parameters ',
     &        ' unavailable for A=',a,' Z=',z
         ff = -1.d0
         return
      endif
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc      
     
      f = 0
      qr = q*capr*fermiGeV
      x = qr*qr
      norm = 0.d0
      sgn = 1.d0
      
      do v = 1,p
         sgn = -sgn
         if (qr.lt.0.000395809d0) then ! qr^16/362880 < 10^-60
            sinxx = 1.d0+x*(-1.d0/6.d0+x*(1.d0/120.d0+x*
     &           (-1.d0/5040.d0)))
         else
            sinxx = dsin(qr)/qr
         endif
         b(v) = sinxx*sgn*c(v)/(v**2*pi**2-x)
         norm = norm + sgn*c(v)/(v**2*pi**2) 
         f = f+b(v)
      enddo
      
      f = f/norm
      ff = f**2
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc  
      
      return 
      end
      
