      subroutine dssuspectsugra(unphys,errmess)
c_______________________________________________________________________
c  interface subroutine between Suspect & Darksusy in mSUGRA case
c  this routine should be called instead of dssusy
c  
c  call for each given set of mSUGRA variables: 
c     m0var,mhfvar,a0var,tgbetavar,sgnmuvar
c  given in the sugrainput common block; additional standard inputs for 
c  Suspect are set in this routine as specified in the body below. 
c
c  if error checks in Suspect are all ok, the routine sets up: 
c  1) SM inputs in Darksusy according to SM standard inputs in Suspect
c  2) Darksusy SUSY variables and soft terms 
c  3) full mass spectrum and mixing matrices according to Suspect output
c  4) global constants needed to run Darksusy (relic density + detection 
c     rates + constraint checks), set with dssuconst
c  5) interaction vertices, set with dsvertx
c  6) particle widths, set with dshwidths + plus a few set explicitly in
c     this routine
c  7) prepares the rate routines for a new model, calling dsprep
c
c  in output if unphys=0, everything is ok
c  if unphys < 0, one or more errmess(10) are <0, check error message
c  with dssuspecterr.f
c
c  expanded from a version by Emmanuel NEZRI, nezri@in2p3.fr,
c  February 2001  
c
c  author: Piero Ullio, ullio@sissa.it, September 2001
c=======================================================================
      implicit none
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Suspect input variables:
c   in the call of Suspect:  
      integer IKNOWL, INPUT
      integer ICHOICE(10)
      real*8 errmess(10)
c   in common Suspect blocks:  
      real*8 dalfinv,dsw2,dalphas,dmt,dmb,dmc,dmtau
      COMMON/SMpar/dalfinv,dsw2,dalphas,dmt,dmb,dmc,dmtau
      real*8 susym,ehigh,elow
      COMMON/RGscal/susym,ehigh,elow
      real*8 dmhu2,dmhd2,dMA,dMU
      COMMON/MSSMhpar/dmhu2,dmhd2,dMA,dMU
      real*8 dM1,dM2,dM3
      COMMON/MSSMgpar/dM1,dM2,dM3
      real*8 dMSL,dMTAUR,dMEL,dMER
      COMMON/MSSMslep/dMSL,dMTAUR,dMEL,dMER
      real*8 dMSQ,dMTR,dMBR,dMUQ,dMUR,dMDR
      COMMON/MSSMsqua/dMSQ,dMTR,dMBR,dMUQ,dMUR,dMDR
      real*8 dAL,dAU,dAD
      COMMON/MSSMAtri/dAL,dAU,dAD
      real*8 m0,mhalf,A0,sgnmu0,tgbeta
      COMMON/MSUGRA/m0,mhalf,A0,sgnmu0,tgbeta
c Suspect output :
c   rad. corrected (final) bottom, top tau masses 
      real*8 mbcr,mtcr,mtaucr
      common/rcmass/mbcr,mtcr,mtaucr
c   stop 1,2 and sup 1,2 = scharm 1,2 masses
      real*8 dmst1,dmst2,dmsu1,dmsu2
      COMMON/OUTSQU/dmst1,dmst2,dmsu1,dmsu2
c   sbottom 1,2 and sdown 1,2 = sstrange 1,2 masses
      real*8 dmsb1,dmsb2,dmsd1,dmsd2
      COMMON/OUTSQD/dmsb1,dmsb2,dmsd1,dmsd2
c   stau 1,2 ; selectron (=smuon) 1,2; sneut_e,mu, sneut_tau masses
      real*8 dmsl1,dmsl2,dmse1,dmse2,dmsn1,dmsntau
      COMMON/OUTSLEP/dmsl1,dmsl2,dmse1,dmse2,dmsn1,dmsntau
c   stop, sbottom, stau mixing angles
      real*8 thet,theb,thel
      common/mixang/thet,theb,thel
c   neutralino, chargino, gluino parameters
      real*8 dmc1,dmc2,dmn1,dmn2,dmn3,dmn4,mgluino,U(2,2),V(2,2)
     & ,Z(4,4),dxmn(4) 
      COMMON/OUTGINOS/dmc1,dmc2,dmn1,dmn2,dmn3,dmn4,mgluino
      COMMON/matino/U,V,Z,dxmn
c   higgs sector:
      real*8 dml,dmh,dmch,alfa 
      COMMON/OUTHIGGS/dml,dmh,dmch,alfa 
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c declarations in Darksusy and for this routine
      include 'dssusy.h'
      include 'dsidtag.h'
      integer unphys
      integer i,j,g
      real*8 d,m2ll,m2rr,theta
      real*8 zgm,dsabsq
      character*80 message
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c part pasted in from supect2_call.f
c
c                        SET RUNNING INPUT :
c The calling of the SUSPECT2 subroutine is :
c       CALL suspect2(IKNOWL, INPUT, ICHOICE,ERRMESS)
c  where IKNOWL, INPUT, ICHOICE,ERRMESS are important control parameters:
c************************************************************
c 1. "ALGORITHM  CONTROL" PARAMETERS:
c************************************************************ 
c  set degree of control on various part of the calculation: 
c  IKNOWL=0: "blind" use: i.e. no control on any "algorithmic"
c  parameters, no warning etc messages (i.e. just the results from you
c  physical input. "Reasonable" default values are set for control param.
c  IKNOWL=1:  no control on any algorithmic
c  but some warning/error messages are collected in suspect.out output  
c  IKNOWL= 2: more "educated" use: you can control (some) algorithmic
c parameters AND you get all warning/error messages.
c Caution: IKNOWL=2 is not encouraged for a "standard" user,
c unless you want to "debug". With IKNOWL=2 many unconvenient comments
c will be also print on the screen.
c 
        IKNOWL = 1
c
c*****************************************************************
c 2. INPUT CHOICE (OPTION/CONTROL PARAMETERS and MODELS): ICHOICE(1)-(10)
c !!!!!!  2 possibilities: of entering input:
c  1) set  all ichoice(1)-(10) values in INPUT FILE: suspect2.in (see there)
c  then you have to set HERE: 
c           INPUT = 0
c AND you will define all relevant MSSM/mSUGRA etc parameters in suspect2.in
c (see suspect2.in for more details)
c//////////////////////////////////////////////////////////////////////
c  2)  SET ALL ICHOICE(1)-(10) values HERE (convenient e.g. for scan on
c  some MSSM parameters)
c  THEN you have to set HERE:
c           INPUT = 1
c AND you also have to define ALL relevant physical input parameters: 
c the required list of parameters with consistent names etc, can 
c be found in the commons given above, and their meaning is explained in
c details in the suspect2.f file. See example of a consistent call below.)
c  3) INPUT = 11:
c Same as INPUT = 1 , but NO OUTPOUT File (suspect.out) generated
c (convenient e.g. for scan on MSSM/mSUGRA parameters)
c NB: if you are not sure what to choose (or make "wrong" choice here), note 
c "reasonable" default values for these crucial input will be taken anyway
c
c       INPUT = 0
c
c NB UNcomment assignement CC lines below for call example with INPUT = 1:
c
      INPUT = 11
c control parameters input:
      ichoice(1) = 10
c (minimal SUGRA case)
c
      ichoice(2) = 21
c (2-loop RGE for gauge, yukawas, gauginos)
      ichoice(3) = 0
c (ichoice(3)= 0: GUT scale imposed (then EHIGH = input!); 
c            = 1: gauge unif scale calculated from gauge cpls. input
c
      ichoice(4) = 1
c  (RG accuracy: 1: moderately accurate and fast (generally sufficient)
c                 2: very accurate but rather slow!
      ichoice(5) = 1
c (consistent EWSB)
      ichoice(6) = 1
c (M_Hu, M_Hd (= m_0 in mSUGRA) input)
      ichoice(7) = 3
c ICHOICE(7):  SUSY radiative corrections to the (s)particles masses: 
c                      No Radiative corretions     : 0 
c                      only in mb,mt,mtau +Yukawas : 1  
c                      all squarks R.C. in addition: 2
c                      all gaugino R.C in addition : 3
      ichoice(8) = 1
c (ichoice(8) = 1 for default EWSB scale=(m_t_L*m_t_R)^(1/2), =0 if not)
c then IF ichoice(8)=0 EWSB scale is set by user from input file/calling
c routine by the value of Qewsb
      ichoice(9) = 3
c ichoice(9) >= 3: Nb of (long: RGE + Full spectrum) iterations
      ichoice(10) = 1
c ICHOICE(10): Running Higgs masses: 0; pole Higgs masses: 1
c
c  "SM-like" input:
      dalfinv= 127.938d0
      dsw2 = .23117d0
      dalphas =.119d0
      dmt =174.3d0
      dmb = 4.62d0
      dmtau =1.7771d0
c
c RG evolution parameters: 
      ehigh = 1.9d16
      elow =91.19d0
c   
c  minimal SUGRA case input:
c
      m0 = m0var
      mhalf = mhfvar
      A0 = a0var
      sgnmu0 = sgnmuvar
      tgbeta = tgbetavar
c
c   non-universal case input:
c
CC        MHU2     = .5d4
CC        MHD2     = 5.d5
CC        M1       = 50.D0
CC        M2       = 200.D0
CC        M3       = 350.D0
CC        MSL      = 4.7D2
CC        MTAUR    = 4.7D2
CC        MSQ      = 367.5d0
CC        MTR      = 123.5d0
CC        MBR      = 4.7D2
CC        MEL      = 4.7D2
CC        MER      = 4.7D2
CC        MUQ      = 4.7D2
CC        MUR      = 4.7D2
CC        MDR      = 4.7D2
CC        AL       = 1.5D3
CC        AU       = -300.d0
CC        AD       = 1.5D3
c  special case of MA, MU input (instead of MHU2, MHD2)
CC        MA       = 1000.d0
CC        MU       = 100.d0
c      
c    At this stage you can call the main subroutine suspect:
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
       CALL suspect2(IKNOWL, INPUT, ICHOICE,ERRMESS)
c
c  (ALL relevant OUTPUT will be written in suspect.out file;
c   except if INPUT=11 chosen, and you may continue with output 
c   values within this program)
c 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c 3. !!!!!!! IMPORTANT:     WARNING/ERRORS FLAGS OPTIONS 
c  SUSPECT version 2 provides a useful set of warning/error flags,
c which are automatically written in the output file suspect.out,
c and are of course returned as output of suspect2 (see above last
c argument of suspect2(..). At the moment, this indicates:
c 
c errmess(i)= 0: Everything is fine.
c errmess(1)=-1: tachyon 3rd gen. sfermion from RGE
c errmess(2)=-1: tachyon 1,2 gen. sfermion from RGE
c errmess(3)=-1: tachyon A (maybe temporary:    see final mass) 
c errmess(4)=-1: tachyon 3rd gen. sf from mix.
c errmess(5)=-1: MU(GUT) guess inconsistent 
c errmess(6)=-1: unconvergent MU from EWSB 
c errmess(7)=-1: EWSB maybe inconsistent (!but RG-improved only check)
c errmess(8)=-1: V_Higgs maybe UFB or CCB (!but RG-improved only check)
c errmess(9)=-1: Higgs masses are NaN 
c errmess(10)=-1: RGE problems (non-pert and/or Landau poles)
c
c
c   ......sequel of your own program continues e.g. here c
c   ....
c//////////////////////////////////////////////////////////////////
c Appendix: more detailed explanations on the meaning of some
c of the ICHOICE(1)-(10): 
c***************************************************************** 
c   CHOICE OF different models: 4 different "models"/options presently 
c   available:
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c 1.  ichoice(1) = 0:  unconstrained MSSM, NO RGE
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c (only spectrum calculation: relevant input assumed to be at LOW scale);
c 
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c 2.  ichoice(1) = 1: unconstrained MSSM (i.e. non-universality) with RGE
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
c  (input are thus assumed at high scale (!except tgbeta, which is assumed 
c  to be given at MZ scale value!) )   
c
c  !CAUTION!:  ichoice(1) option is rather fragile, as it is notoriously
c  difficult to guess a (totally non-universal) HIGH-energy input that
c  will give a consistent spectrum: i.e. for a "random" choice of
c  input, one can very easily end up with either some slepton
c  and/or squark with negative mass^2, and/or inconsistent EWSB, etc...)
c Thus use it only if you are reasonably confident on the consistency of
c your input (for instance, smooth non-universal departures from the
c minimal SUGRA cases should normally work). 
c Note however that appropriate error/warning messages are returned from
c SUSPECT2 output whenever problems like tachyonic sfermions, inconsistent
c EWSB, etc appear (see more details below on ERRMESS(1)-(10) error flags)
c
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c 3.  ichoice(1) = 2: unconstrained MSSM with RGE DOWN TO UP:
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c relevant input: like ichoice(1) = 0 , but the final output is all
c the soft, etc parameters  at HIGH scale. (This may be useful,
c e.g., prior to non-universality studies with case ichoice(1)=1, where
c it is generally difficult to guess a set of high energy parameters
c that give a consistent low-energy spectrum). 
c
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c 4.  ichoice(1) = 10:  minimal SUGRA  model 
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c  The only parameters to define from the calling code (or input file) are
c  the usual mSUGRA parameters:
c  m0  universal GUT-scale scalar mass
c  mhalf    "     "    "   gaugino mass
c  A0       "     "    "   trilinear scalar coupling
c  sgnmu0  the sign of the mu(GUT) parameter
c  tgbeta : ! value at the Z scale (not GUT scale)
c all are passed via the COMMON/MSUGRA/m0,mhalf,A0,sgnmu0,tgbeta
c  (the 5 relevant input are assumed at high scale (!except tgbeta).
c
c++++++++++++++++++++++++++++++++++++++++++++++++++++
c  CHOICE OF scalar sector input: in the UNconstrained MSSM, it is often
c  convenient to choose M_A (the physical CP-odd scalar mass) and MU as
c input instead of M^2_Hu, M^2_Hd (whose consistent values are often
c difficult to guess). Such a possibility is set by:
c++++++++++++++++++++++++++++++++++++++++++
c ichoice(6) = 0   ->   MA,MU input
c++++++++++++++++++++++++++++++++++++++++++
c (NB if RG evolution is set on, MU is understood as MU(high scale))
c++++++++++++++++++++++++++++++++++++++++++
c ichoice(6)= 1    ->  M^2_Hu, M^2_Hd  input
c++++++++++++++++++++++++++++++++++++++++++
c (this is of course irrelevant in minimal SUGRA case where by def.
c it is always M^2_Hu, M^2_Hd(high scale) input)
c then in our algorithm the parameters which are not input are
c calculated consistently from electroweak symmetry breaking.
c IMPORTANT! note that for ichoice(6)=1, in all cases (unconstrained 
c MSSM with or withour RGE, etc) you have to SPECIFY the sign(MU)
c which is an EXTRA parameter (just like in minimal SUGRA case).
c
c
c end of part pasted in from supect2_call.f
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c check if the model is ok, otherwise return
c
      unphys=0
      if(errmess(1).lt.0) unphys=unphys-1
      if(errmess(2).lt.0) unphys=unphys-1
      if(errmess(3).lt.0) unphys=unphys-1
      if(errmess(4).lt.0) unphys=unphys-1
      if(errmess(5).lt.0) unphys=unphys-1
      if(errmess(6).lt.0) unphys=unphys-1
      if(errmess(7).lt.0) unphys=unphys-1
      if(errmess(8).lt.0) unphys=unphys-1
      if(errmess(9).lt.0) unphys=unphys-1
      if(errmess(10).lt.0) unphys=unphys-1
      if(unphys.lt.0) then
        return
      endif  
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c 1) connection between Suspect and Darksusy SM inputs
c
      alphem = 1.d0/dalfinv
      s2thw = dsw2
      alph3 = dalphas
      mass(ktau)   = dmtau
      mass(kt)     = dmt
      mass(kb)     = dmb
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c 2) connection between Suspect and Darksusy variables and soft terms
c
      m1=dM1
      m2=dM2
      m3=dM3
      mu=dMU
      ma=dMA
      tanbe=tgbeta
c      mqtilde=dMSQ  ! Not needed anymore
c
c   squark and sleptons mass matrices
c      
c  3rd family
      mass2q(3)=dMSQ*dMSQ
      mass2u(3)=dMTR*dMTR
      mass2d(3)=dMBR*dMBR
      mass2l(3)=dMSL*dMSL
      mass2e(3)=dMTAUR*dMTAUR
      asoftu(3)=dAU
      asoftd(3)=dAD
      asofte(3)=dAL
c  1rst and 2nd families
      do i=1,2
       mass2q(i)=dMUQ*dMUQ
       mass2u(i)=dMUR*dMUR
       mass2d(i)=dMDR*dMDR
       mass2l(i)=dMEL*dMEL
       mass2e(i)=dMER*dMER
       asoftu(i)=0
       asoftd(i)=0
       asofte(i)=0
      enddo
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c 3) full mass spectrum and mixing matrices according to Suspect output
c    (it replaces the call to dsspectrum)
c
c  rad. corrected (final) bottom, top and tau masses
c
c  this is missing in Darksusy
c
      mass(kb)=mbcr
      mass(kt)=mtcr
      mass(ktau)=mtaucr
c
c gluino mass:
      mass(kgluin)=dabs(mgluino)
c
c  sfermion sector:
c
c  entries in the vector mass from mass(ksnue) to mass(ksb2) 
c  have to be defined here.
c  mixing matrices: slulmx, sldlmx, sldrmx, squlmx, squrmx, 
c  sqdlmx, sqdrmx have to be defined as well.
c
c  in DarkSUSY the sfermion mass eigenstates are defined as:
c  sfermion_1 = sfermion_left * cos(theta_sfermion)
c               + sfermion_right * sin(theta_sfermion)
c  sfermion_2 = - sfermion_left * sin(theta_sfermion)
c               + sfermion_right * cos(theta_sfermion)
c
c  in SUSPECT the sfermion mass eigenstates are defined with
c  the first eigenstate being the lighest.
c
c  initializations:
      do g=1,3
        do i=1,3
          slulmx(i,g) = cmplx(0.d0,0.d0)
        enddo
        do i=1,6
          sldlmx(i,g) = cmplx(0.d0,0.d0)
          sldrmx(i,g) = cmplx(0.d0,0.d0)
          squlmx(i,g) = cmplx(0.d0,0.d0)
          squrmx(i,g) = cmplx(0.d0,0.d0)
          sqdlmx(i,g) = cmplx(0.d0,0.d0)
          sqdrmx(i,g) = cmplx(0.d0,0.d0)
        enddo
      enddo
      d = mass(kz)**2*(1.d0-tanbe*tanbe)/(1.d0+tanbe*tanbe) ! mz^2 cos(2beta)
c  sneutrino_electron
      mass(ksnue)=dmsn1
      slulmx(1,1)=cmplx(1.d0,0.d0)
c  selectrons:
      m2ll = mass2l(1)+d*(-0.5d0+s2thw)
      m2rr = mass2e(1)+d*(-s2thw)
      if(dabs(m2ll).lt.dabs(m2rr)) then
        mass(kse1)=dmse1
        mass(kse2)=dmse2
      else  
        mass(kse1)=dmse2
        mass(kse2)=dmse1
      endif  
      theta = 0.d0
      sldlmx(1,1) = cmplx(cos(theta),0.d0)
      sldlmx(4,1) = cmplx(-sin(theta),0.d0)
      sldrmx(1,1) = cmplx(sin(theta),0.d0)
      sldrmx(4,1) = cmplx(cos(theta),0.d0)
c  sneutrino_muon
      mass(ksnumu)=dmsn1
      slulmx(2,2)=cmplx(1.d0,0.d0)
c  smuons:      
      m2ll = mass2l(2)+d*(-0.5d0+s2thw)
      m2rr = mass2e(2)+d*(-s2thw)
      if(dabs(m2ll).lt.dabs(m2rr)) then
        mass(ksmu1)=dmse1
        mass(ksmu2)=dmse2
      else  
        mass(ksmu1)=dmse2
        mass(ksmu2)=dmse1
      endif  
      theta = 0.d0
      sldlmx(2,2) = cmplx(cos(theta),0.d0)
      sldlmx(5,2) = cmplx(-sin(theta),0.d0)
      sldrmx(2,2) = cmplx(sin(theta),0.d0)
      sldrmx(5,2) = cmplx(cos(theta),0.d0)
c  sneutrino_tau
      mass(ksnutau)=dmsntau
      slulmx(3,3)=cmplx(1.d0,0.d0)
c  staus:
      m2ll = mass2l(3)+d*(-0.5d0+s2thw)
      m2rr = mass2e(3)+d*(-s2thw)
      if(dabs(m2ll).lt.dabs(m2rr)) then
        mass(kstau1)=dmsl1
        mass(kstau2)=dmsl2
        theta = thel
      else
        mass(kstau1)=dmsl2
        mass(kstau2)=dmsl1
        theta = thel - pi/2.d0
      endif  
      sldlmx(3,3) = cmplx(cos(theta),0.d0)
      sldlmx(6,3) = cmplx(-sin(theta),0.d0)
      sldrmx(3,3) = cmplx(sin(theta),0.d0)
      sldrmx(6,3) = cmplx(cos(theta),0.d0)
c  up-squarks:
      m2ll = mass2q(1)+d*(0.5d0-2.d0/3.d0*s2thw)
      m2rr = mass2u(1)+d*(2.d0/3.d0*s2thw)
      if(dabs(m2ll).lt.dabs(m2rr)) then
        mass(ksu1)=dmsu1
        mass(ksu2)=dmsu2
      else
        mass(ksu1)=dmsu2
        mass(ksu2)=dmsu1
      endif  
      theta = 0.d0
      squlmx(1,1) = cmplx(cos(theta),0.d0)
      squlmx(4,1) = cmplx(-sin(theta),0.d0)
      squrmx(1,1) = cmplx(sin(theta),0.d0)
      squrmx(4,1) = cmplx(cos(theta),0.d0)
c  down-squarks:
      m2ll = mass2q(1)+d*(-0.5d0+1.d0/3.d0*s2thw)
      m2rr = mass2d(1)+d*(-1.d0/3.d0*s2thw)
      if(dabs(m2ll).lt.dabs(m2rr)) then
        mass(ksd1)=dmsd1
        mass(ksd2)=dmsd2
      else
        mass(ksd1)=dmsd2
        mass(ksd2)=dmsd1
      endif  
      theta = 0.d0
      sqdlmx(1,1) = cmplx(cos(theta),0.d0)
      sqdlmx(4,1) = cmplx(-sin(theta),0.d0)
      sqdrmx(1,1) = cmplx(dsin(theta),0.d0)
      sqdrmx(4,1) = cmplx(cos(theta),0.d0)
c  charm-squarks:
      m2ll = mass2q(2)+d*(0.5d0-2.d0/3.d0*s2thw)
      m2rr = mass2u(2)+d*(2.d0/3.d0*s2thw)
      if(dabs(m2ll).lt.dabs(m2rr)) then
        mass(ksc1)=dmsu1
        mass(ksc2)=dmsu2
      else  
        mass(ksc1)=dmsu2
        mass(ksc2)=dmsu1
      endif  
      theta = 0.d0
      squlmx(2,2) = cmplx(cos(theta),0.d0)
      squlmx(5,2) = cmplx(-sin(theta),0.d0)
      squrmx(2,2) = cmplx(sin(theta),0.d0)
      squrmx(5,2) = cmplx(cos(theta),0.d0)
c  strange-squarks:
      m2ll = mass2q(2)+d*(-0.5d0+1.d0/3.d0*s2thw)
      m2rr = mass2d(2)+d*(-1.d0/3.d0*s2thw)
      if(dabs(m2ll).lt.dabs(m2rr)) then
        mass(kss1)=dmsd1
        mass(kss2)=dmsd2
      else
        mass(kss1)=dmsd2
        mass(kss2)=dmsd1
      endif  
      theta = 0.d0
      sqdlmx(2,2) = cmplx(cos(theta),0.d0)
      sqdlmx(5,2) = cmplx(-sin(theta),0.d0)
      sqdrmx(2,2) = cmplx(dsin(theta),0.d0)
      sqdrmx(5,2) = cmplx(cos(theta),0.d0)
c  top-squaks:
      m2ll = mass2q(3)+d*(0.5d0-2.d0/3.d0*s2thw)
      m2rr = mass2u(3)+d*(2.d0/3.d0*s2thw)
      if(dabs(m2ll).lt.dabs(m2rr)) then
        mass(kst1)=dmst1
        mass(kst2)=dmst2
        theta = thet
      else   
        mass(kst1)=dmst2
        mass(kst2)=dmst1
        theta = thet - pi/2.d0
      endif  
      squlmx(3,3) = cmplx(cos(theta),0.d0)
      squlmx(6,3) = cmplx(-sin(theta),0.d0)
      squrmx(3,3) = cmplx(sin(theta),0.d0)
      squrmx(6,3) = cmplx(cos(theta),0.d0)
c  bottom-squarks:
      m2ll = mass2q(3)+d*(-0.5d0+1.d0/3.d0*s2thw)
      m2rr = mass2d(3)+d*(-1.d0/3.d0*s2thw)
      if(dabs(m2ll).lt.dabs(m2rr)) then
        mass(ksb1)=dmsb1
        mass(ksb2)=dmsb2
      else
        mass(ksb1)=dmsb2
        mass(ksb2)=dmsb1
      endif  
      theta = theb 
      sqdlmx(3,3) = cmplx(cos(theta),0.d0)
      sqdlmx(6,3) = cmplx(-sin(theta),0.d0)
      sqdrmx(3,3) = cmplx(dsin(theta),0.d0)
      sqdrmx(6,3) = cmplx(cos(theta),0.d0)
c
c
c  neutralino sector:
c
c  entries for the neutralino masses and for the mixing matrix 
c  neunmx(4,4) (i.e. N) have to be defined here.
c
c  in DarkSUSY the convention with positive mass eigenvalues
c  and complex N are assumed. Eigenvalues are ordered with
c  mass(kn1) being the lightest and mass(kn4) the heaviest.
c
c  in SUSPECT the convention with mass eigenvalues with both signs
c  and real mixing matrix Z is assumed.
      do i=1,4
        mass(kn(i))=dxmn(i)
        do j=1,4
          if(mass(kn(i)).ge.0.d0) then
            neunmx(i,j) = cmplx(Z(i,j),0.d0)
          else
            neunmx(i,j) = cmplx(0.d0,Z(i,j))
          endif
        enddo  
        mass(kn(i))=dabs(dxmn(i))
      enddo  
c  set the code for the lightest neutralino
c  THIS SET IS NEEDED BUT I THINK SHOULD BE REMOVED!
      kln=1
c
c  this sign come up different but this is irrelevant I think.
c      do j=1,4
c        neunmx(1,j) = - neunmx(1,j)
c        neunmx(3,j) = - neunmx(3,j)
c      enddo  
c
c
c  chargino sector:
c
c  entries for the chargino masses and for the mixing matrices
c  chaumx(2,2) (i.e. U) and chavmx(2,2) (i.e. V) have to be defined
c  here.
c
c  in DarkSUSY the mixing convention det(U)=1 and 
c  U^* M_{\chi^{\pm}} V^{dagger} = diag(mass(kcha1),mass(kcha2))
c  with non-negative chargino masses is assumed. 
c  mass(kcha1) is always heavier than mass(kcha2)
c  chaumx and chavmx are defined complex*16 although only the real 
c  part is non-zero.
c
c  in suspect the convention is instead mass(kcha1) is always 
c  lighter than mass(kcha2)
c !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
c  CHECK IF THIS IS THE RIGHT WAY TO MATCH THE SPECTRA !!!!!!!!!!
c !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
c 
      mass(kcha1)=dmc2
      mass(kcha2)=dmc1
      do i=1,2
        do j=1,2
          chaumx(3-i,j)=cmplx(U(i,j),0.d0)
          chavmx(3-i,j)=cmplx(V(i,j),0.d0)
        enddo
      enddo  
c
c
c  higgs sector:
c
c  entries in the vector mass for Higgs particles and for
c  the parameter alpha have to be defined here.
c
c  mass of the heavy CP-even state    
      mass(kh1)=dmh
c  mass of the light CP-even state
      mass(kh2)=dml
c  mass of the CP-odd state
      mass(kh3)=ma
c  mass of the charged state
      mass(khc)=dmch
c  mixing angle for the CP-even states, -Pi/2 < alpha < 0
      alpha=alfa
c
c-------------------------------------- lightest supersymmetric particle
      lsp = kn(kln)
      do i=21,48
         if (abs(mass(i)).lt.abs(mass(lsp))) lsp = i
      enddo
      if (lsp.lt.42.or.lsp.gt.45) then
         unphys=1
         write (message,*)
     &        'lsp is ',pname(lsp),
     &        ' m_lsp=',real(mass(lsp)),
     &        ' (m_x0=',real(mass(kn(kln))),')'
         call dswrite(1,1,message)
      endif
      zg = dsabsq(neunmx(1,1))+dsabsq(neunmx(1,2))
      zgm = dsabsq(neunmx(1,3))+dsabsq(neunmx(1,4))
      lgzg = log10(zg)-log10(zgm)
c
c the replacement to the call to dsspectrum ends here
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c if the lightest neutralino is not the lsp return
      if(unphys.gt.0) return
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c 4), 5), 6) and 7), i.e. other calls in dsmodelsetup and to dsprep
c
      mass(0)=1.d10
c
c------------------------------------------------------ global constants
      call dssuconst
c
c-------------------------------------------------- some useful vertices
      call dsvertx
c
c--------------------------------------------------- and particle widths

c...Add Higgs widhts and QCD correction to them and to vertices
      call dshigwid

      width(kcha(1)) = 5.0d0
      width(kcha(2)) = 5.0d0
      width(kn(1)) = 5.0d0
      width(kn(2)) = 5.0d0
      width(kn(3)) = 5.0d0
      width(kn(4)) = 5.0d0
      do i=1,6
        width(ksl(i))=1.0d0
        width(ksnu(i))=1.0d0
        width(ksqu(i))=1.0d0
        width(ksqd(i))=1.0d0
      enddo
c
c---------------------------------------------- prepare for computations
      call dsprep
c
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      return
      end
